--[[
Name: LibQuestInfo
Revision: $Revision: 1 $
Author(s): Angelike
Website:
Documentation: www.wowace.com
SVN:
Description: Information about quests
License: LGPL v2.1
]]

-- check for missing libraries
local MissingLibraries

if not LibStub then
	print("Libstub not found, LibQuestInfo not loaded.")
	return;
end

do
	local REQUIRED_LIBS = {
		"CallbackHandler-1.0",
		"LibBabble-3.0",
		"LibBabble-Zone-3.0",
		"LibBabble-LibQuestInfoHeader-3.0",
	}
	local missing

	for idx, lib in ipairs(REQUIRED_LIBS) do
		if not LibStub:GetLibrary(lib, true) then
			missing = true
			print(format("The Library %s could not be found LibQuestInfo not loaded.", lib))
		end
	end
	if missing then
		return
	end
end


local MAJOR_VERSION = "LibQuestInfo"
local MINOR_VERSION = tonumber(("$Revision: 1 $"):match("%d+")) or 0

local lib, oldMinor = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

lib.party = {}
lib.NumPartyPlayer = 0

local ZoneKeys = LibStub("LibBabble-Zone-3.0"):GetReverseLookupTable()
local ZoneNames = LibStub("LibBabble-Zone-3.0"):GetUnstrictLookupTable()
local NonZoneKeys = LibStub("LibBabble-LibQuestInfoHeader-3.0"):GetReverseLookupTable()
local NonZoneNames = LibStub("LibBabble-LibQuestInfoHeader-3.0"):GetUnstrictLookupTable()
local ClassNames, ClassKeys, questindex = {}, {}, {}

local questcache = {}
local ZoneCache = {} -- cache for questid's in zones, should only used by questeditor
local loadqueue = {}

FillLocalizedClassList(ClassNames)
for k,v in pairs(ClassNames) do
	ClassKeys[v] = k
end

-- #AUTODOC_NAMESPACE lib
local datakeys = {"O","L","M","H","Y","W","T","S"}
--[[
	Datatypes:
	Q	=	QuestID + number
	O	=	num Objectives + number
	o(#)	= # of ojectivetype
		-P	= Player
		-I 	= Item
		-M	= Monster
		-R	= Reputation
		-O	= Object
		-E	= Event
	L	= Level + number
	M = Min Level + number
	H = Higest Level + number
	Y	=	Daily
	W	=	Weekly
	T	= Tag
		P	=	PvP
		R	=	Raid
		D	=	Dungeon
		G	=	Group + number
		E	=	Elite
		H	=	Heroic
	Z	=	Zone
	S = Source + version (which is lib version number)
		S = Scan = version + 1
		P	= Client Exchange = version + 0
		E = Editor = version - 1

--]]
--[[ -- dump noprogress strings for ID Scanning
QUEST_FACTION_NEEDED_NOPROGRESS = "R";
QUEST_ITEMS_NEEDED_NOPROGRESS = "I";
QUEST_MONSTERS_KILLED_NOPROGRESS = "M";
QUEST_OBJECTS_FOUND_NOPROGRESS = "O";
QUEST_PLAYERS_KILLED_NOPROGRESS = "P";
--]]


-- Objectives Type DB
local ObjectiveTypes = {
	P = "Player",
	I = "Item",
	M = "Monster",
	R = "Reputation",
	O = "Object",
	E = "Event"
}

-- Tag Type DB
local TagTypes = {
	P	=	PVP,
	R	=	RAID,
	D	=	LFG_TYPE_DUNGEON,
	G	=	GROUP,
	E	=	ELITE,
	H	=	ITEM_HEROIC,
}

--[[ insert ZoneZable here
local QuestZones = {
	["Ironforge"] = "",
	["Alterac Mountains"] = "522, 523, 500, 537, 511, 510, 525, 514, 512, 554, 505, 551, 535",
	["Brewfest"] ="12062,12306,12420,12421,11441,11446,11318,11409,12491,12492,12020,12192,13932,13931,11117,11431,12022,12191,11122,11412,11407,11294,11293,11408,11118,11120,12318,12193,12194,11321,11413",
	["The Culling of Stratholme"] = "13243,13149,13151,",
	["Strand of the Ancients"] = "13405,",
	["Stranglethorn Vale"] = "187,188,192,193,195,196,197,208,",
	["Moonglade"] = "8735,",
	["Dragonblight"] = "12166,12055,12060,12000,12004,12006,12013,",
	["UNKNOWN"] = "1,2,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,189,190,191,194,198,199,200,201,202,203,204,205,206,207,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,224,225,226,227,228,229,230,231,232,233,234,235,237,238,239,240,243,244,245,246,247,248,249,250,251,252,253,254,255,256,257,258,261,262,263,264,265,266,267,268,269,270,271,272,273,274,275,276,277,278,279,280,281,282,283,284,285,286,287,288,289,290,291,292,293,294,295,296,297,298,299,301,302,303,304,305,306,307,308,309,310,311,312,313,314,315,317,318,319,320,321,322,323,324,325,328,329,330,331,332,333,334,335,336,337,338,339,340,341,342,343,344,345,346,347,348,349,350,351,353,354,355,356,357,358,359,360,361,362,363,364,365,366,367,368,369,370,371,372,373,374,375,376,377,378,379,380,381,382,383,384,385,386,387,388,389,391,392,393,394,395,396,397,398,399,400,401,402,403,404,405,407,408,409,410,411,412,413,414,415,416,417,418,419,420,421,422,423,424,425,426,427,428,429,430,431,432,433,434,435,436,437,438,439,440,441,442,443,444,445,446,447,448,449,450,451,452,453,454,455,456,457,458,459,460,461,464,465,466,467,468,469,471,472,473,475,476,477,478,479,480,481,482,483,485,486,487,488,489,491,492,493,494,495,496,498,499,500,501,502,503,504,505,506,507,508,509,510,511,512,513,514,515,516,517,518,519,520,521,522,523,524,525,526,527,528,529,530,531,532,533,535,536,537,538,539,540,541,542,543,544,545,546,547,549,550,551,552,553,554,555,556,557,558,559,560,561,562,563,564,565,566,567,568,569,570,571,572,573,574,575,576,577,578,579,580,581,582,583,584,585,586,587,588,589,590,591,592,593,594,595,596,597,598,599,600,601,602,603,604,605,606,607,608,609,610,611,613,614,615,616,617,618,619,620,621,622,623,624,625,626,627,628,629,630,631,632,633,634,635,637,638,639,640,641,642,643,644,645,646,647,648,649,650,651,652,653,654,655,656,657,658,659,660,661,662,663,664,665,666,667,668,669,670,671,672,673,674,675,676,677,678,679,680,681,682,683,684,685,686,687,688,689,690,691,692,693,694,695,696,697,698,699,700,701,702,703,704,705,706,707,708,709,710,711,712,713,714,715,716,717,718,719,720,721,722,723,724,725,726,727,728,729,730,731,732,733,734,735,736,737,738,739,741,742,743,744,745,746,747,748,749,750,751,752,753,754,755,756,757,758,759,760,761,762,763,764,765,766,767,768,769,770,771,772,773,775,776,777,778,779,780,781,782,783,784,785,786,787,788,789,790,791,792,793,794,795,804,805,806,808,809,812,813,815,816,817,818,819,821,822,823,824,825,826,827,828,829,830,831,832,833,834,835,836,837,838,840,841,842,843,844,845,846,847,848,849,850,851,852,853,854,855,857,858,860,861,862,863,864,865,866,867,868,869,870,871,872,873,874,875,876,877,878,879,880,881,882,883,884,885,886,887,888,889,890,891,892,893,894,895,896,897,898,899,900,901,902,903,905,906,907,908,909,910,911,912,913,914,915,916,917,918,919,920,921,922,923,924,925,926,927,928,929,930,931,932,933,934,935,936,937,938,939,940,941,942,943,944,945,947,948,949,950,951,952,953,954,955,956,957,958,959,960,961,962,963,964,965,966,967,968,969,970,971,972,973,974,975,976,977,978,979,980,981,982,983,984,985,986,990,991,992,993,994,995,996,997,998,999,1000,1001,1002,1003,1004,1005,1006,1007,1008,1009,1010,1011,1012,1013,1014,1015,1016,1017,1018,1019,1020,1021,1022,1023,1024,1025,1026,1027,1028,1029,1030,1031,1032,1033,1034,1035,1036,1037,1038,1039,1040,1041,1042,1043,1044,1045,1046,1047,1048,1049,1050,1051,1052,1053,1054,1055,1056,1057,1058,1059,1060,1061,1062,1063,1064,1065,1066,1067,1068,1069,1070,1071,1072,1073,1074,1075,1076,1077,1078,1079,1080,1081,1082,1083,1084,1085,1086,1087,1088,1089,1090,1091,1092,1093,1094,1095,1096,1097,1098,1099,1100,1101,1102,1103,1104,1105,1106,1107,1108,1109,1110,1111,1112,1113,1114,1115,1116,1117,1118,1119,1120,1121,1122,1123,1124,1125,1126,1127,1130,1131,1133,1134,1135,1136,1137,1138,1139,1140,1141,1142,1143,1144,1145,1146,1147,1148,1149,1150,1151,1152,1153,1154,1159,1160,1164,1166,1167,1168,1169,1170,1171,1172,1173,1174,1175,1176,1177,1178,1179,1180,1181,1182,1183,1184,1185,1186,1187,1188,1189,1190,1191,1192,1193,1194,1195,1196,1197,1198,1199,1200,1201,1202,1203,1204,1205,1206,1218,1219,1220,1221,1222,1238,1239,1240,1241,1242,1243,1244,1245,1246,1247,1248,1249,1250,1251,1252,1253,1258,1259,1260,1261,1262,1264,1265,1266,1267,1268,1269,1270,1271,1273,1274,1275,1276,1282,1284,1285,1286,1287,1288,1301,1302,1318,1319,1320,1321,1322,1323,1324,1338,1339,1358,1359,1360,1361,1362,1363,1364,1365,1366,1367,1368,1369,1370,1371,1372,1373,1374,1375,1380,1381,1382,1383,1384,1385,1386,1387,1388,1389,1391,1392,1393,1394,1395,1396,1398,1418,1419,1420,1421,1422,1423,1424,1425,1426,1427,1428,1429,1430,1431,1432,1433,1434,1435,1436,1437,1438,1439,1440,1442,1444,1445,1446,1447,1448,1449,1450,1451,1452,1453,1454,1455,1456,1457,1458,1459,1462,1463,1464,1465,1466,1467,1468,1469,1470,1471,1472,1473,1474,1475,1476,1477,1478,1479,1480,1481,1482,1483,1484,1485,1486,1487,1488,1489,1490,1491,1492,1498,1499,1500,1501,1502,1503,1504,1505,1506,1507,1508,1509,1510,1511,1512,1513,1514,1515,1516,1517,1518,1519,1520,1521,1522,1523,1524,1525,1526,1527,1528,1529,1530,1531,1532,1534,1535,1536,1558,1559,1560,1578,1579,1580,1581,1582,1598,1599,1618,1638,1639,1640,1641,1642,1643,1644,1645,1646,1647,1648,1649,1650,1651,1652,1653,1654,1655,1656,1657,1658,1661,1665,1666,1667,1678,1679,1680,1681,1682,1683,1684,1685,1686,1687,1688,1689,1690,1691,1692,1693,1698,1699,1700,1701,1702,1703,1704,1705,1706,1707,1708,1709,1710,1711,1712,1713,1714,1715,1716,1717,1718,1719,1738,1739,1740,1758,1778,1779,1780,1781,1782,1783,1784,1785,1786,1787,1788,1789,1790,1791,1792,1793,1794,1795,1796,1798,1799,1800,1801,1802,1803,1804,1805,1806,1818,1819,1820,1821,1822,1823,1824,1825,1838,1839,1840,1841,1842,1843,1844,1845,1846,1847,1848,1858,1859,1860,1861,1878,1879,1880,1881,1882,1883,1884,1885,1886,1898,1899,1918,1919,1920,1921,1938,1939,1940,1941,1942,1943,1944,1945,1946,1947,1948,1949,1950,1951,1952,1953,1954,1955,1956,1957,1958,1959,1960,1961,1962,1963,1978,1998,1999,2000,2018,2019,2038,2039,2040,2041,7673,7737,7801,7833,7865,7897,7929,7961,8025,8057,8121,8153,8185,8243,8307,8371,8435,8499,8563,8627,8691,8755,8819,8883,8947,9011,9075,9139,9203,9267,9331,9395,9459,9523,9587,9651,9715,9779,9843,9907,9971,10035,10099,10163,10227,10291,10355,10419,10483,10547,10611,10675,10739,10803,10867,10931,10995,11059,11123,11187,11251,11315,11379,11443,11507,11571,11635,11699,11763,11827,11891,11955,12019,12083,12147,12211,12275,12339,12403,12467,12531,12595,12659,12723,12787,12979,13043,13171,13235,13299,13363,13427,13491,13747,13811,4122,8308,4186,8436,8500,4282,8628,8692,8756,8820,4442,8948,4506,9076,9140,4602,9268,9332,9396,4730,4762,9588,9652,9716,9780,9844,9908,4986,10036,10100,10164,10228,10292,10356,10420,10484,10548,10612,10676,10740,10804,10868,10932,10996,11060,11124,11188,11252,11316,11380,11444,11508,11572,11636,11700,11764,11828,11892,11956,12020,12084,12148,12212,12276,12340,12404,12468,12532,12596,12660,12724,12788,12852,12916,12980,13044,13108,13172,13236,13300,13364,13428,13492,13556,13684,13748,13812,7002,7066,7162,7386,7482,7642,7674,7738,7802,7834,7866,7898,7930,7962,8026,8058,8122,8154,8186,8309,8373,8437,8501,8565,8629,8693,8757,2205,8885,8949,9013,9077,9141,9205,9269,9333,9397,9461,2381,9589,9653,9717,9781,9845,9909,9973,10037,10101,2541,10229,10293,10357,2605,2621,10549,10613,10677,10741,2701,10869,10933,2749,2765,11125,11189,11253,11317,11381,11445,11509,11573,11637,11701,11765,11829,11893,11957,12021,12085,12149,12213,12277,12341,12405,12469,12533,12597,12661,12725,12789,12917,13045,13109,13173,13237,13301,13365,13429,13493,13685,13749,13813,3501,3517,3565,3629,3645,3661,3741,3789,3821,3901,3981,4061,24426,4123,8310,8374,8438,8502,4283,8630,8694,8758,8822,4443,8950,4507,9078,9142,4603,9270,9334,9398,4731,4763,9590,9654,9718,9782,9846,9910,4987,10038,10102,10166,10230,10294,10358,10422,10486,10550,10614,10678,10742,10806,10870,10934,10998,11062,11126,11190,11254,11318,11382,11446,11510,11574,11638,11702,11766,11830,11894,11958,12022,12086,12150,24427,12278,12342,12406,12470,12534,12598,12662,12726,12790,12854,12918,12982,13046,13110,13174,13238,13302,13366,13430,13494,13622,13686,13750,13814,7003,7067,7163,7483,7643,7675,7803,7835,7867,7899,7931,8059,8123,24428,8187,8311,8375,8439,8503,8567,8631,8695,8759,8823,8887,8951,9015,9079,9143,9207,9271,9335,9399,9463,9527,9591,9655,9719,9783,9847,9911,9975,10039,10103,10167,10231,10295,10359,10423,10487,10551,10615,10679,10743,10807,10871,10935,10999,11063,11127,11191,11255,11319,11383,11447,11511,11575,11639,11703,11767,11831,11895,11959,12023,12087,12151,12215,12279,12343,12407,12471,12535,12599,12663,12727,12791,12855,12919,13047,13111,13175,13239,13303,13367,13431,13495,13559,13687,4124,8312,8376,8440,8504,4284,8632,8696,8760,8824,4444,8952,4508,9080,9144,4604,9272,9336,9400,4732,4764,9592,9720,9784,9848,9912,9976,10040,10104,10168,10232,10296,10360,10424,10488,10552,10680,10744,10808,10872,10936,11000,11064,11128,11192,11256,11320,11384,11448,11512,11576,11640,11704,11768,11832,11896,11960,12024,12088,12152,24431,12280,12344,12408,12472,12536,12600,12664,12728,12792,12920,12984,13048,13112,13176,13240,13304,13368,13432,13496,13688,13752,13816,14136,14200,7164,7484,7644,7676,7804,7836,7868,7900,7932,8060,8124,8156,8188,8249,2078,8377,8441,8505,8569,2158,8697,8761,2206,8889,2238,9017,9081,9145,9209,2318,9337,9401,9465,2382,2398,9657,9721,9785,9849,2478,9977,10041,10105,10169,10233,10297,10361,10425,10489,10553,10617,10681,10745,10809,10873,10937,11001,11065,11129,11193,11257,11321,11385,11449,11513,11577,11641,11705,11769,11833,11897,11961,12025,12089,12153,12217,12281,12345,12409,12473,12537,12601,12665,12729,12793,12857,13049,13113,13241,13305,13369,13433,13497,13625,13689,13817,3502,3518,3566,3630,3646,3790,3822,3902,3982,4062,4125,8314,8378,4221,8506,4285,8634,8698,4381,8826,4445,8954,4509,9146,4605,9274,9338,4701,4733,4765,9594,9658,4861,9786,9850,9914,9978,10042,10106,10170,10234,10298,10362,10426,10490,10554,10618,10682,10746,10810,10874,10938,11002,11066,11130,11194,11258,11322,11386,11450,11514,11578,11642,11706,11770,11834,11898,11962,12026,12090,12154,12218,12282,12346,12410,12474,12538,12602,12666,12730,12794,12858,12986,13114,13178,13242,13306,13370,13434,13498,6781,13690,13818,6941,14074,7101,7165,7261,7421,7485,7581,7645,7677,7805,7837,7869,7901,7933,8061,8157,8189,8251,8315,8379,8443,8507,8635,8699,8763,8827,8891,8955,9019,9083,9147,9211,9275,9339,9403,9467,9531,9595,9659,9723,9787,9851,9915,9979,10043,10107,10171,10235,10299,10363,10427,10491,10555,10619,10683,10747,10811,10875,10939,11003,11067,11131,11195,11259,11323,11387,11451,11515,11579,11643,11707,11771,11835,11899,11963,12027,12091,12155,12219,12283,12347,12411,12475,12539,12603,12667,12795,12859,13115,13179,13307,13371,13435,13499,13627,13691,13819,14203,4126,8316,8380,4222,8508,4286,8636,8700,4382,8828,4446,8956,4510,4542,9148,4606,9276,9340,9404,4734,4766,9596,9660,4862,9788,9852,9916,9980,10044,10108,10172,10236,10300,10364,10428,10492,10556,10620,10684,10748,10812,10876,10940,11004,11068,11132,11196,11260,11324,11388,11452,11516,11580,11644,11708,11772,11836,11900,11964,12028,12092,12156,12220,12284,12348,12412,12476,12540,12604,12668,12732,12796,12860,12988,13052,13116,13180,13308,13372,13436,13500,13692,13820,6942,14076,14140,7102,7166,7422,7486,7582,7646,7678,7806,7838,7870,7902,7934,8062,8158,8190,8253,8317,8381,8509,8573,2159,8701,8765,8829,8893,2239,9021,9085,9149,9213,9277,9341,9405,9469,2383,2399,9661,9725,9789,9853,2479,9981,10045,10109,10173,10237,10301,10365,10429,10493,10557,10621,10685,10749,10813,10877,10941,11005,11069,11133,11197,11261,11325,11389,11453,11517,11581,11645,11709,11773,11837,11901,11965,12029,12093,12157,12221,12285,12349,12413,12477,12541,12605,12669,12733,12797,12861,13053,13117,13245,13309,13373,13437,13501,13629,13693,13757,13821,3503,14077,14141,3567,3631,3647,3791,3823,3903,4063,24442,4127,8318,8382,4223,8510,4287,8638,8702,4383,8830,4447,8958,4511,9086,9150,9214,9278,9342,9406,4735,4767,9598,9662,4863,9790,9854,9918,9982,10046,10110,10174,10238,10302,10366,10430,10494,10558,10622,10686,10750,10814,10878,10942,11006,11070,11134,11198,11262,11326,11390,11454,11518,11582,11646,11710,11774,11838,11902,11966,12030,12094,12158,12222,12286,12350,12414,12478,12542,12606,12670,12734,12798,12862,12926,12990,13054,13118,13182,13246,13310,13374,13438,13502,13694,13758,13822,6943,13950,14142,7167,7423,7487,7583,7647,7807,7839,7871,7903,7935,8063,8159,8191,8255,8319,8383,8447,8511,8575,8639,8703,8767,8831,8895,8959,9023,9087,9151,9279,9343,9407,9471,9535,9663,9727,9791,9855,9919,9983,10047,10111,10175,10239,10303,10367,10431,10495,10559,10623,10687,10751,10815,10879,10943,11007,11071,11135,11199,11263,11327,11391,11455,11519,11583,11647,11711,11775,11839,11903,11967,12031,12095,12159,12223,12287,12351,12415,12479,12543,12607,12671,12735,12799,12863,12927,12991,13055,13119,13183,13247,13311,13375,13439,13503,13631,13695,13759,13823,13887,13951,14079,14143,8192,4128,8320,8384,4224,8512,4288,8640,8704,4384,8832,4448,8960,4512,9088,9152,9216,9280,9344,9408,4736,4768,9600,9664,4864,9792,9856,9920,9984,10048,10112,10176,10240,10304,10368,10432,10496,10560,10624,10688,10752,10816,10880,10944,11008,11072,11136,11200,11264,11328,11392,11456,11520,11584,11648,11712,11776,11840,11904,11968,12032,12096,12160,12224,12288,12352,12416,12480,12544,12608,12672,12736,12800,12864,12928,12992,13056,13120,13184,13248,13312,13376,13440,13696,13760,13824,13952,14016,14080,14144,7168,7424,7488,7648,7808,7840,7872,7936,8064,8160,8193,8257,8321,8385,8513,8577,2160,8705,8769,8833,8897,2240,9025,9089,9153,9217,9281,9345,9409,9473,9537,9601,9665,9729,9793,9857,2480,9985,10049,10113,10177,10241,10305,10369,10433,10497,10561,10625,10689,10753,10817,10881,10945,11009,11073,11137,11201,11265,11329,11393,11457,11521,11585,11649,11713,11777,11841,11905,11969,12033,12097,12161,12225,12289,12353,12417,12481,12545,12609,12673,12737,12801,12865,12929,12993,13121,13185,13249,13313,13377,13441,3376,13633,13697,13761,13825,13889,14017,14081,14145,3568,3632,3792,3824,3904,8194,4129,4161,8386,8514,4289,4321,8706,4385,8834,4449,4481,4513,9090,9154,9218,4641,9346,9410,4737,4769,4801,9666,4865,9794,9858,4961,9986,10050,10114,10178,10242,10306,10370,10434,10498,5281,10626,10690,10754,10818,10882,10946,11010,11074,11138,11202,11266,11330,11394,11458,5761,11586,11650,11714,11778,11842,11906,11970,12034,12098,12162,12226,12290,12354,12418,12482,12546,12610,12674,12738,12802,12866,12930,12994,13058,13122,13250,13314,13378,13442,13634,13762,13826,13954,14082,7169,7201,7361,7425,7489,7521,7649,7681,7809,7841,7873,7905,7937,8001,8065,8161,8195,8259,8323,8387,8515,8579,8643,8707,8771,8835,8899,8963,9027,9091,9155,9219,9283,9347,9411,9475,9539,9603,9667,9731,9795,9859,9923,9987,10051,10115,10179,10243,10307,10371,10435,10499,10563,10627,10691,10755,10819,10883,10947,11011,11075,11139,11203,11267,11331,11395,11459,11523,11587,11651,11715,11779,11843,11907,11971,12035,12099,12163,12227,12291,12355,12419,12483,12547,12611,12675,12739,12803,12867,12931,12995,13059,13123,13187,13251,13315,13379,13443,13571,13699,13763,13827,13955,14083,14147,8196,4130,8324,8388,8516,4290,4322,8708,4386,8836,4450,4482,9028,9092,9156,4642,9348,9412,4738,4770,4802,9668,4866,9796,9860,4962,9988,10052,10116,10180,10244,10308,10372,10436,10500,10564,10628,10692,10756,10820,10884,10948,11012,11076,11140,11204,11268,11332,11396,11460,11524,11588,11652,11716,11780,11844,11908,11972,12036,12100,12164,12228,12292,12356,12420,12484,12548,12612,12676,12740,12804,12868,12932,13060,13124,13188,13316,13380,13444,13700,13764,13828,13956,14084,14148,7170,7202,7362,7426,7490,7522,7650,7682,7810,7842,7874,7906,7938,8002,8066,8162,8261,8325,8389,8517,8581,2161,8709,8773,8837,8901,2241,9029,9093,9157,9221,9285,9349,9413,9477,9541,9605,9669,9733,9797,9861,9925,9989,10053,10117,10181,10245,10309,10373,10437,10501,10565,10629,10693,10757,10821,10885,10949,11013,11077,11141,11205,11269,11333,11397,11461,11525,11589,11653,11717,11781,11845,11909,11973,12037,12101,12165,12229,12293,12357,12421,12485,12549,12613,12677,12741,12805,12869,12933,13189,13253,13317,13381,13445,3377,13701,13765,13829,13957,3505,14085,14149,3569,3601,3633,3681,3761,3825,3841,3905,3921,4001,4081,4131,8326,8390,8518,4291,8710,8774,8838,4451,4483,9030,9094,9158,9222,9286,9350,9414,4739,4771,4803,9670,4867,9798,9862,4963,9990,10054,10118,10182,10246,10310,10374,10438,10502,10566,10630,10694,10758,10822,10886,10950,11014,11078,11142,11206,11270,11334,11398,11462,11526,11590,11654,11718,11782,11846,11910,11974,12038,12102,12230,12294,12358,12422,12486,12550,12614,12678,12742,12806,12870,12934,12998,13126,13254,13318,13382,13446,13702,14022,14086,14150,7171,7363,7427,7491,7651,7811,7843,7875,7907,7939,8067,8163,8263,8327,8391,8519,8583,8647,8711,8775,8839,8903,8967,9031,9095,9159,9223,9287,9351,9415,9479,9543,9607,9671,9735,9799,9863,9927,9991,10055,10119,10183,10247,10311,10375,10439,10503,10567,10631,10695,10759,10823,10887,10951,11015,11079,11143,11207,11271,11335,11399,11463,11527,11591,11655,11719,11783,11847,11911,11975,12039,12103,12167,12231,12295,12359,12423,12487,12551,12615,12679,12743,12807,12871,12935,12999,13127,13191,13319,13383,13447,13703,13767,13959,14023,14087,14151,5242,5507,5306,5148,6562,8957,6147,5213,5891,6843,8892,6363,8894,9922,5150,2751,9536,9734,9926,10997,9724,9921,5245,13682,4132,8328,8392,9928,8520,4292,4324,8712,8776,8840,8904,4484,9032,9096,9160,9224,9288,9352,9416,4740,9544,4804,9672,9736,9800,9864,4964,9992,10056,10120,10184,10248,10312,10376,10440,10504,10568,10632,10696,10760,10824,10888,10952,11016,11080,11144,11208,11272,11400,11464,11528,11592,11656,11720,11784,11848,11912,11976,12040,12104,12168,12232,12296,12360,12424,12488,12552,12616,12680,12744,12808,12872,12936,13000,13128,13192,6628,13320,13384,13448,5889,2942,9722,13704,13768,13362,13960,14024,14088,14152,9924,12214,7172,13170,8968,13042,9464,2750,7364,9478,7428,12786,7492,12722,8953,2878,3441,7652,3281,12402,2992,5405,7812,7844,7876,7908,7940,5505,12146,12082,8068,2498,11017,8164,8201,8265,8329,2098,2882,8521,8585,8649,2178,8777,8841,8905,2242,2258,9097,9161,9225,9289,2338,9417,9481,9545,9609,2418,9737,9801,9865,9929,9993,10057,10121,10185,10249,10313,10377,10441,10505,10569,10633,10697,10761,10825,10889,10953,2754,2770,11145,11209,11273,11337,2850,2866,11529,11593,11657,2930,2946,2962,2978,2994,12041,12105,3042,12233,12297,12361,12425,12489,12553,12617,12681,12745,12809,12873,12937,13001,13065,13129,13193,13257,13321,13385,13449,3378,5088,13641,13705,13769,13833,11762,3421,3506,14089,8393,5724,3570,3104,14409,11442,3634,11378,11314,11250,9526,11122,11058,3088,3762,10930,8252,5401,6081,3842,8572,5305,8580,3906,3922,5241,8970,5145,4002,9930,5081,5049,12042,4082,4101,4133,8330,8394,4261,4293,8650,8714,8778,4421,8906,4485,9034,9098,4581,9226,9290,9354,9418,4741,9546,4805,9674,9738,4901,9866,4965,9994,10058,10122,10186,10250,10314,10378,10442,10506,10570,10634,10698,10762,10826,10954,11018,11082,11146,11210,11274,11338,11402,11466,11530,11594,11658,11722,11786,11850,11914,11978,6021,12106,12170,12234,12298,12362,12426,12490,12554,12618,12682,12746,12810,12874,12938,13002,13066,13194,13258,13322,13386,13450,9970,9273,6821,13706,13770,13834,4921,6981,5059,14090,11401,9732,7141,3442,9522,9458,5157,7301,2861,7365,3117,7429,7461,7493,9202,5924,9608,7621,7653,9010,9673,8882,7781,7813,7845,7877,9728,7941,5091,5536,3373,8069,8101,8562,8165,5893,8267,8331,8395,8523,8587,8651,8715,8779,8843,8907,9035,9099,9163,9227,9291,9355,9419,9483,9547,9611,9675,9739,9803,9867,9931,9995,10059,10123,10187,10251,10315,10379,10443,10507,10571,10635,10699,10763,10827,10891,10955,11019,11083,11147,11211,11275,11339,11403,11467,11531,11595,11659,11723,11787,11851,11915,11979,12043,12107,12171,12235,12299,12363,12427,12491,12555,12619,12683,12747,12811,12875,12939,13003,13067,13195,13259,13323,13387,13451,8370,13643,13707,13771,13835,3375,8242,9802,2846,9402,11913,9540,8961,8641,8965,6075,9593,9472,6622,5214,6144,5631,5730,3602,9482,5250,8898,2607,5054,6082,5155,8762,9532,9530,9462,9020,8522,4806,4902,4966,5158,12044,12684,4102,4134,8332,8396,8460,4262,4294,8652,8716,8780,8844,8908,4486,9036,9100,9164,9228,9292,9356,9420,4742,9548,9612,9676,9740,9804,9868,9932,9996,10060,10124,10188,5126,10316,10380,10444,10508,10572,10636,10700,10764,10828,10892,10956,11020,11084,11148,11212,11276,11404,11468,11532,11596,11660,11724,11788,11852,11916,11980,6022,12108,12172,12236,12300,12364,12428,12492,12556,12620,6342,12748,12812,12876,12940,13004,13068,13196,13260,13324,13388,13452,8250,6148,6822,13708,13772,13836,3118,6982,14028,14092,3452,9468,7142,2561,6394,3085,3372,7302,9606,7366,9484,5089,7462,7494,7045,6561,8582,7622,7654,5055,6362,7782,7814,7846,6395,2941,7942,2339,9997,3091,8070,24216,3107,8166,3116,8269,8333,8397,8461,8525,8589,8653,8717,8781,8845,8909,2259,9101,9165,9229,9293,9357,9421,9485,9549,9613,9677,9741,9805,9869,9933,2499,10061,10125,10189,10253,10317,10381,10509,10573,10637,10701,10765,10829,10893,10957,2755,2771,11149,11213,11277,11341,2851,2867,11533,11597,11661,2931,2947,2963,11917,11981,12045,24217,12173,12237,12301,12365,12429,12493,12557,12621,12685,12749,12813,12877,12941,13069,13133,13197,13261,13325,13389,13453,3379,3100,3084,13709,13773,13837,9033,9082,3507,3523,9018,11469,9208,2988,2972,6145,3635,2940,2924,2927,9353,2876,11441,2844,3763,8962,5635,6083,3843,2764,2748,8764,3907,3923,9614,9934,5159,5511,4003,12046,24218,6567,4083,4103,4135,8334,8398,8462,4263,4295,8654,8718,8782,8846,8910,4487,9038,9102,9166,9230,9294,9358,9422,4743,9550,4807,9678,9742,9806,9870,4967,9998,10062,10126,10190,10254,10318,10382,10446,10510,10574,10638,10702,10766,10830,10894,10958,11022,11086,11214,11278,11342,11406,11470,11534,11598,11662,11726,11790,11854,11918,11982,6023,12110,12174,12238,12302,12366,12430,12494,12558,12622,12686,12750,12814,12878,6503,13070,13134,13198,13262,13326,13390,13454,10417,2721,6823,13710,13774,13838,8713,13966,14030,4903,6241,5927,6076,9284,6086,2460,5156,6401,7367,9282,9521,7463,7495,2768,3182,9206,7623,7655,8900,9201,9137,7783,7815,7847,3520,6563,7943,5630,8817,8071,24220,5886,8167,8271,8335,8399,8463,8527,8591,8655,8719,8783,8847,8911,9039,9103,9295,9359,9423,9487,9551,9615,9679,9743,9807,9871,9935,9999,10063,10127,10191,10255,10319,10383,10447,10511,10575,10639,10703,10767,10831,10895,11023,11087,11151,11215,11279,11343,11407,11471,11535,11599,11663,11727,11791,11855,11919,11983,12047,12111,12175,12239,12303,12367,12431,12495,12559,12623,12687,12751,12815,12879,12943,13007,13071,13199,13263,13327,13391,13455,8102,9528,9204,13711,13775,13839,13903,8969,3121,14095,6522,9162,5251,5153,9542,2606,6181,11061,11725,5821,9024,6146,9470,5763,8890,8766,2702,6042,9466,5729,5503,9212,5402,8272,8528,8592,4488,9808,10128,5160,5544,24222,12112,4104,4136,8336,8400,8464,4264,4296,8656,8720,8784,8848,8912,9040,9104,9168,9232,9360,9424,9488,9552,9616,9680,9744,4904,9872,4968,10000,10064,5064,10192,10256,10320,10384,10448,10512,10576,10640,10704,10768,10832,10896,10960,11088,11152,11216,11280,11344,11408,11472,11536,11600,11664,11728,11792,11856,11920,11984,12048,24223,6088,12240,12304,12368,12432,12496,12560,12624,12688,12752,12816,12880,12944,13008,13072,13136,13200,13264,13328,13392,13456,2973,9022,6824,11465,13776,13840,13904,6984,14032,14096,14160,6364,5063,5728,9476,5861,5981,5284,3519,7368,9474,8104,2260,7496,9361,10001,11025,7624,7656,2772,13168,2852,7784,7816,7848,11729,11793,7944,11921,24225,12784,8072,24224,12497,8168,13777,8273,8401,8465,8529,8593,8657,8721,8785,8849,8913,8977,9041,9105,9169,9233,2340,9425,9489,9553,9617,9681,9809,9873,9937,2500,10065,10129,10193,10257,10321,10385,10449,10513,10577,10641,10705,10769,10833,10897,10961,2756,11089,11153,11217,11281,11345,11409,11537,11601,11665,2932,2948,11857,2980,11985,12049,12113,12177,12241,12305,12369,12433,3124,12561,12625,12689,12753,12817,12881,12945,13073,13137,13201,13265,13329,13393,13457,3380,14033,13649,13713,3444,8530,13905,8594,3508,3524,8978,9362,12272,9618,12144,4841,3636,9810,9938,11888,5001,10130,11696,5673,3764,11602,11858,5961,11376,3844,11312,11248,12050,3908,3924,6121,11056,6185,10928,4004,12562,13138,24226,13202,4084,4105,8274,8338,4201,8466,4265,4297,8658,4361,8786,8850,8914,4489,4521,9106,9170,9234,4681,9426,9490,9554,4809,9682,9746,9874,4969,10002,10066,5065,10194,10258,10322,10386,10450,10514,10578,10642,10706,10770,10834,10898,10962,11026,11090,11154,11218,11282,11346,11410,11474,11538,5801,11666,11730,11794,5929,11922,11986,6025,12114,12178,12242,12306,12370,12434,12498,6281,12626,12690,12754,12818,12882,12946,13074,6569,6601,13266,13330,13394,13458,6761,7044,6825,13714,13778,6921,13906,6985,8884,7081,9026,5144,7241,3123,8821,5048,8446,7401,9968,8644,7497,5215,7561,7046,7625,7657,2945,7721,5501,7785,7817,7849,7881,5342,7945,7042,9456,8041,8073,8105,8264,8169,6183,8275,8339,8403,8467,8531,8595,8659,8723,8787,8851,8915,8979,9043,9107,9171,9235,9299,9363,9427,9491,9555,9619,9683,9747,9811,9875,9939,10003,10067,10131,10195,10259,10323,10387,10451,10515,10579,10643,10707,10771,10835,10899,10963,11027,11091,11155,11219,11283,11347,11411,11475,11539,11603,11667,11731,11795,11859,11923,11987,12051,12115,12179,12243,12307,12371,12435,12499,12563,12627,12691,12755,12819,12883,12947,13075,13203,13267,13331,13395,13459,5637,11085,6983,13715,13779,13843,6365,9008,14035,8842,14163,7043,6568,8524,9014,5636,5884,6566,8368,5383,8240,5535,3108,5504,5928,9012,8268,5534,5533,5532,5382,8270,8262,8442,11081,8532,8980,9620,9684,10772,11284,11860,13524,4106,8276,8340,8404,8468,4266,4298,8660,4362,8788,8852,8916,4490,9044,9108,9172,9236,9300,9364,9428,9492,9556,4810,4842,9748,4906,9876,9940,10004,10068,5066,10196,10260,5162,10388,10452,10516,10580,10644,10708,5386,10836,10900,10964,11028,11092,11156,11220,5642,11348,11412,11476,11540,11604,11668,11732,11796,5930,11924,11988,12052,12116,12180,12244,12308,12372,12436,12500,12564,12628,12692,12756,12820,12884,12948,13012,13076,13140,13204,13268,13332,13396,13460,6762,5149,6826,13716,13780,6922,13908,8402,14036,14100,14164,5926,8313,5925,6442,6441,8564,8570,5097,5638,7402,5531,5530,7498,2341,7562,2501,7626,7658,2741,7722,7786,7818,7850,7882,11861,7946,2981,2997,8042,8074,8106,12501,8170,12565,8277,8341,8405,8469,8533,8597,8661,8725,8789,8853,8917,8981,9045,9109,9173,9237,9301,9365,9429,9493,9557,9621,9685,9749,9813,9877,9941,10005,10069,10133,10197,10261,2581,10389,10453,10517,10581,2661,10709,10773,10837,10901,10965,2757,11093,11157,11221,11285,11349,11413,11477,11541,11605,11669,11733,2949,2965,11925,11989,12053,12117,12181,12245,12309,3093,3109,3125,3141,12629,12693,12757,12821,12885,12949,13013,13077,13205,13269,13333,13397,13461,3381,8772,5092,13717,13781,13845,5050,14037,3509,14101,3541,8902,8646,8633,5381,3621,3637,5096,6402,5094,3701,6722,4267,3765,3781,4363,4811,3845,3861,4907,4971,3909,11030,3941,5643,11606,5931,4005,4021,6123,12374,6571,6603,4107,8278,8342,8406,8470,8534,8598,8662,8726,8790,8854,8918,4491,9046,9110,9174,9238,9302,9366,9430,9494,9558,9622,9686,9750,9814,9878,9942,10006,10070,10134,10198,10262,10326,10390,10454,10518,10582,10646,10710,10774,10838,10902,10966,5515,11094,11158,11222,11286,11350,11414,11478,11542,5803,11670,11734,11798,11862,11926,11990,12054,12118,12182,12246,12310,6187,12438,12502,12566,12630,12694,12758,12822,12950,13014,13078,13142,13206,13270,13334,13398,13462,5086,5085,13654,13718,13782,13846,5084,5083,5082,14102,14166,5181,6344,3221,8103,5057,6282,6461,3364,6881,8596,5052,5407,7499,6026,7563,8648,7627,7659,2773,7723,6504,7787,7819,3461,7883,5095,6501,6462,5220,8043,8075,8107,5513,8171,6403,8279,8343,8407,8471,8535,8599,8663,8727,8791,8855,8919,8983,9047,9111,9175,9239,9303,9367,9431,9495,9559,9623,9687,9751,9815,9879,9943,10007,10071,10135,10199,10263,10327,10391,10455,10519,10583,10647,10711,10775,10839,10903,10967,11031,11095,11159,11223,11287,11351,11415,11479,11543,11607,11671,11735,11799,11863,11927,11991,12119,12183,12247,12311,12375,12439,12503,12567,12631,12695,12759,12823,12887,12951,13015,13079,13143,13271,13335,13399,13463,6024,5225,8256,13719,13783,13847,8254,8246,14103,14167,2769,3181,5227,8886,5223,5090,5882,6602,8982,5922,5923,8526,9084,24221,9042,5762,2979,5627,5888,5219,5662,5221,2975,5224,2864,8600,4492,9624,5068,5228,11352,5804,6028,6284,4108,8280,8344,8408,8472,8536,4300,8664,8792,8856,8920,8984,9048,9112,9176,9240,9304,9368,9432,9496,9560,4812,9688,9752,9816,9880,4972,10008,10072,10136,10200,10264,10328,10392,10456,10520,10584,10648,10712,10776,10840,10904,10968,11032,11096,11160,11224,11288,5676,11416,11480,11544,11608,11672,11736,11800,11864,11928,11992,12056,12120,12184,12248,12312,12376,12440,12504,12568,12632,12696,12760,12824,12888,12952,13016,13080,13144,6604,13272,13336,13400,13464,24219,13592,5892,13720,13784,5663,5249,5062,14040,14104,14168,8574,5887,5248,5601,6721,6847,5246,3119,5406,5226,5212,7500,5056,7564,6283,7628,7660,5408,7724,2118,7788,7820,2278,7884,9369,2518,2582,10969,8044,8076,8108,2902,2966,12441,8281,8345,8409,8473,8537,8601,8665,8729,8793,8857,8921,8985,9049,9113,9177,9241,9305,2342,9433,9497,9561,9625,9689,9753,9817,9881,9945,10009,10073,10137,10201,10265,10329,10393,10457,10521,10585,10649,10713,10777,10841,10905,2742,11033,11097,11161,11225,2822,11353,11417,11481,11545,11609,11673,11737,11801,11865,11929,11993,12057,12121,12185,12249,12313,12377,3110,3126,12569,12633,12697,12761,12825,12889,13017,13081,13145,3366,13273,13337,13401,13465,3382,13593,14169,13721,13785,3462,8282,8602,14041,14105,3542,8986,9242,9562,3638,4813,13166,4941,3702,4973,5101,5421,11162,3782,12782,5645,11354,11482,5805,11802,12058,6125,3942,12270,12206,12142,6221,4022,12570,12762,11886,6541,4109,4141,8346,8410,8474,8538,4301,8666,8730,8794,8858,8922,4493,9050,9114,9178,4621,9306,9370,9434,9498,4781,9626,9690,13210,9818,9882,9946,10010,10074,10138,10202,10266,10330,10394,10458,10522,10586,10650,10714,10778,10842,10906,10970,11034,11098,5581,11226,11290,5677,11418,5741,11546,11610,11674,11738,5901,11866,11930,11994,6029,12122,12186,12250,12314,12378,12442,12506,6285,12634,12698,6381,12826,12890,12954,13018,13082,13146,6605,13274,13338,13402,13466,11694,5514,6861,13722,13786,13850,13914,6085,7021,14106,14170,11374,7181,11310,11246,6089,5542,7341,11054,5229,10926,5863,7501,5545,6701,5932,7629,7661,7725,5058,7789,7821,5538,7885,6623,7981,5143,8045,8077,8109,8141,8825,5727,8283,8347,8411,8475,8539,8603,8667,8731,8795,8859,8923,8987,9051,9115,9179,9243,9307,9371,9435,9499,9563,9627,9691,5047,9819,9883,9947,10011,10075,10139,10203,10267,10331,10395,10459,10523,10587,10651,10715,10779,10843,10907,10971,11035,11099,11163,11227,11291,11355,11419,11483,11547,11611,11675,11739,11803,11867,11931,11995,12059,12123,12187,12251,12315,12379,12443,12507,12571,12635,12699,12763,12827,12891,12955,13019,13083,13147,13211,13275,13339,13403,13467,9966,6844,13723,13787,13851,13915,6662,14043,14107,14171,6848,5537,5283,5163,5023,9454,3102,2982,3101,5161,5626,5165,9006,3103,3094,3445,8686,2993,4142,8924,8366,4494,8238,10460,5262,11164,5646,11484,12764,6542,13212,13404,4110,8284,8348,8412,8476,8540,8604,8668,8732,8796,8860,4462,8988,9052,9116,9180,9244,9308,9372,9436,9500,4782,9628,9692,9756,9820,9884,9948,10012,10076,10140,5102,10268,10332,10396,5230,10524,10588,10652,10716,10780,10844,10908,10972,5518,11100,5582,11228,11292,5678,11420,5742,11548,11612,11676,11740,5902,11868,11932,11996,6030,12124,12188,12252,12316,12380,12444,12508,12572,12636,12700,6382,12828,12892,13020,13084,13148,6606,13276,13340,13468,11021,5441,5244,13724,13788,13852,13916,6626,7022,14108,14172,5321,5261,5384,5022,5051,2863,7342,8372,5061,7502,3510,5252,6184,7630,7662,5124,7726,2279,7822,2359,7886,2743,2983,11997,3095,8046,8078,8110,8142,12509,8221,8285,8349,8477,8541,8605,8669,8733,2199,8861,8925,8989,9053,9117,9181,9245,9309,9373,9437,9501,9565,9629,9693,2439,9821,9885,9949,10013,10077,10141,10205,10269,10333,10397,10461,10525,10589,10653,10717,10781,10845,10909,10973,2759,11101,11165,11229,11293,11357,11421,11485,11549,11613,11677,11741,11805,11869,11933,2999,12061,12125,12189,12253,12317,12381,3127,12573,12637,12701,12765,12829,12893,12957,13021,13085,13213,13277,13341,13469,13789,13725,3447,3463,13917,9757,3511,3527,14173,13933,5725,6627,6061,3639,5121,8222,4143,3371,8926,8990,9950,5103,3783,10334,5231,10526,5519,11294,11358,11422,3911,5903,6031,6063,6127,3115,3099,4023,3083,12766,13086,6607,4111,8286,8350,8414,8478,8542,8606,8670,8734,8798,8862,4463,4495,9054,9118,9182,9246,9310,9374,9438,9502,9566,9630,9694,9758,9822,9886,4975,10014,10078,10142,10206,10270,5167,10398,10462,5263,10590,10654,10718,10782,10846,10910,10974,11038,11102,11166,11230,5647,5679,11486,11550,11614,11678,11742,11806,11870,11934,11998,12126,12190,12254,12318,12382,12446,12510,12574,12638,12702,6383,12830,12894,13022,6543,13150,13214,13278,13342,13406,13470,5466,5164,13662,13726,13790,2987,7023,14174,11757,11693,6027,5517,11501,11437,11373,6624,5123,7082,8724,11053,7503,10989,6661,5151,7631,7663,7727,7791,7823,2998,7887,5541,2758,6124,10413,8047,8079,8111,8143,3092,8223,8287,8351,8415,8479,8543,8607,8671,8799,8863,8927,8991,9055,9119,9183,9247,9311,9375,9439,9503,9567,9631,9695,9759,9823,9887,9951,10015,10079,10143,10207,10271,10335,10399,10463,10527,10591,10655,10719,10783,10847,10911,10975,11039,11103,11167,11231,11295,11359,11423,11487,11551,11615,11679,11743,11807,11871,11935,11999,12063,12127,12191,12255,12319,12383,12447,12511,12575,12639,12703,12767,12895,13023,13087,13215,13279,13343,13407,13471,5885,5341,13663,13727,13791,13855,6122,10093,14047,14111,14175,6621,4783,2459,5253,7070,5802,3443,9517,5723,6629,8224,8288,9197,9133,4464,8992,9568,9952,8813,5168,5232,5264,8557,11296,11360,11424,11808,12064,6064,6128,13216,4112,4144,8352,8416,8480,8544,8608,8672,8736,8800,8864,8928,4496,9056,9120,9184,9248,9312,9376,9440,9504,4784,9632,9696,9760,9824,9888,4976,10016,10080,10144,10208,10272,10336,10400,10464,10528,10592,10656,10720,10784,10848,10912,10976,11040,11104,11168,11232,5648,5680,11488,11552,11616,11680,11744,5904,11872,11936,6032,12128,12192,12256,12320,12384,12448,12512,12576,12640,12704,6384,12896,13024,6544,13152,6608,13280,13344,13408,13472,2870,13600,13664,13728,13088,13856,10325,7024,14048,14112,14176,2869,2865,5282,2753,3363,8642,2995,3201,2853,5658,12169,8896,7504,2990,2280,2360,7632,7664,2440,7728,11041,7792,7824,11425,7888,11745,11937,3000,3112,8048,8080,8112,8144,12513,8225,8289,8353,8417,8481,8545,8609,8673,8737,8801,8865,8929,8993,9057,9121,9185,9249,9313,9377,9441,9505,9569,9633,9697,9761,9825,9889,9953,10017,10081,10145,10209,10273,2584,10401,10465,10529,10593,10657,10721,10785,10849,10913,2744,2760,11105,11169,11233,11297,11361,2856,11489,11553,11617,11681,2936,2952,2968,2984,12001,12065,12129,12193,12321,12385,12449,3128,12577,12641,12705,12769,12833,12897,12961,13025,13089,13153,13217,13281,13345,13409,13473,3448,13665,13729,13793,13857,3512,3528,14177,2849,11721,5343,3640,4113,4145,8802,8930,4721,4785,4881,13420,3784,10082,13292,13164,10466,5265,10722,3912,11042,11298,12780,5713,5841,6001,4024,12258,6321,12770,8226,8290,8354,8418,4241,8546,8610,8674,8738,4401,8866,4465,8994,9058,9122,9186,9250,9314,9378,9442,9506,9570,9634,9698,9762,9826,9890,9954,10018,5041,10146,10210,10274,10338,5201,5233,10530,10594,10658,5361,10786,10850,10914,10978,5521,11106,11170,11234,5649,11362,11426,11490,11554,11618,11682,11746,11810,11874,11938,12002,12066,6065,12194,6129,6161,12386,12450,12514,12578,12642,12706,6385,12834,12898,6481,13026,13090,13154,6609,13282,13346,13410,13474,13538,6801,13666,12268,13794,13858,6961,13986,7025,12204,14178,7121,12140,13218,6545,7281,11884,5403,2991,11692,7441,6641,7505,6625,7601,7633,7665,11849,7729,7761,7793,7825,11372,7889,7921,11308,11244,11029,8049,8081,8113,8145,3090,8227,8291,8355,8419,8483,8547,8611,8675,8739,8803,8867,8931,8995,9059,9123,9187,9251,9315,9379,9443,9507,9571,9635,9699,9763,9827,9891,9955,10019,10083,10147,10211,10275,10339,10403,10467,10531,10595,10659,10723,10787,10851,10915,10979,11043,11107,11171,11235,11299,11363,11427,11491,11555,11619,11683,11747,11811,11875,11939,12003,12067,12131,12195,12259,12323,12387,12451,12515,12579,12643,12707,12771,12835,12899,13027,13091,13155,13219,13283,13347,13411,13475,13539,13603,13667,13731,13795,13859,11052,6142,14051,5462,14179,5021,2519,10604,12216,5238,5206,2520,10284,5675,4114,10092,4146,9964,4242,4466,4786,9764,9644,10404,11044,9452,5650,11364,5842,6002,6066,6162,9004,12644,6386,13092,6610,8228,8292,8356,8420,8484,8548,8612,8676,8740,8804,8868,8932,8996,9060,9124,9188,9252,9316,9380,9444,9508,9572,9636,9700,4882,9828,9892,9956,10020,10084,10148,10212,10276,10340,5202,10468,10532,10596,10660,10724,10788,10852,10916,10980,5522,11108,11172,11236,11300,11428,11492,11556,11620,11684,11748,11812,11876,11940,12068,12132,12196,12260,12324,12388,12452,12516,12580,6322,12708,12772,12836,6482,13028,6546,13156,13220,13284,13348,13412,13476,8684,13604,13668,13732,3087,13860,6962,10485,7026,12964,14180,7122,5060,4182,14436,12109,7282,8236,2926,2904,2879,3374,7506,2201,2281,7602,7634,7666,2361,7730,2585,7794,7826,7890,7922,2937,11877,2985,8050,3001,8114,8146,12645,8229,8293,8357,8421,8485,8549,8613,8677,8741,8805,8869,8933,8997,9061,9125,9189,9253,9317,9381,9445,9509,9573,9637,9701,9765,9829,9893,9957,10021,10085,10149,10213,10277,10341,10405,10469,10533,10597,10661,2681,10789,10853,10917,2745,11045,11109,11173,11237,11301,2841,11429,2873,11557,11621,11685,11749,11813,2969,11941,12005,12069,12133,12197,12261,12325,3097,3113,3129,12581,3161,12709,12773,12837,12901,13029,13093,13221,13285,13349,13477,3385,13669,13733,3449,13861,3481,2609,3513,14181,3561,3522,2601,14437,3625,3641,2781,2935,2622,2623,3721,2641,5672,5502,3785,3801,2767,3526,2929,10470,3881,5651,3913,5843,3961,6067,6131,12326,4041,6547,13222,13414,4115,4147,8358,8422,8486,8550,8614,8678,8742,8806,8870,8934,8998,9062,9126,9190,9254,9318,9382,9446,9510,9574,9638,9702,9766,9830,9894,9958,10022,5043,10150,10214,10278,10342,5203,5235,10534,10598,10662,10726,10790,10854,10918,10982,11110,11238,11302,11366,11430,11494,11558,11622,11686,11750,11814,11878,11942,12070,12134,12198,12262,6163,12390,12454,12518,12582,6323,12710,6387,12838,12902,13030,13094,13158,6611,13286,13350,13478,24429,13606,13670,13734,7027,13862,6963,13990,14054,2855,14182,7123,10421,2608,14438,12257,24227,2974,2977,12774,4402,2880,6130,7507,5481,10805,7603,7635,7667,5154,7731,12130,7795,7827,2976,7891,7923,10165,2903,10645,8051,2783,8115,8147,8231,8295,8359,8423,8487,8551,8615,8679,8743,8807,8871,8935,8999,9063,9127,9191,9255,9319,9383,9447,9511,9575,9639,9703,9767,9831,9895,9959,10023,10087,10151,8294,10279,10343,10407,10471,10535,10599,10663,10727,10791,10855,10919,10983,11047,11111,11175,11239,11303,11367,11431,11495,11559,11623,11687,11751,11815,11879,11943,12007,12071,12135,12199,12263,12327,12391,12455,12519,12583,12647,12711,12775,12839,12903,13031,13095,13159,13223,13287,13351,13415,13479,5166,13607,13671,13735,2847,13863,13927,14053,14055,14119,14183,2766,2782,2928,2662,2933,4722,2996,11785,2862,2950,2951,4116,8488,9448,9576,5044,5204,5236,5524,5652,11368,11688,6004,6068,6132,12328,6324,12776,6548,6612,6644,8232,8296,8360,8424,4244,8552,8616,8680,8744,8808,8872,8936,9000,9064,9128,9192,9256,9320,9384,4724,9512,4788,9640,9704,9768,9832,9896,9960,10024,10088,10152,10216,10280,10344,10408,10472,10536,10600,10664,10728,10792,10856,10920,10984,11048,11112,11176,11240,11304,11432,11496,11560,11624,5844,11752,11816,11880,11944,12008,12072,12136,12200,12264,6164,12392,12456,12520,12584,12648,12712,6388,12840,12904,13032,13096,13160,13224,13288,13352,13416,13480,6804,13672,13736,2138,13864,6964,2202,7028,8873,9129,7124,2298,2378,2458,2522,2586,10409,10985,11049,2842,2858,2874,7508,2922,2938,7604,7636,7668,2954,7732,11881,7796,7828,2986,7892,7924,3002,3082,3098,8052,12457,8116,8148,12521,8233,8297,8361,8425,8489,8553,8617,8681,8745,8809,2218,8937,9001,9065,2282,9193,9257,9321,9385,9449,9513,9577,9641,9705,9769,9833,9897,9961,10025,10089,10153,10217,10281,10345,2602,10473,10537,10601,10665,10729,10793,10857,10921,2746,2762,11113,11177,11241,11305,11369,11433,11497,11561,11625,11689,11753,11817,2970,11945,12009,12073,12137,12201,12265,12329,12393,3114,3130,12585,12649,12713,12777,13033,13097,13161,13225,13289,13353,13417,13481,13609,3402,13673,13737,3450,13929,2953,3514,14185,3562,4148,2521,8234,3626,3642,8362,4245,9002,9322,9642,4981,5045,10282,3786,3802,10602,5461,11050,3882,5621,3914,11306,5781,3962,5845,6069,12202,6261,12778,13290,13482,4117,8298,4181,8426,8490,8554,8618,4341,8746,8810,8874,8938,4501,9066,9130,9194,9258,4661,9386,9450,9514,9578,4821,9706,9770,9834,9898,9962,10026,10090,10154,10218,5141,10346,10410,10474,10538,5301,10666,10730,10794,10858,10922,10986,5525,11114,11178,11242,11434,11498,11562,11626,11690,11754,11818,5941,11946,12010,12074,12138,6101,12266,12330,12394,12458,12522,12586,12650,12714,6389,6421,13034,13098,6581,13226,6645,13354,6741,13610,6805,13674,13738,6901,2989,13930,2964,7029,7061,14186,3454,2943,11853,7221,14058,11789,3370,13418,7381,13162,12842,6133,7509,7541,11882,11370,7637,7669,7701,7733,5237,7797,7829,7861,7893,7925,4725,8682,8021,8053,3106,8117,8149,8181,8235,8299,8363,8427,8491,8555,8619,8683,8747,8811,8875,8939,9003,9067,9131,9195,9259,9323,9387,9451,9515,9579,9643,9707,9771,9835,9899,9963,10027,10091,10155,10219,10283,10347,10411,10475,10539,10603,10667,10731,10795,10859,10923,10987,11051,11115,11179,11243,11307,11371,11435,11499,11563,11627,11691,11755,11819,11883,11947,12011,12075,12139,12203,12267,12331,12395,12459,12523,12587,12651,12715,12779,13099,13163,13227,13291,13355,13419,13483,11405,13611,13675,13739,13926,13931,6643,14059,12646,14187,11046,10406,10086,4883,4787,4723,4467,4403,4243,8230,5211,2881,2583,3525,12517,6642,5128,5042,3369,3321,12965,12453,12389,3065,2857,11365,2761,10981,10725,2871,5234,3365,4118,8300,8364,8428,8492,8556,8620,4342,8748,8812,8876,8940,4502,9068,9132,9196,9260,9324,9388,4726,9516,9580,4822,9708,9772,9836,9900,4982,10028,5046,10156,10220,5142,10348,10412,10476,10540,5302,10668,10732,10796,10860,10924,10988,5526,11116,11180,5622,5654,11436,11500,11564,11628,5846,11756,11820,5942,11948,12012,12076,6070,6102,6134,12332,12396,12460,12524,12588,12652,12716,6390,13036,6582,13228,6646,13356,13484,13548,4561,13676,13740,8482,2848,13932,6141,14060,7062,2854,3022,3521,3368,7222,3096,11873,11809,2872,7382,10977,10337,2200,11473,5674,5067,7638,7670,7702,7734,3089,7798,7830,7862,7894,7926,2821,2752,8022,8054,12768,8118,8150,8182,8301,8365,8429,8493,2139,8621,8685,8749,2203,8877,8941,9005,9069,2283,2299,9261,9325,9389,9453,2379,9581,9645,9709,9773,9837,9901,9965,10029,2523,10157,10221,10285,10349,2603,10477,10541,10605,10669,10733,10797,10861,10925,2747,2763,11117,11181,11245,11309,2843,2859,2875,11565,11629,2923,2939,11821,11885,11949,12077,12141,12205,12269,12333,12397,12461,12525,12589,12653,12717,12781,12845,12909,12973,13037,13101,13165,13229,13293,13357,13421,13485,13549,13677,13741,3451,5122,3483,13997,14061,3367,3563,3063,2967,11037,3627,3643,8797,5125,11356,3105,2934,6565,5543,3787,3803,3453,5883,3446,6846,3883,14044,6862,5127,6523,6126,6062,11804,11036,10204,4974,9564,4119,8302,4183,8430,8494,8558,8622,4343,8750,8814,8878,8942,4503,9070,9134,9198,9262,9326,9390,4727,9518,9582,9646,9710,9774,9838,9902,4983,10030,10094,10158,10222,10286,10350,10414,10478,10542,10606,10670,10734,10798,10862,5463,10990,5527,11118,11182,5623,5655,11438,11502,11566,11630,5847,11758,11822,5943,11950,12014,12078,6071,6103,6135,12334,12398,12462,12526,12590,12654,12718,6391,12910,13038,6583,13230,13294,13358,13486,4461,13614,13678,13742,12505,3062,13934,11289,14062,7063,2438,2358,2198,5661,7223,5862,5216,5385,6827,7383,5890,5218,5632,5247,5629,5660,7639,7671,7703,7735,3301,7799,7831,7863,7895,7927,5644,9944,8023,8055,5628,8119,8151,8183,8239,8303,8367,8431,8495,8559,8623,8687,8751,8815,8879,8943,9007,9071,9135,9199,9263,9327,9391,9455,9519,9583,9647,9711,9775,9839,9903,9967,10031,10095,10159,10223,10287,10351,10415,10479,10543,10607,10671,10735,10799,10863,10927,10991,11055,11119,11183,11247,11311,11375,11439,11503,11567,11631,11695,11759,11823,11887,11951,12015,12079,12143,12207,12271,12335,12399,12463,12527,12591,12655,12719,12783,12847,12911,13039,13167,13231,13295,13359,13487,12437,12373,13679,13743,13807,5722,5093,3120,5641,8578,6343,8155,8770,8568,8576,8258,8260,8590,8584,8266,9016,6570,6186,5482,10324,5098,10132,5002,4970,9812,5508,6087,6084,3362,8722,6301,5921,3361,6564,6502,5510,5146,3122,5639,4120,8304,4184,8432,8496,8560,8624,8688,8752,8816,8880,8944,4504,9072,9136,9200,9264,9328,9392,4728,9520,9584,9648,9712,9776,9840,9904,4984,10032,10096,10160,10224,10288,10352,10416,10480,10544,10608,10672,10736,10800,10864,5464,10992,5528,11120,11184,5624,5656,11440,11504,11568,11632,5848,11760,11824,5944,11952,12016,12080,6072,12208,6136,12336,12400,12464,12528,12592,12656,12720,6392,12848,12912,13040,13104,6584,13232,13296,13360,13488,11977,13616,13680,13744,8645,2801,8888,14064,7064,8322,8586,8588,12176,7224,5640,11024,9936,4808,7384,5344,9480,3504,9210,9037,9486,5243,7640,7672,7704,7736,2845,7800,7832,7864,7896,7928,9533,5404,8024,8056,8637,8120,8184,8241,8305,8369,8433,8497,8561,8625,8689,8753,2204,8881,8945,9009,9073,2284,2300,9265,9329,9393,9457,2380,9585,9649,9713,9777,9841,9905,9969,10033,10097,10161,10225,10289,10353,2604,10481,10545,10609,10673,10737,10801,10865,10929,10993,11057,11121,11185,11249,11313,11377,2860,11505,11569,11633,11697,11761,11825,11889,11953,12017,12081,12145,12209,12273,12337,12401,12465,12529,12593,12657,12721,12785,12849,12913,13041,13105,13169,13233,13297,13361,13425,13489,13681,13745,13809,5222,13937,5307,14065,6182,11340,3564,10252,8966,6341,3628,3644,7041,7068,5633,2944,5726,9534,9538,3788,3804,6074,8964,9460,9524,3884,2877,2925,9525,3086,5147,5509,5217,9529,9602,5634,9604,9610,4121,8306,4185,8434,8498,4281,8626,8690,8754,8818,4441,8946,4505,9074,9138,4601,9266,9330,9394,4729,4761,9586,9650,9714,9778,9842,9906,4985,10034,10098,10162,10226,10290,10354,10418,10482,10546,10610,10674,10738,10802,10866,5465,10994,5529,5561,11186,5625,5657,5721,11506,11570,11634,11698,5881,11826,11890,11954,12018,6041,6073,12210,12274,12338,12466,12530,12594,12658,6361,6393,12850,12914,12978,6521,13106,6585,13234,13298,6681,13426,13490,9726,13256,13746,13810,6845,13938,7001,8566,7065,9730,9913,7161,5087,3261,9917,2784,7321,9972,7385,8768,9974,7481,10562,6143,3341,5210,7641,5152,",
	["Tournement"] = "13753,13754,13755,13756,13854,",
	["Ghostlands"] = "9215,9220,9167,",
	["Halls of Lightning"] = "13244,",
	["Black Temple"] = "10959,",
	["Alterac Valley"] = "11336,",
	["Dustwallow Marsh"] = "11150,11174,",
	["Jewelcrafting"] = "12958,12959,12960,12962,12963,",
	["Borean Tundra"] = "11797,",
	["Coldarra"] = "13413,",
	["Caverns of Time"] = "10445,",
	["Wetlands"] = "463,470,474,484,1132,",
	["nyi"] = "73,108,137,241,242,259,260,316,326,327,352,390,406,462,490,497,534,548,612,636,740,774,796,797,798,799,800,801,802,803,807,810,811,814,820,839,856,859,904,946,987,988,989,1128,1129,1155,1156,1157,1158,1161,1162,1163,1165,1263,1272,1277,1278,1279,1280,1281,1283,1289,1290,1291,1292,1293,1294,1295,1296,1297,1298,1299,1300,1390,1397,1441,1443,1460,1461,1533,1537,1538,1659,1660,1662,1663,1664,2020,8245,3405,3581,3885,8247,10616,4541,7069,7741,8571,8445,9599,7904,8449,8450,8451,8452,8453,8454,4323,5053,5506,5666,5209,8458,10890,8459,8971,8248,3516,9597,8973,5699,5703,5698,5691,8975,9231,5701,5704,8976,9296,8337,9297,9745,2868,9298,4905,5304,5208,5696,5695,8444,6842,4299,8244,5692,3622,3910,5659,5303,5207,5693,5694,3425,6702,5710,5708,5706,6202,7790,3111,3383,3515,3623,5711,2971,5702,5707,5697,5709,8237,6704,5712,3064,3384,3023,5671,5667,3624,10402,5681,5516,5520,5669,5682,3422,11493,3401,3529,6003,5683,6707,6703,3423,6706,6708,5684,2058,3482,3530,5205,5653,6165,6709,5685,6705,5523,5668,3241,5700,5686,6710,3424,2059,3403,3531,5690,5687,6711,5705,8974,5688,8972,5512,8152,8448,3404,5670,5665,5664,5689,6201,6841,",
	["Utgarde Pinnacle"] = "13131,13132,",
	["Zul'Drak"] = "12974,",
	["Tanaris"] = "8728,",
	["SHAMAN"] = "8413,",
	["Ahn'kahet: The Old Kingdom"] = "13190,13255,",
	["Halls of Stone"] = "13252,13207,",
	["The Storm Peaks"] = "12851,12915,12853,12981,12983,12856,12921,12985,12922,13050,12987,13051,12924,12925,12989,13057,12996,12997,13061,13062,13063,13064,13005,13006,12942,13009,13010,13011,12886,12953,13422,12956,12831,12832,12900,12966,12967,12968,12841,12905,12969,12906,12970,12843,12907,12971,13035,12844,12908,12972,12846,12975,13423,12976,13424,12977,",
	["Fishing"] = "13830,13832,",
	["Wintergrasp"] = "236,13177,13181,13186,",
	["Icecrown"] = "13125,13130,13135,13139,13141,13157,",
	["Trial of the Champion"] = "14199,",
	["Cooking"] = "13107,13100,13102,13103,",
}
--]]
local QuestZones = {
-- kalimdor
["Ahn'Qiraj War"] = "8529|8516|8500|8499|8796|8505|8518|8495|8521|8520|8797|8506|8795|8493|8525|8503|8510|8504|8527|8524|8522|8517|8513|8515|8511|8494|8492|8528|8512|8509|8514|8526|8523|10501|",
["Alcaz Island"] = "8970|",
["Ammen Vale"] = "9312|9798|9311|9309|10304|10303|9305|9303|9294|9799|10302|9371|9409|9293|9283|9278|9280|9279|9369|",
["Red Cloud Mesa"] = "1656|3376|763|781|780|757|750|755|753|747|752|5844",
["Shadowglen"] = "920|928|921|917|3522|3521|916|3519|4495|459|457|456|458|5842|",
["Treasure Map"] = "2882|",

-- azeroth
["Booty Bay"] = "3721|",
["Deathknell"] = "6395|8|590|383|382|6396|380|381|3902|3901|376|364|5847|363|",
["Kharanos"] = "6722|",
["Sunstrider Isle"] = "8347|8335|9705|9704|8350|8338|8345|8330|8336|8334|8327|8326|8346|10071|10073|10070|10069|10072|10068|8325|8547|",
["Alterac Mountains"] = "551|554|504|537|523|522|500|512|535|510|525|514|511|505|",
["Alterac Valley"] = "7365|7364|8387|6901|7426|7425|7424|8375|8383|7366|7142|11336|7122|7163|8272|13428|7241|7141|7082|7124|6985|7224|7282|7121|7223|7301|6982|11340|7081|7101|7164|7166|7167|7165|7172|7171|7170|7169|7168|7027|7161|7162|7102|13427|6846|7261|8271|7002|6943|7302|7123|6801|5893|6941|7026|6942|6881|6781|6827|6826|7001|6825|7386|7385|6741|5892|",
["Anvilmar"] = "420|282|218|3365|3364|182|234|183|233|3361|170|179|5841|",
["Arathi Basin"] = "8262|8265|8114|8121|8297|8115|8122|8120|8080|8105|8154|8166|8169|8264|8261|8156|8170|8167|8260|8263|8168|8171|8155|11339|11335|14178|14181|",
["Arathi Highlands"] = "656|667|652|646|645|644|643|688|653|673|687|680|679|640|685|641|670|669|668|666|665|664|662|694|696|695|697|693|684|651|678|682|702|847|642|701|639|691|635|663|672|655|675|674|671|677|681|",

-- outland
["Skettis"] = "11098|11073|11085|11008|11006|11885|11005|11021|11072|11004|11093|11029|11028|11056|",
["The Barrier Hills"] = "10997|",

-- northrend
["Coldarra"] = "13414|13413|13412|11910|11969|11967|11951|11946|11943|11957|11931|11940|11900|11914|11936|11941|11918|11912|11919|",
["Ahn'kahet: The Old Kingdom"] = "13190|13255|13187|13204|",

-- special
["Alchemy"] = "10905|10897|10907|10902|10906|10899|1581|14151|",
["Battleground"] = "8388|8371|13475|13477|13478|8367|8385|13476|",
["Brewfest"] = "12062|11454|12022|11441|11294|12318|11419|12492|11293|11446|11408|12194|11431|11118|13649|11321|12193|12421|12278|12491|12420|11412|11409|12191|12192|12020|13932|11318|11400|12306|11407|13931|11486|11413|11120|11117|11487|11122|",
--["Collector's Edition"] = "5805|5844|",
--["Dalaran"] = "9826|9860|",
["Epic"] = "8447|8446|",
["Midsummer"] = "11955|11691|11954|11953|11952|11948|11947|9389|9388|11917|11970|11972|11923|9332|9386|9339|11925|9325|9324|11926|11971|11915|9331|11886|9319|9367|9323|11933|11966|11882|11921|12012|11922|11924|11964|11731|9322|9365|11883|11657|9330|11891|9368|11935|9326|",
["Legendary"] = "7785|9271|9269|9257|9270|9251|9250|7787|7786|7522|7521|",
["Lunar Festival"] = "8686|8647|8872|8719|8725|8682|8654|8871|8724|8684|8873|8644|8652|8874|8645|13022|8678|8883|8718|13017|8648|13021|8670|13067|8680|8635|8643|8727|13027|13023|13030|13020|8882|8881|8864|8862|8676|13032|8675|13028|8653|13065|8726|8685|8679|13014|8672|8671|13018|8717|8720|13033|13029|13031|8650|13024|8681|8722|8880|8716|8877|8714|8688|8651|8649|8646|8636|13066|8878|8865|8863|13013|8673|8674|13025|8870|8866|8683|8721|8713|13012|8875|13015|8879|8876|8677|8715|13016|13026|13019|8723|8867|8642|8619|",
["Invasion"] = "12816|12817|9301|9300|9154|9085|9153|9299|9295|9292|9302|9247|9304|",
["Seasonal"] = "11392|11135|11696|11220|9025|11581|8354|13467|13469|8763|12387|12133|12382|6961|8311|12397|11357|11131|13473|13472|12348|12347|12336|13490|13448|12286|12345|12404|11753|8903|12400|11748|11746|11361|12401|11745|12399|11743|12135|1658|12341|12338|11737|12373|12335|12332|11807|8373|11528|11356|11450|8993|12331|13460|12365|13459|9027|12364|12357|13443|12351|12409|12950|12408|12407|12406|12359|12358|12403|11558|12356|12355|12354|8860|12353|13493|12352|8861|12360|11760|11823|13444|11757|11756|13440|11755|13491|11751|11779|11749|13455|11742|11739|13445|11738|7042|6983|11735|11732|8979|12155|13449|13447|12402|13500|13497|7045|13438|13462|8828|8788|11439|13465|12410|13466|11832|8983|12945|8982|7024|8981|12405|8980|11744|7023|8901|8762|7061|12334|11767|12337|11801|8897|12941|6963|8353|13446|11242|13461|13471|12385|12384|7022|12368|13498|12363|11440|8359|12367|11860|8358|12940|13470|11862|11859|12946|13468|11855|11854|11852|11851|9026|11850|8898|11849|12386|11846|13452|12383|11844|12381|11843|12379|11841|12378|11840|12377|11839|13433|12375|11837|12374|11836|11835|13439|12362|11763|11762|7021|11761|11759|11758|11754|12346|11752|11750|11747|12398|11741|8900|11740|11736|11734|8984|12396|12350|12349|12376|12342|11863|12371|11775|11857|11449|12370|12333|11845|12369|11842|11838|11856|13495|11853|8746|8149|13499|11847|11861|11584|12139|12361|11848|9028|13496|13494|12380|11583|13488|13442|13487|13486|13485|12947|13453|13450|13437|11830|13441|7043|11834|8803|11825|11833|11829|11828|13434|8799|11821|11827|8868|11826|11824|8356|11822|11401|11820|11819|11405|8312|1657|8769|11799|11818|8322|8768|11787|11817|8409|13435|11816|11782|11815|11778|11814|8744|11812|13464|11811|11810|11809|7063|13451|11808|11806|11805|11804|8899|11803|12395|11802|11800|12393|11786|12392|11785|12391|11783|7025|12389|11781|11360|6964|12388|11780|6962|12390|11776|11774|11976|9024|11773|11858|11403|11772|11771|12394|11770|8357|11769|11766|11765|11764|11831|11813|11784|11404|11777|11768|11580|13454|13456|13463|13489|8150|12343|13436|12339|8904|12344|8355|12340|8902|8827|13203|12944|13492|8767|13474|6984|7062|12366|13458|13457|8360|11219|",
["Special"] = "3861|10966|172|12784|13927|12773|10952|10950|925|13954|10953|10942|13957|13956|10945|12776|13955|12783|13933|13950|12774|11975|10963|10951|1687|12787|915|13951|13934|10960|13929|13930|4822|12772|558|10943|5502|10962|1479|1558|12753|10967|13937|12775|12788|12782|13926|12752|911|910|1800|1468|12811|171|13938|13959|12812|12771|12786|12785|10968|12809|12808|10956|12777|10954|",
--["The Thorium Brotherhood"] = "7848|6646|6644|6643|6645|6642|8578|7487|",
--["Thunder Falls"] = "5999|",
["Unknown"] = "14048|12781|13629|13845|14203|",



["Ahn'Qiraj"] = "8665|8602|8659|8711|8661|8703|8784|8596|8594|8593|8691|8592|8707|8699|8603|8658|8660|8704|8696|8632|8626|8558|8557|8690|8622|8710|8702|8694|8663|8641|8637|8657|8656|8629|8655|8801|8662|8669|8668|8667|8624|8621|8709|8639|8802|8640|8638|8712|8790|8695|8664|8689|8705|8697|8789|8633|8631|8625|8634|8560|8562|8561|8559|8544|8693|8701|8556|8706|8628|8698|8627|8630|8700|8692|8708|8666|8623|",
["Ashenvale"] = "7865|7868|7867|7864|7866|7863|1012|9536|9522|1031|1032|1021|1044|1045|1043|1042|1041|9535|1039|2|9534|6504|1035|6922|9526|1046|1022|247|9520|9516|909|1038|1037|1040|1011|4581|1030|9518|1055|1028|9517|1029|1027|1918|824|6571|908|1026|9519|24|6581|6441|6621|1054|25|6546|1017|9533|1009|9521|6482|6503|6462|216|1025|1016|976|23|6544|6641|1034|1033|1024|1023|1134|1085|1070|6547|9432|1010|9428|6383|1020|1007|10752|990|991|6442|1008|6545|6543|1056|",
["Auchindoun"] = "10178|10094|11374|11376|11373|10095|10091|10177|10649|11372|11375|10098|10097|10167|10227|10164|10253|10228|",
["Azjol-Nerub"] = "13254|13182|13167|",
["Azshara"] = "6821|6822|6823|8729|8575|7486|6824|3509|3621|3602|3511|3510|3508|3382|6805|3141|6804|3421|3503|5534|3601|3517|3564|3561|3565|3518|3562|3541|3563|3542|5536|5535|",
["Azuremyst Isle"] = "14082|9570|9564|9622|9625|9623|9573|9544|9566|9571|9515|10428|9528|9540|9539|9559|9562|9560|9542|9541|9565|9527|9537|9531|9530|9602|9612|9538|10324|9523|9473|9514|9513|9456|9505|9455|9512|9506|9616|9463|9454|9453|9452|9313|9314|",
["Badlands"] = "4062|3821|795|717|779|793|706|778|762|735|2203|782|732|716|712|739|777|734|692|723|726|727|725|724|733|1420|1419|703|9439|708|728|737|2258|711|718|738|705|710|714|715|713|720|719|",
["Black Temple"] = "10957|10959|10958|",
["Blackfathom Deeps"] = "6921|1200|6561|6565|1199|1198|1275|971|6563|6564|6562|",
["Blackrock Depths"] = "4004|7604|9015|4361|4342|4341|4003|4363|4362|4122|4063|4024|4322|4121|4132|4282|4264|4263|4286|3907|4126|4123|4083|4134|4133|4128|4082|3911|4002|4242|4201|4001|3982|4241|7201|4136|4324|3801|3802|4081|3981|4295|",
["Blackrock Mountain"] = "8962|9032|8961|8964|8985|8963|8996|8987|8965|8988|8986|8994|8960|4262|3906|",
["Blackrock Spire"] = "4764|5089|4743|4765|8995|5047|5127|4867|5160|5081|7761|4768|6602|4903|4866|5102|4766|4907|4734|8989|8966|4742|4769|4974|6569|6502|4735|4981|4729|4724|4862|5001|4982|4983|4701|5002|4788|",
["Blacksmithing"] = "7654|7657|5103|7655|7656|7658|7651|7649|5307|5306|5305|7652|7653|7650|7659|10891|3321|10892|2771|2763|2762|2761|2765|2764|2773|2772|5283|2757|2759|2758|5301|5284|2756|2760|5302|2753|2754|2755|2752|2751|1618|1578|",
["Blackwing Lair"] = "8730|8288|",
["Blade's Edge Mountains"] = "10742|11057|10983|11022|11078|11079|11023|11513|11066|10996|11059|11027|11062|11065|11061|11119|11058|11080|11010|11009|10995|11030|11025|11000|10998|10982|11102|10976|11514|11051|11026|10989|10806|10975|10974|11091|11060|10910|10904|10912|10803|10802|10911|10830|10829|10607|10867|10865|10845|10843|10825|10820|10819|10810|10786|10748|10747|10723|10722|10821|10675|10812|10860|10753|10682|10846|10818|10771|10770|10584|10797|10796|10795|11040|11036|10594|11043|10798|10800|10609|10608|10801|10799|11047|10805|10674|10671|10657|10632|10620|10581|10580|10894|10893|10859|10853|10851|10517|10785|10783|10749|10720|10719|10717|10714|10709|10718|10715|10543|10526|10721|10713|10518|10784|10724|10512|10506|10489|10486|10567|10566|10542|10511|10510|10502|10516|10456|10556|10690|10455|10555|10504|10565|10617|10614|10545|10544|10525|10524|10505|10503|10488|10487|10457|10618|9794|10927|10928|9795|10710|10557|10711|10615|10712|10716|",
["Blasted Lands"] = "2744|3628|3627|10259|2743|2721|2681|2801|2783|2702|2701|2522|2521|3501|3502|2585|2583|2581|2784|2586|2584|2582|2601|2602|2621|2604|2603|",
["Bloodmyst Isle"] = "9689|9762|9761|9760|9756|9759|9753|9669|10067|9670|9688|9683|9749|10064|10066|9585|9671|9578|9569|9649|9748|9746|9740|9682|9674|9672|9741|9751|9711|9706|9579|9687|10063|9646|9703|9549|9548|10065|9696|9647|9550|9699|9700|9580|9698|9561|9557|9779|9642|9633|9632|9643|9694|9693|9641|9584|9628|9574|9620|9594|9567|9648|9667|9629|9668|9666|9576|9663|9624|9634|9581|9604|9606|9605|9603|",
["Borean Tundra"] = "13257|11592|11942|11623|11867|12086|11961|11950|11949|11610|11609|11945|11868|11612|11605|11637|11892|11687|11798|11566|11673|11878|11872|11871|11869|11864|11670|11659|11884|11633|11654|11630|11647|11641|11640|11876|11866|11865|11627|11930|11929|12088|11944|11938|11909|11628|11607|11879|12019|11617|11870|11699|11629|11622|11790|11587|11956|12471|11712|12486|11677|11675|11678|11683|11650|11968|11586|11730|11725|11692|11788|11913|11908|11903|11689|11639|11638|11576|11632|11618|11928|11616|11600|11615|11599|11887|11614|11927|11611|11596|11645|11561|11626|11602|11559|11902|11598|11575|11625|11620|11901|11726|11574|11701|11680|11564|11710|11700|11562|11897|11697|11708|11694|11702|11707|11693|11704|11571|11582|11570|12490|11569|11565|11563|11648|11646|11796|11606|11733|11728|11671|11873|11713|11724|11722|11795|11682|11681|11679|11663|11658|11653|11715|11898|11718|11889|11791|11789|11613|11920|11727|11672|11635|12141|11619|11797|11881|11662|11603|11655|11703|11652|11651|11709|11711|11644|11643|11642|11686|11688|11594|11690|11593|11676|11721|11590|11720|12728|11719|11624|11717|11890|11716|11714|11664|11661|11965|11660|11963|11705|11962|11656|11932|12035|11604|11601|11585|11904|11794|11793|11792|11729|11916|11888|11591|11706|11907|11906|11597|11695|11899|11684|11896|11895|11894|11893|11685|11723|11560|11698|11674|11608|11636|11634|11595|11649|11631|11621|",
["Burning Steppes"] = "5522|4283|4484|4481|4463|4483|4482|4061|4022|4808|4223|4023|4186|4185|4184|4182|4224|4183|3825|3824|3822|4726|3823|4296|",
["Caverns of Time"] = "11378|11382|9836|10298|10297|13432|10470|10461|10474|10445|10469|10472|11106|11105|11103|10465|11104|10473|10561|10468|10464|10463|11383|10462|10466|10460|10296|10471|10467|10475|12515|10282|10277|10284|10283|10285|12513|10279|",
["Ironforge"] = "8951|7806|7342|7805|10492|8952|7808|8959|8958|8954|7807|8956|8955|8953|8906|8932|8977|9005|9003|9002|9001|8908|8999|10498|8935|9004|9000|10494|8937|8934|7804|8933|7803|8911|7802|8905|8907|8936|8909|8926|7812|9006|8912|8910|8931|10496|8997|7811|7809|1019|8275|3702|3701|4513|3449|3451|3448|3450|3461|3483|4512|2946|2977|2964|3790|2963|2439|3371|3201|707|2948|1453|689|700|686|683|637|2039|2041|7905|6721|",
["DEATHKNIGHT"] = "12849|12711|12698|12697|12687|12744|12733|12747|12719|12680|12679|12678|12670|12779|12778|12757|12756|12755|12750|12742|13189|12739|12718|12748|12801|12800|12754|12751|12743|12738|12625|12727|12725|12724|12723|12722|12720|12716|12715|12714|12706|12701|12700|13188|13166|13165|12717|12641|12636|12619|12850|12848|12745|12842|12746|12593|12657|12626|12749|",
["DRUID"] = "11011|9052|9053|9051|9063|3094|3120|6130|6129|6128|26|10993|10992|10991|10990|10988|10987|10986|10980|10979|10978|6001|10965|10961|10955|5931|5929|5923|5921|6124|6002|6123|5932|6122|6121|10964|10994|5928|11001|29|5926|30|5927|28|27|6125|5061|5924|6126|6127|272|5930|5922|31|5925|",
["HUNTER"] = "7633|7636|7635|7632|7634|8151|8232|8231|8153|3108|9393|3087|3092|3082|3117|6072|6073|6075|6085|6071|9617|9757|9675|9593|9592|9591|10530|6074|9486|9485|9484|6082|6083|6089|6088|6087|6061|9673|6070|6103|6102|6101|6063|6076|10529|6062|6081|6084|6086|6064|",
["MAGE"] = "12172|12173|9362|8253|8250|8252|8251|1952|1956|1953|1957|1958|1951|1948|1955|1954|1947|1949|1950|1938|1945|1944|1943|1946|1942|1940|1939|9488|1960|1920|9487|1959|1961|1921|1962|1941|1919|9595|1882|1881|1879|9403|9402|9404|1880|1861|1860|1883|1884|9290|3104|3098|8328|3086|3114|9364|",
["PALADIN"] = "10592|8418|10590|8416|8414|10593|9601|8415|4486|4485|1661|9712|9692|9710|9707|1654|1806|1442|1655|9599|9287|3101|9676|3107|1789|1788|1787|9678|1645|7641|9737|1779|7642|7643|7638|7670|10366|1652|1651|1649|7666|9723|7647|1783|1778|2999|1648|1650|1647|9598|1646|9722|1793|1790|7637|9735|9600|2997|9684|9681|1794|1785|1784|3000|9677|9725|9736|9721|9690|7639|7646|7648|7645|9691|7644|9686|1786|1780|1644|1653|1643|1642|3681|1641|1781|2998|9685|7640|",
["PRIEST"] = "7622|7621|8256|8255|8257|8254|5632|10364|9489|9586|5622|5625|5624|5651|5650|5626|5621|5623|5649|5648|3110|3097|3119|8564|3103|3085|5630|5629|5679|5676|5674|5672|5644|5641|5628|5647|5675|5627|5677|5673|5678|5646|5633|5631|5645|",
["ROGUE"] = "8249|6701|8234|8233|8236|8235|6681|10794|2607|2608|2609|2282|1999|9491|10548|2381|2382|2298|10372|2380|2299|2259|2281|2300|2379|2260|2019|1998|2378|1963|1899|1898|1886|1978|1858|9460|2206|2205|2239|2218|9618|2238|1885|2242|2241|9532|1859|9392|3096|3118|3088|3109|3102|3113|3083|2458|2359|2478|2460|2360|2480|2479|",
["SHAMAN"] = "7667|7669|8258|8259|8413|8411|8410|8412|14100|14111|1463|1521|1520|1519|1518|1517|1516|1462|3093|3089|3084|2986|1527|1526|1464|9554|9553|9552|1534|9551|1523|9503|100|9451|9508|2983|9509|9467|9502|9504|9501|1528|1524|9461|9555|2985|9464|10490|1529|1531|220|1536|1535|1103|1530|1525|63|9500|96|1532|10491|9468|9465|2984|9450|1522|9547|9449|9462|",
["WARLOCK"] = "8419|8422|8420|8421|4964|1799|4963|4961|4962|4975|4976|4786|4785|4784|4783|4968|4965|4969|4967|4781|4782|4737|4738|4739|1796|4736|1740|8563|3115|3099|3105|3090|7630|7629|7628|7627|7626|7625|7624|1470|7602|10790|9529|9619|1506|1804|1801|4490|4489|4488|4487|10605|10789|7564|8344|1478|7562|3631|1499|1798|1598|1685|2996|1510|7623|1795|7563|1802|10788|3001|7603|1471|7582|7581|1476|1504|1501|1485|1805|1599|1474|1803|1716|1515|1739|7583|1513|1738|1512|1511|1509|1508|1507|7631|1473|1472|1758|1717|7601|1689|1715|1688|",
["WARRIOR"] = "8423|8425|8424|8417|1713|1792|1712|1714|1847|1846|1845|1844|1843|1842|1841|1840|1839|1838|1711|1706|1791|1709|1710|1848|1708|1705|1700|1703|1701|1704|1699|1702|1825|1824|1698|1823|10371|1680|1821|1681|9582|1503|1502|1693|1692|1820|1667|1686|10350|1666|9289|3106|3091|3095|3116|2383|3065|3100|3112|1719|1684|1679|1498|1683|1678|1638|1819|1818|1505|1718|1665|1640|1639|",
["Coilfang Reservoir"] = "11370|11371|11369|9766|11368|10459|9738|9876|",
["Cooking"] = "13571|11377|11379|11380|11381|8317|8313|8307|6612|6611|6610|7321|862|2178|384|4161|13115|13090|13087|13113|13114|13088|13100|13103|13089|13102|13107|13112|13101|13116|13825|",
["Dalaran"] = "9826|9860|12790|12794|12796|12791|",
["Darkmoon Faire"] = "13325|13326|13327|13324|10938|10939|10940|10941|7893|7891|7903|7933|7902|7927|7937|8222|7981|7943|7942|7897|7938|7898|7884|7901|7930|9249|7929|7885|7944|7939|7945|7900|7895|7931|7894|7882|7890|7932|7892|7889|7940|7899|7883|7907|7881|7936|7941|7935|7934|7946|7896|7928|",
["Darkshore"] = "1143|981|1140|1167|943|973|994|970|5321|942|968|2098|951|741|1133|2078|993|995|729|986|731|4731|4733|4732|945|2139|4740|4763|967|966|965|950|949|944|948|982|947|1138|1003|2138|963|4730|4727|5713|4761|4725|4762|1002|2118|1141|985|4811|4728|4681|4813|4812|984|956|3524|4722|4723|957|955|953|1001|954|958|6342|6341|983|6343|",
["Darnassus"] = "14409|7799|7800|10354|10352|7801|6762|6761|3803|1047|4510|4493|5091|3785|3763|3781|3764|14085|730|2518|2520|952|2519|6344|",
["Deadwind Pass"] = "9680|",
["Deeprun Tram"] = "6662|6661|",
["Desolace"] = "6568|1381|1380|1052|1467|1466|1488|1456|1457|261|1455|1454|6132|6134|6141|5581|5943|5381|6027|1373|1374|1375|5386|6143|6161|1385|1382|1370|1459|1482|1371|5821|1365|6142|5561|5741|1438|1465|1458|1440|1439|1437|1367|1369|1368|1481|1480|1484|1361|1386|5501|1384|1362|1387|5763|1366|5421|",
["Dire Maul"] = "8949|7503|7502|7500|5528|1193|7703|7482|7429|7877|8948|7499|7485|8967|7484|7508|7483|7506|8990|7481|7461|7462|5518|7501|1318|7504|7498|7507|5519|5525|7505|8950|7441|7494|7488|7489|7492|7463|",
["Dragonblight"] = "13372|13375|13078|13343|12124|12450|12267|12095|12470|12097|12418|12768|12123|12419|12769|12046|12045|12044|12043|12441|12442|12438|12265|12264|12263|12262|12261|12500|12496|12266|12089|12091|12459|12488|12458|12447|12150|12090|13377|12416|13370|12497|12080|12460|13371|13369|12545|12499|12417|12498|12542|13347|12495|12079|12077|12461|12076|12456|12454|12151|13242|12078|12049|12052|12075|12449|12448|12469|12112|12372|12435|12050|12047|12142|12021|12025|12005|11959|12274|12462|12478|12148|12147|12145|12004|12146|12143|12017|12473|12140|12132|12285|12477|12476|12475|12474|12069|12472|12467|12466|12465|12463|12136|12457|12455|12224|12464|12319|12312|12326|12325|12321|12013|12320|12111|12144|12104|12102|12127|12126|12125|12273|12149|12060|12055|12237|12056|12065|12000|12063|12053|12039|12034|12048|12100|12157|12032|12031|12117|12030|12083|12028|12098|12067|12119|12107|12009|12041|12033|12214|12245|12189|12230|12303|11980|12283|11978|12221|12101|12260|12254|12188|12304|11979|12200|12008|11983|11999|11996|12211|12209|12206|11958|12118|12182|12205|12440|12297|12298|12439|12115|11995|12064|12234|12240|12232|12243|12040|12301|12016|12766|12011|12305|12291|12085|12167|12084|12106|12071|12066|12059|12057|12096|12171|12767|12311|12122|12309|12290|12166|12258|12110|12253|12235|12169|12287|12168|12282|12281|12277|12092|12276|12275|12272|12269|12174|12251|12006|12036|12061|12218|11960|12252|11977|12072|12271|12239|",
["Drak'Tharon Keep"] = "13249|12238|12037|13129|",
["Dun Morogh"] = "467|466|14083|14084|312|314|417|433|419|291|415|413|412|414|432|315|287|320|319|313|311|308|318|317|5541|310|400|2160|403|",
["Durotar"] = "14088|926|924|809|842|827|829|828|832|806|840|816|835|826|837|813|812|808|834|825|815|786|817|831|830|823|818|791|784|2161|",
["Duskwood"] = "1395|1391|1388|1383|1372|254|55|98|222|223|229|160|253|181|231|58|227|225|401|134|228|252|251|526|322|321|221|324|325|270|269|67|69|66|323|173|70|74|72|97|80|79|78|75|68|133|57|90|230|95|174|177|101|165|453|175|337|268|266|265|262|156|157|154|149|148|56|158|159|164|245|226|163|9429|93|5|240|",
["Dustwallow Marsh"] = "6584|6583|6582|6570|6601|6585|1173|1168|1166|1169|1205|1171|11162|11159|1172|1170|11217|1258|7541|1262|1261|1259|11174|11172|11156|11205|11215|11160|11211|11152|11151|11161|11149|11150|11206|11214|11173|11184|11208|11207|11158|1285|11201|11148|11144|11143|1260|11186|1204|1286|1287|11185|11169|11204|11203|11200|1270|11146|11145|1222|11142|11134|1323|1269|9437|1276|11213|1271|1273|1322|11222|11212|11138|11137|11141|11140|11139|11209|11223|11147|11210|11198|1177|11225|11194|11177|11136|11180|11183|11181|1284|1319|1321|1239|1268|1251|1238|1202|1282|1201|11124|11126|11123|11193|11192|11191|1219|1218|1240|1320|1203|11133|11128|1206|1220|1253|1252|",
["Eastern Plaguelands"] = "9222|9221|9213|9211|6146|6145|9185|5513|9142|6147|9121|9226|9186|9141|8930|9206|5281|8947|9122|9165|5721|8929|6133|9187|9136|9198|9191|8946|9227|9179|9194|9128|9126|5265|9124|5264|9205|5942|9197|6185|5206|5941|9204|9223|9125|6024|9202|5517|9195|9190|9196|9178|6187|9127|9181|9224|9131|9123|5464|9665|6136|6135|9137|6148|6144|9200|9228|9129|9664|9201|9188|9225|9182|9203|9183|9184|9132|5065|6026|5845|6022|6042|6041|5248|5846|5181|5247|5781|5168|5241|5152|5246|5742|5153|5544|5543|5542|5210|5149|6021|5142|5211|5601|5503|6164|5510|5509|5508|",
["Elwynn Forest"] = "14079|176|52|37|35|76|123|109|45|39|59|71|40|46|11|239|147|83|5545|88|87|62|60|112|47|114|61|106|84|86|85|111|107|2158|16|",
["Engineering"] = "9635|9636|3644|3646|3645|3638|3634|3647|3640|3642|1559|3643|3632|4181|3639|3637|3641|3630|3526|3635|3633|3629|",
["Eversong Woods"] = "14081|9360|8479|9363|8894|9258|9394|8476|9359|8474|10166|8477|8889|8888|9253|8490|8891|8890|9358|9395|9254|8487|8473|9067|9252|8488|9255|8885|9076|8886|8887|8480|8892|8491|9256|8483|8884|9352|9035|9062|8468|8486|8482|8895|9066|9064|8475|9119|8472|8463|",
["Eye of the Storm"] = "11337|14179|14182|11341|",
["Feathermoon Stronghold"] = "3792|3788|3791|",
["Felwood"] = "4721|5242|4741|5204|5385|8469|4521|4261|5249|8467|8466|4462|4461|4114|5887|4118|4111|4222|4343|4444|8462|4116|4115|5882|4109|3363|5889|4101|5884|4443|4464|1514|4113|4105|4103|4447|996|4467|4106|4117|4107|4110|4104|4119|4403|998|4401|5885|2523|4466|4465|4102|5886|5165|4221|4112|5890|5883|8465|4445|5891|4446|4448|5888|4108|8470|5203|5202|4906|5156|939|4505|4442|4441|4421|5159|4506|5157|5158|5155|6162|6131|8460|",
["Feralas"] = "7726|7725|2878|4120|3445|2879|3129|2942|3063|3062|3128|3127|3126|2845|2844|2944|2943|7732|4298|7735|2941|7733|7721|7003|7734|7738|2970|2969|7731|3843|3842|3841|4297|2939|2972|2940|3002|3124|3123|4135|4267|4266|4265|2821|2979|2822|2871|2870|2976|7730|3122|2767|3121|2974|2973|3125|2766|2982|4127|4131|4130|4129|4281|2980|4125|2903|2866|4124|3130|2869|2867|2902|2987|2978|2975|2863|2862|3567|",
["First Aid"] = "6625|6623|6622|6624|",
["Fishing"] = "13827|11531|11667|11668|11665|11666|11669|6607|6609|6608|1579|1580|13834|8224|8221|13833|8194|13836|13832|13830|13826|8225|8193|",
["Ghostlands"] = "11163|9167|9156|9328|9811|9151|9215|9220|9170|9164|9877|9277|9275|9219|9281|9276|9218|9214|9193|9199|9212|9176|9166|9217|9282|9169|9161|9216|9172|9162|9171|9159|9173|9175|9140|9192|9155|9163|9207|9158|9174|9149|9150|9145|9157|9146|9143|9274|9160|9152|9139|9315|9144|9133|9758|9148|9327|9135|9130|9138|9134|9147|",
["Gnomeregan"] = "2842|2843|2929|2841|2949|2947|2945|4601|2953|4602|4605|4604|4603|2952|2951|2925|2928|2930|2924|2904|2962|4606|2931|2926|2927|2923|2922|",
["Grizzly Hills"] = "12152|12082|12058|12121|12164|12068|12330|12329|12328|12327|12411|12134|12184|12196|12185|12483|12002|12160|12159|12161|12425|12094|12204|12203|12202|12201|12198|12197|12199|12770|11998|12099|12250|12248|12247|12431|12430|12415|11981|12014|12412|12010|12003|12129|12093|12165|11993|12180|11988|12128|11986|12249|12414|12158|12195|12236|12241|12231|12074|12070|12116|12054|12113|12763|12114|12026|12153|12131|12130|12183|12138|12154|11997|11985|12081|11982|12120|12073|12279|12137|12299|12307|12280|12288|12042|12270|12284|12029|12308|12427|11984|12190|12434|12207|12317|12432|12007|12484|11990|12213|12487|12424|12423|12246|12422|12429|12178|12428|12802|12177|12175|12413|12324|12315|12294|12217|12216|12446|12215|12437|12212|12255|12295|12293|12292|12027|12444|12105|12109|12219|12220|12443|11991|12310|12316|12302|12300|12323|12170|12314|12223|12229|12222|12296|12289|12242|12244|12268|12259|12257|12256|12468|12176|12453|12451|12208|12433|12436|11989|12210|12511|12225|12227|12226|12038|",
["Gundrak"] = "13250|13096|13111|13098|",
["Halls of Lightning"] = "13244|13109|13108|13253|",
["Halls of Stone"] = "13252|13207|",
["Hellfire Citadel"] = "10558|9525|9495|11364|11354|9496|9492|9524|11362|11363|9494|9493|9608|9590|9589|9607|9572|9575|",
["Hellfire Peninsula"] = "10764|10762|11516|11515|11002|11003|10754|10763|10756|10755|10758|10757|9447|9545|9442|10400|9490|10159|10255|10351|10349|10134|10132|9372|9383|9399|9418|10136|9441|9387|9466|9424|9417|9430|9385|10093|9381|9543|10876|9376|9375|10295|10157|9400|10139|10162|10937|10129|10369|10368|10367|9498|10147|10133|9426|9398|10403|9390|9499|9588|9472|9397|9374|9396|9370|9483|9391|9366|9340|10287|10286|10347|9587|9558|9427|9423|9438|9410|9406|10146|9405|9401|9420|9563|10163|10208|10144|10834|10813|10346|10214|10909|10916|10158|10061|10156|10054|10153|10152|10151|10121|10120|10289|10138|10055|10288|10087|10086|10124|9373|10123|10140|10099|10141|10119|10160|9342|10391|10060|9355|10050|9345|10278|10220|10079|10919|10142|10390|10238|10236|9351|10443|10630|10629|9407|9361|9344|10393|10230|10059|10936|10935|10078|10399|10903|9356|10397|10161|10396|10395|10394|10258|10382|10792|10250|10154|10058|10875|10057|10047|10229|9350|10442|10864|10213|9347|10103|9346|9349|10838|10143|10388|10835|10150|10155|10294|10389|10392|10344|10254|10340|10485|10110|10106|13408|10449|13411|13410|10291|10450|10242|10538|13409|10809|10895|10484|10482|10483|",
["Herbalism"] = "866|",
["Hillsbrad Foothills"] = "521|520|507|519|566|508|518|660|661|658|657|503|506|545|557|533|544|565|564|552|553|659|550|676|560|559|561|556|563|562|555|517|536|515|547|524|541|9435|509|539|567|513|532|529|528|546|502|501|527|499|496|498|549|9425|494|",
["Howling Fjord"] = "11355|11238|11236|11453|11367|11296|11266|11264|11237|12181|11452|11358|11261|11260|11365|11366|11267|11189|11250|11249|11359|11573|11319|11448|11316|11311|11333|11275|11426|11278|11420|11343|11300|11414|11299|11432|11289|11288|11277|11239|11463|11471|11433|11241|11417|11415|11399|11462|11232|12481|11230|11416|11229|11227|11424|11458|11457|11285|11456|11270|11221|11352|11170|11279|11168|11167|11254|11253|11324|12566|11295|11317|11325|11301|11298|11175|11282|11305|11303|11233|11467|11286|11466|11512|11398|11511|11351|11283|11510|11268|11508|11507|11459|11504|11234|11464|11310|12482|11473|11455|11154|11530|11396|11529|11393|11434|11527|11182|11479|11391|11495|11309|11572|11494|11308|11476|11491|11307|11568|11567|11306|11519|11322|11280|11480|11315|11423|11470|11469|11468|11327|11443|11251|11436|11430|11410|11429|11349|11297|11348|11427|11224|11199|11190|11344|11187|11428|11284|11188|11313|11478|11269|11218|11314|11509|11346|11155|11395|11394|11259|11202|11258|11257|11421|11176|11390|11256|11304|11157|11501|11153|11460|11489|11485|11350|11287|11484|11483|11475|11465|11312|11281|11265|11271|11263|11323|11231|11332|11331|11397|11247|11330|11328|11248|11422|11246|11326|11245|11235|11474|11240|11291|11276|11418|11329|11290|11274|11273|11255|11244|11228|11292|11302|11477|11406|11243|11411|11472|",
["Hyjal Summit"] = "10947|",
["Icecrown"] = "13360|13331|13309|13308|13349|13072|13144|13068|13340|13383|13136|13074|13134|13364|13305|13380|13293|13282|13119|13275|13271|13117|13263|13236|13394|13392|13391|13120|13390|13283|13389|13297|13388|13387|13176|13175|13403|13402|13401|13400|13399|13398|13397|13396|13395|12838|13160|13168|13482|13235|13147|13220|13076|12814|13075|13077|13313|13079|13281|12810|13329|13328|12807|13228|13121|13319|13069|13320|13294|13238|13237|13302|12898|12896|13393|13481|12839|13332|13314|13221|13146|13145|13143|13334|13091|13059|13333|12813|13083|13082|13081|13080|13259|13367|13366|13357|13310|13354|13301|13352|13339|13351|13344|13323|13322|13418|13289|13330|13404|13304|13279|13277|13276|13264|13262|13381|13353|13365|13212|13306|13317|13261|13373|13278|13214|13164|13142|13140|12852|13379|13138|13239|13133|13376|12806|13233|13342|13106|13318|13315|13321|13337|13338|13406|13335|12847|13292|12840|13288|13071|13346|13345|13382|13042|13260|13358|12887|13350|13298|13219|13295|13218|13216|13215|13213|12815|13258|13152|13234|12943|13217|12939|13211|13043|12938|13356|12897|13312|12893|13316|12999|13359|12891|12995|12992|12899|12982|13163|13084|13229|13162|13161|13092|12892|13230|13085|13155|13174|12955|13172|13171|12951|13170|12949|13169|13378|13137|13307|13184|13355|13173|13093|13374|13300|13419|13296|13073|13341|13290|13386|13291|13336|13287|13286|13284|13280|13363|13232|13362|13231|13361|13368|13225|13224|13135|13130|13125|13157|13141|13139|13110|13122|13118|13104|13105|13008|13070|13227|13086|13045|13044|13040|13039|13036|13226|",
["Inscription"] = "13311|12518|12798|12517|",
["Isle of Conquest"] = "14164|14163|",
["Isle of Quel'Danas"] = "11541|11482|11524|11525|11526|11532|11523|11548|11546|11556|11481|11539|11537|11536|11533|11547|11538|11542|11535|11549|11545|11555|11550|11557|11543|11540|11496|11554|",
["Jewelcrafting"] = "12918|14160|13148|13004|13041|12963|12962|12961|12959|12958|12960|13002|12952|",
["Karazhan"] = "10733|9638|10727|10735|9637|10729|11031|10725|10734|9838|10740|9843|9640|11033|9644|10739|11216|11034|12616|10731|9639|10736|10730|11032|9844|9840|10741|10732|10728|10738|9645|10726|9825|9824|9630|",
["Leatherworking"] = "5145|5148|5146|5143|5144|5141|2856|2852|2858|2859|2851|2860|2855|2854|2853|2850|2849|2848|2847|2857|1582|",
["Loch Modan"] = "2501|3182|2500|256|271|531|283|255|161|250|199|436|297|274|280|278|258|217|257|307|1339|237|263|454|2038|298|309|273|302|301|385|1338|224|267|418|416|6387|6388|6392|6391|",
["Magisters' Terrace"] = "11499|11492|11490|11488|11500|",
["Magtheridon's Lair"] = "13430|10888|",
["Mana-Tombs"] = "10981|10977|10165|10218|10216|",
["Maraudon"] = "7066|7065|7064|7046|7044|7067|7028|7029|7041|7068|7070|",
["Moonglade"] = "8741|8736|5526|8735|5527|6845|1185|1124|",
["Mulgore"] = "14087|776|770|854|765|773|764|759|833|775|861|760|749|758|751|766|746|743|772|756|11129|771|754|767|761|745|748|",
["Nagrand"] = "10669|10668|10641|11502|9887|9885|10646|11503|10689|9886|10640|11880|9884|9991|9999|10477|10212|10102|10101|10044|10082|10085|9977|10081|10045|10175|10172|10171|10170|10168|10004|10001|9852|10478|10011|9983|9982|9946|9956|9872|9868|9866|9944|9954|9937|10076|10077|9938|9948|9945|9867|9864|9865|9855|10075|9850|10074|9873|9955|9973|9900|9972|9856|9970|9851|9967|11037|10251|10252|9962|9893|9879|9853|9878|9874|9862|9861|9871|11044|9849|11048|10479|11042|9892|10476|9913|9883|9924|9870|9939|10108|9936|9915|9789|9858|9916|9863|9907|9932|9931|9925|9882|9933|9934|9940|9928|9927|9914|9859|10109|9869|10107|10111|9810|9922|9935|9815|9821|9918|9920|10113|10114|9921|9910|9906|9891|9897|9857|9854|9917|9805|9818|9804|9923|9819|9800|",
["Naxxramas"] = "9102|9101|9100|9099|9098|9097|9096|9095|9111|9050|9048|9047|9046|9045|9044|9043|9243|9242|9241|9240|9236|9234|9235|9237|9109|9108|9106|9112|9105|9104|9110|9103|9086|9115|9230|9107|9229|9116|9113|9033|9233|9120|9114|9077|9056|9083|9041|9034|9049|9246|9245|9244|9232|9239|9238|9118|9117|9093|9092|9091|9090|9089|9088|9087|9084|9082|9081|9080|9079|9078|9075|9074|9073|9072|9071|9070|9069|9068|9061|9060|9059|9058|9057|9055|9054|9042|9040|9039|9038|9037|9036|",
["Netherstorm"] = "10341|10969|10317|10439|10318|10310|10202|10311|10365|10422|10411|10970|10437|9631|10406|10335|10315|10385|10275|10384|10353|10973|10345|10336|10276|10971|10323|10405|10339|10972|10438|10425|10432|10257|10290|10413|10508|10274|10273|10272|10271|10338|10281|11877|10431|10381|10380|10408|10328|10322|10409|10410|10850|10507|10270|10407|10234|10348|10188|10240|10321|10343|10300|10330|10652|10433|10319|10209|10307|10426|10856|10306|10305|10203|10266|10423|10924|10237|10333|10332|10316|10312|10267|10301|10256|10435|10235|10223|10857|10314|10239|10320|10233|10232|10855|10337|10434|10334|10424|10331|10440|10269|10268|10436|10430|10293|10187|10429|10427|10205|10244|10248|10249|10247|10192|10222|10265|10189|10264|10309|10221|10206|10185|10184|10173|10313|10299|10342|10199|10246|10245|10417|10308|10198|10200|10197|10701|10194|10262|10261|10174|10182|10176|10183|10204|10193|10190|10329|10191|10225|10224|10186|10226|10243|10241|10418|10263|",
["Noblegarden"] = "13484|13503|13479|13480|13502|13483|",
["Northshire Valley"] = "54|21|6|3905|18|3904|3903|15|5261|33|7|783|5805",
["Onyxia's Lair"] = "7509|",
["Orgrimmar"] = "13266|9007|8938|6566|8957|9022|8942|8944|8943|8941|9010|8940|8939|9020|7833|9019|10497|9016|9021|8927|7784|10493|7783|8919|6567|8918|8917|8915|8913|9018|9017|7827|10499|7826|7837|7835|7831|7490|7834|8916|7832|7824|8914|8978|7491|9014|9012|7836|4941|7341|7493|9009|10495|8920|9013|9011|9008|8998|3506|3507|1018|8276|4511|4494|3504|3505|4300|936|650|649|81|2981|2950|1433|1436|1435|1434|1146|1432|1431|9626|14086|235|5730|5729|5727|5726|7926|",
["Ragefire Chasm"] = "5725|5722|5728|5761|5724|5723|",
["Razorfen Downs"] = "3341|3636|3523|3525|6522|6521|6626|",
["Razorfen Kraul"] = "1142|1144|1102|1101|1221|1109|",
["Redridge Mountains"] = "169|249|180|219|19|128|34|91|115|178|248|94|127|20|126|124|89|150|145|122|119|118|92|146|246|244|125|116|3741|130|131|129|144|143|120|121|",
["Ruins of Ahn'Qiraj"] = "8791|",
["Scarlet Monastery"] = "1048|1053|1049|1050|1160|1113|1051|",
["Scholomance"] = "5341|5343|5382|5515|8992|5384|5466|8969|5531|4771|5582|5529|",
["Searing Gorge"] = "8241|7737|13662|8242|7736|3463|3481|7701|3462|3454|3453|3452|7722|3378|3377|7724|7723|7727|3442|3441|7729|7728|3443|4451|3372|3566|3368|3367|4450|4449|3181|",
["Shadow Labyrinth"] = "9831|",
["Shadowfang Keep"] = "1014|1013|1098|",
["Shadowmoon Valley"] = "11071|10519|10515|10577|10836|10775|10814|13429|10804|10772|10680|10781|11018|11016|10692|10776|10823|11101|11100|11099|10458|11086|10648|10644|10824|10688|10684|11108|11107|10817|11019|11014|11013|11012|10769|11069|10870|11083|11082|11081|11077|11075|10628|11055|11053|10622|11041|11017|10637|10673|10676|10866|10679|10944|10650|10691|10634|10670|10667|10807|10575|10687|10683|11052|10550|10528|10527|11084|10547|11068|10816|10546|11067|10522|11064|10574|10521|11063|10451|10523|11050|10541|10588|10540|10537|10773|10678|11054|10985|10949|11498|11095|10706|10793|11544|10685|10707|11049|10858|10619|11090|10872|11089|11076|10854|10837|10645|10576|11035|10571|10570|10480|11020|10568|11015|10481|10651|10708|10635|10636|10514|11094|11092|10513|11097|10751|10639|10768|10647|10750|10745|10681|11070|10765|10871|10811|10587|10633|10774|10578|10744|11497|10579|10686|10589|10612|10606|10597|10623|10627|10662|10598|10759|10760|10703|10677|10643|10642|10661|10778|10572|10569|10564|10563|10562|10573|10621|10585|10583|10582|10626|10604|10603|10808|10782|10602|10601|10600|10761|10664|10599|10702|10660|10777|10780|10666|10665|10663|10625|10613|10596|10611|10767|10672|10595|10624|10586|10766|",
["Shattrath City"] = "10416|10654|10655|10421|10826|11517|11024|11113|10420|10280|10884|10415|11114|10886|10885|10419|10883|10656|10828|10948|10659|10658|11534|9837|10984|11110|10653|11109|9829|10414|10827|11112|11875|10822|9832|11451|11111|10509|11096|10231|11039|11045|11038|11046|10327|10326|10553|10325|10412|10551|10554|10552|10211|10169|10024|10025|10019|10021|10020|10017|9979|",
["Sholazar Basin"] = "12758|12732|12737|12726|12736|12734|12735|12741|12803|12703|12548|12546|12761|12797|12608|12691|12760|12762|12759|12705|12671|12699|12581|12702|12603|12681|12683|12614|12658|12607|12605|12689|12595|12611|12561|12695|12573|12571|12570|12538|12537|12536|12660|12621|12620|12617|12696|12613|12559|12580|12578|12576|12575|12574|12572|12579|12534|12533|12532|12530|12529|12692|12582|12539|12805|12612|12540|12586|13052|12704|12535|12531|12528|12577|12651|12523|12522|12654|12624|12550|12592|12804|12525|12551|12544|12569|12560|12558|12556|12526|12644|12634|12645|12489|12521|12543|12524|12589|12688|12549|12520|",
["Silithus"] = "8786|8806|8540|8496|8739|9415|8319|8341|8745|8781|8804|8501|8538|8541|8303|8352|8785|8310|8787|8810|8519|8306|8809|8305|8859|8858|8732|8302|8539|8779|9422|8773|8349|8857|8380|8805|8376|8314|8315|8364|8343|8782|8348|8808|8778|8378|8351|8534|8379|9338|8807|8507|8321|9419|8331|8304|8332|8320|8318|8536|8772|8309|8770|8731|8829|8301|8497|8316|8286|8687|9248|8308|8279|8377|8508|8362|8287|8800|8780|8535|8361|8771|8537|8502|8363|9023|8775|8333|8783|8777|8776|8774|8498|8574|8740|8573|8572|8738|8548|8737|8382|8342|8743|8381|9416|8323|8324|8283|8285|8284|8282|1126|8281|8278|6844|8277|8280|1125|",
["Silvermoon City"] = "10360|10359|10363|10362|10361|9627|",
["Silverpine Forest"] = "480|516|493|442|530|461|451|491|460|444|443|479|438|439|448|446|441|440|9189|450|452|424|99|437|477|423|478|482|481|447|428|425|3221|449|435|430|422|429|421|6323|6324|6321|6322|",
["Stonetalon Mountains"] = "1088|1089|5881|1084|1081|1096|1057|1095|1059|1058|6282|1083|6283|1078|1087|6393|6381|6301|1068|1092|1090|1091|1080|1079|1082|1094|1093|1074|1073|6284|1075|1071|1072|1077|1076|6481|6461|6542|1062|6421|6523|1063|6401|6548|6629|1061|",
["Stormwind City"] = "7791|7796|7794|7782|7781|7497|7795|6501|7793|6183|7496|6186|6403|6402|7495|6182|1015|5048|3787|3789|5066|1477|1449|1448|1364|1363|543|212|1324|542|538|1267|540|1266|1301|1265|1264|1250|1249|690|1447|1247|1246|2745|350|396|434|2746|395|394|1245|397|335|336|1244|393|392|1274|1242|1241|1243|343|347|346|3765|345|344|389|373|399|1097|353|333|334|332|579|",
["The Stockade"] = "391|378|377|388|387|386|",
["Strand of the Ancients"] = "13405|13407|",
["Stranglethorn Vale"] = "4621|1036|3626|3625|7810|9272|626|580|620|8552|206|591|593|592|586|588|589|348|613|609|621|573|1127|625|208|197|624|623|617|576|8553|8551|209|8554|615|630|608|604|598|594|618|614|619|196|607|587|606|595|599|597|571|572|584|600|205|585|193|611|202|342|341|339|338|340|610|628|207|574|192|570|596|638|569|629|582|601|603|627|602|578|210|616|622|328|331|330|329|188|568|9457|213|577|195|200|187|605|349|189|581|9436|194|204|215|203|191|186|198|201|185|190|575|583|9259|9266|",
["Stratholme"] = "8945|5848|5213|5282|5125|5263|5243|5262|8968|5212|5251|5214|8991|6163|5122|5463|",
["The Culling of Stratholme"] = "13149|13151|13251|13243|",
["Swamp of Sorrows"] = "3374|3512|2623|2622|1428|1422|1429|1430|1427|1426|1424|9610|1398|699|1425|1423|698|1392|9448|9440|1393|1396|9609|1116|1421|1389|1418|",
["Tailoring"] = "10832|10831|10833|3379|3385|3402|13270|13269|13268|13265|13272|",
["Tanaris"] = "8728|2747|8766|8598|8759|8758|8555|8757|8599|8764|8756|8750|8924|8586|8765|8584|8922|8925|8742|8587|8620|8597|8606|8928|8577|2748|8585|8576|8760|8755|8754|8753|8752|8751|8749|8748|8747|8923|8921|8761|2750|2749|8182|5801|5802|4508|4507|4509|4504|4496|3444|1560|3362|2954|162|2606|2641|2661|2662|5863|2605|648|110|351|113|10|32|3161|8893|3022|82|2741|379|243|2781|841|654|992|2873|8366|8365|2874|2876|2872|2875|1691|1707|3520|1878|1690|9268|",
["Teldrassil"] = "8734|938|486|927|941|934|940|935|7383|937|931|2399|930|2561|2499|933|483|923|2498|2541|2459|487|489|932|919|918|922|2026|2438|476|475|997|929|488|2159|",
["Tempest Keep"] = "10560|11388|10882|11387|10705|11384|11007|11386|11385|10946|11389|10704|",
["Terokkar Forest"] = "11520|11521|11074|10180|10009|10010|10877|9985|10920|9984|10915|10036|10031|10913|10930|10929|10926|10923|10922|10921|10914|10889|10898|10879|10874|10196|10873|10842|10446|10042|10040|9890|9889|10896|10918|10917|10043|10030|10041|10034|9959|10210|10840|10035|10033|9888|10029|10447|10201|9980|9947|9943|10023|10018|9950|10881|9990|10008|10861|10012|10448|10013|9997|9995|10880|10039|10027|9996|9994|10848|10444|10049|10048|10852|10839|9987|10887|10037|9949|10052|9951|10112|10038|10022|10026|10016|10002|10007|9986|10791|10003|10051|9961|10863|9942|9941|9960|9930|10908|10849|10005|10006|10862|10032|10000|10878|9957|9971|9968|9929|9978|10028|9998|9992|9993|10868|10869|10847|11505|11506|",
["Tournament"] = "13762|13681|13757|13666|13675|13695|13726|14112|13668|13710|13793|13791|13707|13770|13665|13616|13788|13847|13592|13783|13706|13820|13772|13600|13669|13789|13851|13790|13717|13855|14142|13761|13634|13750|13749|13748|13852|14017|14016|13742|13678|13715|13752|13663|13704|13641|13672|13671|13670|13708|13794|13625|13743|13760|13719|13731|13813|13795|13643|13812|13864|13740|13769|13664|13863|13633|13679|13862|13860|13858|13857|13745|13744|13741|13654|13811|13810|13699|13735|13673|13809|13714|13765|13839|13777|13838|13776|13773|13829|13771|13689|13688|13684|14104|13721|13690|13720|14152|13758|14105|13764|13696|13763|13767|13697|13856|14145|13759|14143|14136|13718|14107|14092|14095|13705|13861|13723|14074|13787|13786|13814|13739|13693|13785|13729|13837|13859|13722|13774|13733|14090|14096|14080|13755|13746|14076|13716|13732|14102|13724|13781|13780|13784|14101|13713|13734|13702|13725|13627|13738|13728|13768|14141|14140|13828|13691|13754|13835|13694|13667|14144|13603|13782|13703|13779|13854|13778|13756|13775|13753|13680|13846|13677|13737|13676|13727|13674|13711|14108|13709|13685|13682|13747|13593|13736|14077|",
["The Barrens"] = "1145|857|3514|1153|873|874|849|846|885|3369|906|3513|879|3370|884|822|893|897|843|868|883|5052|878|1483|5046|5045|5044|5042|6382|5043|898|899|1060|913|889|891|1069|4921|4021|876|882|3924|852|6541|863|907|3261|3281|858|3923|865|896|851|905|895|850|902|881|880|877|875|888|821|853|867|903|3301|848|3922|872|819|5041|894|887|855|901|900|3921|892|890|869|870|845|871|844|1492|6365|886|6364|6362|6361|860|6363|6386|6385|6384|9267|",
["The Deadmines"] = "166|167|2040|168|214|",
["The Exodar"] = "10356|7798|10358|10357|7792|10520|10373|",
["The Eye of Eternity"] = "13385|13384|",
["The Hinterlands"] = "2938|2937|2994|7862|7845|7847|7861|7846|7815|4787|7850|7849|7843|7840|7841|2877|7816|2989|7844|7830|7829|7828|836|485|7842|7839|77|2782|8273|2742|2992|2993|2990|9475|9470|9469|9476|2881|2988|2880|2935|2934|1444|9471|1450|1451|2933|1469|1452|2932|",
["Molten Core"] = "11116|9952|7838|7676|1782|7673|7678|7677|7672|9029|8229|8228|",
["The Nexus"] = "13246|13094|11905|11911|11973|13095|",
["The Oculus"] = "13128|13127|13247|13240|13126|13124|",
["The Slave Pens"] = "10901|13431|10900|",
["The Steamvault"] = "9765|9764|9763|",
["The Storm Peaks"] = "13046|13007|13057|13010|13005|12874|12976|12973|12876|12875|12873|12889|13051|13006|13423|13559|12998|12991|13037|12988|12984|12981|13426|12977|13285|13274|12965|12964|13038|12937|13034|13417|12957|12978|12931|13425|12917|12888|12900|12966|12886|12877|12856|12851|12975|12986|12878|12880|12879|13064|13063|13062|13050|12985|13035|13011|12997|12996|12989|12972|12971|12970|12969|12968|13047|12967|12956|12953|13424|12942|13009|12925|12924|12922|13058|13048|12872|12987|13049|12915|12983|12994|13416|12860|12881|12895|12854|13415|12928|12871|13003|12858|13001|13056|12855|13055|12885|13054|13273|12993|12913|12980|12979|12930|12929|12927|12926|12920|12910|12909|13422|12905|12844|12843|12841|13061|12921|12908|12907|12906|13000|12868|12867|12825|12853|12846|12836|12833|12832|12831|12829|12828|12827|12869|13060|12820|12819|12818|12830|12826|12824|12823|12822|12821|12866|12865|12864|12863|12862|12870|12882|13421|13420|",
["The Temple of Atal'Hakkar"] = "8733|3373|1446|3528|4146|3446|3447|3380|1475|1445|",
["The Underbog"] = "9715|9714|9717|9719|",
["Undercity"] = "13267|7814|7817|7813|7819|7818|4294|5961|1004|4642|5049|3568|3569|3570|4293|4561|4661|3784|2995|864|232|238|736|495|1164|9812|9621|1358|9180|9177|1359|357|",
["The Violet Hold"] = "13256|13158|13159|",
["Thousand Needles"] = "2769|1120|1119|1122|1121|1118|1192|1194|1190|1187|1188|1189|1108|1137|1181|1186|1178|1183|1180|1182|1115|1114|1112|1117|1111|1394|1147|1148|1184|1106|1107|5361|1175|1110|5762|1152|1151|1150|6627|1159|1154|1176|1105|1104|6628|5151|1179|4904|4770|4767|5147|5088|4881|9434|9433|4966|5064|9431|5062|1100|1149|4821|4865|4542|4841|",
["Thunder Bluff"] = "13986|7823|7821|7825|7822|7820|3804|1123|1000|3761|3782|2966|3762|2965|2968|2967|3786|2440|1136|1130|1131|1196|1197|1195|1086|1067|1066|742|1065|1064|264|744|769|768|",
["Timbermaw Hold"] = "8485|8481|8484|8464|6241|6221|8461|6031|6032|",
["Tirisfal Glades"] = "5901|5902|14089|408|411|409|372|354|492|369|356|371|355|445|362|398|431|410|360|359|370|368|405|375|358|427|426|366|361|407|374|365|367|5482|404|5481|",
["Trial of the Champion"] = "14199|",
["Uldaman"] = "2279|2278|2280|1139|2339|2361|2204|2340|2341|2342|1360|2201|2200|2202|17|3375|2318|2338|2199|2284|2198|2283|722|2398|2240|709|704|2418|721|",
["Ulduar"] = "13607|13609|13824|13823|13818|13604|13614|13816|13822|13622|13611|13817|13610|13606|13821|",
["Un'Goro Crater"] = "12547|4292|3962|980|5150|4301|4491|4289|4501|974|4502|4492|4084|4005|3961|3942|13903|4284|4148|4245|4321|13906|13915|4290|4384|13850|4287|4244|3881|13914|4288|4385|4285|13916|4291|13917|13887|4243|13905|13904|4386|13889|4383|4382|4381|4144|3883|3913|3941|3914|4145|3908|3845|3844|4143|4141|4147|3909|3912|4142|3882|4503|3884|",
["Utgarde Keep"] = "13245|11252|13206|11262|13205|11272|",
["Utgarde Pinnacle"] = "13248|13241|13131|13132|",
["Valley of Trials"] = "805|804|794|790|792|6394|5441|4402|789|788|5843|4641|",
["Wailing Caverns"] = "6981|999|3366|914|1487|959|1491|962|1486|1490|1489|",
["Warsong Gulch"] = "11338|14183|14180|11342|",
["Western Plaguelands"] = "5166|5164|5505|211|5803|5511|5461|5344|5465|5167|5521|5804|5342|5162|6184|5507|5524|5462|105|5944|5862|5861|5504|9444|9474|5235|5226|5225|5237|9443|5227|9446|6025|5236|5238|5537|964|5538|6023|5514|4973|5098|4987|6004|4971|5154|4985|5097|4986|4972|5408|5405|6390|6389|5533|5058|5904|5903|5220|5222|5404|5403|5401|5224|5221|5402|5407|5406|5219|5223|5231|5234|5233|5232|5059|5060|838|4984|5051|5216|5215|5217|5229|5218|5096|5230|5228|5092|5050|5023|5021|5022|10374|5093|5095|5094|",
["Westfall"] = "53|48|49|50|51|104|152|142|141|135|65|132|155|14|103|140|139|138|136|153|9|117|102|151|13|38|64|22|12|6181|6285|6261|6281|36|184|",
["Wetlands"] = "1302|634|633|631|304|632|303|290|293|292|647|296|289|299|295|288|474|473|465|464|471|275|472|284|281|286|285|294|306|305|470|277|279|484|468|463|455|469|276|1132|",
["Wintergrasp"] = "13199|13180|13223|13198|13222|13153|13192|13181|13186|13201|13193|13195|13179|13183|13196|13154|13200|13191|13185|13178|13539|13538|13194|13177|13197|236|13156|13202|",
["Winterspring"] = "4970|4803|5056|969|8798|4806|4807|6606|4801|5161|975|5201|4805|5068|4804|5126|5057|5981|5067|5124|5063|4802|4883|4882|4864|5128|5123|6604|4863|5121|4901|4861|5252|5253|5163|977|5055|979|4902|5087|5086|5244|5085|5084|5083|5250|5054|6603|5082|5245|3783|8471|4842|978|4810|4809|6605|6030|6028|6029|3661|",
["Zangarmarsh"] = "9729|9823|9822|9834|9833|9830|9905|10116|9904|9903|9702|9902|9792|10118|9809|9841|9784|9820|9727|9814|10117|9797|9776|9845|9835|9848|9839|10115|9919|9744|9730|9728|9724|9720|9785|9778|9807|9733|9743|9847|9842|9816|9709|9726|9911|9808|9806|9708|9701|9739|9742|9875|9732|9718|9716|9817|9697|9802|9731|9752|10104|9898|9896|9783|9770|9801|9791|9901|9793|9786|9782|9781|9899|9777|9796|9775|9774|9772|9771|9846|9790|9780|9747|9912|9895|9788|10096|9894|9828|10355|9803|9827|9787|10105|9769|9773|",
["Zul'Aman"] = "11132|11196|11166|11164|11171|11165|11195|11178|11130|",
["Zul'Drak"] = "12948|13099|12730|13097|12656|12642|12632|13556|13549|12655|12650|12712|12932|12954|12936|12935|12934|12933|12974|12662|12659|12640|12639|12635|12623|12615|12721|12709|12708|12707|12685|12684|12675|12674|12672|12668|12622|12667|12665|12729|12666|12628|12627|12647|12618|12653|12646|12585|12568|12564|12563|12713|12904|12903|12504|12602|12601|12610|12609|12606|12914|12599|12597|12596|12901|12583|12565|12562|12557|12554|12527|12598|12514|12512|12510|12508|12507|12506|12505|12503|12584|12740|12555|12553|12552|12502|12604|12594|12509|12591|12588|12919|12541|12916|12519|12587|12912|12516|12567|12663|12793|12648|12884|12883|12637|12631|12789|12630|12629|12861|12859|12857|12690|12686|12677|12673|12902|12669|12664|12894|12661|12652|12799|12795|12649|12643|12638|12633|12710|12792|12676|",
["Zul'Farrak"] = "2770|2768|3527|2991|2861|2846|2864|2865|2936|3042|",
["Zul'Gurub"] = "8072|8063|8071|8103|8066|8188|8112|8111|8109|8107|8147|8148|8146|8192|8190|8058|8187|8077|8186|8101|8108|8116|8046|8045|8142|8240|8119|8113|8057|8065|8064|8104|8106|8059|8052|8067|8044|8047|8062|8073|8227|8145|8143|9209|8060|8141|8069|8068|8246|8189|8239|8238|9210|8184|8074|8076|8049|8061|8048|8110|8201|8041|8102|8051|8191|8118|8079|8117|9208|8070|8042|8053|8055|8144|8183|8050|8043|8243|8196|8195|8075|8056|8054|8185|8078|8181|",
}
-- Insert QuestDB here
local QuestDB = {
	"|Q1;Y.1;T.P;|Q2;Y.1;T.D;|Q5;|Q6;O.1;o1.I;|Q7;O.1;o1.M;|Q8;|Q9;O.1;o1.M;|Q10;O.1;o1.I;|Q11;O.1;o1.I;|Q12;O.2;o1.M;o2.M;|Q13;O.2;o1.M;o2.M;|Q14;O.3;o1.M;o2.M;o3.M;|Q15;O.1;o1.M;|Q16;O.1;o1.I;|Q17;O.1;o1.I;|Q18;O.1;o1.I;|Q19;O.1;o1.I;|Q20;O.1;o1.I;|Q21;O.1;o1.M;|Q22;O.1;o1.I;|Q23;|Q24;|Q25;O.2;o1.E;o2.M;|Q26;|Q27;|Q28;O.1;o1.E;|Q29;O.1;o1.E;|Q30;O.1;o1.I;|Q31;|Q32;|Q33;O.1;o1.I;|Q34;O.1;o1.I;|Q35;|Q36;|Q37;|Q38;O.4;o1.I;o2.I;o3.I;o4.I;|Q39;|Q40;|Q45;|Q46;O.1;o1.I;|Q47;O.1;o1.I;|Q48;O.1;o1.I;|Q49;O.3;o1.I;o2.I;o3.I;|Q50;O.1;o1.I;|Q51;O.1;o1.I;|Q52;O.2;o1.M;o2.M;|Q53;O.1;o1.I;|Q54;|Q55;O.1;o1.O;|Q56;O.2;o1.M;o2.M;|Q57;O.2;o1.M;o2.M;|Q58;O.1;o1.M;|Q59;|Q60;O.1;o1.I;|Q61;|Q62;O.1;o1.E;|Q63;O.1;o1.I;|Q64;O.1;o1.I;|Q65;|Q66;|Q67;|Q68;|Q69;|Q70;O.1;o1.I;|Q71;|Q72;|Q73;|Q74;|Q75;O.1;o1.I;|Q76;O.1;o1.E;|Q77;O.1;o1.I;|Q78;|Q79;|Q80;|Q81;|Q82;O.1;o1.I;|Q83;O.1;o1.I;|Q84;|Q85;|Q86;O.1;o1.I;|Q87;O.1;o1.I;|Q88;O.1;o1.I;|Q89;O.2;o1.I;o2.I;|Q90;O.2;o1.I;o2.I;|Q91;O.1;o1.I;|Q92;O.3;o1.I;o2.I;o3.I;|Q93;O.1;o1.I;|Q94;|Q95;|Q96;|Q97;|Q98;O.1;o1.I;|Q99;O.1;o1.I;|Q100;|", -- [1]
	"|Q101;O.3;o1.I;o2.I;o3.I;|Q102;O.1;o1.I;|Q103;O.1;o1.I;|Q104;O.1;o1.I;|Q105;O.1;o1.I;|Q106;|Q107;|Q108;|Q109;|Q110;|Q111;|Q112;O.1;o1.I;|Q113;|Q114;|Q115;O.1;o1.I;|Q116;O.4;o1.I;o2.I;o3.I;o4.I;|Q117;O.1;o1.I;|Q118;|Q119;|Q120;|Q121;|Q122;O.1;o1.I;|Q123;|Q124;O.2;o1.M;o2.M;|Q125;O.1;o1.I;|Q126;O.1;o1.I;|Q127;O.1;o1.I;|Q128;O.1;o1.M;|Q129;|Q130;|Q131;|Q132;|Q133;O.1;o1.I;|Q134;O.1;o1.I;|Q135;|Q136;|Q137;O.1;o1.I;|Q138;|Q139;|Q140;|Q141;|Q142;O.1;o1.I;|Q143;|Q144;|Q145;|Q146;|Q147;O.1;o1.I;|Q148;|Q149;|Q150;O.1;o1.I;|Q151;O.1;o1.I;|Q152;O.4;o1.M;o2.M;o3.M;o4.M;|Q153;O.1;o1.I;|Q154;|Q155;O.1;o1.E;|Q156;O.1;o1.I;|Q157;|Q158;|Q159;|Q160;|Q161;|Q162;|Q163;|Q164;|Q165;|Q166;O.1;o1.I;|Q167;O.1;o1.I;|Q168;O.1;o1.I;|Q169;O.1;o1.I;|Q170;O.2;o1.M;o2.M;|Q171;O.1;o1.I;|Q172;|Q173;O.1;o1.M;|Q174;O.1;o1.I;|Q175;|Q176;O.1;o1.I;|Q177;O.1;o1.I;|Q178;|Q179;O.1;o1.I;|Q180;O.1;o1.I;|Q181;O.1;o1.I;|Q182;O.1;o1.M;|Q183;O.1;o1.M;|Q184;|Q185;O.1;o1.M;|Q186;O.1;o1.M;|Q187;O.1;o1.M;|Q188;O.1;o1.I;|Q189;O.1;o1.I;|Q190;O.1;o1.M;|Q191;O.1;o1.M;|Q192;O.1;o1.M;|Q193;O.1;o1.I;|Q194;O.1;o1.M;|Q195;O.1;o1.M;|Q196;O.1;o1.M;|Q197;O.1;o1.I;|Q198;|Q199;|Q200;|", -- [2]
	"|Q201;O.1;o1.E;|Q202;O.3;o1.M;o2.M;o3.I;|Q203;O.1;o1.M;|Q204;O.2;o1.I;o2.I;|Q205;O.1;o1.I;|Q206;O.1;o1.I;|Q207;O.4;o1.I;o2.I;o3.I;o4.I;|Q208;T.G.3;O.1;o1.I;|Q209;O.1;o1.I;|Q210;|Q211;O.1;o1.I;|Q212;O.1;o1.I;|Q213;O.1;o1.I;|Q214;O.1;o1.I;|Q215;|Q216;O.2;o1.M;o2.M;|Q217;O.3;o1.M;o2.M;o3.M;|Q218;O.1;o1.I;|Q219;O.1;o1.E;|Q220;|Q221;O.1;o1.M;|Q222;O.2;o1.M;o2.M;|Q223;|Q224;O.2;o1.M;o2.M;|Q225;|Q226;O.2;o1.M;o2.M;|Q227;|Q228;O.1;o1.I;|Q229;|Q230;|Q231;|Q232;|Q233;|Q234;|Q235;|Q236;T.P;O.1;o1.I;|Q237;O.2;o1.M;o2.M;|Q238;|Q239;|Q240;|Q241;|Q242;O.3;o1.M;o2.M;o3.M;|Q243;|Q244;|Q245;O.1;o1.M;|Q246;O.2;o1.M;o2.M;|Q247;|Q248;|Q249;O.1;o1.I;|Q250;|Q251;|Q252;|Q253;O.1;o1.I;|Q254;|Q255;O.3;o1.M;o2.M;o3.M;|Q256;O.1;o1.I;|Q257;O.1;o1.M;|Q258;O.1;o1.M;|Q259;|Q260;O.2;o1.I;o2.I;|Q261;O.1;o1.M;|Q262;|Q263;O.2;o1.M;o2.M;|Q264;|Q265;|Q266;|Q267;O.1;o1.I;|Q268;|Q269;|Q270;|Q271;O.1;o1.I;|Q272;O.1;o1.I;|Q273;O.1;o1.E;|Q274;|Q275;O.1;o1.M;|Q276;O.2;o1.M;o2.M;|Q277;O.1;o1.I;|Q278;O.3;o1.I;o2.I;o3.I;|Q279;O.2;o1.M;o2.I;|Q280;|Q281;|Q282;|Q283;|Q284;|Q285;|Q286;|Q287;O.2;o1.E;o2.M;|Q288;O.1;o1.I;|Q289;O.3;o1.M;o2.M;o3.I;|Q290;O.1;o1.I;|Q291;|Q292;|Q293;|Q294;O.2;o1.M;o2.M;|Q295;O.2;o1.M;o2.M;|Q296;O.1;o1.I;|Q297;O.1;o1.I;|Q298;|Q299;O.4;o1.I;o2.I;o3.I;o4.I;|", -- [3]
	"|Q301;|Q302;|Q303;O.4;o1.M;o2.M;o3.M;o4.M;|Q304;O.1;o1.I;|Q305;|Q306;|Q307;O.1;o1.I;|Q308;O.1;o1.I;|Q309;O.1;o1.E;|Q310;|Q311;|Q312;O.1;o1.I;|Q313;O.1;o1.I;|Q314;O.1;o1.I;|Q315;O.1;o1.I;|Q316;O.1;o1.I;|Q317;O.2;o1.I;o2.I;|Q318;|Q319;O.3;o1.M;o2.M;o3.M;|Q320;|Q321;|Q322;|Q323;O.3;o1.M;o2.M;o3.M;|Q324;O.1;o1.I;|Q325;|Q326;O.1;o1.I;|Q327;|Q328;|Q329;|Q330;|Q331;|Q332;|Q333;|Q334;|Q335;O.2;o1.I;o2.I;|Q336;|Q337;|Q338;O.4;o1.I;o2.I;o3.I;o4.I;|Q339;O.4;o1.I;o2.I;o3.I;o4.I;|Q340;O.4;o1.I;o2.I;o3.I;o4.I;|Q341;O.4;o1.I;o2.I;o3.I;o4.I;|Q342;O.3;o1.I;o2.I;o3.I;|Q343;|Q344;|Q345;|Q346;|Q347;O.1;o1.I;|Q348;O.1;o1.I;|Q349;O.1;o1.I;|Q350;|Q351;|Q352;|Q353;|Q354;O.3;o1.I;o2.I;o3.I;|Q355;|Q356;O.2;o1.M;o2.M;|Q357;O.1;o1.I;|Q358;O.3;o1.M;o2.M;o3.I;|Q359;|Q360;|Q361;|Q362;O.1;o1.I;|Q363;|Q364;O.2;o1.M;o2.M;|Q365;O.1;o1.I;|Q366;|Q367;O.1;o1.I;|Q368;O.1;o1.I;|Q369;O.1;o1.I;|Q370;O.3;o1.M;o2.M;o3.M;|Q371;O.2;o1.M;o2.M;|Q372;O.2;o1.M;o2.M;|Q373;|Q374;O.1;o1.I;|Q375;O.2;o1.I;o2.I;|Q376;O.2;o1.I;o2.I;|Q377;O.1;o1.I;|Q378;O.1;o1.I;|Q379;O.1;o1.I;|Q380;O.2;o1.M;o2.M;|Q381;O.1;o1.I;|Q382;O.1;o1.I;|Q383;|Q384;O.2;o1.I;o2.I;|Q385;O.2;o1.I;o2.I;|Q386;O.1;o1.I;|Q387;O.3;o1.M;o2.M;o3.M;|Q388;O.1;o1.I;|Q389;|Q390;|Q391;O.1;o1.I;|Q392;|Q393;|Q394;|Q395;|Q396;|Q397;|Q398;O.1;o1.I;|Q399;O.1;o1.I;|Q400;|", -- [4]
	"|Q401;|Q402;|Q403;|Q404;O.1;o1.I;|Q405;|Q406;|Q407;|Q408;O.3;o1.M;o2.M;o3.I;|Q409;O.1;o1.M;|Q410;O.1;o1.I;|Q411;|Q412;O.2;o1.I;o2.I;|Q413;|Q414;|Q415;|Q416;O.1;o1.I;|Q417;O.1;o1.I;|Q418;O.3;o1.I;o2.I;o3.I;|Q419;|Q420;|Q421;O.1;o1.M;|Q422;O.1;o1.I;|Q423;O.2;o1.I;o2.I;|Q424;O.1;o1.I;|Q425;O.1;o1.I;|Q426;O.2;o1.I;o2.I;|Q427;O.1;o1.M;|Q428;|Q429;O.1;o1.I;|Q430;|Q431;|Q432;O.1;o1.M;|Q433;O.1;o1.M;|Q434;O.3;o1.E;o2.M;o3.M;|Q435;O.1;o1.E;|Q436;|Q437;O.2;o1.E;o2.I;|Q438;|Q439;|Q440;|Q441;|Q442;O.1;o1.I;|Q443;O.1;o1.I;|Q444;|Q445;|Q446;|Q447;O.2;o1.I;o2.I;|Q448;|Q449;|Q450;O.1;o1.I;|Q451;O.3;o1.I;o2.I;o3.I;|Q452;O.1;o1.E;|Q453;|Q454;|Q455;O.3;o1.E;o2.M;o3.M;|Q456;O.2;o1.M;o2.M;|Q457;O.2;o1.M;o2.M;|Q458;|Q459;O.1;o1.I;|Q460;|Q461;|Q462;O.1;o1.M;|Q463;|Q464;O.1;o1.I;|Q465;|Q466;O.1;o1.I;|Q467;|Q468;|Q469;|Q470;O.1;o1.I;|Q471;O.1;o1.I;|Q472;|Q473;|Q474;O.1;o1.I;|Q475;|Q476;|Q477;|Q478;|Q479;O.1;o1.I;|Q480;O.1;o1.I;|Q481;|Q482;|Q483;O.4;o1.I;o2.I;o3.I;o4.I;|Q484;O.1;o1.I;|Q485;|Q486;O.1;o1.M;|Q487;O.1;o1.M;|Q488;O.3;o1.I;o2.I;o3.I;|Q489;O.1;o1.I;|Q490;O.1;o1.I;|Q491;|Q492;|Q493;|Q494;|Q495;|Q496;O.2;o1.I;o2.I;|Q497;|Q498;O.2;o1.O;o2.O;|Q499;|Q500;O.1;o1.I;|", -- [5]
	"|Q501;O.1;o1.I;|Q502;|Q503;O.2;o1.E;o2.I;|Q504;O.1;o1.M;|Q505;O.2;o1.M;o2.M;|Q506;|Q507;O.1;o1.M;|Q508;|Q509;O.1;o1.I;|Q510;|Q511;|Q512;O.1;o1.I;|Q513;|Q514;|Q515;O.3;o1.I;o2.I;o3.I;|Q516;O.2;o1.M;o2.M;|Q517;O.1;o1.I;|Q518;O.1;o1.M;|Q519;O.3;o1.I;o2.I;o3.I;|Q520;O.2;o1.I;o2.I;|Q521;|Q522;|Q523;O.1;o1.I;|Q524;|Q525;|Q526;O.1;o1.I;|Q527;O.4;o1.M;o2.M;o3.M;o4.M;|Q528;O.1;o1.M;|Q529;O.3;o1.M;o2.M;o3.I;|Q530;O.1;o1.I;|Q531;|Q532;O.4;o1.M;o2.M;o3.O;o4.I;|Q533;O.1;o1.I;|Q534;|Q535;O.1;o1.I;|Q536;O.2;o1.M;o2.M;|Q537;O.2;o1.M;o2.I;|Q538;|Q539;O.2;o1.M;o2.M;|Q540;O.2;o1.I;o2.I;|Q541;O.4;o1.M;o2.M;o3.M;o4.M;|Q542;|Q543;O.1;o1.I;|Q544;O.4;o1.I;o2.I;o3.I;o4.I;|Q545;O.2;o1.M;o2.M;|Q546;O.1;o1.I;|Q547;O.1;o1.I;|Q548;|Q549;O.2;o1.M;o2.M;|Q550;|Q551;|Q552;O.1;o1.I;|Q553;O.3;o1.O;o2.O;o3.O;|Q554;|Q555;O.1;o1.I;|Q556;O.1;o1.I;|Q557;O.1;o1.I;|Q558;O.1;o1.I;|Q559;O.1;o1.I;|Q560;|Q561;|Q562;O.2;o1.M;o2.M;|Q563;|Q564;O.2;o1.M;o2.M;|Q565;O.4;o1.I;o2.I;o3.I;o4.I;|Q566;O.1;o1.I;|Q567;O.4;o1.M;o2.M;o3.M;o4.M;|Q568;O.1;o1.M;|Q569;O.2;o1.M;o2.M;|Q570;O.2;o1.I;o2.I;|Q571;O.1;o1.I;|Q572;O.1;o1.I;|Q573;O.2;o1.M;o2.I;|Q574;O.2;o1.M;o2.M;|Q575;O.1;o1.I;|Q576;O.1;o1.I;|Q577;O.1;o1.I;|Q578;O.1;o1.E;|Q579;O.1;o1.I;|Q580;O.1;o1.I;|Q581;O.1;o1.I;|Q582;O.1;o1.I;|Q583;|Q584;O.2;o1.I;o2.I;|Q585;O.3;o1.I;o2.I;o3.I;|Q586;O.4;o1.M;o2.M;o3.M;o4.I;|Q587;O.1;o1.I;|Q588;|Q589;O.1;o1.I;|Q590;O.1;o1.E;|Q591;O.1;o1.I;|Q592;O.1;o1.I;|Q593;O.1;o1.I;|Q594;|Q595;|Q596;O.1;o1.I;|Q597;|Q598;O.1;o1.I;|Q599;|Q600;O.1;o1.I;|", -- [6]
	"|Q601;O.1;o1.I;|Q602;|Q603;|Q604;O.3;o1.M;o2.I;o3.I;|Q605;O.1;o1.I;|Q606;O.1;o1.I;|Q607;|Q608;O.3;o1.M;o2.M;o3.M;|Q609;O.3;o1.I;o2.I;o3.I;|Q610;O.1;o1.I;|Q611;O.1;o1.I;|Q612;O.1;o1.I;|Q613;O.1;o1.I;|Q614;O.1;o1.I;|Q615;|Q616;|Q617;O.1;o1.I;|Q618;O.1;o1.I;|Q619;O.2;o1.I;o2.I;|Q620;|Q621;O.1;o1.I;|Q622;|Q623;|Q624;|Q625;|Q626;|Q627;O.1;o1.I;|Q628;O.1;o1.I;|Q629;O.1;o1.I;|Q630;O.1;o1.I;|Q631;|Q632;|Q633;O.1;o1.O;|Q634;|Q635;|Q636;|Q637;|Q638;|Q639;O.1;o1.I;|Q640;O.1;o1.I;|Q641;|Q642;O.1;o1.I;|Q643;O.1;o1.I;|Q644;O.1;o1.I;|Q645;|Q646;|Q647;|Q648;O.1;o1.E;|Q649;|Q650;|Q651;O.3;o1.I;o2.I;o3.I;|Q652;O.1;o1.I;|Q653;|Q654;O.4;o1.I;o2.I;o3.I;o4.I;|Q655;|Q656;O.1;o1.I;|Q657;|Q658;O.1;o1.I;|Q659;|Q660;O.1;o1.E;|Q661;|Q662;O.4;o1.I;o2.I;o3.I;o4.I;|Q663;|Q664;O.2;o1.M;o2.M;|Q665;O.1;o1.E;|Q666;O.1;o1.I;|Q667;O.1;o1.E;|Q668;|Q669;|Q670;|Q671;O.1;o1.I;|Q672;O.1;o1.I;|Q673;O.1;o1.I;|Q674;|Q675;|Q676;O.2;o1.M;o2.M;|Q677;O.3;o1.M;o2.M;o3.M;|Q678;O.2;o1.M;o2.M;|Q679;O.2;o1.M;o2.M;|Q680;O.1;o1.I;|Q681;O.2;o1.M;o2.M;|Q682;O.1;o1.I;|Q683;|Q684;O.1;o1.I;|Q685;O.2;o1.I;o2.I;|Q686;|Q687;|Q688;|Q689;O.1;o1.I;|Q690;|Q691;O.3;o1.I;o2.I;o3.I;|Q692;O.3;o1.I;o2.I;o3.I;|Q693;O.1;o1.I;|Q694;O.1;o1.I;|Q695;|Q696;O.3;o1.I;o2.I;o3.I;|Q697;|Q698;O.1;o1.I;|Q699;O.1;o1.I;|Q700;|", -- [7]
	"|Q701;O.1;o1.I;|Q702;|Q703;O.1;o1.I;|Q704;O.1;o1.I;|Q705;O.1;o1.I;|Q706;O.1;o1.I;|Q707;|Q708;|Q709;O.1;o1.I;|Q710;O.1;o1.I;|Q711;O.1;o1.I;|Q712;O.1;o1.I;|Q713;O.1;o1.I;|Q714;O.1;o1.I;|Q715;O.2;o1.I;o2.I;|Q716;O.1;o1.I;|Q717;O.2;o1.I;o2.I;|Q718;O.1;o1.I;|Q719;O.1;o1.I;|Q720;|Q721;|Q722;O.1;o1.I;|Q723;|Q724;|Q725;|Q726;|Q727;|Q728;|Q729;|Q730;|Q731;O.1;o1.E;|Q732;O.1;o1.I;|Q733;O.1;o1.I;|Q734;|Q735;O.3;o1.I;o2.I;o3.I;|Q736;O.3;o1.I;o2.I;o3.I;|Q737;|Q738;|Q739;O.2;o1.M;o2.M;|Q740;|Q741;|Q742;|Q743;O.1;o1.I;|Q744;O.2;o1.I;o2.I;|Q745;O.3;o1.M;o2.M;o3.M;|Q746;O.1;o1.I;|Q747;O.2;o1.I;o2.I;|Q748;O.2;o1.I;o2.I;|Q749;|Q750;O.1;o1.I;|Q751;|Q752;|Q753;O.1;o1.I;|Q754;O.1;o1.E;|Q755;|Q756;O.2;o1.I;o2.I;|Q757;O.1;o1.I;|Q758;O.1;o1.E;|Q759;O.1;o1.I;|Q760;O.1;o1.E;|Q761;O.1;o1.I;|Q762;O.1;o1.I;|Q763;|Q764;O.2;o1.M;o2.M;|Q765;O.1;o1.I;|Q766;O.4;o1.I;o2.I;o3.I;o4.I;|Q767;|Q768;O.1;o1.I;|Q769;O.2;o1.I;o2.I;|Q770;|Q771;O.2;o1.I;o2.I;|Q772;|Q773;|Q774;|Q775;|Q776;O.1;o1.I;|Q777;|Q778;O.1;o1.I;|Q779;O.3;o1.I;o2.I;o3.I;|Q780;O.2;o1.I;o2.I;|Q781;|Q782;O.1;o1.I;|Q783;|Q784;O.3;o1.M;o2.M;o3.M;|Q785;|Q786;O.3;o1.O;o2.O;o3.O;|Q787;|Q788;O.1;o1.M;|Q789;O.1;o1.I;|Q790;O.1;o1.I;|Q791;O.1;o1.I;|Q792;O.1;o1.M;|Q793;O.2;o1.I;o2.I;|Q794;O.1;o1.I;|Q795;O.3;o1.I;o2.I;o3.I;|Q796;|Q797;|Q798;|Q799;|Q800;|", -- [8]
	"|Q801;|Q802;|Q803;|Q804;|Q805;|Q806;O.1;o1.I;|Q807;O.1;o1.I;|Q808;O.1;o1.I;|Q809;O.1;o1.E;|Q810;|Q811;|Q812;O.1;o1.I;|Q813;O.1;o1.I;|Q814;|Q815;O.1;o1.I;|Q816;O.1;o1.I;|Q817;O.1;o1.I;|Q818;O.2;o1.I;o2.I;|Q819;|Q820;|Q821;O.3;o1.I;o2.I;o3.I;|Q822;O.3;o1.I;o2.I;o3.I;|Q823;|Q824;|Q825;O.1;o1.I;|Q826;O.3;o1.M;o2.M;o3.I;|Q827;O.1;o1.I;|Q828;|Q829;|Q830;|Q831;|Q832;|Q833;O.1;o1.M;|Q834;O.1;o1.I;|Q835;O.2;o1.M;o2.M;|Q836;O.1;o1.E;|Q837;O.4;o1.M;o2.M;o3.M;o4.M;|Q838;|Q839;|Q840;|Q841;O.1;o1.I;|Q842;|Q843;O.3;o1.M;o2.M;o3.I;|Q844;O.1;o1.I;|Q845;O.1;o1.I;|Q846;O.3;o1.I;o2.I;o3.I;|Q847;|Q848;O.1;o1.I;|Q849;O.1;o1.O;|Q850;O.1;o1.I;|Q851;O.1;o1.I;|Q852;O.1;o1.I;|Q853;|Q854;|Q855;O.1;o1.I;|Q856;|Q857;O.1;o1.I;|Q858;O.1;o1.I;|Q859;|Q860;|Q861;O.1;o1.I;|Q862;O.1;o1.I;|Q863;O.1;o1.E;|Q864;|Q865;O.1;o1.I;|Q866;O.1;o1.I;|Q867;O.1;o1.I;|Q868;O.1;o1.I;|Q869;O.1;o1.I;|Q870;O.1;o1.E;|Q871;O.3;o1.M;o2.M;o3.M;|Q872;O.3;o1.M;o2.M;o3.I;|Q873;O.1;o1.I;|Q874;|Q875;O.1;o1.I;|Q876;O.1;o1.I;|Q877;O.1;o1.O;|Q878;O.3;o1.M;o2.M;o3.M;|Q879;O.3;o1.I;o2.I;o3.I;|Q880;O.1;o1.I;|Q881;O.1;o1.I;|Q882;O.1;o1.I;|Q883;|Q884;|Q885;|Q886;|Q887;O.2;o1.M;o2.M;|Q888;O.2;o1.I;o2.I;|Q889;O.1;o1.I;|Q890;|Q891;O.4;o1.M;o2.M;o3.M;o4.I;|Q892;|Q893;O.3;o1.I;o2.I;o3.I;|Q894;|Q895;O.1;o1.I;|Q896;O.1;o1.I;|Q897;|Q898;O.1;o1.E;|Q899;O.1;o1.I;|Q900;O.3;o1.O;o2.O;o3.O;|", -- [9]
	"|Q901;O.1;o1.I;|Q902;|Q903;O.1;o1.I;|Q904;|Q905;O.3;o1.O;o2.O;o3.O;|Q906;|Q907;O.1;o1.I;|Q908;O.1;o1.I;|Q909;|Q910;O.1;o1.E;|Q911;O.1;o1.E;|Q912;|Q913;O.1;o1.I;|Q914;O.4;o1.I;o2.I;o3.I;o4.I;|Q915;O.1;o1.I;|Q916;O.1;o1.I;|Q917;O.1;o1.I;|Q918;O.1;o1.I;|Q919;O.1;o1.I;|Q920;|Q921;O.1;o1.I;|Q922;|Q923;O.1;o1.I;|Q924;O.1;o1.O;|Q925;O.1;o1.I;|Q926;|Q927;|Q928;|Q929;O.1;o1.I;|Q930;|Q931;|Q932;O.1;o1.I;|Q933;O.1;o1.I;|Q934;O.1;o1.I;|Q935;|Q936;|Q937;O.1;o1.I;|Q938;O.1;o1.E;|Q939;O.1;o1.I;|Q940;|Q941;|Q942;|Q943;O.2;o1.I;o2.I;|Q944;O.1;o1.E;|Q945;O.1;o1.E;|Q946;|Q947;O.2;o1.I;o2.I;|Q948;|Q949;|Q950;|Q951;O.1;o1.I;|Q952;|Q953;O.2;o1.O;o2.O;|Q954;|Q955;O.1;o1.I;|Q956;O.1;o1.I;|Q957;O.1;o1.O;|Q958;O.1;o1.I;|Q959;O.1;o1.I;|Q960;|Q961;|Q962;O.1;o1.I;|Q963;O.1;o1.I;|Q964;O.1;o1.I;|Q965;|Q966;O.1;o1.I;|Q967;|Q968;|Q969;O.1;o1.I;|Q970;O.1;o1.I;|Q971;O.1;o1.I;|Q972;|Q973;O.1;o1.I;|Q974;O.1;o1.O;|Q975;|Q976;O.1;o1.E;|Q977;O.1;o1.I;|Q978;O.1;o1.I;|Q979;|Q980;|Q981;|Q982;O.2;o1.I;o2.I;|Q983;O.1;o1.I;|Q984;O.1;o1.E;|Q985;O.2;o1.M;o2.M;|Q986;O.1;o1.I;|Q987;|Q988;O.1;o1.I;|Q989;|Q990;|Q991;|Q992;O.1;o1.I;|Q993;|Q994;O.1;o1.E;|Q995;O.1;o1.E;|Q996;O.1;o1.I;|Q997;|Q998;O.1;o1.I;|Q999;|Q1000;|", -- [10]
	"|Q1001;O.1;o1.I;|Q1002;O.1;o1.I;|Q1003;O.1;o1.I;|Q1004;|Q1005;|Q1006;|Q1007;O.1;o1.I;|Q1008;O.1;o1.I;|Q1009;O.1;o1.I;|Q1010;O.1;o1.I;|Q1011;O.1;o1.I;|Q1012;O.3;o1.M;o2.M;o3.M;|Q1013;O.1;o1.I;|Q1014;O.1;o1.I;|Q1015;|Q1016;O.1;o1.I;|Q1017;O.1;o1.I;|Q1018;|Q1019;|Q1020;|Q1021;|Q1022;O.1;o1.O;|Q1023;O.1;o1.I;|Q1024;|Q1025;O.4;o1.M;o2.M;o3.M;o4.M;|Q1026;O.1;o1.I;|Q1027;O.1;o1.I;|Q1028;|Q1029;|Q1030;|Q1031;O.1;o1.I;|Q1032;O.1;o1.I;|Q1033;O.1;o1.I;|Q1034;O.1;o1.I;|Q1035;O.1;o1.I;|Q1036;O.1;o1.R;|Q1037;|Q1038;O.1;o1.I;|Q1039;|Q1040;|Q1041;|Q1042;|Q1043;O.1;o1.O;|Q1044;|Q1045;O.3;o1.M;o2.M;o3.I;|Q1046;|Q1047;|Q1048;O.4;o1.M;o2.M;o3.M;o4.M;|Q1049;O.1;o1.I;|Q1050;O.1;o1.I;|Q1051;O.1;o1.I;|Q1052;|Q1053;O.4;o1.M;o2.M;o3.M;o4.M;|Q1054;O.1;o1.I;|Q1055;|Q1056;|Q1057;O.4;o1.M;o2.M;o3.M;o4.M;|Q1058;O.4;o1.I;o2.I;o3.I;o4.I;|Q1059;|Q1060;|Q1061;|Q1062;O.1;o1.M;|Q1063;|Q1064;|Q1065;|Q1066;O.1;o1.I;|Q1067;|Q1068;O.2;o1.M;o2.M;|Q1069;O.1;o1.I;|Q1070;|Q1071;O.2;o1.M;o2.M;|Q1072;|Q1073;O.2;o1.I;o2.I;|Q1074;|Q1075;|Q1076;O.1;o1.I;|Q1077;|Q1078;O.1;o1.I;|Q1079;O.1;o1.I;|Q1080;O.1;o1.I;|Q1081;|Q1082;|Q1083;O.1;o1.I;|Q1084;O.1;o1.I;|Q1085;|Q1086;O.1;o1.E;|Q1087;O.3;o1.M;o2.M;o3.M;|Q1088;O.1;o1.I;|Q1089;O.3;o1.I;o2.I;o3.I;|Q1090;O.1;o1.E;|Q1091;|Q1092;|Q1093;O.1;o1.I;|Q1094;|Q1095;|Q1096;O.1;o1.I;|Q1097;|Q1098;|Q1099;O.1;o1.I;|Q1100;|", -- [11]
	"|Q1101;O.1;o1.I;|Q1102;O.1;o1.I;|Q1103;O.1;o1.I;|Q1104;O.1;o1.I;|Q1105;O.1;o1.I;|Q1106;|Q1107;O.1;o1.I;|Q1108;O.1;o1.I;|Q1109;O.1;o1.I;|Q1110;O.1;o1.I;|Q1111;|Q1112;|Q1113;O.1;o1.I;|Q1114;|Q1115;|Q1116;O.1;o1.I;|Q1117;|Q1118;|Q1119;|Q1120;|Q1121;|Q1122;|Q1123;|Q1124;|Q1125;O.2;o1.M;o2.M;|Q1126;O.1;o1.I;|Q1127;O.1;o1.I;|Q1128;O.1;o1.I;|Q1129;O.1;o1.I;|Q1130;|Q1131;O.1;o1.I;|Q1132;|Q1133;|Q1134;O.1;o1.I;|Q1135;O.1;o1.I;|Q1136;O.1;o1.I;|Q1137;|Q1138;O.1;o1.I;|Q1139;O.1;o1.I;|Q1140;O.2;o1.O;o2.O;|Q1141;O.1;o1.I;|Q1142;O.1;o1.I;|Q1143;O.1;o1.I;|Q1144;O.1;o1.E;|Q1145;|Q1146;|Q1147;O.3;o1.M;o2.M;o3.M;|Q1148;O.3;o1.I;o2.I;o3.I;|Q1149;|Q1150;O.1;o1.I;|Q1151;O.1;o1.I;|Q1152;|Q1153;O.1;o1.I;|Q1154;O.1;o1.I;|Q1155;|Q1156;|Q1157;|Q1158;|Q1159;|Q1160;O.1;o1.I;|Q1161;|Q1162;|Q1163;|Q1164;O.3;o1.I;o2.I;o3.I;|Q1165;O.1;o1.I;|Q1166;O.3;o1.I;o2.I;o3.I;|Q1167;|Q1168;O.3;o1.M;o2.M;o3.M;|Q1169;O.2;o1.I;o2.I;|Q1170;|Q1171;|Q1172;O.1;o1.O;|Q1173;O.1;o1.E;|Q1174;O.1;o1.I;|Q1175;O.3;o1.M;o2.M;o3.M;|Q1176;O.1;o1.I;|Q1177;O.1;o1.I;|Q1178;|Q1179;|Q1180;|Q1181;|Q1182;O.1;o1.I;|Q1183;|Q1184;|Q1185;|Q1186;|Q1187;O.1;o1.I;|Q1188;|Q1189;|Q1190;|Q1191;|Q1192;O.1;o1.I;|Q1193;O.2;o1.I;o2.I;|Q1194;|Q1195;O.1;o1.I;|Q1196;|Q1197;O.1;o1.I;|Q1198;|Q1199;O.1;o1.I;|Q1200;O.1;o1.I;|", -- [12]
	"|Q1201;O.1;o1.M;|Q1202;O.1;o1.I;|Q1203;O.1;o1.I;|Q1204;O.1;o1.I;|Q1205;O.1;o1.I;|Q1206;O.1;o1.I;|Q1218;O.1;o1.I;|Q1219;|Q1220;|Q1221;O.4;o1.I;o2.I;o3.I;o4.I;|Q1222;O.1;o1.E;|Q1238;|Q1239;|Q1240;|Q1241;|Q1242;|Q1243;|Q1244;O.1;o1.I;|Q1245;|Q1246;|Q1247;|Q1248;|Q1249;O.1;o1.E;|Q1250;|Q1251;|Q1252;|Q1253;|Q1258;O.1;o1.I;|Q1259;|Q1260;|Q1261;O.1;o1.I;|Q1262;|Q1263;|Q1264;|Q1265;O.1;o1.E;|Q1266;|Q1267;|Q1268;|Q1269;|Q1270;O.1;o1.E;|Q1271;|Q1272;|Q1273;O.1;o1.E;|Q1274;|Q1275;O.1;o1.I;|Q1276;|Q1277;|Q1278;|Q1279;|Q1280;|Q1281;|Q1282;|Q1283;|Q1284;|Q1285;|Q1286;|Q1287;|Q1288;|Q1289;|Q1290;|Q1291;|Q1292;|Q1293;|Q1294;|Q1295;|Q1296;|Q1297;|Q1298;|Q1299;|Q1300;|", -- [13]
	"|Q1301;|Q1302;|Q1318;O.1;o1.I;|Q1319;|Q1320;|Q1321;|Q1322;O.1;o1.I;|Q1323;|Q1324;O.1;o1.E;|Q1338;|Q1339;|Q1358;|Q1359;|Q1360;O.1;o1.I;|Q1361;|Q1362;|Q1363;|Q1364;O.1;o1.I;|Q1365;O.1;o1.I;|Q1366;O.1;o1.I;|Q1367;O.1;o1.R;|Q1368;O.1;o1.R;|Q1369;O.2;o1.I;o2.R;|Q1370;O.2;o1.I;o2.R;|Q1371;O.2;o1.I;o2.R;|Q1372;|Q1373;O.2;o1.I;o2.R;|Q1374;O.2;o1.I;o2.R;|Q1375;O.2;o1.I;o2.R;|Q1380;O.2;o1.I;o2.E;|Q1381;O.2;o1.I;o2.E;|Q1382;O.1;o1.R;|Q1383;O.3;o1.I;o2.I;o3.I;|Q1384;O.2;o1.I;o2.R;|Q1385;O.1;o1.R;|Q1386;O.4;o1.M;o2.M;o3.M;o4.R;|Q1387;O.1;o1.I;|Q1388;|Q1389;O.1;o1.I;|Q1390;|Q1391;|Q1392;|Q1393;O.1;o1.E;|Q1394;|Q1395;|Q1396;O.3;o1.M;o2.M;o3.M;|Q1397;|Q1398;O.1;o1.I;|", -- [14]
	"|Q1418;|Q1419;O.1;o1.I;|Q1420;|Q1421;O.1;o1.I;|Q1422;|Q1423;|Q1424;O.1;o1.I;|Q1425;|Q1426;O.3;o1.M;o2.M;o3.M;|Q1427;|Q1428;O.3;o1.M;o2.M;o3.M;|Q1429;|Q1430;O.1;o1.I;|Q1431;|Q1432;|Q1433;|Q1434;O.4;o1.M;o2.M;o3.M;o4.M;|Q1435;O.1;o1.I;|Q1436;|Q1437;|Q1438;|Q1439;O.1;o1.I;|Q1440;O.1;o1.E;|Q1441;|Q1442;O.1;o1.I;|Q1443;|Q1444;|Q1445;O.1;o1.I;|Q1446;O.1;o1.I;|Q1447;O.1;o1.E;|Q1448;O.1;o1.E;|Q1449;|Q1450;|Q1451;|Q1452;O.3;o1.I;o2.I;o3.I;|Q1453;|Q1454;|Q1455;|Q1456;O.1;o1.I;|Q1457;|Q1458;O.2;o1.I;o2.I;|Q1459;O.2;o1.I;o2.I;|Q1460;|Q1461;|Q1462;|Q1463;|Q1464;|Q1465;|Q1466;O.3;o1.I;o2.I;o3.I;|Q1467;|Q1468;|Q1469;|Q1470;O.1;o1.I;|Q1471;O.1;o1.M;|Q1472;|Q1473;O.1;o1.I;|Q1474;O.1;o1.M;|Q1475;O.1;o1.I;|Q1476;O.2;o1.I;o2.I;|Q1477;|Q1478;|Q1479;O.1;o1.E;|Q1480;|Q1481;O.1;o1.I;|Q1482;O.1;o1.I;|Q1483;|Q1484;|Q1485;O.1;o1.I;|Q1486;O.1;o1.I;|Q1487;O.4;o1.M;o2.M;o3.M;o4.M;|Q1488;O.2;o1.M;o2.M;|Q1489;|Q1490;|Q1491;O.1;o1.I;|Q1492;|Q1498;O.1;o1.I;|Q1499;|Q1500;|", -- [15]
	"|Q1501;O.1;o1.I;|Q1502;|Q1503;O.1;o1.I;|Q1504;O.1;o1.M;|Q1505;|Q1506;|Q1507;|Q1508;|Q1509;|Q1510;|Q1511;|Q1512;|Q1513;O.1;o1.M;|Q1514;O.1;o1.I;|Q1515;|Q1516;O.1;o1.I;|Q1517;|Q1518;|Q1519;O.1;o1.I;|Q1520;|Q1521;|Q1522;|Q1523;|Q1524;|Q1525;O.2;o1.I;o2.I;|Q1526;O.1;o1.I;|Q1527;|Q1528;|Q1529;|Q1530;|Q1531;|Q1532;|Q1533;|Q1534;O.1;o1.I;|Q1535;O.1;o1.I;|Q1536;O.1;o1.I;|Q1537;|Q1538;|Q1558;O.1;o1.E;|Q1559;|Q1560;O.1;o1.E;|Q1578;O.2;o1.I;o2.I;|Q1579;O.1;o1.I;|Q1580;O.1;o1.I;|Q1581;O.2;o1.I;o2.I;|Q1582;O.3;o1.I;o2.I;o3.I;|Q1598;O.1;o1.I;|Q1599;O.1;o1.I;|", -- [16]
	"|Q1618;O.2;o1.I;o2.I;|Q1638;|Q1639;|Q1640;O.1;o1.E;|Q1641;|Q1642;|Q1643;|Q1644;O.1;o1.I;|Q1645;|Q1646;|Q1647;|Q1648;O.1;o1.I;|Q1649;|Q1650;|Q1651;O.1;o1.E;|Q1652;|Q1653;|Q1654;O.4;o1.I;o2.I;o3.I;o4.I;|Q1655;O.1;o1.I;|Q1656;|Q1657;O.1;o1.O;|Q1658;O.1;o1.E;|Q1659;|Q1660;|Q1661;|Q1662;|Q1663;|Q1664;|Q1665;|Q1666;|Q1667;O.1;o1.I;|Q1678;O.1;o1.I;|Q1679;|Q1680;|Q1681;O.1;o1.I;|Q1682;|Q1683;O.1;o1.I;|Q1684;|Q1685;|Q1686;O.2;o1.I;o2.I;|Q1687;O.1;o1.E;|Q1688;O.1;o1.I;|Q1689;O.1;o1.M;|Q1690;O.2;o1.M;o2.M;|Q1691;O.3;o1.M;o2.M;o3.M;|Q1692;|Q1693;|Q1698;|Q1699;O.1;o1.E;|Q1700;|", -- [17]
	"|Q1701;O.4;o1.I;o2.I;o3.I;o4.I;|Q1702;|Q1703;|Q1704;|Q1705;O.2;o1.I;o2.I;|Q1706;|Q1707;O.1;o1.I;|Q1708;O.1;o1.I;|Q1709;|Q1710;O.1;o1.I;|Q1711;|Q1712;O.3;o1.I;o2.I;o3.I;|Q1713;O.1;o1.I;|Q1714;O.3;o1.I;o2.I;o3.I;|Q1715;|Q1716;|Q1717;|Q1718;|Q1719;O.2;o1.E;o2.M;|Q1738;O.1;o1.I;|Q1739;O.1;o1.M;|Q1740;O.2;o1.I;o2.I;|Q1758;|Q1778;|Q1779;|Q1780;|Q1781;|Q1782;|Q1783;|Q1784;O.1;o1.I;|Q1785;|Q1786;|Q1787;O.1;o1.I;|Q1788;|Q1789;|Q1790;|Q1791;|Q1792;|Q1793;|Q1794;|Q1795;O.1;o1.M;|Q1796;O.1;o1.I;|Q1798;|Q1799;O.1;o1.I;|Q1800;O.1;o1.E;|", -- [18]
	"|Q1801;|Q1802;O.2;o1.I;o2.I;|Q1803;O.2;o1.I;o2.I;|Q1804;O.1;o1.I;|Q1805;O.1;o1.I;|Q1806;|Q1818;|Q1819;O.1;o1.M;|Q1820;|Q1821;O.4;o1.I;o2.I;o3.I;o4.I;|Q1822;|Q1823;|Q1824;O.1;o1.I;|Q1825;|Q1838;O.4;o1.I;o2.I;o3.I;o4.I;|Q1839;|Q1840;|Q1841;|Q1842;O.1;o1.I;|Q1843;|Q1844;O.1;o1.I;|Q1845;|Q1846;O.1;o1.I;|Q1847;|Q1848;|Q1858;O.1;o1.I;|Q1859;|Q1860;|Q1861;O.1;o1.I;|Q1878;O.1;o1.I;|Q1879;|Q1880;O.1;o1.I;|Q1881;|Q1882;O.1;o1.I;|Q1883;|Q1884;O.1;o1.O;|Q1885;|Q1886;O.1;o1.I;|Q1898;|Q1899;O.1;o1.I;|", -- [19]
	"|Q1918;|Q1919;|Q1920;O.3;o1.I;o2.I;o3.I;|Q1921;O.2;o1.I;o2.I;|Q1938;O.1;o1.I;|Q1939;|Q1940;O.1;o1.I;|Q1941;|Q1942;|Q1943;|Q1944;O.1;o1.I;|Q1945;O.1;o1.I;|Q1946;|Q1947;|Q1948;O.2;o1.I;o2.I;|Q1949;|Q1950;O.1;o1.E;|Q1951;O.1;o1.I;|Q1952;|Q1953;|Q1954;O.1;o1.I;|Q1955;O.1;o1.M;|Q1956;O.1;o1.I;|Q1957;O.1;o1.M;|Q1958;|Q1959;|Q1960;O.3;o1.I;o2.I;o3.I;|Q1961;O.2;o1.I;o2.I;|Q1962;|Q1963;O.1;o1.I;|Q1978;|Q1998;O.1;o1.I;|Q1999;O.1;o1.I;|Q2000;|", -- [20]
	"|Q2018;O.1;o1.I;|Q2019;|Q2020;O.1;o1.I;|Q2038;O.4;o1.I;o2.I;o3.I;o4.I;|Q2039;|Q2040;O.1;o1.I;|Q2041;|Q2078;O.1;o1.O;|Q2098;O.3;o1.I;o2.I;o3.I;|Q2058;O.1;o1.I;|Q2059;O.1;o1.I;|", -- [21]
	"|Q2158;|Q2159;|Q2160;|Q2161;|Q2178;O.1;o1.I;|Q2118;O.1;o1.O;|Q2199;O.1;o1.I;|Q2138;O.1;o1.M;|Q2200;|Q2139;O.1;o1.M;|Q2198;|", -- [22]
	"|Q2205;|Q2206;O.1;o1.I;|Q2238;|Q2239;|Q2240;O.1;o1.E;|Q2241;|Q2242;O.1;o1.I;|Q2258;O.3;o1.I;o2.I;o3.I;|Q2259;|Q2260;|Q2278;O.1;o1.E;|Q2279;|Q2280;|Q2201;O.3;o1.I;o2.I;o3.I;|Q2281;|Q2202;O.1;o1.I;|Q2298;|Q2218;|Q2282;O.1;o1.I;|Q2203;O.1;o1.I;|Q2283;O.1;o1.I;|Q2299;|Q2204;O.1;o1.I;|Q2284;|Q2300;|", -- [23]
	"|Q2381;O.1;o1.I;|Q2318;|Q2382;|Q2398;|Q2383;|Q2399;|Q2338;|Q2339;O.4;o1.I;o2.I;o3.I;o4.I;|Q2340;|Q2341;|Q2342;O.1;o1.I;|Q2359;O.2;o1.I;o2.I;|Q2360;|Q2361;|Q2378;|Q2379;|Q2358;O.1;o1.I;|Q2380;|", -- [24]
	"|Q2478;O.6;o1.M;o2.M;o3.M;o4.I;o5.I;o6.I;|Q2479;|Q2480;O.1;o1.E;|Q2498;|Q2418;O.2;o1.I;o2.I;|Q2499;O.1;o1.I;|Q2460;O.1;o1.E;|Q2500;O.3;o1.I;o2.I;o3.I;|Q2439;|Q2459;O.2;o1.M;o2.I;|Q2440;|Q2458;|Q2438;O.1;o1.I;|", -- [25]
	"|Q2541;O.1;o1.I;|Q2561;O.1;o1.E;|Q2501;O.1;o1.I;|Q2581;O.3;o1.I;o2.I;o3.I;|Q2518;O.1;o1.I;|Q2582;O.3;o1.I;o2.I;o3.I;|Q2584;O.3;o1.I;o2.I;o3.I;|Q2519;|Q2520;O.1;o1.E;|Q2585;O.3;o1.I;o2.I;o3.I;|Q2522;O.1;o1.I;|Q2586;O.3;o1.I;o2.I;o3.I;|Q2521;O.1;o1.I;|Q2583;O.3;o1.I;o2.I;o3.I;|Q2523;O.1;o1.I;|", -- [26]
	"|Q2605;O.1;o1.I;|Q2621;|Q2607;|Q2606;|Q2661;|Q2681;O.4;o1.O;o2.O;o3.O;o4.O;|Q2609;O.4;o1.I;o2.I;o3.I;o4.I;|Q2601;O.2;o1.I;o2.I;|Q2622;|Q2623;O.1;o1.I;|Q2641;O.1;o1.I;|Q2608;O.1;o1.E;|Q2662;|Q2602;O.2;o1.I;o2.I;|Q2603;O.2;o1.I;o2.I;|Q2604;O.2;o1.I;o2.I;|", -- [27]
	"|Q2701;|Q2749;O.1;o1.I;|Q2765;O.1;o1.E;|Q2751;O.3;o1.I;o2.I;o3.I;|Q2750;O.1;o1.I;|Q2754;O.3;o1.I;o2.I;o3.I;|Q2770;O.1;o1.I;|Q2755;O.1;o1.E;|Q2771;O.2;o1.I;o2.I;|Q2764;|Q2748;O.1;o1.I;|Q2721;|Q2768;O.1;o1.I;|Q2702;|Q2772;O.2;o1.I;o2.I;|Q2756;O.2;o1.I;o2.I;|Q2741;O.1;o1.I;|Q2757;|Q2773;O.2;o1.I;o2.I;|Q2769;|Q2742;O.1;o1.E;|Q2743;|Q2759;|Q2758;O.1;o1.I;|Q2753;O.3;o1.I;o2.I;o3.I;|Q2744;O.1;o1.E;|Q2760;|Q2745;|Q2781;O.1;o1.I;|Q2767;O.1;o1.E;|Q2783;|Q2766;|Q2782;|Q2746;O.2;o1.I;o2.I;|Q2762;O.2;o1.I;o2.I;|Q2761;O.2;o1.I;o2.I;|Q2752;O.2;o1.I;o2.I;|Q2747;O.1;o1.I;|Q2763;O.2;o1.I;o2.I;|Q2784;O.1;o1.E;|", -- [28]
	"|Q2878;O.1;o1.I;|Q2882;O.1;o1.I;|Q2850;O.3;o1.I;o2.I;o3.I;|Q2866;|Q2861;|Q2846;|Q2851;O.3;o1.I;o2.I;o3.I;|Q2867;|Q2876;|Q2844;|Q2852;O.3;o1.I;o2.I;o3.I;|Q2868;|Q2864;|Q2822;O.1;o1.E;|Q2863;O.1;o1.M;|Q2870;O.1;o1.I;|Q2869;O.1;o1.I;|Q2865;O.1;o1.I;|Q2853;|Q2856;O.3;o1.I;o2.I;o3.I;|Q2849;O.3;o1.I;o2.I;o3.I;|Q2879;O.1;o1.I;|Q2841;O.2;o1.E;o2.E;|Q2873;O.1;o1.I;|Q2855;O.2;o1.I;o2.I;|Q2880;O.1;o1.I;|Q2847;O.1;o1.I;|Q2862;O.1;o1.I;|Q2842;|Q2858;O.3;o1.I;o2.I;o3.I;|Q2874;|Q2881;O.1;o1.I;|Q2857;O.3;o1.I;o2.I;o3.I;|Q2871;|Q2848;O.2;o1.I;o2.I;|Q2854;O.1;o1.I;|Q2872;|Q2821;O.1;o1.E;|Q2843;O.1;o1.E;|Q2859;O.3;o1.I;o2.I;o3.I;|Q2875;O.1;o1.I;|Q2801;O.1;o1.E;|Q2845;O.2;o1.E;o2.E;|Q2860;|Q2877;O.2;o1.M;o2.M;|", -- [29]
	"|Q2942;O.1;o1.I;|Q2992;O.1;o1.E;|Q2930;O.1;o1.I;|Q2946;|Q2962;O.1;o1.I;|Q2978;|Q2994;O.1;o1.O;|Q2941;|Q2931;|Q2947;|Q2963;|Q2988;O.3;o1.O;o2.O;o3.O;|Q2972;|Q2940;|Q2924;O.1;o1.I;|Q2927;|Q2973;O.1;o1.I;|Q2932;O.1;o1.E;|Q2948;O.2;o1.I;o2.I;|Q2980;O.3;o1.M;o2.M;o3.M;|Q2945;|Q2981;|Q2997;|Q2949;|Q2965;|Q2979;O.1;o1.I;|Q2975;O.3;o1.M;o2.M;o3.M;|Q2902;|Q2966;|Q2982;O.3;o1.M;o2.M;o3.M;|Q2993;|Q2983;|Q2999;|Q2987;O.1;o1.I;|Q2971;O.1;o1.E;|Q2998;|Q2995;O.3;o1.O;o2.O;o3.O;|Q2990;|Q3000;|Q2936;O.1;o1.E;|Q2952;|Q2968;|Q2984;|Q2991;O.1;o1.I;|Q2926;O.1;o1.I;|Q2904;O.1;o1.E;|Q2937;O.1;o1.I;|Q2985;|Q2969;O.1;o1.E;|Q2935;|Q2929;O.1;o1.M;|Q2974;O.1;o1.I;|Q2977;|Q2976;|Q2903;|Q2928;O.1;o1.I;|Q2933;|Q2996;|Q2950;O.2;o1.I;o2.I;|Q2951;O.1;o1.I;|Q2922;O.1;o1.I;|Q2938;|Q2954;O.1;o1.E;|Q2986;|Q2970;O.3;o1.M;o2.M;o3.M;|Q2953;O.1;o1.I;|Q2989;O.1;o1.E;|Q2964;|Q2943;|Q2923;|Q2939;|Q2967;|Q2934;O.1;o1.I;|Q2944;O.1;o1.I;|Q2925;|", -- [30]
	"|Q3042;O.1;o1.I;|Q3088;|Q3085;|Q3091;|Q3100;|Q3084;|Q3093;|Q3094;|Q3095;|Q3099;|Q3083;|Q3092;|Q3064;|Q3023;|Q3090;|Q3087;|Q3001;|Q3097;|Q3002;|Q3082;|Q3098;|Q3065;|Q3022;|Q3096;|Q3089;|Q3063;O.4;o1.M;o2.M;o3.M;o4.M;|Q3062;O.1;o1.I;|Q3086;|", -- [31]
	"|Q3104;|Q3117;|Q3118;|Q3107;|Q3116;|Q3182;|Q3121;|Q3124;O.1;o1.I;|Q3123;O.1;o1.I;|Q3108;|Q3109;|Q3125;O.1;o1.I;|Q3141;O.1;o1.E;|Q3181;|Q3119;|Q3110;|Q3126;O.1;o1.I;|Q3102;|Q3101;|Q3103;|Q3111;|Q3127;O.1;o1.I;|Q3115;|Q3112;|Q3128;O.4;o1.I;o2.I;o3.I;o4.I;|Q3113;|Q3129;|Q3161;O.1;o1.I;|Q3114;|Q3130;|Q3106;|Q3105;|Q3120;|Q3122;|", -- [32]
	"|Q3281;O.1;o1.I;|Q3221;|Q3201;|Q3241;|Q3261;|", -- [33]
	"|Q3376;O.1;o1.I;|Q3377;O.1;o1.E;|Q3378;O.1;o1.I;|Q3373;|Q3375;O.3;o1.I;o2.I;o3.I;|Q3372;O.1;o1.I;|Q3379;O.1;o1.M;|Q3380;|Q3381;O.1;o1.I;|Q3364;|Q3366;O.1;o1.E;|Q3382;O.1;o1.E;|Q3383;O.4;o1.M;o2.M;o3.M;o4.M;|Q3371;|Q3363;O.1;o1.I;|Q3384;O.1;o1.I;|Q3374;O.1;o1.I;|Q3385;O.3;o1.M;o2.M;o3.I;|Q3370;|Q3369;|Q3321;O.1;o1.E;|Q3365;|Q3368;|Q3367;O.1;o1.E;|Q3301;|Q3362;O.2;o1.M;o2.M;|Q3361;O.3;o1.I;o2.I;o3.I;|Q3341;O.1;o1.I;|", -- [34]
	"|Q3405;|Q3441;O.1;o1.E;|Q3421;|Q3442;O.2;o1.I;o2.I;|Q3452;O.1;o1.I;|Q3444;O.1;o1.I;|Q3461;|Q3462;|Q3445;|Q3425;|Q3447;|Q3463;O.4;o1.O;o2.O;o3.O;o4.O;|Q3443;O.1;o1.I;|Q3448;|Q3422;|Q3401;|Q3449;O.4;o1.I;o2.I;o3.I;o4.I;|Q3481;|Q3423;|Q3402;|Q3450;|Q3482;|Q3454;|Q3424;|Q3403;|Q3451;|Q3483;|Q3453;O.1;o1.E;|Q3446;|Q3404;|", -- [35]
	"|Q3501;O.1;o1.I;|Q3517;O.4;o1.I;o2.I;o3.I;o4.I;|Q3565;|Q3581;|Q3502;O.1;o1.I;|Q3518;|Q3566;O.2;o1.I;o2.I;|Q3503;|Q3567;|Q3568;O.4;o1.I;o2.I;o3.I;o4.I;|Q3505;O.3;o1.E;o2.M;o3.M;|Q3569;|Q3506;O.1;o1.I;|Q3570;|Q3516;|Q3507;|Q3523;|Q3520;O.1;o1.O;|Q3519;|Q3508;|Q3524;O.1;o1.I;|Q3509;|Q3541;|Q3542;|Q3510;O.3;o1.I;o2.I;o3.I;|Q3515;|Q3511;|Q3527;O.2;o1.I;o2.I;|Q3512;|Q3528;O.1;o1.I;|Q3513;|Q3529;|Q3561;|Q3522;|Q3526;|Q3514;O.1;o1.M;|Q3530;|Q3562;|Q3525;O.1;o1.E;|Q3521;O.3;o1.I;o2.I;o3.I;|Q3531;|Q3563;|Q3504;|Q3564;|", -- [36]
	"|Q3629;|Q3645;|Q3661;O.1;o1.I;|Q3630;|Q3646;|Q3631;|Q3647;|Q3632;|Q3601;O.4;o1.I;o2.I;o3.I;o4.I;|Q3633;|Q3681;|Q3634;|Q3602;O.1;o1.I;|Q3635;|Q3636;O.1;o1.M;|Q3621;|Q3637;|Q3622;|Q3638;O.1;o1.I;|Q3623;|Q3639;O.3;o1.I;o2.I;o3.I;|Q3624;|Q3640;O.1;o1.I;|Q3625;O.1;o1.E;|Q3641;O.3;o1.I;o2.I;o3.I;|Q3626;|Q3642;O.1;o1.I;|Q3627;O.3;o1.I;o2.I;o3.I;|Q3643;O.3;o1.I;o2.I;o3.I;|Q3628;O.1;o1.I;|Q3644;|", -- [37]
	"|Q3741;O.1;o1.I;|Q3789;|Q3790;|Q3791;O.1;o1.I;|Q3792;O.1;o1.I;|Q3761;O.1;o1.I;|Q3762;|Q3763;|Q3764;O.1;o1.I;|Q3701;O.1;o1.O;|Q3765;|Q3781;|Q3702;O.1;o1.E;|Q3782;|Q3783;O.1;o1.I;|Q3784;|Q3721;|Q3785;O.1;o1.I;|Q3786;O.1;o1.I;|Q3787;|Q3788;|", -- [38]
	"|Q3821;|Q3885;O.1;o1.E;|Q3822;O.1;o1.I;|Q3823;O.3;o1.M;o2.M;o3.M;|Q3824;O.1;o1.I;|Q3825;O.1;o1.O;|Q3841;|Q3842;O.1;o1.I;|Q3843;|Q3844;|Q3845;O.3;o1.I;o2.I;o3.I;|Q3861;O.1;o1.I;|Q3801;|Q3881;O.2;o1.I;o2.I;|Q3802;O.1;o1.I;|Q3882;O.1;o1.I;|Q3803;O.1;o1.I;|Q3883;O.1;o1.I;|Q3804;O.1;o1.I;|Q3884;|", -- [39]
	"|Q3901;O.1;o1.M;|Q3981;|Q3902;O.1;o1.I;|Q3982;O.1;o1.E;|Q3903;|Q3904;O.1;o1.I;|Q3905;|Q3921;|Q3906;O.1;o1.M;|Q3922;O.1;o1.I;|Q3907;O.2;o1.M;o2.I;|Q3923;|Q3908;|Q3924;O.1;o1.I;|Q3909;O.1;o1.I;|Q3941;|Q3910;O.1;o1.E;|Q3942;|Q3911;O.1;o1.I;|Q3912;|Q3913;|Q3961;|Q3914;|Q3962;O.2;o1.M;o2.I;|", -- [40]
	"|Q4061;O.1;o1.I;|Q4062;|Q4063;O.2;o1.I;o2.I;|Q4001;O.1;o1.E;|Q4081;O.3;o1.M;o2.M;o3.M;|Q4002;|Q4082;O.3;o1.M;o2.M;o3.M;|Q4003;O.1;o1.M;|Q4083;O.3;o1.I;o2.I;o3.I;|Q4004;|Q4084;O.2;o1.I;o2.I;|Q4005;O.1;o1.I;|Q4021;O.1;o1.I;|Q4022;O.2;o1.E;o2.I;|Q4023;O.1;o1.I;|Q4024;O.1;o1.I;|Q4041;O.1;o1.I;|", -- [41]
	"|Q4122;|Q4186;|Q4123;O.1;o1.I;|Q4124;|Q4125;|Q4126;O.1;o1.I;|Q4127;|Q4128;|Q4129;|Q4161;O.1;o1.I;|Q4130;|Q4131;|Q4132;O.1;o1.M;|Q4101;O.1;o1.I;|Q4133;|Q4102;O.1;o1.I;|Q4134;O.1;o1.I;|Q4103;O.1;o1.I;|Q4135;|Q4104;O.1;o1.I;|Q4136;O.1;o1.I;|Q4105;O.1;o1.I;|Q4106;O.1;o1.I;|Q4107;O.1;o1.I;|Q4108;O.1;o1.I;|Q4109;O.1;o1.I;|Q4141;O.1;o1.I;|Q4142;|Q4110;O.1;o1.I;|Q4143;O.1;o1.I;|Q4111;O.1;o1.I;|Q4112;O.1;o1.I;|Q4144;O.1;o1.I;|Q4113;O.1;o1.I;|Q4145;O.4;o1.M;o2.M;o3.M;o4.M;|Q4114;O.1;o1.I;|Q4146;O.1;o1.I;|Q4182;O.4;o1.M;o2.M;o3.M;o4.M;|Q4115;O.1;o1.I;|Q4147;|Q4116;O.1;o1.I;|Q4148;O.1;o1.I;|Q4117;O.1;o1.I;|Q4181;|Q4118;O.1;o1.I;|Q4119;O.1;o1.I;|Q4183;|Q4120;O.2;o1.M;o2.M;|Q4184;|Q4121;O.2;o1.E;o2.I;|Q4185;O.1;o1.E;|", -- [42]
	"|Q4282;O.2;o1.I;o2.I;|Q4283;O.1;o1.I;|Q4284;O.4;o1.I;o2.I;o3.I;o4.I;|Q4221;O.1;o1.I;|Q4285;O.1;o1.E;|Q4222;O.1;o1.I;|Q4286;O.1;o1.I;|Q4223;|Q4287;O.1;o1.E;|Q4224;O.1;o1.E;|Q4288;O.1;o1.E;|Q4289;O.3;o1.I;o2.I;o3.I;|Q4290;O.1;o1.I;|Q4291;O.1;o1.I;|Q4292;O.1;o1.I;|Q4261;O.1;o1.E;|Q4293;O.1;o1.I;|Q4262;O.1;o1.M;|Q4294;O.1;o1.I;|Q4263;O.1;o1.M;|Q4295;O.1;o1.I;|Q4264;|Q4296;O.1;o1.I;|Q4201;O.3;o1.I;o2.I;o3.I;|Q4265;O.1;o1.E;|Q4297;O.1;o1.I;|Q4266;|Q4298;|Q4267;|Q4299;O.1;o1.E;|Q4300;O.1;o1.I;|Q4241;|Q4242;|Q4244;O.1;o1.I;|Q4245;O.1;o1.E;|Q4243;|Q4281;|", -- [43]
	"|Q4381;O.2;o1.I;o2.I;|Q4382;O.2;o1.I;o2.I;|Q4383;O.2;o1.I;o2.I;|Q4384;O.2;o1.I;o2.I;|Q4321;|Q4385;O.2;o1.I;o2.I;|Q4322;O.1;o1.E;|Q4386;O.2;o1.I;o2.I;|Q4323;O.1;o1.I;|Q4324;|Q4361;|Q4362;O.1;o1.M;|Q4363;|Q4301;O.1;o1.I;|Q4341;|Q4342;O.1;o1.E;|Q4343;O.1;o1.I;|", -- [44]
	"|Q4442;|Q4443;O.1;o1.I;|Q4444;O.1;o1.I;|Q4445;O.1;o1.I;|Q4446;O.1;o1.I;|Q4447;O.1;o1.I;|Q4448;O.1;o1.I;|Q4449;O.2;o1.M;o2.I;|Q4481;O.4;o1.I;o2.I;o3.I;o4.I;|Q4450;O.4;o1.I;o2.I;o3.I;o4.I;|Q4482;O.4;o1.I;o2.I;o3.I;o4.I;|Q4451;|Q4483;O.4;o1.I;o2.I;o3.I;o4.I;|Q4484;O.4;o1.I;o2.I;o3.I;o4.I;|Q4421;O.4;o1.M;o2.M;o3.M;o4.M;|Q4485;|Q4486;|Q4487;|Q4488;|Q4489;|Q4490;|Q4491;O.1;o1.E;|Q4492;|Q4493;|Q4494;|Q4462;O.1;o1.I;|Q4463;O.4;o1.I;o2.I;o3.I;o4.I;|Q4495;|Q4464;O.1;o1.I;|Q4496;O.2;o1.I;o2.I;|Q4401;O.1;o1.I;|Q4465;O.1;o1.I;|Q4466;O.1;o1.I;|Q4402;O.1;o1.I;|Q4467;O.1;o1.I;|Q4403;O.1;o1.I;|Q4461;O.1;o1.I;|Q4441;O.1;o1.I;|", -- [45]
	"|Q4506;O.1;o1.E;|Q4507;O.1;o1.I;|Q4508;|Q4509;|Q4541;|Q4510;|Q4542;|Q4511;|Q4512;O.2;o1.I;o2.I;|Q4513;O.1;o1.I;|Q4581;|Q4521;O.2;o1.M;o2.M;|Q4501;O.1;o1.M;|Q4502;O.1;o1.I;|Q4561;O.1;o1.I;|Q4503;O.2;o1.I;o2.I;|Q4504;O.1;o1.I;|Q4505;O.1;o1.I;|", -- [46]
	"|Q4602;O.1;o1.I;|Q4603;O.1;o1.I;|Q4604;O.1;o1.I;|Q4605;|Q4606;|Q4641;|Q4642;O.1;o1.I;|Q4681;O.1;o1.I;|Q4621;O.3;o1.M;o2.M;o3.R;|Q4661;O.1;o1.I;|Q4601;O.1;o1.I;|", -- [47]
	"|Q4730;|Q4762;O.1;o1.I;|Q4731;|Q4763;O.1;o1.I;|Q4732;|Q4764;O.1;o1.I;|Q4701;O.1;o1.M;|Q4733;|Q4765;|Q4734;O.1;o1.E;|Q4766;|Q4735;O.1;o1.I;|Q4767;O.1;o1.I;|Q4736;|Q4768;O.1;o1.I;|Q4737;|Q4769;|Q4738;|Q4770;O.1;o1.E;|Q4739;|Q4771;O.2;o1.E;o2.M;|Q4740;O.1;o1.M;|Q4741;O.1;o1.M;|Q4742;O.4;o1.I;o2.I;o3.I;o4.I;|Q4743;O.1;o1.I;|Q4781;O.1;o1.I;|Q4782;|Q4783;O.2;o1.I;o2.I;|Q4784;O.3;o1.I;o2.I;o3.I;|Q4721;O.1;o1.M;|Q4785;|Q4786;O.1;o1.E;|Q4722;|Q4724;O.1;o1.M;|Q4788;O.2;o1.I;o2.I;|Q4725;|Q4787;O.1;o1.I;|Q4723;|Q4726;O.1;o1.I;|Q4727;|Q4728;|Q4729;O.1;o1.I;|Q4761;|", -- [48]
	"|Q4861;|Q4862;O.1;o1.I;|Q4863;|Q4864;O.2;o1.I;o2.I;|Q4801;O.1;o1.I;|Q4865;|Q4802;O.1;o1.I;|Q4866;O.1;o1.E;|Q4803;O.1;o1.I;|Q4867;O.1;o1.I;|Q4804;O.1;o1.I;|Q4805;O.1;o1.I;|Q4806;O.1;o1.I;|Q4807;O.1;o1.I;|Q4841;O.3;o1.M;o2.M;o3.M;|Q4809;O.1;o1.I;|Q4810;|Q4842;O.1;o1.E;|Q4811;O.1;o1.E;|Q4812;O.1;o1.I;|Q4813;|Q4881;|Q4882;|Q4821;O.1;o1.I;|Q4883;|Q4822;O.1;o1.I;|Q4808;|", -- [49]
	"|Q4986;|Q4987;|Q4961;O.1;o1.M;|Q4962;O.1;o1.I;|Q4963;O.1;o1.I;|Q4964;O.1;o1.E;|Q4901;O.1;o1.E;|Q4965;|Q4921;O.1;o1.O;|Q4902;|Q4966;O.1;o1.E;|Q4967;|Q4903;O.4;o1.M;o2.M;o3.M;o4.I;|Q4904;O.1;o1.E;|Q4968;|Q4905;|Q4969;|Q4906;O.4;o1.M;o2.M;o3.M;o4.M;|Q4907;|Q4971;O.1;o1.M;|Q4972;O.1;o1.I;|Q4941;O.1;o1.E;|Q4973;O.1;o1.I;|Q4975;O.1;o1.E;|Q4976;|Q4981;|Q4982;O.1;o1.I;|Q4974;O.1;o1.I;|Q4983;|Q4970;O.2;o1.I;o2.I;|Q4984;O.1;o1.M;|Q4985;O.1;o1.M;|", -- [50]
	"|Q5053;|Q5088;O.2;o1.E;o2.I;|Q5081;O.3;o1.M;o2.M;o3.M;|Q5049;|Q5059;O.1;o1.I;|Q5091;|Q5054;O.1;o1.M;|Q5089;|Q5055;O.1;o1.M;|Q5064;O.3;o1.I;o2.I;o3.I;|Q5063;O.4;o1.I;o2.I;o3.I;o4.I;|Q5001;O.1;o1.I;|Q5065;O.2;o1.I;o2.I;|Q5048;|Q5066;|Q5097;O.4;o1.O;o2.O;o3.O;o4.O;|Q5092;O.2;o1.M;o2.M;|Q5050;|Q5096;O.1;o1.E;|Q5094;|Q5086;O.1;o1.I;|Q5085;|Q5084;|Q5083;|Q5082;O.3;o1.M;o2.M;o3.M;|Q5057;|Q5052;O.1;o1.I;|Q5095;|Q5090;|Q5068;O.4;o1.I;o2.I;o3.I;o4.I;|Q5062;O.1;o1.I;|Q5056;O.1;o1.M;|Q5058;|Q5047;|Q5023;|Q5022;|Q5051;O.1;o1.I;|Q5061;|Q5041;O.1;o1.I;|Q5021;|Q5060;O.1;o1.I;|Q5043;O.1;o1.I;|Q5044;O.1;o1.I;|Q5045;O.1;o1.I;|Q5042;O.1;o1.I;|Q5046;O.1;o1.I;|Q5067;O.4;o1.I;o2.I;o3.I;o4.I;|Q5093;|Q5098;O.4;o1.O;o2.O;o3.O;o4.O;|Q5002;|Q5087;O.1;o1.I;|", -- [51]
	"|Q5148;O.2;o1.I;o2.I;|Q5150;O.1;o1.I;|Q5145;O.3;o1.I;o2.I;o3.I;|Q5157;O.1;o1.I;|Q5155;O.4;o1.M;o2.M;o3.M;o4.M;|Q5158;|Q5126;O.1;o1.E;|Q5159;|Q5156;O.3;o1.E;o2.M;o3.M;|Q5153;O.1;o1.I;|Q5160;|Q5144;O.4;o1.I;o2.I;o3.I;o4.I;|Q5162;|Q5149;O.1;o1.I;|Q5181;O.2;o1.I;o2.I;|Q5101;O.1;o1.M;|Q5143;O.2;o1.I;o2.I;|Q5163;O.3;o1.O;o2.O;o3.O;|Q5161;|Q5165;O.4;o1.O;o2.O;o3.O;o4.O;|Q5102;O.1;o1.M;|Q5124;O.4;o1.I;o2.I;o3.I;o4.I;|Q5121;O.1;o1.M;|Q5103;|Q5167;O.4;o1.I;o2.I;o3.I;o4.I;|Q5164;|Q5123;|Q5151;O.1;o1.I;|Q5168;O.2;o1.I;o2.I;|Q5154;O.1;o1.I;|Q5166;O.4;o1.I;o2.I;o3.I;o4.I;|Q5141;O.3;o1.I;o2.I;o3.I;|Q5128;|Q5142;|Q5122;O.1;o1.I;|Q5125;|Q5127;O.2;o1.I;o2.I;|Q5146;O.4;o1.I;o2.I;o3.I;o4.I;|Q5147;O.1;o1.I;|Q5152;|", -- [52]
	"|Q5281;|Q5242;O.3;o1.M;o2.M;o3.I;|Q5213;O.1;o1.I;|Q5245;O.4;o1.I;o2.I;o3.I;o4.I;|Q5241;|Q5209;|Q5214;O.1;o1.I;|Q5250;|Q5251;O.2;o1.M;o2.O;|Q5284;O.4;o1.I;o2.I;o3.I;o4.I;|Q5208;|Q5215;|Q5220;|Q5225;O.1;o1.I;|Q5227;O.3;o1.I;o2.I;o3.I;|Q5223;|Q5219;O.1;o1.I;|Q5221;O.3;o1.I;o2.I;o3.I;|Q5224;O.3;o1.I;o2.I;o3.I;|Q5228;|Q5249;|Q5248;|Q5246;O.1;o1.I;|Q5226;|Q5212;O.1;o1.I;|Q5229;O.1;o1.I;|Q5207;|Q5283;O.3;o1.I;o2.I;o3.I;|Q5262;|Q5230;|Q5244;|Q5261;|Q5252;|Q5231;O.1;o1.I;|Q5263;O.1;o1.I;|Q5253;|Q5232;|Q5264;|Q5282;O.1;o1.O;|Q5265;|Q5201;O.2;o1.M;o2.M;|Q5233;O.1;o1.I;|Q5238;|Q5206;O.1;o1.I;|Q5202;|Q5203;O.1;o1.E;|Q5235;O.1;o1.I;|Q5204;O.1;o1.M;|Q5236;|Q5205;|Q5237;|Q5211;O.1;o1.O;|Q5234;|Q5216;O.1;o1.I;|Q5218;O.3;o1.I;o2.I;o3.I;|Q5247;O.3;o1.I;o2.I;o3.I;|Q5243;O.1;o1.I;|Q5222;O.1;o1.I;|Q5217;|Q5210;|", -- [53]
	"|Q5306;O.1;o1.I;|Q5305;O.1;o1.I;|Q5304;|Q5342;O.1;o1.I;|Q5383;|Q5382;O.3;o1.M;o2.O;o3.O;|Q5386;O.1;o1.I;|Q5381;O.1;o1.I;|Q5303;|Q5321;O.2;o1.E;o2.I;|Q5384;O.1;o1.M;|Q5341;O.4;o1.I;o2.I;o3.I;o4.I;|Q5343;O.4;o1.I;o2.I;o3.I;o4.I;|Q5361;|Q5301;O.3;o1.I;o2.I;o3.I;|Q5302;O.4;o1.I;o2.I;o3.I;o4.I;|Q5385;|Q5344;O.1;o1.I;|Q5307;O.1;o1.I;|", -- [54]
	"|Q5405;|Q5401;|Q5402;O.1;o1.I;|Q5407;O.1;o1.I;|Q5406;O.1;o1.I;|Q5408;O.1;o1.I;|Q5421;O.1;o1.I;|Q5441;O.1;o1.O;|Q5466;O.1;o1.I;|Q5403;O.1;o1.I;|Q5462;|Q5481;O.1;o1.I;|Q5461;O.1;o1.I;|Q5463;|Q5482;O.1;o1.I;|Q5464;|Q5404;O.1;o1.I;|Q5465;|", -- [55]
	"|Q5507;O.2;o1.I;o2.R;|Q5506;|Q5505;|Q5536;O.3;o1.M;o2.M;o3.M;|Q5511;|Q5503;|Q5544;O.1;o1.I;|Q5512;|Q5501;O.1;o1.I;|Q5535;O.2;o1.M;o2.M;|Q5504;O.2;o1.I;o2.R;|Q5534;O.1;o1.I;|Q5533;|Q5532;O.2;o1.I;o2.R;|Q5531;|Q5530;O.2;o1.I;o2.R;|Q5515;O.1;o1.I;|Q5513;O.2;o1.I;o2.R;|Q5581;O.1;o1.O;|Q5514;|Q5542;O.3;o1.M;o2.M;o3.M;|Q5545;O.1;o1.I;|Q5538;|Q5537;O.1;o1.I;|Q5518;O.4;o1.I;o2.I;o3.I;o4.I;|Q5582;|Q5519;O.4;o1.I;o2.I;o3.I;o4.I;|Q5517;O.2;o1.I;o2.R;|Q5541;O.1;o1.I;|Q5521;O.2;o1.I;o2.R;|Q5516;|Q5520;|Q5522;|Q5502;O.1;o1.I;|Q5523;|Q5524;O.2;o1.I;o2.R;|Q5525;O.1;o1.I;|Q5526;O.1;o1.I;|Q5543;O.1;o1.M;|Q5527;O.1;o1.I;|Q5508;O.1;o1.I;|Q5510;O.1;o1.I;|Q5528;|Q5509;O.1;o1.I;|Q5529;O.1;o1.M;|Q5561;O.1;o1.O;|", -- [56]
	"|Q5666;|Q5631;|Q5670;|Q5635;|Q5699;|Q5630;|Q5698;|Q5691;|Q5673;|Q5696;|Q5637;|Q5695;|Q5636;|Q5642;|Q5638;|Q5643;|Q5627;|Q5662;|Q5676;|Q5663;|Q5692;|Q5601;|Q5645;|Q5677;|Q5659;|Q5693;|Q5626;|Q5694;|Q5646;|Q5678;|Q5647;|Q5679;|Q5697;|Q5648;O.1;o1.O;|Q5680;|Q5658;|Q5671;|Q5667;|Q5649;|Q5681;|Q5669;|Q5675;|Q5650;O.1;o1.O;|Q5682;|Q5672;|Q5651;|Q5683;|Q5652;|Q5684;|Q5621;O.1;o1.O;|Q5653;|Q5685;|Q5668;|Q5700;|Q5622;|Q5654;|Q5686;|Q5674;|Q5690;|Q5623;|Q5655;|Q5687;|Q5661;|Q5632;|Q5629;|Q5660;|Q5644;|Q5628;|Q5641;|Q5639;|Q5624;O.1;o1.O;|Q5656;|Q5688;|Q5640;|Q5633;|Q5665;|Q5634;|Q5664;|Q5625;O.1;o1.O;|Q5657;|Q5689;|", -- [57]
	"|Q5761;O.1;o1.I;|Q5724;|Q5730;|Q5703;|Q5763;|Q5701;|Q5729;|Q5704;|Q5728;O.2;o1.M;o2.M;|Q5762;|Q5741;O.1;o1.I;|Q5727;O.1;o1.E;|Q5742;O.1;o1.E;|Q5708;|Q5706;|Q5725;O.2;o1.I;o2.I;|Q5711;|Q5702;|Q5707;|Q5709;|Q5723;O.2;o1.M;o2.M;|Q5712;|Q5713;O.1;o1.E;|Q5781;O.1;o1.I;|Q5710;|Q5705;|Q5722;|Q5726;O.1;o1.I;|Q5721;O.1;o1.O;|", -- [58]
	"|Q5891;O.1;o1.I;|Q5889;O.1;o1.I;|Q5893;O.1;o1.I;|Q5886;O.1;o1.I;|Q5821;O.1;o1.E;|Q5861;|Q5801;O.1;o1.I;|Q5884;O.1;o1.I;|Q5803;O.1;o1.I;|Q5882;O.1;o1.I;|Q5888;O.1;o1.I;|Q5804;O.1;o1.I;|Q5892;O.1;o1.I;|Q5887;O.1;o1.I;|Q5805;|Q5863;O.3;o1.M;o2.M;o3.M;|Q5885;O.1;o1.I;|Q5802;O.1;o1.I;|Q5841;|Q5842;|Q5843;|Q5844;|Q5845;O.1;o1.I;|Q5846;|Q5883;O.1;o1.I;|Q5847;|Q5862;|Q5890;O.1;o1.I;|Q5848;O.1;o1.I;|Q5881;|", -- [59]
	"|Q5924;|Q5927;|Q5981;O.2;o1.M;o2.M;|Q5961;|Q5929;O.1;o1.E;|Q5928;|Q5930;O.1;o1.E;|Q5926;|Q5925;|Q5931;|Q5922;|Q5923;|Q5901;O.1;o1.I;|Q5932;|Q5902;|Q5903;O.1;o1.I;|Q5904;|Q5941;|Q5942;|Q5943;O.1;o1.E;|Q5921;|Q5944;O.1;o1.E;|", -- [60]
	"|Q6081;|Q6021;O.1;o1.I;|Q6075;|Q6082;O.1;o1.E;|Q6022;O.1;o1.I;|Q6083;O.1;o1.E;|Q6023;O.2;o1.M;o2.M;|Q6076;|Q6086;|Q6042;O.2;o1.M;o2.M;|Q6088;O.1;o1.E;|Q6025;O.1;o1.E;|Q6026;O.4;o1.I;o2.I;o3.I;o4.I;|Q6024;O.1;o1.I;|Q6028;|Q6029;|Q6085;O.1;o1.E;|Q6089;|Q6030;|Q6061;O.1;o1.E;|Q6031;O.1;o1.I;|Q6063;O.1;o1.E;|Q6027;O.1;o1.I;|Q6064;O.1;o1.E;|Q6032;O.1;o1.I;|Q6001;O.1;o1.E;|Q6065;|Q6002;O.1;o1.E;|Q6066;|Q6067;|Q6003;|Q6004;O.4;o1.M;o2.M;o3.M;o4.M;|Q6068;|Q6069;|Q6070;|Q6062;O.1;o1.E;|Q6071;|Q6087;O.1;o1.E;|Q6084;O.1;o1.E;|Q6072;|Q6074;|Q6041;O.1;o1.O;|Q6073;|", -- [61]
	"|Q6147;|Q6144;|Q6148;|Q6145;O.1;o1.I;|Q6181;|Q6146;O.1;o1.I;|Q6121;|Q6185;O.4;o1.E;o2.I;o3.I;o4.I;|Q6183;|Q6123;O.2;o1.I;o2.I;|Q6187;O.1;o1.M;|Q6125;|Q6184;|Q6127;O.1;o1.I;|Q6124;O.1;o1.O;|Q6122;O.1;o1.I;|Q6128;O.2;o1.I;o2.I;|Q6129;O.1;o1.O;|Q6161;O.2;o1.I;o2.I;|Q6142;O.1;o1.I;|Q6162;O.1;o1.I;|Q6131;O.3;o1.M;o2.M;o3.M;|Q6163;O.1;o1.I;|Q6130;|Q6132;|Q6164;O.1;o1.I;|Q6165;|Q6101;O.1;o1.E;|Q6133;O.4;o1.M;o2.M;o3.M;o4.I;|Q6102;O.1;o1.E;|Q6134;O.1;o1.I;|Q6141;|Q6126;|Q6103;|Q6135;O.1;o1.I;|Q6186;|Q6136;O.1;o1.M;|Q6182;|Q6143;O.3;o1.M;o2.M;o3.M;|", -- [62]
	"|Q6241;O.3;o1.M;o2.M;o3.M;|Q6281;|Q6282;O.4;o1.M;o2.M;o3.M;o4.M;|Q6284;O.1;o1.I;|Q6283;O.1;o1.I;|Q6221;O.3;o1.M;o2.M;o3.M;|Q6285;|Q6202;O.2;o1.I;o2.I;|Q6261;|Q6201;O.2;o1.I;o2.I;|", -- [63]
	"|Q6363;|Q6342;|Q6394;O.1;o1.I;|Q6362;|Q6395;O.1;o1.O;|Q6364;|Q6365;|Q6344;|Q6381;O.1;o1.O;|Q6382;|Q6383;|Q6384;|Q6321;|Q6385;|Q6386;|Q6322;|Q6323;|Q6387;|Q6324;|Q6388;|Q6389;|Q6390;|Q6391;|Q6343;|Q6301;O.1;o1.I;|Q6392;|Q6341;|Q6361;|Q6393;O.1;o1.I;|", -- [64]
	"|Q6401;|Q6442;O.1;o1.I;|Q6441;O.1;o1.I;|Q6402;|Q6461;O.2;o1.M;o2.M;|Q6462;O.1;o1.I;|Q6403;O.1;o1.E;|Q6481;O.1;o1.M;|Q6482;O.1;o1.E;|Q6421;O.2;o1.E;o2.I;|", -- [65]
	"|Q6562;|Q6561;O.1;o1.I;|Q6567;|Q6503;O.1;o1.M;|Q6563;O.1;o1.I;|Q6522;|Q6569;O.1;o1.I;|Q6568;|Q6566;O.1;o1.E;|Q6571;O.4;o1.I;o2.I;o3.I;o4.I;|Q6504;O.3;o1.I;o2.I;o3.I;|Q6501;|Q6541;|Q6542;|Q6543;O.3;o1.I;o2.I;o3.I;|Q6544;O.1;o1.E;|Q6545;O.1;o1.I;|Q6546;O.1;o1.I;|Q6547;O.1;o1.I;|Q6548;O.2;o1.M;o2.M;|Q6581;O.1;o1.I;|Q6582;O.1;o1.I;|Q6565;O.1;o1.M;|Q6523;O.1;o1.E;|Q6583;O.1;o1.I;|Q6570;|Q6564;|Q6502;O.1;o1.I;|Q6584;O.1;o1.I;|Q6521;O.1;o1.I;|Q6585;O.1;o1.I;|", -- [66]
	"|Q6628;O.1;o1.E;|Q6622;O.1;o1.E;|Q6601;|Q6603;|Q6602;O.1;o1.I;|Q6604;|Q6605;|Q6623;|Q6662;|Q6606;|Q6626;O.3;o1.M;o2.M;o3.M;|Q6627;O.1;o1.E;|Q6607;O.4;o1.I;o2.I;o3.I;o4.I;|Q6624;O.1;o1.E;|Q6661;O.1;o1.O;|Q6621;O.1;o1.I;|Q6629;O.2;o1.M;o2.M;|Q6608;|Q6609;|Q6641;O.1;o1.E;|Q6625;|Q6610;O.3;o1.I;o2.I;o3.I;|Q6611;|Q6612;|Q6644;O.1;o1.I;|Q6645;O.1;o1.I;|Q6643;O.1;o1.I;|Q6642;O.1;o1.I;|Q6646;O.1;o1.I;|Q6681;O.1;o1.O;|", -- [67]
	"|Q6781;O.1;o1.I;|Q6761;|Q6762;|Q6722;|Q6721;|Q6701;O.1;o1.I;|Q6702;|Q6704;|Q6707;|Q6703;|Q6706;|Q6708;|Q6709;|Q6741;O.1;o1.I;|Q6705;|Q6710;|Q6711;|", -- [68]
	"|Q6843;|Q6821;O.1;o1.I;|Q6822;O.4;o1.M;o2.M;o3.M;o4.M;|Q6823;O.1;o1.R;|Q6824;O.4;o1.I;o2.I;o3.I;o4.I;|Q6825;O.1;o1.I;|Q6826;O.1;o1.I;|Q6842;|Q6881;O.1;o1.I;|Q6847;O.1;o1.E;|Q6861;O.4;o1.I;o2.I;o3.I;o4.I;|Q6844;|Q6848;O.1;o1.E;|Q6801;O.1;o1.I;|Q6804;O.1;o1.I;|Q6805;O.2;o1.M;o2.M;|Q6846;O.1;o1.I;|Q6862;O.4;o1.I;o2.I;o3.I;o4.I;|Q6827;O.1;o1.I;|Q6841;|Q6845;|", -- [69]
	"|Q6941;O.1;o1.I;|Q6942;O.1;o1.I;|Q6943;O.1;o1.I;|Q6981;O.1;o1.E;|Q6982;O.1;o1.I;|Q6984;|Q6921;O.1;o1.I;|Q6985;O.1;o1.I;|Q6983;O.1;o1.I;|Q6922;|Q6961;|Q6962;O.2;o1.I;o2.I;|Q6963;|Q6964;|Q6901;O.1;o1.I;|", -- [70]
	"|Q7002;O.1;o1.I;|Q7066;|Q7003;O.1;o1.I;|Q7067;O.1;o1.I;|Q7069;|Q7045;|Q7044;O.2;o1.I;o2.I;|Q7081;O.1;o1.O;|Q7046;O.1;o1.E;|Q7042;|Q7043;O.1;o1.I;|Q7021;|Q7022;|Q7023;|Q7082;O.1;o1.O;|Q7070;O.1;o1.I;|Q7024;|Q7025;O.2;o1.I;o2.I;|Q7026;O.1;o1.I;|Q7027;O.1;o1.E;|Q7028;O.1;o1.I;|Q7029;O.2;o1.O;o2.I;|Q7061;|Q7062;|Q7063;|Q7064;O.1;o1.M;|Q7041;O.2;o1.O;o2.I;|Q7068;O.1;o1.I;|Q7001;O.1;o1.E;|Q7065;O.1;o1.M;|", -- [71]
	"|Q7162;O.1;o1.I;|Q7163;O.1;o1.I;|Q7164;O.1;o1.I;|Q7101;O.1;o1.O;|Q7165;O.1;o1.I;|Q7102;O.1;o1.O;|Q7166;O.1;o1.I;|Q7167;O.1;o1.I;|Q7168;O.1;o1.I;|Q7169;O.1;o1.I;|Q7170;O.1;o1.I;|Q7171;O.1;o1.I;|Q7172;O.1;o1.I;|Q7141;O.1;o1.E;|Q7142;O.1;o1.E;|Q7181;O.1;o1.M;|Q7121;|Q7122;O.1;o1.O;|Q7123;|Q7124;O.1;o1.O;|Q7161;O.1;o1.I;|", -- [72]
	"|Q7261;|Q7201;O.1;o1.I;|Q7202;O.1;o1.M;|Q7241;|Q7281;O.1;o1.M;|Q7282;O.1;o1.M;|Q7221;|Q7222;|Q7223;O.1;o1.I;|Q7224;O.1;o1.I;|", -- [73]
	"|Q7386;O.1;o1.I;|Q7361;O.1;o1.I;|Q7362;O.1;o1.I;|Q7363;O.1;o1.I;|Q7364;O.1;o1.I;|Q7301;O.3;o1.O;o2.O;o3.O;|Q7365;O.1;o1.I;|Q7302;O.3;o1.O;o2.O;o3.O;|Q7366;O.1;o1.I;|Q7367;O.1;o1.M;|Q7368;O.1;o1.M;|Q7341;O.1;o1.I;|Q7342;O.1;o1.I;|Q7381;O.1;o1.I;|Q7382;O.1;o1.I;|Q7383;O.1;o1.I;|Q7384;O.1;o1.I;|Q7321;O.1;o1.I;|Q7385;O.1;o1.I;|", -- [74]
	"|Q7482;O.1;o1.E;|Q7483;O.4;o1.I;o2.I;o3.I;o4.I;|Q7484;O.4;o1.I;o2.I;o3.I;o4.I;|Q7421;O.1;o1.I;|Q7485;O.4;o1.I;o2.I;o3.I;o4.I;|Q7422;O.1;o1.I;|Q7486;|Q7423;O.1;o1.I;|Q7487;O.1;o1.I;|Q7424;O.1;o1.I;|Q7488;O.1;o1.I;|Q7425;O.1;o1.I;|Q7489;O.1;o1.I;|Q7426;O.1;o1.I;|Q7490;|Q7427;O.1;o1.I;|Q7491;|Q7428;O.1;o1.I;|Q7492;|Q7429;O.1;o1.I;|Q7461;O.2;o1.M;o2.M;|Q7493;|Q7462;|Q7494;|Q7463;O.1;o1.I;|Q7495;|Q7496;|Q7401;O.1;o1.I;|Q7497;|Q7402;O.1;o1.I;|Q7498;|Q7499;|Q7500;|Q7441;O.1;o1.I;|Q7481;O.1;o1.E;|", -- [75]
	"|Q7581;O.1;o1.I;|Q7582;O.1;o1.I;|Q7583;O.1;o1.I;|Q7521;O.4;o1.I;o2.I;o3.I;o4.I;|Q7522;|Q7561;|Q7562;|Q7563;O.1;o1.I;|Q7564;|Q7501;|Q7502;|Q7503;|Q7504;|Q7505;|Q7506;|Q7507;|Q7508;|Q7509;O.1;o1.I;|Q7541;|", -- [76]
	"|Q7673;O.1;o1.I;|Q7642;O.4;o1.I;o2.I;o3.I;o4.I;|Q7674;O.1;o1.I;|Q7643;O.1;o1.I;|Q7675;O.1;o1.I;|Q7644;|Q7676;O.1;o1.I;|Q7645;O.1;o1.I;|Q7677;O.1;o1.I;|Q7646;O.2;o1.I;o2.I;|Q7678;O.1;o1.I;|Q7647;O.2;o1.I;o2.I;|Q7648;|Q7649;|Q7681;|Q7650;|Q7682;|Q7651;|Q7652;|Q7621;|Q7653;O.1;o1.I;|Q7622;O.1;o1.E;|Q7654;O.1;o1.I;|Q7623;|Q7655;O.1;o1.I;|Q7624;O.1;o1.I;|Q7656;O.1;o1.I;|Q7625;O.1;o1.I;|Q7657;O.1;o1.I;|Q7626;O.1;o1.I;|Q7658;O.1;o1.I;|Q7627;O.2;o1.I;o2.I;|Q7659;O.1;o1.I;|Q7628;O.1;o1.I;|Q7660;O.1;o1.I;|Q7629;O.1;o1.O;|Q7661;O.1;o1.I;|Q7630;O.1;o1.I;|Q7662;O.1;o1.I;|Q7631;|Q7663;O.1;o1.I;|Q7632;|Q7664;O.1;o1.I;|Q7601;|Q7633;|Q7665;O.1;o1.I;|Q7602;O.3;o1.I;o2.I;o3.I;|Q7634;O.1;o1.I;|Q7666;|Q7603;O.1;o1.I;|Q7635;O.1;o1.I;|Q7667;O.2;o1.I;o2.I;|Q7604;O.1;o1.I;|Q7636;O.4;o1.I;o2.I;o3.I;o4.I;|Q7668;O.1;o1.I;|Q7637;|Q7669;|Q7638;|Q7670;|Q7639;|Q7671;O.1;o1.I;|Q7640;O.1;o1.M;|Q7672;O.1;o1.I;|Q7641;|", -- [77]
	"|Q7737;O.1;o1.I;|Q7738;|Q7741;|Q7781;|Q7782;|Q7783;|Q7784;|Q7721;O.1;o1.I;|Q7785;|Q7722;O.1;o1.I;|Q7786;O.4;o1.I;o2.I;o3.I;o4.I;|Q7723;O.1;o1.M;|Q7787;|Q7724;O.1;o1.M;|Q7788;O.1;o1.I;|Q7725;O.1;o1.I;|Q7789;O.1;o1.I;|Q7726;O.1;o1.I;|Q7790;O.1;o1.O;|Q7727;O.1;o1.M;|Q7791;O.1;o1.I;|Q7728;O.2;o1.I;o2.I;|Q7792;O.1;o1.I;|Q7729;O.2;o1.M;o2.M;|Q7761;|Q7793;O.1;o1.I;|Q7730;O.1;o1.I;|Q7794;O.1;o1.I;|Q7731;O.1;o1.I;|Q7795;O.1;o1.I;|Q7732;|Q7796;O.1;o1.I;|Q7701;O.1;o1.I;|Q7733;O.1;o1.I;|Q7797;|Q7702;O.1;o1.I;|Q7734;O.1;o1.I;|Q7798;O.1;o1.I;|Q7703;O.1;o1.I;|Q7735;|Q7799;O.1;o1.I;|Q7704;|Q7736;O.3;o1.I;o2.I;o3.I;|Q7800;O.1;o1.I;|", -- [78]
	"|Q7801;O.1;o1.I;|Q7833;O.1;o1.I;|Q7865;|Q7897;O.1;o1.I;|Q7802;O.1;o1.I;|Q7834;O.1;o1.I;|Q7866;|Q7898;O.1;o1.I;|Q7803;O.1;o1.I;|Q7835;O.1;o1.I;|Q7867;|Q7899;O.1;o1.I;|Q7804;O.1;o1.I;|Q7836;O.1;o1.I;|Q7868;|Q7900;O.1;o1.I;|Q7805;O.1;o1.I;|Q7837;O.1;o1.I;|Q7869;|Q7806;O.1;o1.I;|Q7838;O.1;o1.I;|Q7870;|Q7807;O.1;o1.I;|Q7839;O.1;o1.I;|Q7871;O.1;o1.I;|Q7808;O.1;o1.I;|Q7840;O.1;o1.I;|Q7872;O.1;o1.I;|Q7809;O.1;o1.I;|Q7841;O.4;o1.M;o2.M;o3.M;o4.M;|Q7873;O.1;o1.I;|Q7810;|Q7842;O.1;o1.I;|Q7874;O.1;o1.I;|Q7811;O.1;o1.I;|Q7843;O.1;o1.E;|Q7875;O.1;o1.I;|Q7812;O.1;o1.I;|Q7844;O.2;o1.M;o2.M;|Q7876;O.1;o1.I;|Q7813;O.1;o1.I;|Q7845;|Q7877;|Q7814;O.1;o1.I;|Q7846;O.1;o1.I;|Q7815;O.1;o1.M;|Q7847;|Q7816;O.1;o1.I;|Q7848;O.1;o1.I;|Q7817;O.1;o1.I;|Q7849;O.2;o1.I;o2.I;|Q7881;O.1;o1.I;|Q7818;O.1;o1.I;|Q7850;O.1;o1.I;|Q7882;O.1;o1.I;|Q7819;O.1;o1.I;|Q7883;O.1;o1.I;|Q7820;O.1;o1.I;|Q7884;O.1;o1.I;|Q7821;O.1;o1.I;|Q7885;O.1;o1.I;|Q7822;O.1;o1.I;|Q7886;O.1;o1.I;|Q7823;O.1;o1.I;|Q7887;O.1;o1.I;|Q7824;O.1;o1.I;|Q7888;O.1;o1.I;|Q7825;O.1;o1.I;|Q7889;O.1;o1.I;|Q7826;O.1;o1.I;|Q7890;O.1;o1.I;|Q7827;O.1;o1.I;|Q7891;O.1;o1.I;|Q7828;O.2;o1.M;o2.M;|Q7892;O.1;o1.I;|Q7829;O.1;o1.M;|Q7861;O.2;o1.M;o2.M;|Q7893;O.1;o1.I;|Q7830;O.1;o1.I;|Q7862;O.4;o1.M;o2.M;o3.M;o4.M;|Q7894;O.1;o1.I;|Q7831;O.1;o1.I;|Q7863;|Q7895;O.1;o1.I;|Q7832;O.1;o1.I;|Q7864;|Q7896;O.1;o1.I;|", -- [79]
	"|Q7929;|Q7961;O.1;o1.I;|Q7930;O.1;o1.I;|Q7962;O.1;o1.I;|Q7931;O.1;o1.I;|Q7932;O.1;o1.I;|Q7901;O.1;o1.I;|Q7933;O.1;o1.I;|Q7902;O.1;o1.I;|Q7934;O.1;o1.I;|Q7903;O.1;o1.I;|Q7935;O.1;o1.I;|Q7904;O.1;o1.I;|Q7936;O.1;o1.I;|Q7905;|Q7937;|Q7906;O.1;o1.I;|Q7938;|Q7907;|Q7939;O.1;o1.I;|Q7908;|Q7940;O.1;o1.I;|Q7941;O.1;o1.I;|Q7942;O.1;o1.I;|Q7943;O.1;o1.I;|Q7944;|Q7945;|Q7946;O.1;o1.I;|Q7981;O.1;o1.I;|Q7921;O.1;o1.I;|Q7922;O.1;o1.I;|Q7923;O.1;o1.I;|Q7924;O.1;o1.I;|Q7925;O.1;o1.I;|Q7926;|Q7927;|Q7928;|", -- [80]
	"|Q8025;O.1;o1.I;|Q8057;O.2;o1.I;o2.R;|Q8026;O.1;o1.I;|Q8058;O.2;o1.I;o2.R;|Q8059;O.2;o1.I;o2.R;|Q8060;O.2;o1.I;o2.R;|Q8061;O.2;o1.I;o2.R;|Q8062;O.2;o1.I;o2.R;|Q8063;O.2;o1.I;o2.R;|Q8064;O.2;o1.I;o2.R;|Q8001;O.1;o1.I;|Q8065;O.2;o1.I;o2.R;|Q8002;O.1;o1.I;|Q8066;O.2;o1.I;o2.R;|Q8067;O.2;o1.I;o2.R;|Q8068;O.2;o1.I;o2.R;|Q8069;O.2;o1.I;o2.R;|Q8070;O.2;o1.I;o2.R;|Q8071;O.2;o1.I;o2.R;|Q8072;O.2;o1.I;o2.R;|Q8041;|Q8073;O.2;o1.I;o2.R;|Q8042;O.1;o1.I;|Q8074;O.2;o1.I;o2.R;|Q8043;O.1;o1.I;|Q8075;O.2;o1.I;o2.R;|Q8044;O.1;o1.I;|Q8076;O.2;o1.I;o2.R;|Q8045;|Q8077;O.2;o1.I;o2.R;|Q8046;O.1;o1.I;|Q8078;O.2;o1.I;o2.R;|Q8047;O.1;o1.I;|Q8079;O.2;o1.I;o2.R;|Q8048;O.1;o1.I;|Q8080;O.1;o1.I;|Q8049;|Q8081;O.1;o1.I;|Q8050;O.1;o1.I;|Q8051;O.1;o1.I;|Q8052;O.1;o1.I;|Q8021;O.1;o1.I;|Q8053;O.2;o1.I;o2.R;|Q8022;O.1;o1.I;|Q8054;O.2;o1.I;o2.R;|Q8023;O.1;o1.I;|Q8055;O.2;o1.I;o2.R;|Q8024;O.1;o1.I;|Q8056;O.2;o1.I;o2.R;|", -- [81]
	"|Q8121;O.1;o1.E;|Q8153;O.1;o1.I;|Q8185;O.2;o1.I;o2.I;|Q8122;O.1;o1.E;|Q8154;O.1;o1.I;|Q8186;O.2;o1.I;o2.I;|Q8123;O.1;o1.I;|Q8187;O.2;o1.I;o2.I;|Q8124;O.1;o1.I;|Q8156;O.1;o1.I;|Q8188;O.2;o1.I;o2.I;|Q8157;O.1;o1.I;|Q8189;O.2;o1.I;o2.I;|Q8158;O.1;o1.I;|Q8190;O.2;o1.I;o2.I;|Q8159;O.1;o1.I;|Q8191;O.2;o1.I;o2.I;|Q8192;O.2;o1.I;o2.I;|Q8160;O.1;o1.I;|Q8193;O.1;o1.I;|Q8194;O.1;o1.I;|Q8161;O.1;o1.I;|Q8195;O.3;o1.I;o2.I;o3.I;|Q8196;O.2;o1.I;o2.R;|Q8162;O.1;o1.I;|Q8163;O.1;o1.I;|Q8164;O.1;o1.I;|Q8101;|Q8165;O.1;o1.I;|Q8166;O.4;o1.O;o2.O;o3.O;o4.O;|Q8167;O.4;o1.O;o2.O;o3.O;o4.O;|Q8102;O.1;o1.I;|Q8104;O.1;o1.I;|Q8168;O.4;o1.O;o2.O;o3.O;o4.O;|Q8105;O.4;o1.O;o2.O;o3.O;o4.O;|Q8169;O.4;o1.O;o2.O;o3.O;o4.O;|Q8106;|Q8170;O.4;o1.O;o2.O;o3.O;o4.O;|Q8103;O.1;o1.I;|Q8107;O.1;o1.I;|Q8171;O.4;o1.O;o2.O;o3.O;o4.O;|Q8108;O.1;o1.I;|Q8109;O.1;o1.I;|Q8141;|Q8110;|Q8142;O.1;o1.I;|Q8111;O.1;o1.I;|Q8143;O.1;o1.I;|Q8112;O.1;o1.I;|Q8144;O.1;o1.I;|Q8113;O.1;o1.I;|Q8145;|Q8114;O.1;o1.E;|Q8146;O.1;o1.I;|Q8115;O.1;o1.E;|Q8147;O.1;o1.I;|Q8116;|Q8148;O.1;o1.I;|Q8117;O.1;o1.I;|Q8149;O.1;o1.E;|Q8181;|Q8118;O.1;o1.I;|Q8150;O.1;o1.E;|Q8182;|Q8119;O.1;o1.I;|Q8151;|Q8183;|Q8155;O.1;o1.I;|Q8120;O.4;o1.O;o2.O;o3.O;o4.O;|Q8152;|Q8184;O.2;o1.I;o2.I;|", -- [82]
	"|Q8243;O.2;o1.I;o2.R;|Q8245;O.2;o1.I;o2.R;|Q8247;|Q8249;O.1;o1.I;|Q8251;O.1;o1.I;|Q8253;O.1;o1.I;|Q8255;O.1;o1.I;|Q8257;O.1;o1.I;|Q8259;|Q8261;O.1;o1.R;|Q8263;O.1;o1.R;|Q8201;O.1;o1.I;|Q8265;|Q8252;O.1;o1.I;|Q8267;O.1;o1.I;|Q8248;|Q8242;O.3;o1.I;o2.I;o3.I;|Q8250;|Q8269;O.1;o1.I;|Q8271;|Q8272;|Q8273;|Q8274;O.1;o1.M;|Q8264;O.1;o1.R;|Q8275;|Q8240;|Q8268;O.1;o1.I;|Q8270;|Q8262;|Q8276;|Q8277;O.2;o1.I;o2.I;|Q8278;O.3;o1.I;o2.I;o3.I;|Q8279;O.3;o1.I;o2.I;o3.I;|Q8256;O.1;o1.I;|Q8254;|Q8246;O.2;o1.I;o2.R;|Q8244;O.2;o1.I;o2.R;|Q8280;O.1;o1.M;|Q8281;O.1;o1.M;|Q8282;O.1;o1.I;|Q8283;O.1;o1.I;|Q8238;O.3;o1.I;o2.I;o3.I;|Q8284;O.1;o1.I;|Q8221;O.1;o1.I;|Q8285;|Q8222;O.1;o1.I;|Q8286;O.1;o1.E;|Q8223;O.1;o1.I;|Q8287;|Q8224;O.1;o1.I;|Q8288;O.1;o1.I;|Q8237;|Q8225;O.1;o1.I;|Q8289;O.1;o1.I;|Q8226;O.1;o1.I;|Q8290;O.1;o1.I;|Q8227;|Q8291;O.1;o1.I;|Q8228;|Q8292;O.1;o1.I;|Q8236;O.1;o1.I;|Q8229;|Q8293;O.1;o1.I;|Q8231;O.1;o1.I;|Q8295;O.1;o1.I;|Q8294;O.1;o1.I;|Q8232;O.1;o1.I;|Q8296;O.1;o1.I;|Q8233;|Q8297;O.1;o1.I;|Q8234;O.1;o1.I;|Q8298;O.1;o1.I;|Q8235;O.1;o1.I;|Q8299;O.1;o1.I;|Q8230;|Q8300;O.1;o1.I;|Q8239;O.3;o1.I;o2.I;o3.I;|Q8258;O.1;o1.I;|Q8260;O.1;o1.R;|Q8266;O.1;o1.I;|Q8241;O.3;o1.I;o2.I;o3.I;|", -- [83]
	"|Q8307;|Q8371;O.4;o1.I;o2.I;o3.I;o4.I;|Q8308;|Q8309;O.3;o1.I;o2.I;o3.I;|Q8373;O.1;o1.E;|Q8310;O.3;o1.I;o2.I;o3.I;|Q8374;O.1;o1.I;|Q8311;O.4;o1.I;o2.I;o3.I;o4.I;|Q8375;O.1;o1.I;|Q8312;O.4;o1.I;o2.I;o3.I;o4.I;|Q8376;|Q8377;|Q8314;|Q8378;|Q8315;O.1;o1.I;|Q8379;|Q8316;|Q8380;|Q8317;O.1;o1.I;|Q8381;|Q8318;O.1;o1.I;|Q8382;|Q8319;O.1;o1.I;|Q8383;O.1;o1.I;|Q8320;O.1;o1.M;|Q8384;O.1;o1.I;|Q8321;O.1;o1.I;|Q8385;O.4;o1.I;o2.I;o3.I;o4.I;|Q8386;O.1;o1.I;|Q8323;O.1;o1.I;|Q8387;O.1;o1.I;|Q8324;O.1;o1.I;|Q8388;O.4;o1.I;o2.I;o3.I;o4.I;|Q8325;O.1;o1.M;|Q8389;O.1;o1.I;|Q8326;O.1;o1.I;|Q8390;O.1;o1.I;|Q8327;|Q8391;O.1;o1.I;|Q8328;|Q8392;O.1;o1.I;|Q8329;|Q8393;O.1;o1.I;|Q8330;O.3;o1.I;o2.I;o3.I;|Q8394;O.1;o1.I;|Q8331;|Q8395;O.1;o1.I;|Q8370;O.1;o1.I;|Q8332;O.2;o1.I;o2.I;|Q8396;O.1;o1.I;|Q8333;O.2;o1.I;o2.I;|Q8397;O.1;o1.I;|Q8334;O.2;o1.M;o2.M;|Q8398;O.1;o1.I;|Q8335;O.3;o1.M;o2.M;o3.I;|Q8399;O.1;o1.I;|Q8336;O.1;o1.I;|Q8400;O.1;o1.I;|Q8337;O.1;o1.I;|Q8338;|Q8339;|Q8368;O.1;o1.I;|Q8340;|Q8313;|Q8341;O.2;o1.I;o2.I;|Q8342;O.2;o1.I;o2.I;|Q8343;|Q8344;O.2;o1.I;o2.I;|Q8345;O.1;o1.O;|Q8346;O.1;o1.O;|Q8347;|Q8366;O.4;o1.M;o2.M;o3.M;o4.M;|Q8348;O.1;o1.I;|Q8372;O.1;o1.I;|Q8349;|Q8350;|Q8351;|Q8352;O.1;o1.I;|Q8353;O.1;o1.O;|Q8354;O.1;o1.O;|Q8355;O.1;o1.O;|Q8356;O.1;o1.O;|Q8357;O.1;o1.O;|Q8358;O.1;o1.O;|Q8359;O.1;o1.O;|Q8360;O.1;o1.O;|Q8361;O.1;o1.I;|Q8362;O.1;o1.I;|Q8363;O.1;o1.I;|Q8364;O.1;o1.I;|Q8301;O.1;o1.I;|Q8365;O.1;o1.I;|Q8302;O.1;o1.I;|Q8303;|Q8367;O.4;o1.I;o2.I;o3.I;o4.I;|Q8304;O.2;o1.O;o2.O;|Q8322;|Q8305;|Q8369;O.1;o1.I;|Q8306;O.1;o1.O;|", -- [84]
	"|Q8435;O.1;o1.I;|Q8499;O.1;o1.I;|Q8436;O.1;o1.I;|Q8500;O.1;o1.I;|Q8437;O.1;o1.I;|Q8438;O.1;o1.I;|Q8439;O.1;o1.I;|Q8440;O.1;o1.I;|Q8441;O.1;o1.I;|Q8443;O.1;o1.I;|Q8445;|Q8447;O.1;o1.E;|Q8449;|Q8450;|Q8451;|Q8452;|Q8453;|Q8454;|Q8458;O.1;o1.I;|Q8459;|Q8460;O.3;o1.M;o2.M;o3.M;|Q8461;O.3;o1.M;o2.M;o3.M;|Q8462;|Q8463;O.1;o1.I;|Q8464;O.3;o1.M;o2.M;o3.M;|Q8401;O.1;o1.I;|Q8465;|Q8466;O.1;o1.I;|Q8446;|Q8403;O.1;o1.I;|Q8467;O.1;o1.I;|Q8444;|Q8442;O.1;o1.I;|Q8404;O.1;o1.I;|Q8468;O.1;o1.I;|Q8402;O.1;o1.I;|Q8405;O.1;o1.I;|Q8469;O.1;o1.I;|Q8406;O.1;o1.I;|Q8470;O.1;o1.R;|Q8407;O.1;o1.I;|Q8471;O.1;o1.R;|Q8408;O.1;o1.I;|Q8472;O.1;o1.I;|Q8409;|Q8473;O.1;o1.M;|Q8410;O.4;o1.I;o2.I;o3.I;o4.I;|Q8474;|Q8411;O.4;o1.I;o2.I;o3.I;o4.I;|Q8475;O.1;o1.M;|Q8412;O.2;o1.I;o2.I;|Q8476;O.2;o1.M;o2.M;|Q8413;T.D;O.3;o1.I;o2.I;o3.I;L.52;|Q8477;O.1;o1.I;|Q8414;O.1;o1.I;|Q8478;|Q8415;|Q8479;O.1;o1.I;|Q8416;|Q8480;O.1;o1.I;|Q8417;|Q8481;O.2;o1.I;o2.R;|Q8418;O.3;o1.I;o2.I;o3.I;|Q8419;O.1;o1.I;|Q8483;O.1;o1.I;|Q8420;O.1;o1.I;|Q8484;O.1;o1.R;|Q8421;O.2;o1.I;o2.I;|Q8485;O.1;o1.R;|Q8422;O.3;o1.I;o2.I;o3.I;|Q8486;O.2;o1.M;o2.M;|Q8423;O.1;o1.M;|Q8487;O.1;o1.I;|Q8488;O.1;o1.E;|Q8424;O.3;o1.M;o2.M;o3.M;|Q8425;O.3;o1.I;o2.I;o3.I;|Q8489;|Q8426;O.1;o1.I;|Q8490;O.1;o1.O;|Q8427;O.1;o1.I;|Q8491;O.1;o1.I;|Q8428;O.1;o1.I;|Q8492;O.1;o1.I;|Q8482;|Q8429;O.1;o1.I;|Q8493;O.1;o1.I;|Q8430;O.1;o1.I;|Q8494;O.1;o1.I;|Q8431;O.1;o1.I;|Q8495;O.1;o1.I;|Q8432;O.1;o1.I;|Q8496;O.3;o1.I;o2.I;o3.I;|Q8433;O.1;o1.I;|Q8497;O.3;o1.I;o2.I;o3.I;|Q8448;|Q8434;O.1;o1.I;|Q8498;O.1;o1.I;|", -- [85]
	"|Q8563;|Q8501;O.1;o1.M;|Q8565;O.1;o1.I;|Q8502;O.1;o1.M;|Q8503;O.1;o1.I;|Q8567;O.1;o1.I;|Q8504;O.1;o1.I;|Q8505;O.1;o1.I;|Q8569;O.1;o1.I;|Q8506;O.1;o1.I;|Q8507;O.1;o1.I;|Q8571;O.1;o1.I;|Q8508;O.1;o1.I;|Q8509;O.1;o1.I;|Q8573;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8510;O.1;o1.I;|Q8511;O.1;o1.I;|Q8575;|Q8512;O.1;o1.I;|Q8513;O.1;o1.I;|Q8577;|Q8514;O.1;o1.I;|Q8515;O.1;o1.I;|Q8579;O.1;o1.I;|Q8516;O.1;o1.I;|Q8517;O.1;o1.I;|Q8581;O.1;o1.I;|Q8518;O.1;o1.I;|Q8519;O.1;o1.E;|Q8583;O.1;o1.I;|Q8520;O.1;o1.I;|Q8521;O.1;o1.I;|Q8585;O.2;o1.I;o2.I;|Q8572;O.4;o1.I;o2.I;o3.I;o4.R;|Q8580;O.1;o1.I;|Q8562;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8523;O.1;o1.I;|Q8587;|Q8522;O.1;o1.I;|Q8582;O.1;o1.I;|Q8525;O.1;o1.I;|Q8589;O.1;o1.I;|Q8527;O.1;o1.I;|Q8591;O.1;o1.I;|Q8528;O.1;o1.I;|Q8592;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8529;O.1;o1.I;|Q8593;|Q8530;O.1;o1.I;|Q8594;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8531;O.1;o1.I;|Q8595;O.1;o1.I;|Q8524;O.1;o1.I;|Q8532;O.1;o1.I;|Q8564;|Q8570;O.1;o1.I;|Q8533;O.1;o1.I;|Q8597;|Q8534;O.1;o1.I;|Q8598;|Q8596;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8535;O.1;o1.M;|Q8599;|Q8526;O.1;o1.I;|Q8600;O.1;o1.I;|Q8536;O.1;o1.M;|Q8574;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8537;O.1;o1.M;|Q8538;O.4;o1.M;o2.M;o3.M;o4.M;|Q8539;O.1;o1.M;|Q8540;O.1;o1.I;|Q8541;O.3;o1.I;o2.I;o3.I;|Q8542;O.1;o1.I;|Q8543;O.1;o1.I;|Q8557;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8544;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8545;O.1;o1.I;|Q8546;O.1;o1.I;|Q8547;|Q8548;O.4;o1.I;o2.I;o3.I;o4.R;|Q8549;O.1;o1.I;|Q8550;O.1;o1.I;|Q8551;O.1;o1.I;|Q8552;|Q8553;|Q8554;O.1;o1.I;|Q8555;|Q8556;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8558;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8559;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8578;O.1;o1.I;|Q8568;O.1;o1.I;|Q8576;|Q8590;O.1;o1.I;|Q8584;|Q8560;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8586;O.2;o1.I;o2.I;|Q8588;O.1;o1.I;|Q8561;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8566;O.1;o1.I;|", -- [86]
	"|Q8627;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8691;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8628;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8692;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8629;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8693;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8630;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8694;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8631;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8695;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8632;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8696;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8697;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8634;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8698;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8635;|Q8699;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8636;|Q8700;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8638;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8639;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8640;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8643;|Q8647;|Q8649;|Q8650;|Q8651;|Q8641;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8652;|Q8653;|Q8654;|Q8655;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8656;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8657;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8658;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8644;|Q8659;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8660;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8661;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8646;|Q8633;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8662;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8648;|Q8663;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8664;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8601;O.1;o1.I;|Q8665;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8602;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8666;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8603;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8667;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8686;|Q8604;O.1;o1.I;|Q8668;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8605;O.1;o1.I;|Q8669;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8606;O.1;o1.O;|Q8670;|Q8607;O.1;o1.I;|Q8671;|Q8608;O.1;o1.I;|Q8672;|Q8642;|Q8609;O.1;o1.I;|Q8673;|Q8610;O.1;o1.I;|Q8674;|Q8611;O.1;o1.I;|Q8675;|Q8612;O.1;o1.I;|Q8676;|Q8684;|Q8613;O.1;o1.I;|Q8677;|Q8614;O.1;o1.I;|Q8678;|Q8615;O.1;o1.I;|Q8679;|Q8616;O.1;o1.I;|Q8680;|Q8617;O.1;o1.I;|Q8681;|Q8618;O.1;o1.I;|Q8682;|Q8619;|Q8683;|Q8620;O.1;o1.I;|Q8621;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8685;|Q8622;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8623;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8687;O.1;o1.M;|Q8624;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8688;|Q8645;|Q8637;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8625;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8689;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8626;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8690;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|", -- [87]
	"|Q8755;O.1;o1.I;|Q8756;O.1;o1.I;|Q8757;|Q8758;O.1;o1.I;|Q8759;O.1;o1.I;|Q8760;O.1;o1.I;|Q8761;O.1;o1.I;|Q8763;O.1;o1.I;|Q8701;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8765;O.4;o1.I;o2.I;o3.I;o4.I;|Q8702;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8703;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8767;|Q8704;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8705;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8769;|Q8706;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8707;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8771;O.1;o1.M;|Q8708;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8709;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8773;O.1;o1.M;|Q8710;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8774;O.1;o1.M;|Q8711;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8775;O.1;o1.M;|Q8712;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q8776;O.1;o1.M;|Q8777;O.1;o1.M;|Q8714;|Q8778;O.3;o1.I;o2.I;o3.I;|Q8715;|Q8779;O.3;o1.I;o2.I;o3.I;|Q8762;O.2;o1.O;o2.I;|Q8716;|Q8780;O.2;o1.I;o2.I;|Q8717;|Q8781;O.1;o1.I;|Q8764;O.4;o1.I;o2.I;o3.I;o4.I;|Q8718;|Q8782;O.3;o1.I;o2.I;o3.I;|Q8713;|Q8719;|Q8783;O.2;o1.I;o2.I;|Q8766;O.4;o1.I;o2.I;o3.I;o4.I;|Q8720;|Q8784;|Q8721;|Q8785;O.3;o1.I;o2.I;o3.I;|Q8786;O.1;o1.I;|Q8723;|Q8787;O.2;o1.I;o2.I;|Q8788;|Q8725;|Q8789;O.2;o1.I;o2.I;|Q8772;O.1;o1.M;|Q8726;|Q8790;O.2;o1.I;o2.I;|Q8727;|Q8791;|Q8728;O.4;o1.I;o2.I;o3.I;o4.I;|Q8792;|Q8729;O.1;o1.I;|Q8793;|Q8730;O.1;o1.I;|Q8794;|Q8731;O.1;o1.I;|Q8795;|Q8732;O.1;o1.I;|Q8796;|Q8733;|Q8734;|Q8798;|Q8724;|Q8735;O.4;o1.I;o2.I;o3.I;o4.I;T.R;|Q8799;O.1;o1.I;|Q8736;O.1;o1.E;|Q8800;|Q8737;O.1;o1.M;|Q8738;O.1;o1.I;|Q8739;O.1;o1.I;|Q8740;O.2;o1.M;o2.M;|Q8741;|Q8742;|Q8743;O.1;o1.I;|Q8744;|Q8745;|Q8746;O.2;o1.O;o2.I;|Q8747;|Q8748;O.1;o1.I;|Q8749;O.1;o1.I;|Q8797;|Q8750;O.1;o1.I;|Q8751;O.1;o1.I;|Q8770;O.1;o1.M;|Q8722;|Q8752;|Q8753;O.1;o1.I;|Q8754;O.1;o1.I;|Q8768;|", -- [88]
	"|Q8819;O.1;o1.I;|Q8883;|Q8820;O.1;o1.I;|Q8885;O.1;o1.I;|Q8822;O.1;o1.I;|Q8823;O.1;o1.I;|Q8887;|Q8824;O.1;o1.I;|Q8889;O.3;o1.O;o2.O;o3.O;|Q8826;O.1;o1.I;|Q8827;|Q8891;|Q8828;|Q8829;O.3;o1.I;o2.I;o3.I;|Q8893;O.1;o1.I;|Q8830;O.1;o1.I;|Q8831;O.1;o1.I;|Q8895;|Q8832;O.1;o1.I;|Q8833;O.1;o1.I;|Q8897;|Q8834;O.1;o1.I;|Q8835;O.1;o1.I;|Q8899;|Q8836;O.1;o1.I;|Q8837;O.1;o1.I;|Q8838;O.1;o1.I;|Q8839;O.1;o1.I;|Q8892;O.2;o1.M;o2.M;|Q8894;O.2;o1.M;o2.M;|Q8840;O.1;o1.I;|Q8841;O.1;o1.I;|Q8882;O.1;o1.I;|Q8843;O.1;o1.I;|Q8898;|Q8844;O.1;o1.I;|Q8845;O.1;o1.I;|Q8846;O.1;o1.I;|Q8900;|Q8817;O.1;o1.I;|Q8847;O.1;o1.I;|Q8890;|Q8848;O.1;o1.I;|Q8849;O.1;o1.I;|Q8850;O.1;o1.I;|Q8884;O.1;o1.I;|Q8821;O.1;o1.I;|Q8851;O.1;o1.I;|Q8842;O.1;o1.I;|Q8852;O.1;o1.I;|Q8853;O.1;o1.I;|Q8854;O.1;o1.I;|Q8855;O.1;o1.I;|Q8886;O.1;o1.I;|Q8856;O.3;o1.I;o2.I;o3.I;|Q8857;|Q8858;|Q8825;O.1;o1.I;|Q8859;|Q8860;|Q8861;|Q8862;O.1;o1.I;|Q8863;O.1;o1.I;|Q8813;O.1;o1.I;|Q8864;O.1;o1.I;|Q8896;|Q8801;|Q8865;O.1;o1.I;|Q8802;|Q8866;|Q8803;|Q8867;O.2;o1.O;o2.O;|Q8804;O.3;o1.I;o2.I;o3.I;|Q8868;O.1;o1.E;|Q8805;O.1;o1.I;|Q8869;O.1;o1.I;|Q8806;O.3;o1.I;o2.I;o3.I;|Q8870;|Q8807;O.3;o1.I;o2.I;o3.I;|Q8871;|Q8808;O.3;o1.I;o2.I;o3.I;|Q8872;|Q8873;|Q8809;O.2;o1.I;o2.I;|Q8810;O.3;o1.I;o2.I;o3.I;|Q8874;|Q8811;O.1;o1.I;|Q8875;|Q8812;O.1;o1.I;|Q8876;O.1;o1.I;|Q8877;O.1;o1.I;|Q8814;O.1;o1.I;|Q8878;O.1;o1.I;|Q8815;O.1;o1.I;|Q8879;O.1;o1.I;|Q8816;O.1;o1.I;|Q8880;O.1;o1.I;|Q8888;|Q8881;O.1;o1.I;|Q8818;O.1;o1.I;|", -- [89]
	"|Q8947;O.4;o1.I;o2.I;o3.I;o4.I;|Q8948;|Q8949;O.1;o1.I;|Q8950;O.3;o1.I;o2.I;o3.I;|Q8951;O.3;o1.I;o2.I;o3.I;|Q8952;O.3;o1.I;o2.I;o3.I;|Q8954;O.3;o1.I;o2.I;o3.I;|Q8955;O.3;o1.I;o2.I;o3.I;|Q8956;O.3;o1.I;o2.I;o3.I;|Q8958;O.3;o1.I;o2.I;o3.I;|Q8959;O.3;o1.I;o2.I;o3.I;|Q8960;|Q8963;O.1;o1.I;|Q8901;|Q8903;O.1;o1.I;|Q8967;O.2;o1.M;o2.I;|Q8957;O.3;o1.I;o2.I;o3.I;|Q8904;O.1;o1.I;|Q8968;O.3;o1.M;o2.M;o3.I;|Q8953;O.3;o1.I;o2.I;o3.I;|Q8905;O.2;o1.I;o2.I;|Q8970;O.1;o1.I;|Q8906;O.2;o1.I;o2.I;|Q8907;O.2;o1.I;o2.I;|Q8971;O.3;o1.I;o2.I;o3.I;|Q8961;O.4;o1.I;o2.I;o3.I;o4.I;|Q8965;O.1;o1.I;|Q8908;O.2;o1.I;o2.I;|Q8909;O.2;o1.I;o2.I;|Q8973;O.3;o1.I;o2.I;o3.I;|Q8962;O.1;o1.I;|Q8910;O.2;o1.I;o2.I;|Q8911;O.2;o1.I;o2.I;|Q8975;|Q8969;O.2;o1.M;o2.I;|Q8912;O.2;o1.I;o2.I;|Q8976;|Q8913;O.2;o1.I;o2.I;|Q8977;|Q8978;|Q8914;O.2;o1.I;o2.I;|Q8915;O.2;o1.I;o2.I;|Q8979;O.2;o1.I;o2.I;|Q8980;|Q8916;O.2;o1.I;o2.I;|Q8917;O.2;o1.I;o2.I;|Q8981;O.1;o1.I;|Q8902;|Q8918;O.2;o1.I;o2.I;|Q8919;O.2;o1.I;o2.I;|Q8983;|Q8982;|Q8920;O.2;o1.I;o2.I;|Q8984;|Q8921;O.4;o1.I;o2.I;o3.I;o4.I;|Q8985;O.1;o1.I;|Q8986;O.1;o1.I;|Q8922;|Q8923;|Q8987;O.1;o1.I;|Q8924;O.3;o1.I;o2.I;o3.I;|Q8988;O.1;o1.I;|Q8925;O.1;o1.I;|Q8989;O.2;o1.M;o2.I;|Q8926;O.2;o1.I;o2.I;|Q8990;O.2;o1.M;o2.I;|Q8927;O.2;o1.I;o2.I;|Q8991;O.3;o1.M;o2.M;o3.I;|Q8992;O.2;o1.M;o2.I;|Q8928;O.1;o1.I;|Q8929;|Q8993;O.1;o1.I;|Q8930;|Q8994;O.2;o1.I;o2.I;|Q8931;O.2;o1.I;o2.I;|Q8995;O.2;o1.M;o2.I;|Q8932;O.2;o1.I;o2.I;|Q8996;O.1;o1.I;|Q8933;O.2;o1.I;o2.I;|Q8997;|Q8934;O.2;o1.I;o2.I;|Q8998;|Q8935;O.2;o1.I;o2.I;|Q8999;O.2;o1.I;o2.I;|Q8936;O.2;o1.I;o2.I;|Q9000;O.2;o1.I;o2.I;|Q8937;O.2;o1.I;o2.I;|Q8938;O.2;o1.I;o2.I;|Q8939;O.2;o1.I;o2.I;|Q8940;O.2;o1.I;o2.I;|Q8941;O.2;o1.I;o2.I;|Q8942;O.2;o1.I;o2.I;|Q8943;O.2;o1.I;o2.I;|Q8974;O.3;o1.I;o2.I;o3.I;|Q8944;O.2;o1.I;o2.I;|Q8972;O.3;o1.I;o2.I;o3.I;|Q8945;O.1;o1.O;|Q8966;O.2;o1.M;o2.I;|Q8964;O.1;o1.I;|Q8946;|", -- [90]
	"|Q9011;O.2;o1.I;o2.I;|Q9075;O.4;o1.I;o2.I;o3.I;o4.I;|Q9076;O.1;o1.I;|Q9013;O.2;o1.I;o2.I;|Q9077;O.4;o1.I;o2.I;o3.I;o4.I;|Q9078;O.4;o1.I;o2.I;o3.I;o4.I;|Q9015;O.2;o1.O;o2.I;|Q9079;O.4;o1.I;o2.I;o3.I;o4.I;|Q9080;O.4;o1.I;o2.I;o3.I;o4.I;|Q9017;O.3;o1.I;o2.I;o3.I;|Q9081;O.4;o1.I;o2.I;o3.I;o4.I;|Q9019;O.3;o1.I;o2.I;o3.I;|Q9083;O.4;o1.I;o2.I;o3.I;o4.I;|Q9021;O.3;o1.I;o2.I;o3.I;|Q9085;O.1;o1.M;|Q9086;O.4;o1.I;o2.I;o3.I;o4.I;|Q9023;O.2;o1.I;o2.I;|Q9087;O.4;o1.I;o2.I;o3.I;o4.I;|Q9088;O.4;o1.I;o2.I;o3.I;o4.I;|Q9025;|Q9089;O.4;o1.I;o2.I;o3.I;o4.I;|Q9090;O.4;o1.I;o2.I;o3.I;o4.I;|Q9027;|Q9091;O.4;o1.I;o2.I;o3.I;o4.I;|Q9028;|Q9092;O.4;o1.I;o2.I;o3.I;o4.I;|Q9029;|Q9093;O.4;o1.I;o2.I;o3.I;o4.I;|Q9030;|Q9094;O.1;o1.I;|Q9031;|Q9095;O.4;o1.I;o2.I;o3.I;o4.I;|Q9032;|Q9096;O.4;o1.I;o2.I;o3.I;o4.I;|Q9097;O.4;o1.I;o2.I;o3.I;o4.I;|Q9034;O.4;o1.I;o2.I;o3.I;o4.I;|Q9098;O.4;o1.I;o2.I;o3.I;o4.I;|Q9010;O.2;o1.I;o2.I;|Q9035;|Q9099;O.4;o1.I;o2.I;o3.I;o4.I;|Q9020;O.3;o1.I;o2.I;o3.I;|Q9036;O.4;o1.I;o2.I;o3.I;o4.I;|Q9100;O.3;o1.I;o2.I;o3.I;|Q9033;O.4;o1.M;o2.M;o3.M;o4.M;|Q9082;O.4;o1.I;o2.I;o3.I;o4.I;|Q9018;O.3;o1.I;o2.I;o3.I;|Q9038;O.4;o1.I;o2.I;o3.I;o4.I;|Q9039;O.4;o1.I;o2.I;o3.I;o4.I;|Q9024;O.2;o1.I;o2.I;|Q9040;O.4;o1.I;o2.I;o3.I;o4.I;|Q9022;O.3;o1.I;o2.I;o3.I;|Q9041;O.4;o1.I;o2.I;o3.I;o4.I;|Q9026;|Q9043;O.4;o1.I;o2.I;o3.I;o4.I;|Q9008;O.2;o1.I;o2.I;|Q9014;O.2;o1.I;o2.I;|Q9012;O.2;o1.I;o2.I;|Q9044;O.4;o1.I;o2.I;o3.I;o4.I;|Q9045;O.4;o1.I;o2.I;o3.I;o4.I;|Q9046;O.4;o1.I;o2.I;o3.I;o4.I;|Q9047;O.4;o1.I;o2.I;o3.I;o4.I;|Q9084;O.4;o1.I;o2.I;o3.I;o4.I;|Q9042;O.4;o1.I;o2.I;o3.I;o4.I;|Q9048;O.4;o1.I;o2.I;o3.I;o4.I;|Q9049;O.4;o1.I;o2.I;o3.I;o4.I;|Q9050;O.4;o1.I;o2.I;o3.I;o4.I;|Q9051;O.1;o1.E;|Q9006;O.2;o1.I;o2.I;|Q9052;O.2;o1.I;o2.I;|Q9053;O.1;o1.I;|Q9054;O.4;o1.I;o2.I;o3.I;o4.I;|Q9055;O.4;o1.I;o2.I;o3.I;o4.I;|Q9056;O.4;o1.I;o2.I;o3.I;o4.I;|Q9057;O.4;o1.I;o2.I;o3.I;o4.I;|Q9058;O.4;o1.I;o2.I;o3.I;o4.I;|Q9059;O.4;o1.I;o2.I;o3.I;o4.I;|Q9004;O.2;o1.I;o2.I;|Q9060;O.4;o1.I;o2.I;o3.I;o4.I;|Q9061;O.4;o1.I;o2.I;o3.I;o4.I;|Q9062;O.1;o1.I;|Q9063;|Q9064;|Q9001;O.2;o1.I;o2.I;|Q9065;O.1;o1.M;|Q9002;O.2;o1.I;o2.I;|Q9066;O.2;o1.O;o2.O;|Q9003;O.2;o1.I;o2.I;|Q9067;O.3;o1.I;o2.I;o3.I;|Q9068;O.4;o1.I;o2.I;o3.I;o4.I;|Q9005;O.2;o1.I;o2.I;|Q9069;O.4;o1.I;o2.I;o3.I;o4.I;|Q9070;O.4;o1.I;o2.I;o3.I;o4.I;|Q9007;O.2;o1.I;o2.I;|Q9071;O.4;o1.I;o2.I;o3.I;o4.I;|Q9016;O.3;o1.I;o2.I;o3.I;|Q9072;O.4;o1.I;o2.I;o3.I;o4.I;|Q9037;O.4;o1.I;o2.I;o3.I;o4.I;|Q9009;O.2;o1.I;o2.I;|Q9073;O.4;o1.I;o2.I;o3.I;o4.I;|Q9074;O.4;o1.I;o2.I;o3.I;o4.I;|", -- [91]
	"|Q9139;O.2;o1.M;o2.M;|Q9140;O.2;o1.I;o2.I;|Q9141;O.1;o1.I;|Q9142;O.1;o1.I;|Q9143;O.1;o1.I;|Q9144;|Q9145;|Q9146;|Q9147;O.1;o1.I;|Q9148;|Q9149;O.1;o1.I;|Q9150;O.1;o1.I;|Q9151;|Q9152;O.1;o1.I;|Q9153;O.1;o1.I;|Q9154;|Q9155;O.2;o1.M;o2.M;|Q9156;O.2;o1.I;o2.I;|Q9157;O.1;o1.I;|Q9158;O.1;o1.M;|Q9159;O.2;o1.M;o2.M;|Q9160;O.2;o1.E;o2.M;|Q9161;|Q9163;O.3;o1.I;o2.I;o3.I;|Q9164;O.3;o1.O;o2.O;o3.O;|Q9101;O.4;o1.I;o2.I;o3.I;o4.I;|Q9165;O.1;o1.O;|Q9102;O.4;o1.I;o2.I;o3.I;o4.I;|Q9166;|Q9137;O.1;o1.I;|Q9103;O.4;o1.I;o2.I;o3.I;o4.I;|Q9167;T.G.5;O.1;o1.I;|Q9162;|Q9104;O.4;o1.I;o2.I;o3.I;o4.I;|Q9168;O.1;o1.M;|Q9105;O.4;o1.I;o2.I;o3.I;o4.I;|Q9169;O.1;o1.O;|Q9106;O.4;o1.I;o2.I;o3.I;o4.I;|Q9170;O.4;o1.M;o2.M;o3.M;o4.M;|Q9107;O.4;o1.I;o2.I;o3.I;o4.I;|Q9171;O.1;o1.I;|Q9108;O.3;o1.I;o2.I;o3.I;|Q9172;|Q9109;O.4;o1.I;o2.I;o3.I;o4.I;|Q9173;O.2;o1.M;o2.M;|Q9110;O.4;o1.I;o2.I;o3.I;o4.I;|Q9174;O.1;o1.M;|Q9111;O.4;o1.I;o2.I;o3.I;o4.I;|Q9175;|Q9112;O.4;o1.I;o2.I;o3.I;o4.I;|Q9176;O.2;o1.I;o2.I;|Q9113;O.4;o1.I;o2.I;o3.I;o4.I;|Q9177;|Q9114;O.4;o1.I;o2.I;o3.I;o4.I;|Q9178;O.1;o1.I;|Q9115;O.4;o1.I;o2.I;o3.I;o4.I;|Q9179;O.1;o1.I;|Q9116;O.3;o1.I;o2.I;o3.I;|Q9180;|Q9117;O.4;o1.I;o2.I;o3.I;o4.I;|Q9181;O.1;o1.I;|Q9118;O.4;o1.I;o2.I;o3.I;o4.I;|Q9182;O.1;o1.I;|Q9119;|Q9183;O.1;o1.I;|Q9197;O.1;o1.I;|Q9133;|Q9120;|Q9184;O.1;o1.I;|Q9121;O.4;o1.I;o2.I;o3.I;o4.R;|Q9185;O.1;o1.I;|Q9122;O.3;o1.I;o2.I;o3.R;|Q9186;O.1;o1.I;|Q9123;O.1;o1.R;|Q9187;O.1;o1.I;|Q9124;O.1;o1.I;|Q9188;O.1;o1.I;|Q9125;O.1;o1.I;|Q9189;|Q9126;O.1;o1.I;|Q9190;O.1;o1.I;|Q9127;O.1;o1.I;|Q9191;O.1;o1.I;|Q9128;O.1;o1.I;|Q9192;O.3;o1.M;o2.M;o3.M;|Q9129;O.1;o1.I;|Q9193;O.2;o1.E;o2.O;|Q9130;|Q9194;O.1;o1.I;|Q9131;O.1;o1.I;|Q9195;O.1;o1.I;|Q9132;O.1;o1.I;|Q9196;O.1;o1.I;|Q9134;|Q9198;O.1;o1.I;|Q9135;|Q9199;O.1;o1.I;|Q9136;O.1;o1.I;|Q9200;O.1;o1.I;|Q9138;O.1;o1.M;|", -- [92]
	"|Q9203;O.1;o1.I;|Q9267;O.2;o1.I;o2.I;|Q9268;O.2;o1.I;o2.I;|Q9205;O.1;o1.I;|Q9269;O.1;o1.O;|Q9270;O.1;o1.O;|Q9207;O.1;o1.I;|Q9271;O.1;o1.O;|Q9272;|Q9209;O.1;o1.I;|Q9274;O.2;o1.M;o2.M;|Q9211;O.1;o1.I;|Q9275;O.3;o1.O;o2.O;o3.O;|Q9276;O.2;o1.M;o2.M;|Q9213;O.1;o1.I;|Q9277;O.2;o1.M;o2.M;|Q9214;O.4;o1.I;o2.I;o3.I;o4.I;|Q9278;|Q9215;T.G.2;O.1;o1.I;|Q9279;|Q9216;O.1;o1.I;|Q9280;O.1;o1.I;|Q9217;O.1;o1.I;|Q9281;O.2;o1.M;o2.M;|Q9218;O.1;o1.I;|Q9219;O.1;o1.I;|Q9283;O.1;o1.O;|Q9220;O.3;o1.M;o2.M;o3.M;|Q9221;O.2;o1.I;o2.I;|Q9285;|Q9222;O.2;o1.I;o2.I;|Q9286;|Q9223;O.2;o1.I;o2.I;|Q9287;|Q9224;O.2;o1.I;o2.I;|Q9288;|Q9225;O.2;o1.I;o2.I;|Q9289;|Q9226;O.2;o1.I;o2.I;|Q9290;|Q9273;O.1;o1.I;|Q9202;O.1;o1.I;|Q9227;O.2;o1.I;o2.I;|Q9291;|Q9228;O.2;o1.I;o2.I;|Q9292;|Q9229;O.1;o1.I;|Q9293;O.1;o1.I;|Q9208;O.1;o1.I;|Q9230;O.3;o1.I;o2.I;o3.I;|Q9294;O.1;o1.O;|Q9284;|Q9282;|Q9206;O.1;o1.I;|Q9201;O.1;o1.I;|Q9231;|Q9295;|Q9204;O.1;o1.I;|Q9212;O.1;o1.E;|Q9232;O.3;o1.I;o2.I;o3.I;|Q9296;|Q9233;|Q9297;|Q9234;O.4;o1.I;o2.I;o3.I;o4.I;|Q9298;|Q9235;O.4;o1.I;o2.I;o3.I;o4.I;|Q9299;|Q9236;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q9300;|Q9237;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q9238;O.4;o1.I;o2.I;o3.I;o4.I;|Q9239;O.4;o1.I;o2.I;o3.I;o4.I;|Q9240;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q9241;O.4;o1.I;o2.I;o3.I;o4.I;|Q9242;O.4;o1.I;o2.I;o3.I;o4.I;|Q9243;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q9244;O.4;o1.I;o2.I;o3.I;o4.I;|Q9245;O.4;o1.I;o2.I;o3.I;o4.I;|Q9246;O.5;o1.I;o2.I;o3.I;o4.I;o5.R;|Q9247;|Q9248;O.2;o1.I;o2.R;|Q9249;O.1;o1.I;|Q9250;|Q9251;|Q9252;O.2;o1.M;o2.M;|Q9253;|Q9254;|Q9255;|Q9256;|Q9257;O.1;o1.O;|Q9258;|Q9259;O.2;o1.I;o2.I;|Q9260;O.2;o1.E;o2.I;|Q9261;O.2;o1.E;o2.I;|Q9262;O.2;o1.E;o2.I;|Q9263;O.2;o1.E;o2.I;|Q9264;O.2;o1.E;o2.I;|Q9210;O.1;o1.I;|Q9265;O.2;o1.E;o2.I;|Q9266;O.2;o1.I;o2.I;|", -- [93]
	"|Q9331;|Q9395;|Q9332;|Q9396;O.1;o1.I;|Q9333;O.1;o1.I;|Q9397;O.1;o1.I;|Q9334;O.1;o1.I;|Q9398;O.2;o1.M;o2.M;|Q9335;O.1;o1.I;|Q9399;O.1;o1.M;|Q9336;O.1;o1.I;|Q9400;O.1;o1.E;|Q9337;O.1;o1.I;|Q9338;O.3;o1.I;o2.I;o3.I;|Q9339;|Q9340;O.2;o1.M;o2.M;|Q9341;O.1;o1.I;|Q9342;O.1;o1.M;|Q9343;O.1;o1.I;|Q9344;|Q9345;O.1;o1.I;|Q9346;O.1;o1.I;|Q9347;|Q9348;O.2;o1.M;o2.I;|Q9349;O.1;o1.I;|Q9350;O.2;o1.I;o2.I;|Q9351;O.1;o1.I;|Q9352;O.1;o1.O;|Q9354;O.2;o1.I;o2.I;|Q9355;O.1;o1.M;|Q9356;O.1;o1.I;|Q9357;|Q9353;O.1;o1.I;|Q9358;|Q9359;|Q9360;|Q9361;O.1;o1.I;|Q9362;O.1;o1.I;|Q9363;|Q9364;O.1;o1.M;|Q9301;|Q9365;|Q9302;|Q9366;O.1;o1.I;|Q9303;O.1;o1.O;|Q9367;O.3;o1.O;o2.O;o3.O;|Q9304;|Q9368;O.3;o1.O;o2.O;o3.O;|Q9369;O.1;o1.I;|Q9305;O.1;o1.I;|Q9306;|Q9370;O.1;o1.M;|Q9307;O.1;o1.M;|Q9371;|Q9308;|Q9372;O.1;o1.I;|Q9309;|Q9373;|Q9310;|Q9374;O.1;o1.I;|Q9311;O.1;o1.M;|Q9375;O.1;o1.E;|Q9312;|Q9376;O.1;o1.I;|Q9313;|Q9377;O.1;o1.I;|Q9314;|Q9378;O.1;o1.R;|Q9315;O.1;o1.M;|Q9379;O.1;o1.I;|Q9316;O.1;o1.I;|Q9380;O.1;o1.I;|Q9317;O.1;o1.I;|Q9381;O.1;o1.I;|Q9318;O.1;o1.I;|Q9382;O.1;o1.M;|Q9319;O.4;o1.O;o2.O;o3.O;o4.O;|Q9383;O.1;o1.I;|Q9320;O.1;o1.I;|Q9384;O.1;o1.O;|Q9321;O.1;o1.I;|Q9385;O.1;o1.M;|Q9322;O.4;o1.O;o2.O;o3.O;o4.O;|Q9386;O.4;o1.O;o2.O;o3.O;o4.O;|Q9323;O.4;o1.O;o2.O;o3.O;o4.O;|Q9387;O.1;o1.I;|Q9324;|Q9388;O.4;o1.O;o2.O;o3.O;o4.O;|Q9325;|Q9389;O.4;o1.O;o2.O;o3.O;o4.O;|Q9326;|Q9390;|Q9327;|Q9391;O.3;o1.O;o2.O;o3.O;|Q9328;|Q9392;|Q9329;|Q9393;|Q9330;|Q9394;|", -- [94]
	"|Q9459;O.1;o1.I;|Q9461;|Q9463;O.1;o1.I;|Q9401;|Q9465;O.1;o1.I;|Q9403;O.1;o1.I;|Q9467;O.2;o1.I;o2.I;|Q9404;O.1;o1.I;|Q9405;|Q9469;|Q9406;|Q9407;|Q9471;O.2;o1.M;o2.M;|Q9408;|Q9409;|Q9473;O.1;o1.I;|Q9410;|Q9411;O.1;o1.I;|Q9475;O.1;o1.I;|Q9412;O.1;o1.I;|Q9413;O.1;o1.I;|Q9477;O.1;o1.I;|Q9414;O.1;o1.I;|Q9415;|Q9479;O.1;o1.I;|Q9416;|Q9464;|Q9478;O.1;o1.I;|Q9417;O.2;o1.M;o2.M;|Q9481;O.1;o1.I;|Q9418;|Q9458;O.1;o1.I;|Q9419;O.1;o1.O;|Q9483;O.1;o1.I;|Q9402;O.1;o1.I;|Q9472;O.1;o1.O;|Q9482;O.1;o1.I;|Q9462;|Q9420;O.1;o1.I;|Q9468;|Q9484;O.1;o1.E;|Q9421;|Q9485;O.1;o1.E;|Q9422;O.1;o1.O;|Q9423;|Q9487;O.1;o1.I;|Q9470;O.2;o1.M;o2.M;|Q9466;O.1;o1.I;|Q9424;O.1;o1.I;|Q9488;O.2;o1.I;o2.I;|Q9476;|Q9474;O.1;o1.I;|Q9425;|Q9489;O.1;o1.O;|Q9426;O.2;o1.M;o2.M;|Q9490;O.1;o1.I;|Q9456;O.1;o1.M;|Q9427;O.1;o1.O;|Q9491;O.1;o1.I;|Q9428;|Q9492;O.1;o1.I;|Q9429;|Q9493;O.3;o1.M;o2.M;o3.M;|Q9430;O.1;o1.I;|Q9494;O.1;o1.I;|Q9431;O.1;o1.I;|Q9495;O.1;o1.I;|Q9432;|Q9496;O.3;o1.M;o2.M;o3.M;|Q9433;O.1;o1.I;|Q9497;O.2;o1.M;o2.I;|Q9434;|Q9498;|Q9435;O.1;o1.I;|Q9499;|Q9454;O.1;o1.I;|Q9436;O.1;o1.I;|Q9500;|Q9437;O.2;o1.O;o2.I;|Q9438;|Q9439;O.2;o1.I;o2.I;|Q9440;O.3;o1.O;o2.O;o3.O;|Q9441;|Q9442;O.1;o1.I;|Q9443;O.1;o1.I;|Q9452;O.1;o1.I;|Q9444;O.1;o1.O;|Q9445;O.1;o1.I;|Q9446;O.1;o1.E;|Q9447;O.1;o1.O;|Q9448;O.1;o1.M;|Q9449;|Q9450;O.1;o1.M;|Q9451;|Q9453;|Q9455;|Q9480;O.1;o1.I;|Q9486;O.1;o1.E;|Q9457;O.1;o1.I;|Q9460;O.1;o1.I;|", -- [95]
	"|Q9523;O.1;o1.I;|Q9587;|Q9588;|Q9589;O.1;o1.I;|Q9590;O.1;o1.I;|Q9527;O.1;o1.I;|Q9591;O.1;o1.E;|Q9592;O.1;o1.E;|Q9594;O.2;o1.M;o2.M;|Q9531;O.1;o1.O;|Q9595;O.1;o1.M;|Q9596;O.1;o1.M;|Q9598;|Q9535;|Q9599;|Q9600;O.1;o1.O;|Q9537;O.1;o1.I;|Q9539;|Q9541;|Q9543;|Q9536;O.2;o1.M;o2.M;|Q9544;O.1;o1.O;|Q9545;O.1;o1.O;|Q9526;O.1;o1.O;|Q9546;O.1;o1.O;|Q9522;O.2;o1.M;o2.M;|Q9547;|Q9540;|Q9593;O.1;o1.E;|Q9532;|Q9530;O.2;o1.I;o2.I;|Q9548;O.1;o1.I;|Q9597;|Q9549;O.2;o1.I;o2.I;|Q9550;|Q9521;|Q9551;|Q9528;O.1;o1.E;|Q9542;|Q9552;|Q9553;|Q9554;|Q9555;|Q9556;O.1;o1.I;|Q9557;|Q9558;|Q9559;|Q9560;O.1;o1.I;|Q9561;|Q9562;O.1;o1.I;|Q9563;O.2;o1.I;o2.R;|Q9501;|Q9565;|Q9502;|Q9566;|Q9503;O.1;o1.I;|Q9567;O.1;o1.I;|Q9517;O.2;o1.I;o2.I;|Q9568;O.3;o1.M;o2.M;o3.M;|Q9504;O.1;o1.I;|Q9505;|Q9569;O.4;o1.M;o2.M;o3.M;o4.I;|Q9506;O.2;o1.I;o2.I;|Q9570;O.1;o1.I;|Q9507;|Q9571;|Q9508;O.1;o1.I;|Q9572;O.3;o1.I;o2.I;o3.I;|Q9509;|Q9573;O.2;o1.M;o2.M;|Q9510;O.1;o1.I;|Q9574;O.1;o1.I;|Q9511;|Q9575;O.3;o1.I;o2.I;o3.I;|Q9576;|Q9512;O.1;o1.I;|Q9513;O.3;o1.M;o2.M;o3.M;|Q9577;|Q9514;|Q9578;|Q9515;O.1;o1.M;|Q9579;O.1;o1.I;|Q9516;O.3;o1.M;o2.M;o3.M;|Q9580;O.1;o1.I;|Q9581;O.1;o1.I;|Q9564;|Q9518;O.4;o1.M;o2.M;o3.M;o4.M;|Q9582;O.1;o1.M;|Q9519;O.1;o1.I;|Q9583;O.1;o1.M;|Q9520;|Q9584;O.1;o1.I;|Q9533;|Q9585;O.1;o1.I;|Q9534;O.3;o1.M;o2.M;o3.M;|Q9538;O.1;o1.E;|Q9524;O.1;o1.O;|Q9525;O.1;o1.O;|Q9529;|Q9586;O.1;o1.O;|", -- [96]
	"|Q9651;|Q9652;O.1;o1.I;|Q9653;O.1;o1.I;|Q9654;O.2;o1.I;o2.I;|Q9655;O.2;o1.I;o2.I;|Q9657;O.2;o1.I;o2.I;|Q9658;O.2;o1.I;o2.I;|Q9659;O.2;o1.I;o2.I;|Q9660;O.2;o1.I;o2.I;|Q9661;O.2;o1.I;o2.I;|Q9662;O.1;o1.M;|Q9663;O.3;o1.O;o2.O;o3.O;|Q9664;O.4;o1.O;o2.O;o3.O;o4.O;|Q9601;|Q9665;O.4;o1.O;o2.O;o3.O;o4.O;|Q9666;O.2;o1.E;o2.M;|Q9603;|Q9667;O.1;o1.O;|Q9668;|Q9605;|Q9669;O.3;o1.M;o2.M;o3.M;|Q9670;O.1;o1.O;|Q9607;O.1;o1.E;|Q9671;|Q9672;|Q9609;|Q9674;O.1;o1.M;|Q9608;O.1;o1.E;|Q9673;|Q9611;|Q9675;|Q9612;|Q9676;|Q9606;|Q9613;O.1;o1.R;|Q9677;|Q9614;O.3;o1.I;o2.I;o3.I;|Q9678;O.1;o1.O;|Q9615;O.3;o1.I;o2.I;o3.I;|Q9679;|Q9616;|Q9680;O.1;o1.I;|Q9617;|Q9681;|Q9618;|Q9682;O.1;o1.I;|Q9619;O.1;o1.M;|Q9683;O.1;o1.M;|Q9620;|Q9684;O.1;o1.I;|Q9621;|Q9685;O.1;o1.O;|Q9622;|Q9686;O.1;o1.E;|Q9623;|Q9687;O.1;o1.I;|Q9624;O.1;o1.I;|Q9688;O.2;o1.M;o2.M;|Q9625;|Q9689;O.1;o1.M;|Q9626;|Q9690;|Q9627;|Q9691;|Q9628;O.1;o1.I;|Q9692;O.4;o1.I;o2.I;o3.I;o4.I;|Q9629;O.1;o1.O;|Q9693;|Q9630;|Q9694;O.1;o1.M;|Q9631;|Q9695;|Q9632;|Q9696;|Q9633;|Q9697;|Q9634;O.1;o1.M;|Q9698;|Q9635;O.1;o1.I;|Q9699;|Q9644;O.1;o1.I;|Q9636;O.1;o1.I;|Q9700;O.2;o1.E;o2.M;|Q9637;O.2;o1.I;o2.I;|Q9638;|Q9639;|Q9640;O.1;o1.I;|Q9641;O.1;o1.I;|Q9642;O.1;o1.I;|Q9643;O.1;o1.I;|Q9645;O.1;o1.E;|Q9646;O.1;o1.I;|Q9647;O.1;o1.M;|Q9648;O.4;o1.I;o2.I;o3.I;o4.I;|Q9649;O.1;o1.I;|Q9602;|Q9604;|Q9610;O.1;o1.I;|Q9650;|", -- [97]
	"|Q9715;O.1;o1.I;|Q9779;O.1;o1.I;|Q9716;O.1;o1.E;|Q9780;O.1;o1.I;|Q9717;O.1;o1.I;|Q9781;O.1;o1.M;|Q9718;O.1;o1.E;|Q9782;O.1;o1.I;|Q9719;O.1;o1.I;|Q9783;O.1;o1.M;|Q9720;O.4;o1.O;o2.O;o3.O;o4.O;|Q9784;O.1;o1.I;|Q9721;|Q9785;O.2;o1.O;o2.O;|Q9786;O.1;o1.E;|Q9723;O.4;o1.I;o2.I;o3.I;o4.I;|Q9787;O.1;o1.I;|Q9788;O.1;o1.I;|Q9725;O.2;o1.M;o2.O;|Q9789;O.1;o1.M;|Q9790;O.1;o1.I;|Q9727;O.2;o1.M;o2.M;|Q9791;O.1;o1.M;|Q9792;|Q9729;O.1;o1.E;|Q9793;|Q9794;|Q9731;O.1;o1.E;|Q9795;|Q9796;|Q9733;|Q9797;|Q9798;|Q9735;O.1;o1.I;|Q9799;O.1;o1.I;|Q9734;|Q9724;|Q9736;O.4;o1.I;o2.I;o3.I;o4.I;|Q9800;O.1;o1.I;|Q9722;|Q9737;O.1;o1.O;|Q9738;O.4;o1.O;o2.O;o3.O;o4.O;|Q9732;|Q9728;O.1;o1.I;|Q9739;O.1;o1.I;|Q9740;O.1;o1.O;|Q9741;O.1;o1.M;|Q9742;O.1;o1.I;|Q9743;O.1;o1.I;|Q9744;O.1;o1.I;|Q9745;|Q9746;O.2;o1.M;o2.M;|Q9747;O.4;o1.M;o2.M;o3.M;o4.M;|Q9748;O.1;o1.I;|Q9749;O.1;o1.O;|Q9750;|Q9751;|Q9752;O.1;o1.E;|Q9753;|Q9756;O.1;o1.O;|Q9757;|Q9758;|Q9759;O.1;o1.E;|Q9760;|Q9761;O.2;o1.M;o2.M;|Q9762;|Q9763;O.1;o1.M;|Q9764;|Q9701;O.1;o1.E;|Q9765;O.1;o1.I;|Q9702;O.1;o1.I;|Q9766;O.1;o1.I;|Q9703;O.1;o1.I;|Q9767;O.1;o1.I;|Q9704;|Q9768;O.1;o1.I;|Q9705;|Q9769;O.1;o1.I;|Q9706;|Q9770;O.1;o1.M;|Q9707;|Q9771;|Q9708;O.1;o1.I;|Q9772;O.1;o1.I;|Q9709;O.1;o1.I;|Q9773;O.1;o1.I;|Q9710;|Q9774;O.1;o1.I;|Q9711;O.1;o1.E;|Q9775;|Q9712;|Q9776;|Q9713;|Q9777;O.1;o1.I;|Q9714;O.1;o1.I;|Q9778;|Q9726;O.2;o1.M;o2.M;|Q9730;O.1;o1.M;|", -- [98]
	"|Q9843;O.1;o1.I;|Q9844;O.1;o1.O;|Q9845;O.1;o1.M;|Q9846;O.1;o1.I;|Q9847;O.1;o1.O;|Q9848;O.2;o1.I;o2.I;|Q9849;O.1;o1.M;|Q9850;O.1;o1.M;|Q9851;O.1;o1.I;|Q9852;O.1;o1.I;|Q9853;O.1;o1.I;|Q9854;O.1;o1.M;|Q9855;O.1;o1.M;|Q9856;O.1;o1.I;|Q9857;O.1;o1.M;|Q9858;O.1;o1.M;|Q9859;O.1;o1.I;|Q9860;|Q9861;|Q9862;O.1;o1.M;|Q9863;O.1;o1.I;|Q9864;|Q9801;O.2;o1.I;o2.I;|Q9865;O.2;o1.M;o2.M;|Q9866;|Q9803;O.1;o1.I;|Q9867;O.1;o1.I;|Q9802;O.1;o1.I;|Q9804;O.1;o1.M;|Q9868;O.1;o1.E;|Q9805;O.4;o1.O;o2.O;o3.O;o4.O;|Q9869;|Q9806;O.1;o1.I;|Q9870;|Q9807;O.1;o1.I;|Q9871;O.1;o1.R;|Q9808;O.1;o1.I;|Q9872;O.1;o1.R;|Q9809;O.1;o1.I;|Q9873;O.1;o1.M;|Q9810;O.2;o1.M;o2.M;|Q9874;O.1;o1.O;|Q9811;|Q9875;|Q9876;|Q9813;|Q9877;|Q9814;O.1;o1.I;|Q9878;O.2;o1.M;o2.M;|Q9815;O.1;o1.I;|Q9879;O.1;o1.E;|Q9816;O.1;o1.O;|Q9880;O.1;o1.I;|Q9817;O.1;o1.M;|Q9881;O.1;o1.I;|Q9818;|Q9882;O.1;o1.I;|Q9819;O.1;o1.M;|Q9883;O.1;o1.I;|Q9820;O.1;o1.I;|Q9884;|Q9821;O.1;o1.I;|Q9885;|Q9822;O.1;o1.I;|Q9886;|Q9823;O.3;o1.M;o2.M;o3.M;|Q9887;|Q9824;O.2;o1.O;o2.O;|Q9888;|Q9825;O.1;o1.I;|Q9889;O.2;o1.E;o2.M;|Q9826;|Q9890;|Q9827;|Q9891;|Q9828;|Q9892;O.1;o1.I;|Q9829;|Q9893;O.1;o1.I;|Q9830;O.1;o1.I;|Q9894;O.1;o1.M;|Q9831;O.1;o1.I;|Q9895;O.1;o1.M;|Q9832;O.2;o1.I;o2.I;|Q9896;O.1;o1.I;|Q9833;O.1;o1.M;|Q9897;|Q9834;O.1;o1.I;|Q9898;O.1;o1.I;|Q9835;O.2;o1.M;o2.M;|Q9899;O.1;o1.M;|Q9836;O.1;o1.E;|Q9900;O.1;o1.M;|Q9837;|Q9838;|Q9839;O.2;o1.M;o2.M;|Q9812;|Q9840;|Q9841;O.1;o1.M;|Q9842;O.1;o1.I;|", -- [99]
	"|Q9907;O.2;o1.M;o2.M;|Q9971;O.1;o1.O;|Q9908;O.1;o1.I;|Q9909;O.1;o1.I;|Q9973;O.1;o1.E;|Q9910;O.3;o1.O;o2.O;o3.O;|Q9911;|Q9975;O.1;o1.O;|Q9912;|Q9976;O.1;o1.O;|Q9977;O.1;o1.E;|Q9914;O.1;o1.I;|Q9978;|Q9915;O.1;o1.I;|Q9979;|Q9916;O.1;o1.I;|Q9980;O.2;o1.M;o2.O;|Q9981;O.2;o1.M;o2.O;|Q9918;O.1;o1.O;|Q9982;|Q9919;|Q9983;|Q9920;|Q9984;|Q9985;|Q9986;O.3;o1.M;o2.M;o3.M;|Q9923;O.1;o1.O;|Q9987;O.3;o1.M;o2.M;o3.M;|Q9988;O.1;o1.I;|Q9925;O.1;o1.M;|Q9989;O.1;o1.I;|Q9990;O.1;o1.I;|Q9927;O.1;o1.O;|Q9991;O.1;o1.E;|Q9922;O.2;o1.M;o2.M;|Q9926;|Q9921;O.2;o1.M;o2.M;|Q9928;O.1;o1.I;|Q9992;O.1;o1.I;|Q9924;O.1;o1.O;|Q9929;|Q9993;O.1;o1.I;|Q9930;|Q9994;|Q9970;O.1;o1.E;|Q9931;O.1;o1.O;|Q9995;|Q9932;O.1;o1.O;|Q9996;O.3;o1.M;o2.M;o3.M;|Q9997;O.3;o1.M;o2.M;o3.M;|Q9933;|Q9934;|Q9998;O.1;o1.M;|Q9935;O.2;o1.M;o2.M;|Q9999;O.3;o1.M;o2.M;o3.M;|Q10000;O.1;o1.M;|Q9937;O.1;o1.M;|Q9938;O.1;o1.M;|Q9968;O.2;o1.I;o2.I;|Q9939;O.3;o1.M;o2.M;o3.M;|Q9940;O.3;o1.M;o2.M;o3.M;|Q9941;|Q9942;|Q9943;|Q9945;O.2;o1.M;o2.M;|Q9946;O.1;o1.I;|Q9947;|Q9966;O.1;o1.I;|Q9948;O.1;o1.O;|Q9949;O.3;o1.I;o2.I;o3.I;|Q9950;O.3;o1.I;o2.I;o3.I;|Q9951;O.1;o1.M;|Q9952;|Q9953;|Q9954;|Q9955;O.1;o1.O;|Q9964;|Q9956;O.1;o1.I;|Q9957;|Q9958;O.1;o1.O;|Q9959;O.1;o1.O;|Q9960;|Q9961;|Q9962;O.1;o1.E;|Q9963;|Q9901;O.1;o1.M;|Q9965;O.1;o1.I;|Q9902;O.1;o1.M;|Q9944;|Q9903;O.1;o1.M;|Q9967;O.1;o1.E;|Q9904;O.1;o1.M;|Q9936;O.2;o1.M;o2.M;|Q9905;O.1;o1.M;|Q9969;O.2;o1.I;o2.I;|Q9906;O.2;o1.M;o2.M;|Q9913;|Q9917;O.1;o1.I;|Q9972;O.1;o1.E;|Q9974;O.2;o1.I;o2.I;|", -- [100]
	"|Q10035;O.1;o1.I;|Q10099;O.1;o1.M;|Q10036;O.1;o1.I;|Q10100;O.1;o1.M;|Q10037;O.1;o1.I;|Q10038;|Q10039;|Q10040;O.3;o1.O;o2.O;o3.O;|Q10041;O.3;o1.O;o2.O;o3.O;|Q10042;O.3;o1.M;o2.M;o3.M;|Q10043;O.3;o1.M;o2.M;o3.M;|Q10044;O.1;o1.E;|Q10045;O.4;o1.I;o2.I;o3.I;o4.I;|Q10046;|Q10047;O.1;o1.O;|Q10048;O.1;o1.I;|Q10049;O.1;o1.I;|Q10050;O.3;o1.M;o2.M;o3.M;|Q10051;O.1;o1.E;|Q10052;O.1;o1.E;|Q10053;O.1;o1.M;|Q10054;O.4;o1.M;o2.M;o3.M;o4.M;|Q10055;O.2;o1.I;o2.I;|Q10056;O.1;o1.O;|Q10057;O.2;o1.M;o2.M;|Q10058;O.1;o1.I;|Q10059;O.1;o1.M;|Q10060;O.4;o1.M;o2.M;o3.M;o4.M;|Q10061;O.3;o1.M;o2.M;o3.M;|Q10062;O.2;o1.M;o2.M;|Q10063;|Q10064;|Q10001;O.1;o1.I;|Q10065;O.1;o1.M;|Q10002;O.1;o1.M;|Q10066;O.1;o1.M;|Q10003;O.1;o1.M;|Q10067;O.1;o1.M;|Q10004;O.1;o1.E;|Q10068;|Q10005;|Q10069;|Q10006;|Q10070;|Q10007;O.2;o1.M;o2.M;|Q10071;|Q10008;O.2;o1.M;o2.M;|Q10072;|Q10009;O.3;o1.I;o2.I;o3.I;|Q10073;|Q10010;|Q10074;O.1;o1.I;|Q10011;O.2;o1.O;o2.O;|Q10075;O.1;o1.I;|Q10012;|Q10076;O.1;o1.I;|Q10013;|Q10077;O.1;o1.I;|Q10014;O.4;o1.nil;o2.nil;o3.nil;o4.nil;|Q10078;O.4;o1.O;o2.O;o3.O;o4.O;|Q10015;O.4;o1.I;o2.I;o3.I;o4.I;|Q10079;O.1;o1.M;|Q10093;|Q10016;O.1;o1.I;|Q10080;O.3;o1.M;o2.M;o3.M;|Q10017;O.1;o1.I;|Q10081;|Q10082;O.1;o1.M;|Q10018;O.1;o1.I;|Q10019;O.1;o1.I;|Q10083;O.1;o1.I;|Q10092;O.1;o1.M;|Q10020;O.1;o1.I;|Q10084;O.1;o1.M;|Q10021;O.3;o1.O;o2.O;o3.O;|Q10085;O.4;o1.O;o2.O;o3.O;o4.O;|Q10022;O.1;o1.I;|Q10023;O.1;o1.I;|Q10087;O.2;o1.O;o2.O;|Q10024;O.1;o1.I;|Q10088;O.1;o1.M;|Q10025;O.1;o1.I;|Q10089;O.3;o1.M;o2.M;o3.M;|Q10026;O.1;o1.M;|Q10090;O.1;o1.I;|Q10027;O.1;o1.M;|Q10091;O.1;o1.I;|Q10086;O.2;o1.I;o2.I;|Q10028;O.1;o1.I;|Q10029;|Q10030;O.1;o1.I;|Q10094;|Q10031;O.2;o1.M;o2.M;|Q10095;O.1;o1.M;|Q10032;O.1;o1.I;|Q10096;O.2;o1.M;o2.M;|Q10033;O.1;o1.M;|Q10097;O.2;o1.M;o2.O;|Q10034;O.1;o1.M;|Q10098;O.3;o1.I;o2.I;o3.I;|", -- [101]
	"|Q10163;O.3;o1.M;o2.M;o3.O;|Q10164;O.1;o1.M;|Q10101;|Q10102;|Q10166;|Q10103;|Q10167;O.1;o1.M;|Q10104;|Q10168;O.2;o1.M;o2.I;|Q10105;|Q10169;|Q10106;O.3;o1.O;o2.O;o3.O;|Q10170;|Q10107;O.1;o1.E;|Q10171;|Q10108;O.1;o1.E;|Q10172;O.1;o1.E;|Q10109;O.1;o1.I;|Q10173;O.1;o1.I;|Q10110;O.3;o1.O;o2.O;o3.O;|Q10174;|Q10111;O.2;o1.O;o2.I;|Q10175;|Q10112;O.1;o1.I;|Q10176;O.1;o1.M;|Q10113;|Q10177;|Q10114;|Q10178;|Q10115;O.2;o1.M;o2.M;|Q10179;|Q10116;O.1;o1.I;|Q10180;|Q10117;O.1;o1.I;|Q10181;|Q10118;O.2;o1.M;o2.M;|Q10182;O.1;o1.O;|Q10119;|Q10183;|Q10120;|Q10184;O.1;o1.M;|Q10121;|Q10185;O.2;o1.M;o2.M;|Q10122;|Q10186;O.1;o1.I;|Q10123;O.3;o1.M;o2.M;o3.M;|Q10187;|Q10124;|Q10188;O.1;o1.I;|Q10125;O.4;o1.O;o2.O;o3.O;o4.O;|Q10189;O.1;o1.I;|Q10126;|Q10190;O.1;o1.O;|Q10127;O.3;o1.M;o2.M;o3.M;|Q10191;O.1;o1.E;|Q10128;|Q10192;O.3;o1.I;o2.I;o3.I;|Q10129;O.2;o1.O;o2.O;|Q10193;O.3;o1.M;o2.M;o3.M;|Q10130;O.3;o1.M;o2.M;o3.M;|Q10194;|Q10131;O.1;o1.I;|Q10195;O.1;o1.I;|Q10196;O.1;o1.I;|Q10133;O.1;o1.I;|Q10197;O.3;o1.I;o2.I;o3.I;|Q10134;|Q10198;O.1;o1.E;|Q10135;|Q10199;O.1;o1.I;|Q10136;O.1;o1.M;|Q10200;|Q10137;O.2;o1.O;o2.I;|Q10138;O.3;o1.M;o2.M;o3.I;|Q10139;O.1;o1.M;|Q10140;|Q10141;|Q10142;O.3;o1.M;o2.M;o3.M;|Q10143;|Q10144;O.2;o1.O;o2.O;|Q10145;O.3;o1.M;o2.M;o3.M;|Q10146;O.2;o1.O;o2.O;|Q10147;O.1;o1.I;|Q10148;|Q10149;O.1;o1.M;|Q10150;|Q10165;O.1;o1.I;|Q10151;|Q10152;O.3;o1.M;o2.M;o3.M;|Q10153;|Q10154;O.1;o1.I;|Q10155;O.2;o1.O;o2.I;|Q10156;O.3;o1.M;o2.M;o3.I;|Q10157;O.1;o1.M;|Q10158;O.1;o1.O;|Q10159;O.2;o1.M;o2.M;|Q10132;O.1;o1.M;|Q10160;|Q10161;O.1;o1.I;|Q10162;O.3;o1.M;o2.M;o3.O;|", -- [102]
	"|Q10227;|Q10291;|Q10228;|Q10292;O.1;o1.I;|Q10229;|Q10293;O.1;o1.I;|Q10230;O.1;o1.I;|Q10294;O.1;o1.I;|Q10231;O.1;o1.E;|Q10295;O.1;o1.I;|Q10232;O.2;o1.M;o2.M;|Q10296;|Q10233;O.2;o1.O;o2.O;|Q10297;O.1;o1.E;|Q10234;O.1;o1.I;|Q10298;|Q10235;O.1;o1.I;|Q10299;O.2;o1.O;o2.I;|Q10236;O.1;o1.I;|Q10300;O.1;o1.I;|Q10237;|Q10238;O.3;o1.O;o2.O;o3.O;|Q10239;O.1;o1.I;|Q10240;O.3;o1.O;o2.O;o3.O;|Q10241;O.2;o1.M;o2.M;|Q10242;|Q10243;|Q10244;|Q10245;|Q10246;O.2;o1.M;o2.M;|Q10247;|Q10248;O.1;o1.O;|Q10249;|Q10250;O.1;o1.M;|Q10251;|Q10253;O.1;o1.I;|Q10254;|Q10255;O.1;o1.O;|Q10256;O.1;o1.O;|Q10257;O.1;o1.I;|Q10258;|Q10259;O.1;o1.M;|Q10260;|Q10261;O.1;o1.I;|Q10262;O.1;o1.I;|Q10263;|Q10264;|Q10201;O.1;o1.O;|Q10265;O.1;o1.I;|Q10202;|Q10266;|Q10203;O.4;o1.O;o2.O;o3.O;o4.O;|Q10267;O.1;o1.I;|Q10268;|Q10205;O.1;o1.M;|Q10269;O.1;o1.E;|Q10206;O.1;o1.I;|Q10270;O.1;o1.I;|Q10207;|Q10271;O.1;o1.I;|Q10208;O.2;o1.O;o2.O;|Q10272;O.1;o1.I;|Q10209;O.1;o1.I;|Q10273;O.1;o1.M;|Q10210;|Q10274;O.1;o1.O;|Q10211;O.1;o1.E;|Q10275;O.1;o1.E;|Q10284;O.1;o1.O;|Q10212;|Q10276;O.1;o1.I;|Q10213;|Q10277;O.1;o1.E;|Q10214;O.1;o1.M;|Q10278;O.1;o1.I;|Q10279;|Q10216;O.4;o1.M;o2.M;o3.M;o4.M;|Q10280;|Q10217;O.1;o1.I;|Q10281;|Q10282;|Q10218;O.1;o1.E;|Q10219;|Q10283;O.1;o1.O;|Q10220;O.3;o1.M;o2.M;o3.M;|Q10221;O.1;o1.M;|Q10285;|Q10204;O.1;o1.E;|Q10222;O.1;o1.M;|Q10286;|Q10223;O.1;o1.M;|Q10287;|Q10224;O.1;o1.I;|Q10288;|Q10225;|Q10289;|Q10252;O.3;o1.I;o2.I;o3.I;|Q10226;O.1;o1.I;|Q10290;O.1;o1.I;|", -- [103]
	"|Q10355;O.2;o1.I;o2.I;|Q10356;O.1;o1.I;|Q10357;O.1;o1.I;|Q10358;O.1;o1.I;|Q10359;O.1;o1.I;|Q10360;O.1;o1.I;|Q10361;O.1;o1.I;|Q10362;O.1;o1.I;|Q10363;O.1;o1.I;|Q10364;|Q10301;O.1;o1.I;|Q10365;O.2;o1.O;o2.I;|Q10302;O.1;o1.M;|Q10366;|Q10303;O.1;o1.M;|Q10367;O.1;o1.I;|Q10304;|Q10368;O.3;o1.O;o2.O;o3.O;|Q10305;O.1;o1.O;|Q10369;O.1;o1.M;|Q10306;O.1;o1.O;|Q10370;|Q10307;O.1;o1.O;|Q10371;|Q10308;O.1;o1.I;|Q10372;|Q10309;O.1;o1.I;|Q10373;|Q10310;O.1;o1.E;|Q10374;|Q10311;|Q10375;O.1;o1.I;|Q10312;O.1;o1.I;|Q10376;|Q10313;O.4;o1.O;o2.O;o3.O;o4.O;|Q10377;|Q10314;O.1;o1.I;|Q10378;|Q10315;O.1;o1.M;|Q10379;|Q10316;|Q10380;O.3;o1.M;o2.M;o3.M;|Q10317;|Q10381;|Q10318;O.1;o1.M;|Q10382;|Q10319;O.1;o1.I;|Q10383;|Q10320;O.1;o1.M;|Q10384;O.1;o1.I;|Q10321;O.2;o1.O;o2.I;|Q10385;O.1;o1.I;|Q10322;O.2;o1.O;o2.I;|Q10386;|Q10323;O.2;o1.O;o2.I;|Q10387;|Q10388;|Q10389;O.1;o1.M;|Q10326;O.1;o1.I;|Q10390;O.2;o1.M;o2.I;|Q10327;O.1;o1.I;|Q10391;O.1;o1.M;|Q10328;O.2;o1.I;o2.I;|Q10392;O.3;o1.M;o2.O;o3.I;|Q10329;O.2;o1.O;o2.I;|Q10393;|Q10330;O.2;o1.O;o2.I;|Q10394;O.2;o1.M;o2.M;|Q10331;O.1;o1.I;|Q10395;|Q10332;O.1;o1.M;|Q10396;O.1;o1.M;|Q10333;|Q10397;O.3;o1.M;o2.O;o3.I;|Q10334;|Q10398;|Q10335;O.3;o1.O;o2.O;o3.O;|Q10399;O.1;o1.M;|Q10336;O.2;o1.M;o2.M;|Q10400;O.1;o1.M;|Q10325;O.1;o1.I;|Q10338;O.2;o1.O;o2.I;|Q10339;O.4;o1.M;o2.M;o3.M;o4.M;|Q10340;|Q10341;O.3;o1.M;o2.M;o3.M;|Q10342;O.1;o1.I;|Q10343;O.1;o1.I;|Q10344;|Q10345;O.1;o1.O;|Q10346;O.3;o1.M;o2.M;o3.O;|Q10347;O.3;o1.M;o2.M;o3.O;|Q10348;O.1;o1.I;|Q10337;O.1;o1.E;|Q10349;|Q10350;|Q10351;O.1;o1.O;|Q10324;O.1;o1.I;|Q10352;O.1;o1.I;|Q10353;O.1;o1.M;|Q10354;O.1;o1.I;|", -- [104]
	"|Q10419;O.1;o1.I;|Q10483;O.1;o1.I;|Q10420;O.1;o1.I;|Q10484;O.1;o1.I;|Q10422;O.1;o1.O;|Q10486;O.1;o1.M;|Q10423;|Q10487;O.1;o1.I;|Q10424;O.1;o1.I;|Q10488;O.1;o1.O;|Q10425;O.1;o1.E;|Q10489;O.1;o1.I;|Q10426;O.1;o1.O;|Q10490;|Q10427;O.1;o1.O;|Q10491;|Q10428;|Q10492;O.2;o1.I;o2.I;|Q10429;O.1;o1.I;|Q10493;O.2;o1.I;o2.I;|Q10430;|Q10494;O.2;o1.I;o2.I;|Q10431;|Q10495;O.2;o1.I;o2.I;|Q10432;O.1;o1.I;|Q10496;O.3;o1.I;o2.I;o3.I;|Q10433;O.1;o1.I;|Q10497;O.3;o1.I;o2.I;o3.I;|Q10434;|Q10498;O.2;o1.I;o2.I;|Q10435;O.1;o1.I;|Q10499;O.2;o1.I;o2.I;|Q10436;O.1;o1.M;|Q10500;|Q10437;O.1;o1.I;|Q10438;O.1;o1.O;|Q10439;O.2;o1.M;o2.O;|Q10440;|Q10441;|Q10442;|Q10443;|Q10444;|Q10445;T.R;O.2;o1.I;o2.I;|Q10446;O.1;o1.O;|Q10417;O.1;o1.I;|Q10447;O.1;o1.O;|Q10448;|Q10449;|Q10450;O.1;o1.I;|Q10451;O.1;o1.E;|Q10452;|Q10453;|Q10454;|Q10455;O.1;o1.M;|Q10456;O.1;o1.I;|Q10457;O.1;o1.O;|Q10458;O.2;o1.O;o2.O;|Q10459;O.1;o1.R;|Q10460;|Q10461;|Q10462;|Q10413;|Q10463;|Q10464;O.1;o1.R;|Q10401;O.1;o1.M;|Q10465;O.1;o1.R;|Q10402;|Q10466;O.1;o1.R;|Q10403;|Q10467;O.1;o1.R;|Q10404;O.3;o1.M;o2.M;o3.M;|Q10468;O.1;o1.R;|Q10485;O.1;o1.I;|Q10405;O.1;o1.I;|Q10469;O.1;o1.R;|Q10470;O.1;o1.R;|Q10421;O.1;o1.I;|Q10407;O.2;o1.I;o2.I;|Q10471;O.1;o1.R;|Q10408;O.1;o1.M;|Q10472;O.1;o1.R;|Q10409;O.1;o1.E;|Q10473;O.1;o1.R;|Q10410;|Q10474;O.1;o1.R;|Q10411;O.2;o1.M;o2.M;|Q10475;O.1;o1.R;|Q10406;O.1;o1.E;|Q10412;O.1;o1.I;|Q10476;O.1;o1.I;|Q10477;O.1;o1.I;|Q10414;O.1;o1.I;|Q10478;O.1;o1.I;|Q10415;O.1;o1.I;|Q10479;O.1;o1.I;|Q10416;O.1;o1.I;|Q10480;O.1;o1.O;|Q10481;O.1;o1.O;|Q10418;O.1;o1.M;|Q10482;O.1;o1.M;|", -- [105]
	"|Q10547;O.1;o1.I;|Q10548;O.1;o1.I;|Q10549;O.1;o1.I;|Q10550;|Q10551;|Q10552;|Q10553;|Q10554;|Q10555;O.1;o1.I;|Q10556;O.1;o1.O;|Q10557;O.1;o1.E;|Q10558;O.1;o1.R;|Q10559;O.1;o1.R;|Q10560;O.1;o1.R;|Q10561;O.1;o1.R;|Q10563;O.1;o1.O;|Q10564;O.1;o1.O;|Q10501;|Q10565;O.1;o1.I;|Q10502;O.1;o1.O;|Q10566;O.4;o1.O;o2.O;o3.O;o4.O;|Q10503;O.2;o1.O;o2.M;|Q10567;O.1;o1.I;|Q10504;O.1;o1.O;|Q10568;O.1;o1.I;|Q10505;O.1;o1.O;|Q10569;O.3;o1.I;o2.I;o3.I;|Q10506;O.1;o1.O;|Q10570;O.1;o1.I;|Q10507;O.3;o1.M;o2.I;o3.I;|Q10571;O.1;o1.I;|Q10508;O.2;o1.I;o2.I;|Q10572;O.2;o1.I;o2.I;|Q10509;|Q10573;|Q10510;O.1;o1.I;|Q10574;O.4;o1.I;o2.I;o3.I;o4.I;|Q10511;O.1;o1.I;|Q10575;|Q10512;O.1;o1.O;|Q10576;O.1;o1.I;|Q10513;|Q10577;O.1;o1.E;|Q10514;O.1;o1.I;|Q10578;O.1;o1.I;|Q10515;O.1;o1.O;|Q10579;|Q10516;O.2;o1.I;o2.I;|Q10580;|Q10517;O.1;o1.M;|Q10581;|Q10518;O.2;o1.I;o2.I;|Q10582;O.2;o1.M;o2.M;|Q10519;O.1;o1.E;|Q10583;O.1;o1.I;|Q10520;|Q10584;O.1;o1.O;|Q10521;|Q10585;O.2;o1.O;o2.I;|Q10522;O.1;o1.I;|Q10586;O.2;o1.M;o2.I;|Q10523;|Q10587;O.1;o1.I;|Q10524;O.3;o1.I;o2.I;o3.I;|Q10588;O.1;o1.M;|Q10525;O.1;o1.E;|Q10589;O.1;o1.I;|Q10526;O.1;o1.I;|Q10590;O.1;o1.I;|Q10527;|Q10591;|Q10528;O.1;o1.I;|Q10592;|Q10529;|Q10593;O.1;o1.I;|Q10530;|Q10594;O.1;o1.E;|Q10531;O.4;o1.O;o2.O;o3.O;o4.O;|Q10595;O.1;o1.M;|Q10532;O.1;o1.I;|Q10596;O.1;o1.O;|Q10533;O.1;o1.I;|Q10597;O.2;o1.I;o2.I;|Q10534;|Q10598;O.1;o1.O;|Q10535;O.1;o1.I;|Q10599;|Q10536;O.1;o1.I;|Q10600;O.2;o1.M;o2.M;|Q10537;O.1;o1.I;|Q10538;O.1;o1.I;|Q10539;|Q10540;O.1;o1.I;|Q10541;|Q10542;O.2;o1.I;o2.I;|Q10543;O.2;o1.M;o2.M;|Q10544;O.2;o1.O;o2.O;|Q10545;O.1;o1.O;|Q10546;|Q10562;O.1;o1.M;|", -- [106]
	"|Q10611;O.1;o1.I;|Q10675;O.1;o1.I;|Q10612;O.1;o1.O;|Q10676;|Q10613;O.1;o1.O;|Q10677;O.1;o1.I;|Q10614;|Q10678;O.1;o1.I;|Q10615;|Q10679;O.1;o1.I;|Q10616;|Q10680;|Q10617;O.1;o1.I;|Q10681;|Q10618;O.1;o1.I;|Q10682;O.1;o1.E;|Q10619;O.3;o1.M;o2.M;o3.M;|Q10683;O.1;o1.I;|Q10620;O.1;o1.M;|Q10684;O.1;o1.I;|Q10621;|Q10685;O.4;o1.I;o2.I;o3.I;o4.I;|Q10622;O.1;o1.M;|Q10686;|Q10623;|Q10687;O.1;o1.I;|Q10624;O.1;o1.I;|Q10688;O.1;o1.O;|Q10625;O.1;o1.M;|Q10689;|Q10626;O.2;o1.I;o2.I;|Q10690;O.1;o1.M;|Q10627;O.2;o1.I;o2.I;|Q10691;|Q10628;|Q10692;O.4;o1.M;o2.M;o3.M;o4.M;|Q10629;O.1;o1.I;|Q10693;O.1;o1.I;|Q10630;O.1;o1.M;|Q10694;O.1;o1.I;|Q10631;O.1;o1.nil;|Q10695;O.1;o1.I;|Q10632;O.1;o1.I;|Q10696;O.1;o1.I;|Q10633;|Q10697;O.1;o1.I;|Q10634;O.1;o1.I;|Q10698;O.1;o1.I;|Q10635;O.1;o1.I;|Q10699;O.1;o1.I;|Q10636;O.1;o1.I;|Q10700;O.1;o1.I;|Q10637;O.1;o1.O;|Q10638;|Q10639;O.1;o1.M;|Q10640;|Q10641;O.1;o1.M;|Q10642;O.1;o1.I;|Q10643;O.1;o1.M;|Q10644;|Q10646;O.1;o1.E;|Q10647;O.1;o1.I;|Q10648;O.1;o1.I;|Q10649;O.1;o1.I;|Q10650;|Q10651;O.4;o1.M;o2.M;o3.M;o4.M;|Q10652;|Q10653;O.1;o1.I;|Q10654;O.1;o1.I;|Q10655;O.1;o1.I;|Q10656;O.1;o1.I;|Q10657;O.2;o1.O;o2.I;|Q10658;O.1;o1.I;|Q10659;O.1;o1.I;|Q10604;O.1;o1.I;|Q10660;O.1;o1.I;|Q10661;O.1;o1.I;|Q10662;|Q10645;O.1;o1.M;|Q10663;|Q10664;O.3;o1.I;o2.I;o3.I;|Q10601;O.1;o1.I;|Q10665;O.1;o1.I;|Q10602;O.2;o1.O;o2.I;|Q10666;O.1;o1.I;|Q10603;O.2;o1.M;o2.I;|Q10667;O.1;o1.I;|Q10668;O.1;o1.M;|Q10605;|Q10669;O.1;o1.M;|Q10606;O.1;o1.I;|Q10670;O.1;o1.I;|Q10607;O.4;o1.O;o2.O;o3.O;o4.O;|Q10671;O.1;o1.I;|Q10608;O.1;o1.M;|Q10672;O.1;o1.O;|Q10609;O.3;o1.I;o2.I;o3.I;|Q10673;O.1;o1.I;|Q10610;|Q10674;O.1;o1.O;|", -- [107]
	"|Q10739;O.1;o1.R;|Q10740;O.1;o1.R;|Q10741;O.1;o1.R;|Q10742;O.1;o1.M;|Q10743;|Q10744;|Q10745;|Q10746;O.1;o1.R;|Q10747;O.1;o1.I;|Q10748;O.1;o1.M;|Q10749;|Q10750;O.1;o1.E;|Q10751;O.3;o1.M;o2.M;o3.M;|Q10752;|Q10753;O.3;o1.M;o2.M;o3.M;|Q10754;|Q10755;|Q10756;|Q10757;O.3;o1.I;o2.I;o3.I;|Q10758;O.1;o1.I;|Q10759;|Q10760;O.3;o1.I;o2.I;o3.I;|Q10761;|Q10762;|Q10763;O.3;o1.I;o2.I;o3.I;|Q10764;O.1;o1.I;|Q10701;O.1;o1.M;|Q10765;O.3;o1.M;o2.M;o3.I;|Q10702;O.1;o1.M;|Q10766;|Q10703;O.1;o1.M;|Q10767;|Q10704;O.2;o1.I;o2.I;|Q10768;O.1;o1.I;|Q10705;|Q10769;O.2;o1.O;o2.I;|Q10706;|Q10770;O.1;o1.M;|Q10707;O.1;o1.I;|Q10771;O.3;o1.O;o2.O;o3.O;|Q10772;O.1;o1.E;|Q10708;|Q10709;|Q10773;O.3;o1.M;o2.M;o3.M;|Q10710;O.1;o1.E;|Q10774;O.3;o1.M;o2.M;o3.I;|Q10711;O.1;o1.E;|Q10775;O.1;o1.I;|Q10712;O.2;o1.E;o2.I;|Q10776;O.2;o1.O;o2.I;|Q10713;O.1;o1.M;|Q10777;O.1;o1.I;|Q10714;O.1;o1.O;|Q10778;O.2;o1.I;o2.I;|Q10715;O.1;o1.I;|Q10779;|Q10716;O.1;o1.E;|Q10780;O.1;o1.I;|Q10717;O.1;o1.I;|Q10781;O.1;o1.E;|Q10718;|Q10782;O.1;o1.I;|Q10719;|Q10783;|Q10720;O.3;o1.O;o2.O;o3.O;|Q10784;O.2;o1.M;o2.M;|Q10721;O.1;o1.I;|Q10785;|Q10722;O.1;o1.E;|Q10786;O.2;o1.M;o2.M;|Q10723;O.1;o1.O;|Q10787;O.1;o1.I;|Q10724;O.1;o1.O;|Q10788;|Q10789;|Q10726;O.1;o1.R;|Q10790;|Q10727;O.1;o1.R;|Q10791;O.1;o1.O;|Q10728;O.1;o1.R;|Q10792;O.4;o1.O;o2.O;o3.O;o4.O;|Q10729;|Q10793;|Q10730;|Q10794;|Q10731;|Q10795;O.1;o1.M;|Q10725;O.1;o1.R;|Q10732;|Q10796;O.2;o1.M;o2.M;|Q10733;O.1;o1.R;|Q10797;|Q10734;O.1;o1.R;|Q10798;|Q10735;O.1;o1.R;|Q10799;O.1;o1.I;|Q10736;O.1;o1.R;|Q10800;O.1;o1.I;|Q10737;|Q10738;O.1;o1.R;|", -- [108]
	"|Q10803;O.2;o1.M;o2.M;|Q10867;O.1;o1.I;|Q10804;O.1;o1.O;|Q10868;O.2;o1.M;o2.M;|Q10869;O.2;o1.M;o2.M;|Q10806;O.1;o1.M;|Q10870;|Q10807;O.3;o1.M;o2.M;o3.M;|Q10871;|Q10808;O.1;o1.O;|Q10872;O.2;o1.O;o2.M;|Q10809;O.1;o1.I;|Q10873;O.1;o1.O;|Q10810;|Q10874;O.4;o1.O;o2.O;o3.O;o4.O;|Q10811;|Q10875;|Q10812;|Q10876;O.1;o1.I;|Q10813;O.1;o1.O;|Q10877;O.1;o1.I;|Q10814;O.1;o1.E;|Q10878;O.3;o1.M;o2.M;o3.M;|Q10815;|Q10879;O.1;o1.E;|Q10816;O.3;o1.M;o2.M;o3.M;|Q10880;|Q10817;O.3;o1.M;o2.M;o3.M;|Q10881;O.3;o1.I;o2.I;o3.I;|Q10818;|Q10882;O.1;o1.M;|Q10819;|Q10883;|Q10820;O.2;o1.M;o2.M;|Q10884;O.1;o1.I;|Q10821;O.1;o1.M;|Q10885;O.2;o1.I;o2.I;|Q10822;O.1;o1.I;|Q10886;O.1;o1.E;|Q10823;O.1;o1.I;|Q10887;|Q10824;O.1;o1.I;|Q10888;O.1;o1.M;|Q10825;|Q10889;|Q10826;O.1;o1.I;|Q10890;|Q10827;O.1;o1.I;|Q10891;|Q10828;O.1;o1.I;|Q10892;|Q10829;|Q10893;O.1;o1.M;|Q10830;O.1;o1.O;|Q10894;|Q10831;O.1;o1.I;|Q10895;O.4;o1.O;o2.O;o3.O;o4.O;|Q10832;O.1;o1.I;|Q10896;O.1;o1.M;|Q10833;O.1;o1.O;|Q10897;O.4;o1.I;o2.I;o3.I;o4.I;|Q10834;O.1;o1.I;|Q10898;O.1;o1.E;|Q10835;|Q10899;O.1;o1.I;|Q10836;O.1;o1.M;|Q10900;|Q10837;O.1;o1.I;|Q10838;O.1;o1.I;|Q10839;O.1;o1.E;|Q10840;O.2;o1.M;o2.M;|Q10841;|Q10842;|Q10843;O.1;o1.M;|Q10844;|Q10845;O.1;o1.M;|Q10846;O.1;o1.M;|Q10847;O.2;o1.I;o2.I;|Q10848;O.3;o1.M;o2.M;o3.M;|Q10849;|Q10850;O.1;o1.I;|Q10851;O.1;o1.I;|Q10852;O.1;o1.O;|Q10853;O.1;o1.I;|Q10854;O.1;o1.O;|Q10805;O.4;o1.M;o2.M;o3.M;o4.M;|Q10855;O.1;o1.O;|Q10856;O.1;o1.M;|Q10857;O.3;o1.O;o2.O;o3.O;|Q10858;|Q10859;O.1;o1.O;|Q10860;O.2;o1.I;o2.I;|Q10861;O.2;o1.O;o2.M;|Q10862;|Q10863;|Q10864;O.1;o1.O;|Q10801;|Q10865;|Q10802;O.1;o1.O;|Q10866;O.2;o1.O;o2.M;|", -- [109]
	"|Q10931;|Q10995;O.1;o1.I;|Q10932;|Q10996;O.1;o1.I;|Q10933;|Q10934;|Q10998;O.1;o1.I;|Q10935;O.1;o1.O;|Q10999;O.1;o1.I;|Q10936;|Q11000;O.1;o1.I;|Q10937;O.1;o1.M;|Q10938;|Q10939;|Q10940;|Q10941;|Q10942;|Q10943;|Q10944;|Q10945;O.1;o1.E;|Q10946;O.1;o1.E;|Q10947;O.1;o1.I;|Q10948;|Q10949;|Q10950;O.1;o1.E;|Q10951;O.1;o1.E;|Q10997;O.1;o1.I;|Q10952;O.1;o1.E;|Q10953;O.1;o1.E;|Q10930;O.1;o1.I;|Q10954;O.1;o1.E;|Q10955;|Q10956;O.1;o1.E;|Q10957;O.1;o1.M;|Q10958;|Q10959;T.R;O.2;o1.M;o2.I;|Q10960;O.1;o1.E;|Q10961;O.3;o1.I;o2.I;o3.I;|Q10928;O.1;o1.M;|Q10962;O.2;o1.E;o2.I;|Q10963;O.2;o1.E;o2.I;|Q10964;|Q10901;O.2;o1.I;o2.I;|Q10965;O.1;o1.O;|Q10902;O.4;o1.I;o2.I;o3.I;o4.I;|Q10966;O.1;o1.I;|Q10903;|Q10967;O.1;o1.I;|Q10904;O.1;o1.I;|Q10968;O.1;o1.E;|Q10969;|Q10905;|Q10906;|Q10970;O.1;o1.I;|Q10926;|Q10907;|Q10971;O.1;o1.I;|Q10908;|Q10972;O.1;o1.I;|Q10909;O.1;o1.O;|Q10973;O.1;o1.I;|Q10910;|Q10974;O.1;o1.I;|Q10989;|Q10911;O.2;o1.O;o2.O;|Q10975;O.1;o1.I;|Q10912;O.2;o1.M;o2.I;|Q10976;O.1;o1.I;|Q10913;O.2;o1.O;o2.O;|Q10914;O.2;o1.M;o2.M;|Q10978;|Q10915;O.1;o1.O;|Q10979;|Q10916;O.1;o1.I;|Q10980;O.1;o1.O;|Q10917;O.1;o1.I;|Q10918;O.1;o1.I;|Q10982;|Q10919;O.1;o1.I;|Q10983;|Q10920;O.1;o1.M;|Q10984;|Q10985;O.1;o1.E;|Q10921;O.1;o1.M;|Q10922;O.1;o1.E;|Q10986;|Q10923;O.1;o1.M;|Q10987;O.1;o1.I;|Q10981;O.1;o1.I;|Q10924;O.1;o1.I;|Q10988;O.1;o1.I;|Q10977;O.1;o1.E;|Q10925;O.1;o1.M;|Q10990;O.1;o1.I;|Q10927;O.1;o1.M;|Q10991;O.1;o1.I;|Q10992;O.1;o1.I;|Q10929;O.1;o1.I;|Q10993;|Q10994;O.1;o1.I;|", -- [110]
	"|Q11059;O.1;o1.I;|Q11060;O.1;o1.I;|Q11062;|Q11063;|Q11064;O.1;o1.E;|Q11001;O.2;o1.M;o2.I;|Q11065;O.1;o1.O;|Q11002;|Q11066;O.1;o1.O;|Q11003;|Q11067;O.1;o1.E;|Q11004;O.1;o1.I;|Q11068;O.1;o1.E;|Q11005;O.3;o1.M;o2.M;o3.M;|Q11069;O.1;o1.E;|Q11006;O.1;o1.I;|Q11070;O.1;o1.E;|Q11007;|Q11071;O.1;o1.E;|Q11008;O.1;o1.O;|Q11072;O.4;o1.I;o2.I;o3.I;o4.I;|Q11009;|Q11073;O.1;o1.M;|Q11010;O.1;o1.O;|Q11074;O.4;o1.I;o2.I;o3.I;o4.I;|Q11011;|Q11075;|Q11012;|Q11076;O.1;o1.I;|Q11013;|Q11077;O.2;o1.M;o2.M;|Q11014;|Q11078;O.1;o1.I;|Q11015;O.1;o1.I;|Q11079;O.1;o1.I;|Q11016;O.1;o1.I;|Q11080;O.1;o1.E;|Q11017;O.1;o1.I;|Q11058;O.1;o1.E;|Q11018;O.1;o1.I;|Q11082;O.2;o1.E;o2.I;|Q11019;|Q11083;O.2;o1.M;o2.M;|Q11020;O.1;o1.O;|Q11084;|Q11022;|Q11086;O.1;o1.M;|Q11023;O.1;o1.O;|Q11087;O.2;o1.I;o2.I;|Q11061;O.1;o1.I;|Q11088;O.1;o1.I;|Q11025;O.1;o1.I;|Q11089;O.4;o1.I;o2.I;o3.I;o4.I;|Q11056;O.1;o1.I;|Q11026;O.1;o1.O;|Q11090;O.1;o1.E;|Q11027;O.1;o1.I;|Q11091;|Q11085;O.1;o1.E;|Q11081;|Q11028;|Q11092;|Q11093;O.1;o1.O;|Q11030;O.1;o1.I;|Q11094;|Q11031;O.1;o1.I;|Q11095;|Q11032;O.1;o1.I;|Q11096;O.1;o1.O;|Q11033;O.1;o1.I;|Q11097;O.1;o1.E;|Q11034;O.1;o1.I;|Q11098;|Q11054;O.2;o1.I;o2.I;|Q11035;O.1;o1.I;|Q11099;|Q11100;|Q11021;|Q11038;|Q11053;|Q11039;|Q11040;|Q11041;O.2;o1.M;o2.M;|Q11042;|Q11029;O.1;o1.I;|Q11043;|Q11052;|Q11044;|Q11045;|Q11047;|Q11048;|Q11049;O.1;o1.I;|Q11050;O.1;o1.I;|Q11051;O.1;o1.O;|Q11046;|Q11037;|Q11036;|Q11055;O.1;o1.O;|Q11024;|Q11057;|", -- [111]
	"|Q11123;|Q11187;|Q11124;|Q11188;O.1;o1.M;|Q11125;|Q11189;O.1;o1.E;|Q11126;O.1;o1.O;|Q11190;O.1;o1.I;|Q11127;O.1;o1.I;|Q11191;|Q11128;O.1;o1.I;|Q11192;O.1;o1.I;|Q11129;O.1;o1.O;|Q11193;|Q11130;|Q11194;|Q11131;O.1;o1.E;|Q11195;|Q11132;O.1;o1.I;|Q11196;|Q11133;O.1;o1.O;|Q11197;|Q11134;O.1;o1.O;|Q11198;O.1;o1.E;|Q11135;|Q11199;|Q11136;|Q11200;O.1;o1.I;|Q11137;O.1;o1.I;|Q11138;|Q11139;O.2;o1.I;o2.I;|Q11140;O.2;o1.I;o2.I;|Q11141;|Q11142;O.1;o1.E;|Q11143;|Q11144;O.1;o1.I;|Q11145;O.1;o1.O;|Q11122;O.1;o1.O;|Q11146;O.1;o1.O;|Q11147;O.1;o1.O;|Q11148;O.1;o1.I;|Q11149;|Q11150;O.3;o1.M;o2.M;o3.M;L.39;|Q11151;|Q11152;O.1;o1.O;|Q11153;O.2;o1.M;o2.O;|Q11154;O.1;o1.I;|Q11155;O.1;o1.I;|Q11156;O.1;o1.O;|Q11157;O.2;o1.O;o2.M;|Q11158;O.1;o1.I;|Q11159;O.1;o1.O;|Q11160;O.1;o1.I;|Q11161;O.1;o1.I;|Q11162;O.1;o1.O;|Q11163;|Q11164;O.1;o1.I;|Q11101;O.1;o1.E;|Q11165;|Q11102;O.1;o1.O;|Q11166;O.3;o1.O;o2.O;o3.O;|Q11103;O.1;o1.I;|Q11167;O.1;o1.I;|Q11104;O.1;o1.I;|Q11168;O.1;o1.I;|Q11105;O.1;o1.I;|Q11169;O.1;o1.O;|Q11106;O.1;o1.I;|Q11170;O.1;o1.O;|Q11107;|Q11171;O.1;o1.M;|Q11108;O.1;o1.E;|Q11172;|Q11109;|Q11173;O.1;o1.I;|Q11110;|Q11174;O.1;o1.M;L.40;|Q11111;|Q11175;|Q11112;|Q11176;|Q11113;|Q11177;|Q11114;|Q11178;|Q11115;|Q11179;O.1;o1.I;|Q11116;O.1;o1.M;|Q11180;O.1;o1.O;|Q11117;O.1;o1.I;|Q11181;O.1;o1.I;|Q11118;O.3;o1.M;o2.M;o3.M;|Q11182;O.2;o1.M;o2.M;|Q11119;|Q11183;O.1;o1.O;|Q11120;O.3;o1.M;o2.M;o3.M;|Q11184;O.1;o1.M;|Q11121;|Q11185;|Q11186;|", -- [112]
	"|Q11251;|Q11252;O.1;o1.I;|Q11253;|Q11254;|Q11255;O.1;o1.O;|Q11256;|Q11257;O.1;o1.O;|Q11258;O.3;o1.O;o2.O;o3.O;|Q11259;O.4;o1.O;o2.O;o3.O;o4.O;|Q11260;O.1;o1.M;|Q11261;O.1;o1.I;|Q11262;O.1;o1.I;|Q11263;O.3;o1.M;o2.M;o3.M;|Q11264;O.1;o1.M;|Q11201;O.1;o1.I;|Q11265;O.1;o1.O;|Q11202;O.4;o1.O;o2.O;o3.O;o4.O;|Q11266;|Q11203;|Q11267;O.3;o1.M;o2.M;o3.I;|Q11204;|Q11268;O.3;o1.M;o2.M;o3.M;|Q11205;O.3;o1.O;o2.O;o3.O;|Q11269;O.1;o1.I;|Q11206;|Q11270;O.1;o1.O;|Q11207;O.1;o1.I;|Q11271;O.1;o1.I;|Q11208;|Q11272;O.1;o1.M;|Q11209;O.1;o1.O;|Q11273;|Q11250;O.1;o1.I;|Q11210;|Q11274;|Q11211;|Q11275;O.1;o1.I;|Q11212;|Q11276;|Q11213;|Q11277;O.1;o1.I;|Q11214;|Q11278;|Q11215;|Q11279;O.1;o1.I;|Q11216;|Q11280;O.1;o1.O;|Q11217;O.1;o1.I;|Q11281;O.1;o1.O;|Q11248;|Q11218;O.2;o1.I;o2.I;|Q11282;O.3;o1.O;o2.O;o3.O;|Q11219;O.1;o1.E;|Q11283;O.1;o1.O;|Q11284;O.1;o1.M;|Q11220;|Q11221;O.2;o1.O;o2.O;|Q11285;O.4;o1.O;o2.O;o3.O;o4.O;|Q11222;|Q11286;O.1;o1.I;|Q11223;|Q11287;|Q11224;O.1;o1.O;|Q11288;O.1;o1.I;|Q11225;|Q11226;|Q11290;O.1;o1.I;|Q11246;O.1;o1.O;|Q11227;O.1;o1.O;|Q11291;|Q11228;|Q11292;O.1;o1.I;|Q11229;|Q11293;O.4;o1.O;o2.O;o3.O;o4.O;|Q11294;O.4;o1.O;o2.O;o3.O;o4.O;|Q11230;O.1;o1.M;|Q11231;O.1;o1.O;|Q11295;|Q11296;O.1;o1.O;|Q11232;O.2;o1.O;o2.O;|Q11233;O.3;o1.M;o2.M;o3.M;|Q11297;|Q11298;O.1;o1.I;|Q11234;|Q11244;O.1;o1.O;|Q11235;O.3;o1.M;o2.M;o3.M;|Q11299;|Q11236;O.1;o1.M;|Q11300;O.1;o1.I;|Q11237;|Q11238;O.3;o1.M;o2.M;o3.I;|Q11239;O.3;o1.M;o2.M;o3.M;|Q11240;O.1;o1.M;|Q11241;O.1;o1.E;|Q11242;|Q11243;O.1;o1.M;|Q11245;O.4;o1.O;o2.O;o3.O;o4.O;|Q11289;|Q11247;O.3;o1.O;o2.O;o3.O;|Q11249;O.1;o1.M;|", -- [113]
	"|Q11315;O.1;o1.M;|Q11379;O.1;o1.I;|Q11316;O.1;o1.M;|Q11380;O.1;o1.I;|Q11317;O.1;o1.O;|Q11381;O.1;o1.I;|Q11318;O.3;o1.O;o2.O;o3.O;|Q11382;O.1;o1.I;|Q11319;O.1;o1.O;|Q11383;O.1;o1.M;|Q11320;O.3;o1.O;o2.O;o3.O;|Q11384;O.1;o1.I;|Q11321;|Q11385;O.1;o1.M;|Q11322;O.1;o1.O;|Q11386;O.1;o1.I;|Q11323;|Q11387;O.1;o1.M;|Q11324;O.2;o1.M;o2.I;|Q11388;O.1;o1.I;|Q11325;|Q11389;O.1;o1.M;|Q11326;O.2;o1.M;o2.I;|Q11390;O.1;o1.O;|Q11327;O.1;o1.I;|Q11391;O.1;o1.M;|Q11328;|Q11392;|Q11329;O.1;o1.I;|Q11393;|Q11330;O.1;o1.O;|Q11394;O.1;o1.O;|Q11331;|Q11395;|Q11332;O.1;o1.O;|Q11396;O.1;o1.O;|Q11333;O.1;o1.I;|Q11397;O.1;o1.O;|Q11334;|Q11398;|Q11335;O.1;o1.E;|Q11399;O.1;o1.O;|Q11336;Y.1;T.P;O.1;o1.E;|Q11400;|Q11337;O.1;o1.E;|Q11378;O.1;o1.I;|Q11314;O.1;o1.O;|Q11338;O.1;o1.E;|Q11339;O.1;o1.E;|Q11341;O.1;o1.E;|Q11342;O.1;o1.E;|Q11343;O.1;o1.E;|Q11344;O.1;o1.E;|Q11345;|Q11376;O.1;o1.M;|Q11312;|Q11346;O.1;o1.I;|Q11347;O.1;o1.I;|Q11348;O.2;o1.O;o2.M;|Q11349;O.1;o1.I;|Q11350;O.1;o1.I;|Q11351;O.1;o1.I;|Q11352;O.2;o1.O;o2.M;|Q11353;|Q11354;O.1;o1.I;|Q11374;O.1;o1.I;|Q11310;O.1;o1.O;|Q11355;O.1;o1.O;|Q11357;|Q11358;O.1;o1.O;|Q11373;O.1;o1.I;|Q11359;O.1;o1.M;|Q11360;O.1;o1.O;|Q11361;O.1;o1.O;|Q11362;O.1;o1.I;|Q11372;O.1;o1.I;|Q11308;|Q11363;O.1;o1.I;|Q11364;O.1;o1.M;|Q11301;O.1;o1.I;|Q11302;|Q11366;O.1;o1.O;|Q11303;|Q11367;O.1;o1.M;|Q11368;O.1;o1.I;|Q11304;|Q11305;O.1;o1.I;|Q11369;O.1;o1.I;|Q11306;O.1;o1.I;|Q11370;O.1;o1.I;|Q11307;O.1;o1.O;|Q11371;O.1;o1.M;|Q11365;O.1;o1.O;|Q11309;O.2;o1.I;o2.I;|Q11356;|Q11311;O.1;o1.O;|Q11375;O.1;o1.I;|Q11313;O.1;o1.I;|Q11377;O.1;o1.I;|Q11340;O.1;o1.E;|", -- [114]
	"|Q11443;O.1;o1.I;|Q11444;|Q11445;|Q11446;|Q11447;|Q11448;|Q11449;O.1;o1.O;|Q11450;O.1;o1.O;|Q11451;|Q11452;O.1;o1.M;|Q11453;O.1;o1.M;|Q11454;|Q11455;O.1;o1.I;|Q11456;O.1;o1.I;|Q11457;O.1;o1.I;|Q11458;O.1;o1.M;|Q11459;|Q11460;O.1;o1.E;|Q11461;|Q11462;|Q11463;|Q11464;O.1;o1.I;|Q11442;|Q11402;O.1;o1.M;|Q11466;O.1;o1.I;|Q11401;|Q11403;|Q11467;O.1;o1.I;|Q11404;|Q11468;O.1;o1.I;|Q11469;O.1;o1.I;|Q11441;|Q11406;|Q11470;O.1;o1.I;|Q11407;O.4;o1.O;o2.O;o3.O;o4.O;|Q11471;O.1;o1.I;|Q11408;O.4;o1.O;o2.O;o3.O;o4.O;|Q11472;O.1;o1.E;|Q11465;O.1;o1.I;|Q11409;O.3;o1.O;o2.O;o3.O;|Q11410;O.1;o1.M;|Q11474;|Q11411;|Q11475;O.1;o1.I;|Q11412;O.1;o1.O;|Q11476;O.2;o1.I;o2.I;|Q11413;|Q11477;O.3;o1.M;o2.M;o3.M;|Q11414;O.2;o1.M;o2.M;|Q11478;|Q11415;O.2;o1.M;o2.M;|Q11479;O.1;o1.M;|Q11416;O.1;o1.I;|Q11480;|Q11417;O.1;o1.I;|Q11481;|Q11482;|Q11418;O.1;o1.O;|Q11419;|Q11483;O.3;o1.I;o2.I;o3.I;|Q11484;O.2;o1.I;o2.I;|Q11420;O.1;o1.I;|Q11421;O.4;o1.M;o2.O;o3.O;o4.O;|Q11485;O.1;o1.E;|Q11422;O.1;o1.I;|Q11486;|Q11437;|Q11423;O.3;o1.I;o2.I;o3.I;|Q11487;|Q11424;O.1;o1.I;|Q11488;|Q11425;O.1;o1.M;|Q11489;O.1;o1.O;|Q11426;O.1;o1.I;|Q11490;O.1;o1.O;|Q11427;|Q11491;O.1;o1.E;|Q11428;O.1;o1.M;|Q11492;O.1;o1.I;|Q11493;|Q11429;O.2;o1.E;o2.O;|Q11430;O.1;o1.M;|Q11494;O.1;o1.O;|Q11431;O.1;o1.I;|Q11495;O.1;o1.E;|Q11432;O.1;o1.M;|Q11496;O.1;o1.E;|Q11433;O.1;o1.M;|Q11497;|Q11434;O.2;o1.I;o2.I;|Q11498;|Q11435;|Q11499;O.1;o1.I;|Q11405;|Q11436;O.1;o1.E;|Q11500;O.1;o1.M;|Q11473;|Q11438;|Q11439;O.1;o1.O;|Q11440;O.1;o1.O;|", -- [115]
	"|Q11507;|Q11571;O.1;o1.I;|Q11508;|Q11572;|Q11509;|Q11573;|Q11510;O.1;o1.I;|Q11574;|Q11511;O.1;o1.I;|Q11575;|Q11512;O.1;o1.I;|Q11576;O.1;o1.O;|Q11513;O.1;o1.I;|Q11577;|Q11514;O.1;o1.I;|Q11578;O.2;o1.M;o2.P;|Q11515;O.1;o1.M;|Q11579;O.2;o1.M;o2.P;|Q11516;O.1;o1.E;|Q11580;|Q11517;|Q11581;|Q11518;|Q11582;O.1;o1.O;|Q11519;|Q11583;|Q11520;O.1;o1.I;|Q11584;|Q11521;O.1;o1.I;|Q11585;|Q11586;|Q11523;O.1;o1.E;|Q11587;O.1;o1.O;|Q11524;O.1;o1.O;|Q11588;|Q11525;O.1;o1.O;|Q11589;|Q11526;|Q11590;O.1;o1.O;|Q11527;O.1;o1.I;|Q11591;|Q11528;|Q11592;O.1;o1.E;|Q11529;O.1;o1.I;|Q11593;O.1;o1.O;|Q11530;|Q11594;O.1;o1.O;|Q11531;|Q11595;|Q11532;O.3;o1.M;o2.M;o3.M;|Q11596;|Q11533;O.3;o1.M;o2.M;o3.M;|Q11597;|Q11534;|Q11598;O.1;o1.O;|Q11535;O.1;o1.I;|Q11599;|Q11536;O.1;o1.I;|Q11600;|Q11537;O.2;o1.O;o2.M;|Q11538;O.2;o1.O;o2.M;|Q11539;O.3;o1.M;o2.M;o3.M;|Q11540;O.3;o1.M;o2.M;o3.M;|Q11541;O.1;o1.O;|Q11542;O.4;o1.O;o2.O;o3.O;o4.M;|Q11543;O.4;o1.O;o2.O;o3.O;o4.M;|Q11544;O.1;o1.I;|Q11545;|Q11546;O.1;o1.I;|Q11547;O.3;o1.O;o2.O;o3.O;|Q11548;|Q11549;|Q11550;|Q11501;|Q11551;O.1;o1.I;|Q11552;O.1;o1.I;|Q11553;O.1;o1.I;|Q11554;|Q11555;|Q11556;|Q11557;|Q11558;O.1;o1.I;|Q11559;O.1;o1.I;|Q11560;O.1;o1.O;|Q11561;O.1;o1.O;|Q11562;|Q11563;O.1;o1.I;|Q11564;O.1;o1.I;|Q11565;|Q11502;O.1;o1.M;|Q11566;O.1;o1.I;|Q11503;O.1;o1.M;|Q11567;O.1;o1.I;|Q11504;O.4;o1.I;o2.I;o3.I;o4.I;|Q11568;O.4;o1.O;o2.O;o3.O;o4.O;|Q11505;O.1;o1.E;|Q11569;O.1;o1.I;|Q11506;O.1;o1.E;|Q11570;O.1;o1.E;|", -- [116]
	"|Q11635;|Q11699;O.3;o1.I;o2.I;o3.I;|Q11636;|Q11700;|Q11637;O.2;o1.O;o2.I;|Q11638;O.1;o1.I;|Q11639;O.4;o1.M;o2.M;o3.M;o4.M;|Q11640;O.3;o1.I;o2.I;o3.I;|Q11641;O.2;o1.M;o2.M;|Q11642;|Q11643;O.1;o1.I;|Q11644;O.1;o1.I;|Q11645;O.1;o1.I;|Q11646;|Q11647;O.3;o1.O;o2.O;o3.O;|Q11648;O.1;o1.O;|Q11649;O.1;o1.I;|Q11650;O.3;o1.I;o2.I;o3.I;|Q11651;|Q11652;O.3;o1.E;o2.O;o3.O;|Q11653;O.1;o1.O;|Q11654;|Q11655;O.1;o1.I;|Q11656;O.4;o1.O;o2.O;o3.O;o4.O;|Q11657;O.1;o1.E;|Q11658;O.2;o1.I;o2.I;|Q11659;O.1;o1.O;|Q11660;O.1;o1.I;|Q11661;O.1;o1.M;|Q11662;|Q11663;|Q11664;O.1;o1.E;|Q11601;|Q11665;O.1;o1.I;|Q11696;|Q11602;O.1;o1.O;|Q11666;O.1;o1.I;|Q11603;O.1;o1.I;|Q11667;O.1;o1.I;|Q11604;|Q11668;O.1;o1.I;|Q11605;O.3;o1.O;o2.O;o3.O;|Q11669;O.1;o1.I;|Q11606;O.1;o1.I;|Q11670;O.2;o1.O;o2.I;|Q11607;O.2;o1.O;o2.O;|Q11671;O.1;o1.I;|Q11608;O.4;o1.O;o2.O;o3.O;o4.O;|Q11672;|Q11609;O.1;o1.I;|Q11673;O.1;o1.E;|Q11610;O.3;o1.O;o2.O;o3.O;|Q11674;|Q11694;O.1;o1.O;|Q11611;O.1;o1.O;|Q11675;O.1;o1.M;|Q11612;O.1;o1.M;|Q11676;O.1;o1.O;|Q11613;O.2;o1.M;o2.M;|Q11677;O.1;o1.O;|Q11614;|Q11678;|Q11693;O.1;o1.M;|Q11615;|Q11679;|Q11616;|Q11680;|Q11617;O.3;o1.O;o2.O;o3.O;|Q11681;|Q11618;|Q11682;|Q11692;|Q11619;O.1;o1.M;|Q11683;O.1;o1.O;|Q11620;|Q11684;O.3;o1.O;o2.O;o3.O;|Q11621;|Q11685;O.1;o1.I;|Q11622;|Q11686;O.3;o1.O;o2.O;o3.O;|Q11623;O.1;o1.M;|Q11687;O.2;o1.M;o2.M;|Q11688;O.1;o1.M;|Q11624;|Q11625;O.1;o1.I;|Q11689;|Q11626;O.1;o1.M;|Q11690;O.1;o1.O;|Q11627;O.2;o1.O;o2.O;|Q11691;|Q11628;O.1;o1.I;|Q11629;|Q11630;O.1;o1.I;|Q11631;O.1;o1.O;|Q11695;O.2;o1.I;o2.I;|Q11632;|Q11633;O.3;o1.O;o2.O;o3.O;|Q11697;|Q11634;|Q11698;O.1;o1.O;|", -- [117]
	"|Q11763;|Q11764;|Q11701;|Q11765;|Q11702;|Q11766;|Q11703;|Q11767;|Q11704;|Q11768;|Q11705;O.1;o1.E;|Q11769;|Q11706;O.2;o1.M;o2.O;|Q11770;|Q11707;|Q11771;|Q11708;O.1;o1.E;|Q11772;|Q11709;|Q11773;|Q11710;O.1;o1.I;|Q11774;|Q11711;O.1;o1.E;|Q11775;|Q11712;O.1;o1.O;|Q11776;|Q11713;O.3;o1.O;o2.O;o3.O;|Q11777;|Q11714;O.3;o1.M;o2.M;o3.M;|Q11778;|Q11715;O.1;o1.O;|Q11779;|Q11716;O.1;o1.I;|Q11780;|Q11717;O.1;o1.I;|Q11781;|Q11718;O.1;o1.I;|Q11782;|Q11719;O.1;o1.E;|Q11783;|Q11720;|Q11784;|Q11762;|Q11722;|Q11786;|Q11723;O.2;o1.M;o2.O;|Q11787;|Q11724;|Q11788;O.4;o1.M;o2.M;o3.M;o4.M;|Q11726;O.1;o1.I;|Q11790;|Q11727;|Q11791;|Q11725;|Q11728;O.1;o1.I;|Q11792;O.1;o1.I;|Q11729;|Q11793;|Q11730;O.1;o1.O;|Q11794;O.3;o1.O;o2.O;o3.O;|Q11731;O.1;o1.E;|Q11795;O.1;o1.I;|Q11732;|Q11796;O.3;o1.O;o2.O;o3.O;|Q11797;O.1;o1.M;|Q11733;|Q11734;|Q11798;O.2;o1.O;o2.I;|Q11735;|Q11799;|Q11736;|Q11800;|Q11737;|Q11738;|Q11739;|Q11740;|Q11741;|Q11742;|Q11757;|Q11743;|Q11744;|Q11745;|Q11721;O.1;o1.I;|Q11746;|Q11747;|Q11748;|Q11749;|Q11750;|Q11751;|Q11785;|Q11752;|Q11753;|Q11754;|Q11789;O.1;o1.I;|Q11755;|Q11756;|Q11758;|Q11759;|Q11760;|Q11761;|", -- [118]
	"|Q11827;|Q11891;O.1;o1.E;|Q11828;|Q11892;O.1;o1.M;|Q11829;|Q11893;O.1;o1.O;|Q11830;|Q11894;O.1;o1.I;|Q11831;|Q11895;O.1;o1.O;|Q11832;|Q11896;O.1;o1.O;|Q11833;|Q11897;O.2;o1.O;o2.O;|Q11834;|Q11898;O.3;o1.M;o2.M;o3.M;|Q11835;|Q11899;O.1;o1.O;|Q11836;|Q11900;O.4;o1.O;o2.O;o3.O;o4.O;|Q11837;|Q11838;|Q11839;|Q11840;|Q11841;|Q11842;|Q11843;|Q11844;|Q11845;|Q11846;|Q11847;|Q11848;|Q11850;|Q11851;|Q11852;|Q11854;|Q11855;|Q11856;|Q11857;|Q11888;|Q11858;|Q11859;|Q11860;|Q11861;|Q11862;|Q11863;|Q11864;|Q11801;|Q11865;O.1;o1.O;|Q11802;|Q11886;O.1;o1.I;|Q11866;O.1;o1.I;|Q11803;|Q11867;O.1;o1.I;|Q11868;O.1;o1.M;|Q11805;|Q11869;O.1;o1.M;|Q11806;|Q11870;|Q11807;|Q11871;O.1;o1.I;|Q11808;|Q11872;O.1;o1.M;|Q11810;|Q11874;|Q11884;O.2;o1.M;o2.M;|Q11849;|Q11811;|Q11875;O.1;o1.I;|Q11812;|Q11876;O.1;o1.O;|Q11877;O.1;o1.I;|Q11813;|Q11814;|Q11878;O.1;o1.E;|Q11815;|Q11879;O.1;o1.I;|Q11816;|Q11880;O.1;o1.O;|Q11881;O.1;o1.O;|Q11817;|Q11818;|Q11853;|Q11882;|Q11819;|Q11883;|Q11820;|Q11873;|Q11809;|Q11821;|Q11885;O.4;o1.M;o2.M;o3.M;o4.M;|Q11804;|Q11822;|Q11823;|Q11887;O.1;o1.I;|Q11824;|Q11825;|Q11889;O.1;o1.M;|Q11826;|Q11890;O.1;o1.E;|", -- [119]
	"|Q11955;|Q11956;O.1;o1.I;|Q11957;O.1;o1.I;|Q11958;O.1;o1.I;|Q11959;O.1;o1.O;|Q11960;O.1;o1.I;|Q11961;O.1;o1.I;|Q11962;|Q11963;|Q11964;|Q11901;|Q11965;O.1;o1.O;|Q11902;|Q11966;|Q11903;O.1;o1.M;|Q11967;O.1;o1.I;|Q11904;O.1;o1.O;|Q11968;O.1;o1.M;|Q11905;O.1;o1.O;|Q11969;O.1;o1.O;|Q11906;O.1;o1.I;|Q11970;|Q11907;O.4;o1.M;o2.M;o3.M;o4.M;|Q11971;|Q11908;O.1;o1.I;|Q11972;|Q11909;O.2;o1.O;o2.I;|Q11973;O.1;o1.O;|Q11910;O.2;o1.I;o2.I;|Q11974;O.1;o1.E;|Q11911;O.1;o1.I;|Q11975;|Q11912;O.1;o1.I;|Q11976;|Q11914;O.1;o1.I;|Q11978;O.1;o1.I;|Q11915;|Q11979;|Q11913;O.1;o1.O;|Q11916;|Q11980;O.1;o1.M;|Q11917;O.1;o1.M;|Q11981;|Q11918;O.1;o1.M;|Q11982;O.1;o1.O;|Q11919;O.1;o1.O;|Q11983;O.1;o1.O;|Q11920;|Q11984;O.1;o1.O;|Q11921;O.1;o1.E;|Q11985;O.1;o1.M;|Q11922;O.1;o1.E;|Q11986;O.1;o1.I;|Q11923;O.1;o1.E;|Q11987;|Q11924;O.1;o1.E;|Q11988;O.1;o1.I;|Q11925;O.1;o1.E;|Q11989;O.1;o1.O;|Q11926;O.1;o1.E;|Q11990;O.3;o1.I;o2.I;o3.I;|Q11927;|Q11991;|Q11928;|Q11992;O.4;o1.I;o2.I;o3.I;o4.I;|Q11929;|Q11993;O.3;o1.O;o2.O;o3.O;|Q11930;O.1;o1.E;|Q11994;|Q11931;O.2;o1.I;o2.I;|Q11995;|Q11932;|Q11996;|Q11997;|Q11933;|Q11934;O.1;o1.I;|Q11998;O.1;o1.I;|Q11935;|Q11999;O.1;o1.I;|Q11936;O.1;o1.O;|Q12000;O.1;o1.I;|Q11937;|Q11938;O.1;o1.O;|Q11939;|Q11940;O.1;o1.O;|Q11941;|Q11942;O.3;o1.I;o2.I;o3.I;|Q11943;O.2;o1.I;o2.I;|Q11944;O.1;o1.M;|Q11945;O.1;o1.I;|Q11946;|Q11947;O.1;o1.M;|Q11948;O.1;o1.M;|Q11949;O.1;o1.M;|Q11950;|Q11951;O.1;o1.I;|Q11952;O.1;o1.M;|Q11977;|Q11953;O.1;o1.M;|Q11954;O.1;o1.M;|", -- [120]
	"|Q12019;O.1;o1.O;|Q12083;O.1;o1.O;|Q12020;|Q12084;O.1;o1.O;|Q12021;|Q12085;|Q12022;O.1;o1.O;|Q12086;|Q12023;O.1;o1.M;|Q12087;O.1;o1.M;|Q12024;O.1;o1.I;|Q12088;|Q12025;O.1;o1.I;|Q12089;O.1;o1.M;|Q12026;O.1;o1.I;|Q12090;O.1;o1.M;|Q12027;O.1;o1.E;|Q12091;O.1;o1.M;|Q12028;O.1;o1.E;|Q12092;O.1;o1.O;|Q12029;O.1;o1.O;|Q12093;O.4;o1.M;o2.M;o3.M;o4.M;|Q12030;|Q12094;O.3;o1.O;o2.O;o3.O;|Q12031;O.1;o1.O;|Q12095;|Q12032;O.1;o1.E;|Q12096;O.1;o1.O;|Q12033;O.1;o1.E;|Q12097;O.1;o1.I;|Q12034;|Q12098;|Q12035;O.1;o1.O;|Q12099;O.1;o1.O;|Q12036;O.1;o1.E;|Q12100;O.2;o1.M;o2.I;|Q12037;|Q12038;O.1;o1.O;|Q12039;O.1;o1.I;|Q12040;O.1;o1.M;|Q12082;O.1;o1.E;|Q12041;|Q12042;|Q12043;O.2;o1.M;o2.M;|Q12044;O.1;o1.I;|Q12045;O.1;o1.I;|Q12046;O.1;o1.I;|Q12047;O.1;o1.I;|Q12048;O.1;o1.I;|Q12049;O.1;o1.I;|Q12050;O.1;o1.I;|Q12051;O.1;o1.O;|Q12052;O.2;o1.M;o2.O;|Q12053;O.1;o1.E;|Q12054;O.1;o1.I;|Q12055;|Q12056;O.1;o1.I;|Q12057;|Q12058;O.3;o1.O;o2.O;o3.O;|Q12059;|Q12061;O.1;o1.O;|Q12062;Y.1;T.D;|Q12063;|Q12064;O.3;o1.I;o2.I;o3.I;|Q12001;O.1;o1.I;|Q12065;O.1;o1.O;|Q12002;O.1;o1.M;|Q12066;O.1;o1.O;|Q12003;O.3;o1.O;o2.O;o3.O;|Q12067;|Q12004;O.2;o1.I;o2.I;|Q12068;O.1;o1.I;|Q12005;O.2;o1.I;o2.I;|Q12069;O.1;o1.I;|Q12006;O.1;o1.O;|Q12070;O.2;o1.O;o2.O;|Q12007;O.1;o1.I;|Q12071;|Q12008;|Q12072;O.1;o1.M;|Q12009;O.1;o1.I;|Q12073;O.1;o1.O;|Q12010;|Q12074;|Q12011;|Q12075;O.1;o1.I;|Q12012;|Q12076;O.1;o1.I;|Q12013;O.1;o1.M;L.73;|Q12077;|Q12060;O.1;o1.M;L.72;|Q12014;O.2;o1.I;o2.I;|Q12078;O.1;o1.I;|Q12015;O.1;o1.M;|Q12079;O.1;o1.M;|Q12016;O.1;o1.I;|Q12080;O.1;o1.M;|Q12017;O.1;o1.M;|Q12081;|Q12018;O.1;o1.I;|", -- [121]
	"|Q12147;|Q12148;O.1;o1.I;|Q12149;O.3;o1.M;o2.M;o3.M;|Q12150;O.1;o1.E;|Q12151;O.1;o1.I;|Q12152;O.1;o1.O;|Q12153;O.1;o1.M;|Q12154;O.1;o1.O;|Q12155;|Q12156;|Q12157;|Q12158;|Q12159;O.1;o1.O;|Q12160;|Q12161;|Q12162;|Q12163;|Q12164;O.4;o1.M;o2.M;o3.M;o4.O;|Q12101;|Q12165;O.1;o1.I;|Q12102;O.1;o1.I;|Q12166;L.72;O.2;o1.M;o2.M;|Q12103;O.2;o1.M;o2.M;|Q12167;O.1;o1.I;|Q12104;|Q12168;|Q12146;|Q12105;|Q12106;|Q12170;O.1;o1.O;|Q12107;O.2;o1.O;o2.O;|Q12171;|Q12108;|Q12172;O.1;o1.O;|Q12173;O.1;o1.O;|Q12110;O.2;o1.O;o2.O;|Q12174;|Q12111;O.2;o1.O;o2.O;|Q12175;O.1;o1.I;|Q12112;|Q12113;O.2;o1.I;o2.I;|Q12177;O.2;o1.I;o2.I;|Q12144;O.2;o1.O;o2.O;|Q12114;O.2;o1.M;o2.M;|Q12178;|Q12115;|Q12179;O.1;o1.I;|Q12116;O.1;o1.I;|Q12180;O.3;o1.O;o2.O;o3.O;|Q12117;|Q12181;|Q12118;|Q12182;|Q12119;|Q12183;O.1;o1.I;|Q12120;O.1;o1.I;|Q12184;O.1;o1.O;|Q12121;O.1;o1.O;|Q12185;O.1;o1.O;|Q12142;O.2;o1.O;o2.O;|Q12122;|Q12186;|Q12123;|Q12187;|Q12124;|Q12188;O.1;o1.I;|Q12125;O.1;o1.I;|Q12189;|Q12126;O.1;o1.I;|Q12190;|Q12127;O.1;o1.I;|Q12191;O.1;o1.O;|Q12128;|Q12192;|Q12169;O.1;o1.M;|Q12129;O.1;o1.I;|Q12193;|Q12194;|Q12140;O.1;o1.O;|Q12131;O.2;o1.I;o2.I;|Q12195;O.1;o1.I;|Q12132;O.1;o1.M;|Q12196;O.1;o1.I;|Q12109;|Q12133;|Q12197;O.2;o1.I;o2.I;|Q12134;O.1;o1.M;|Q12198;O.1;o1.O;|Q12130;O.1;o1.I;|Q12135;O.1;o1.E;|Q12199;O.1;o1.M;|Q12136;|Q12200;O.1;o1.I;|Q12137;O.2;o1.I;o2.I;|Q12138;O.1;o1.O;|Q12139;O.1;o1.E;|Q12141;|Q12143;O.1;o1.M;|Q12176;O.1;o1.I;|Q12145;O.1;o1.M;|", -- [122]
	"|Q12211;O.1;o1.O;|Q12275;|Q12212;O.1;o1.I;|Q12276;|Q12213;O.3;o1.O;o2.O;o3.O;|Q12277;O.2;o1.O;o2.O;|Q12278;|Q12215;O.1;o1.M;|Q12279;O.1;o1.I;|Q12280;O.3;o1.I;o2.I;o3.I;|Q12217;O.1;o1.M;|Q12281;|Q12218;O.1;o1.M;|Q12282;O.1;o1.I;|Q12219;O.1;o1.I;|Q12283;O.1;o1.I;|Q12220;O.3;o1.O;o2.O;o3.O;|Q12284;O.1;o1.O;|Q12221;|Q12285;O.1;o1.I;|Q12222;O.1;o1.I;|Q12286;|Q12223;O.1;o1.M;|Q12287;|Q12224;|Q12288;O.1;o1.O;|Q12225;|Q12289;O.1;o1.O;|Q12226;O.1;o1.I;|Q12290;O.1;o1.I;|Q12227;O.1;o1.I;|Q12291;O.4;o1.O;o2.O;o3.O;o4.O;|Q12228;|Q12292;O.1;o1.I;|Q12229;O.1;o1.I;|Q12293;|Q12230;O.1;o1.I;|Q12294;|Q12231;O.2;o1.O;o2.O;|Q12295;|Q12232;O.1;o1.O;|Q12296;O.1;o1.O;|Q12214;O.1;o1.O;|Q12233;|Q12297;|Q12234;O.3;o1.I;o2.I;o3.I;|Q12298;|Q12235;|Q12299;O.1;o1.M;|Q12236;O.1;o1.O;|Q12300;O.1;o1.O;|Q12237;O.1;o1.O;|Q12238;|Q12239;|Q12240;O.1;o1.M;|Q12241;O.1;o1.I;|Q12272;O.1;o1.I;|Q12242;O.1;o1.I;|Q12243;O.2;o1.O;o2.I;|Q12244;O.1;o1.O;|Q12245;O.4;o1.M;o2.M;o3.M;o4.M;|Q12246;O.1;o1.I;|Q12247;O.2;o1.O;o2.O;|Q12248;O.1;o1.I;|Q12249;O.1;o1.O;|Q12270;O.1;o1.O;|Q12206;O.1;o1.O;|Q12250;O.1;o1.I;|Q12251;|Q12252;O.2;o1.O;o2.M;|Q12253;O.1;o1.O;|Q12254;O.1;o1.I;|Q12255;O.1;o1.M;|Q12256;O.1;o1.I;|Q12258;O.1;o1.M;|Q12268;O.3;o1.I;o2.I;o3.I;|Q12204;O.2;o1.O;o2.O;|Q12259;O.1;o1.M;|Q12216;O.1;o1.I;|Q12260;O.1;o1.O;|Q12261;O.1;o1.O;|Q12262;O.2;o1.M;o2.M;|Q12257;O.1;o1.M;|Q12263;O.1;o1.E;|Q12264;O.3;o1.M;o2.M;o3.M;|Q12201;O.1;o1.I;|Q12265;O.1;o1.O;|Q12202;O.1;o1.O;|Q12266;|Q12203;O.1;o1.O;|Q12267;O.2;o1.O;o2.M;|Q12205;O.1;o1.O;|Q12269;O.1;o1.I;|Q12207;O.1;o1.I;|Q12271;|Q12208;|Q12209;O.2;o1.I;o2.I;|Q12273;O.4;o1.O;o2.O;o3.O;o4.O;|Q12210;|Q12274;O.2;o1.O;o2.O;|", -- [123]
	"|Q12339;|Q12340;|Q12341;|Q12342;|Q12343;|Q12344;|Q12345;|Q12346;|Q12347;|Q12348;|Q12349;|Q12350;|Q12351;|Q12352;|Q12353;|Q12354;|Q12355;|Q12356;|Q12357;|Q12358;|Q12359;|Q12360;|Q12361;|Q12362;|Q12363;|Q12364;|Q12301;O.1;o1.E;|Q12365;|Q12302;|Q12366;|Q12303;O.1;o1.I;|Q12367;|Q12304;O.1;o1.O;|Q12368;|Q12305;|Q12369;|Q12306;|Q12370;|Q12307;O.1;o1.O;|Q12371;|Q12308;O.1;o1.O;|Q12372;O.3;o1.M;o2.M;o3.O;|Q12309;|Q12374;|Q12310;O.2;o1.M;o2.M;|Q12311;O.1;o1.M;|Q12375;|Q12312;|Q12376;|Q12313;|Q12377;|Q12314;O.1;o1.M;|Q12378;|Q12315;O.1;o1.M;|Q12379;|Q12316;O.1;o1.O;|Q12380;|Q12317;O.1;o1.O;|Q12381;|Q12318;|Q12382;|Q12319;|Q12383;|Q12320;|Q12384;|Q12321;O.1;o1.E;|Q12385;|Q12386;|Q12323;O.1;o1.O;|Q12387;|Q12324;O.1;o1.O;|Q12388;|Q12325;|Q12326;O.2;o1.O;o2.O;|Q12390;|Q12327;O.1;o1.E;|Q12391;|Q12328;O.1;o1.I;|Q12392;|Q12329;|Q12393;|Q12330;O.1;o1.E;|Q12394;|Q12331;|Q12395;|Q12389;|Q12332;|Q12396;|Q12333;|Q12397;|Q12334;|Q12398;|Q12335;|Q12399;|Q12373;|Q12336;|Q12400;|Q12337;|Q12338;|", -- [124]
	"|Q12403;|Q12467;O.1;o1.I;|Q12404;|Q12468;|Q12405;|Q12469;|Q12406;|Q12470;O.1;o1.E;|Q12407;|Q12471;|Q12408;|Q12472;|Q12409;|Q12473;O.1;o1.E;|Q12410;|Q12474;|Q12411;|Q12475;O.1;o1.I;|Q12412;O.2;o1.M;o2.M;|Q12476;O.1;o1.O;|Q12413;O.1;o1.M;|Q12477;O.1;o1.I;|Q12414;O.1;o1.O;|Q12478;O.1;o1.O;|Q12415;O.1;o1.O;|Q12479;|Q12416;O.3;o1.M;o2.M;o3.M;|Q12480;|Q12417;O.1;o1.O;|Q12481;O.2;o1.O;o2.O;|Q12418;O.2;o1.M;o2.O;|Q12482;O.3;o1.M;o2.M;o3.M;|Q12419;|Q12483;O.3;o1.I;o2.I;o3.I;|Q12420;|Q12484;O.1;o1.O;|Q12421;|Q12485;|Q12422;|Q12486;|Q12423;|Q12487;|Q12424;|Q12488;|Q12402;|Q12425;|Q12489;|Q12426;|Q12490;O.3;o1.M;o2.M;o3.M;|Q12427;O.1;o1.E;|Q12491;|Q12428;O.1;o1.E;|Q12492;|Q12429;O.1;o1.E;|Q12493;O.1;o1.P;|Q12430;O.1;o1.E;|Q12494;|Q12431;|Q12495;|Q12432;O.1;o1.O;|Q12496;|Q12497;|Q12433;O.1;o1.I;|Q12434;O.1;o1.I;|Q12498;O.2;o1.M;o2.I;|Q12435;|Q12499;|Q12436;O.1;o1.I;|Q12500;|Q12438;O.1;o1.M;|Q12439;|Q12440;|Q12441;O.1;o1.M;|Q12442;O.1;o1.M;|Q12443;O.1;o1.I;|Q12444;O.1;o1.O;|Q12446;O.1;o1.I;|Q12447;|Q12448;O.3;o1.M;o2.M;o3.M;|Q12449;O.1;o1.O;|Q12450;O.2;o1.M;o2.O;|Q12451;|Q12452;|Q12454;O.1;o1.M;|Q12455;O.1;o1.I;|Q12456;O.1;o1.I;|Q12457;O.1;o1.O;|Q12458;O.1;o1.I;|Q12459;O.3;o1.M;o2.M;o3.M;|Q12453;O.1;o1.O;|Q12460;|Q12461;|Q12462;O.3;o1.M;o2.M;o3.O;|Q12463;|Q12437;O.1;o1.O;|Q12464;O.1;o1.I;|Q12401;|Q12465;O.4;o1.I;o2.I;o3.I;o4.I;|Q12466;|", -- [125]
	"|Q12531;O.1;o1.I;|Q12595;|Q12532;O.1;o1.I;|Q12596;O.4;o1.O;o2.O;o3.O;o4.O;|Q12533;O.2;o1.M;o2.M;|Q12597;O.1;o1.I;|Q12534;O.1;o1.I;|Q12598;O.1;o1.O;|Q12535;O.1;o1.I;|Q12599;O.1;o1.I;|Q12536;O.1;o1.E;|Q12600;|Q12537;O.1;o1.E;|Q12538;O.1;o1.O;|Q12539;|Q12540;O.1;o1.O;|Q12541;O.1;o1.O;|Q12542;|Q12543;O.1;o1.I;|Q12544;O.1;o1.O;|Q12545;O.1;o1.O;|Q12546;O.4;o1.O;o2.M;o3.M;o4.M;|Q12547;O.1;o1.I;|Q12548;|Q12549;O.1;o1.M;|Q12550;O.1;o1.O;|Q12551;O.1;o1.M;|Q12552;O.1;o1.M;|Q12553;O.1;o1.I;|Q12554;O.1;o1.I;|Q12555;O.1;o1.O;|Q12556;O.1;o1.I;|Q12557;O.4;o1.O;o2.O;o3.O;o4.O;|Q12558;O.1;o1.I;|Q12559;O.1;o1.O;|Q12560;O.1;o1.I;|Q12561;O.2;o1.M;o2.M;|Q12562;O.1;o1.M;|Q12563;O.4;o1.O;o2.O;o3.O;o4.O;|Q12564;O.1;o1.I;|Q12501;O.4;o1.O;o2.O;o3.O;o4.O;|Q12565;O.1;o1.I;|Q12502;O.3;o1.O;o2.O;o3.O;|Q12566;|Q12503;O.1;o1.O;|Q12567;O.1;o1.I;|Q12504;O.1;o1.O;|Q12568;O.1;o1.O;|Q12569;O.1;o1.I;|Q12570;O.1;o1.E;|Q12506;O.1;o1.E;|Q12507;|Q12571;O.2;o1.M;o2.M;|Q12508;O.1;o1.O;|Q12572;O.1;o1.I;|Q12509;O.1;o1.O;|Q12573;O.1;o1.E;|Q12510;O.1;o1.I;|Q12574;|Q12511;|Q12575;O.2;o1.M;o2.I;|Q12512;O.3;o1.O;o2.O;o3.O;|Q12576;O.2;o1.M;o2.M;|Q12513;O.1;o1.I;|Q12577;|Q12514;O.1;o1.I;|Q12578;O.1;o1.E;|Q12515;O.1;o1.I;|Q12579;O.1;o1.I;|Q12516;O.1;o1.M;|Q12580;O.1;o1.O;|Q12581;O.1;o1.M;|Q12518;|Q12582;|Q12519;O.1;o1.I;|Q12583;O.1;o1.I;|Q12520;O.1;o1.M;|Q12584;O.1;o1.I;|Q12521;|Q12585;O.1;o1.I;|Q12522;O.1;o1.I;|Q12586;|Q12523;O.1;o1.I;|Q12587;O.4;o1.O;o2.O;o3.O;o4.O;|Q12517;|Q12524;O.1;o1.O;|Q12588;O.1;o1.O;|Q12525;O.2;o1.M;o2.M;|Q12589;O.1;o1.O;|Q12526;|Q12590;|Q12505;|Q12527;O.1;o1.I;|Q12591;O.1;o1.O;|Q12528;|Q12592;O.1;o1.O;|Q12529;O.2;o1.M;o2.M;|Q12593;|Q12530;O.1;o1.M;|Q12594;O.1;o1.O;|", -- [126]
	"|Q12659;O.1;o1.O;|Q12660;O.1;o1.O;|Q12661;O.1;o1.O;|Q12662;O.1;o1.M;|Q12663;O.1;o1.O;|Q12664;O.1;o1.O;|Q12601;O.1;o1.O;|Q12665;O.1;o1.E;|Q12602;O.1;o1.O;|Q12666;O.1;o1.I;|Q12603;O.1;o1.M;|Q12667;|Q12604;|Q12668;O.1;o1.O;|Q12605;O.1;o1.I;|Q12669;O.2;o1.O;o2.O;|Q12606;O.1;o1.O;|Q12670;|Q12607;O.1;o1.O;|Q12671;O.1;o1.E;|Q12608;|Q12672;O.1;o1.I;|Q12609;O.1;o1.I;|Q12673;O.1;o1.O;|Q12610;O.1;o1.I;|Q12674;O.3;o1.O;o2.O;o3.O;|Q12611;O.1;o1.M;|Q12675;|Q12612;|Q12676;O.3;o1.O;o2.O;o3.O;|Q12613;O.1;o1.O;|Q12677;O.2;o1.O;o2.I;|Q12614;O.1;o1.M;|Q12678;O.2;o1.O;o2.O;|Q12615;O.1;o1.I;|Q12679;O.1;o1.I;|Q12616;|Q12680;O.1;o1.O;|Q12617;O.1;o1.M;|Q12681;O.1;o1.I;|Q12618;O.1;o1.I;|Q12682;|Q12619;O.1;o1.I;|Q12683;O.2;o1.O;o2.M;|Q12684;O.1;o1.I;|Q12620;O.1;o1.O;|Q12621;O.1;o1.O;|Q12685;O.1;o1.M;|Q12622;O.3;o1.I;o2.I;o3.I;|Q12686;O.1;o1.O;|Q12623;|Q12687;O.1;o1.E;|Q12624;O.1;o1.I;|Q12688;O.1;o1.E;|Q12625;|Q12689;|Q12626;|Q12690;O.2;o1.O;o2.O;|Q12627;O.4;o1.O;o2.O;o3.O;o4.O;|Q12691;O.1;o1.I;|Q12628;|Q12692;|Q12629;O.2;o1.I;o2.I;|Q12693;|Q12630;O.1;o1.O;|Q12694;|Q12631;|Q12695;|Q12632;O.1;o1.O;|Q12696;|Q12633;|Q12697;|Q12634;O.3;o1.I;o2.I;o3.I;|Q12698;O.1;o1.O;|Q12635;O.1;o1.I;|Q12699;O.1;o1.I;|Q12636;|Q12700;|Q12637;O.1;o1.O;|Q12638;O.1;o1.O;|Q12639;O.1;o1.I;|Q12640;O.1;o1.O;|Q12641;O.4;o1.O;o2.O;o3.O;o4.O;|Q12642;|Q12643;O.2;o1.I;o2.I;|Q12644;O.1;o1.I;|Q12645;O.3;o1.O;o2.O;o3.O;|Q12647;O.1;o1.M;|Q12648;O.1;o1.I;|Q12649;O.1;o1.I;|Q12650;O.1;o1.I;|Q12651;|Q12646;O.1;o1.I;|Q12652;O.1;o1.O;|Q12653;|Q12654;O.1;o1.I;|Q12655;O.1;o1.I;|Q12656;O.1;o1.I;|Q12657;|Q12658;O.1;o1.I;|", -- [127]
	"|Q12723;|Q12787;|Q12724;O.1;o1.I;|Q12788;|Q12725;|Q12789;|Q12726;O.2;o1.O;o2.O;|Q12790;O.2;o1.O;o2.O;|Q12727;O.1;o1.I;|Q12791;|Q12728;O.1;o1.O;|Q12792;|Q12729;O.2;o1.I;o2.I;|Q12793;|Q12730;O.1;o1.M;|Q12794;|Q12795;|Q12732;O.1;o1.I;|Q12796;|Q12733;O.1;o1.O;|Q12797;|Q12734;O.3;o1.O;o2.O;o3.O;|Q12798;|Q12735;O.3;o1.M;o2.M;o3.M;|Q12799;O.1;o1.I;|Q12736;O.4;o1.O;o2.O;o3.O;o4.O;|Q12800;|Q12737;O.1;o1.O;|Q12738;|Q12739;O.1;o1.M;|Q12740;O.1;o1.O;|Q12741;O.1;o1.I;|Q12742;O.1;o1.M;|Q12743;O.1;o1.M;|Q12744;O.1;o1.M;|Q12786;|Q12722;O.2;o1.O;o2.O;|Q12745;O.1;o1.M;|Q12746;O.1;o1.M;|Q12747;O.1;o1.M;|Q12748;O.1;o1.M;|Q12749;O.1;o1.M;|Q12750;O.1;o1.M;|Q12751;|Q12752;|Q12784;|Q12753;|Q12754;O.2;o1.I;o2.I;|Q12755;|Q12756;|Q12757;|Q12758;O.1;o1.I;|Q12759;O.1;o1.O;|Q12760;O.2;o1.O;o2.I;|Q12761;O.1;o1.O;|Q12782;|Q12762;O.2;o1.O;o2.I;|Q12763;|Q12764;|Q12701;O.1;o1.O;|Q12765;|Q12766;|Q12702;O.1;o1.I;|Q12703;O.1;o1.O;|Q12767;|Q12704;O.1;o1.I;|Q12705;O.1;o1.O;|Q12769;|Q12780;|Q12770;|Q12706;|Q12707;O.1;o1.O;|Q12771;|Q12708;O.1;o1.O;|Q12772;|Q12709;O.1;o1.I;|Q12773;|Q12710;O.1;o1.O;|Q12774;|Q12711;|Q12775;|Q12776;|Q12712;O.1;o1.I;|Q12713;O.1;o1.I;|Q12777;|Q12778;|Q12714;|Q12715;|Q12779;O.2;o1.O;o2.O;|Q12716;O.3;o1.I;o2.I;o3.I;|Q12768;|Q12717;|Q12781;|Q12718;O.1;o1.I;|Q12719;O.2;o1.M;o2.I;|Q12783;|Q12720;O.1;o1.E;|Q12721;O.1;o1.E;|Q12785;|", -- [128]
	"|Q12851;O.2;o1.O;o2.O;|Q12852;O.1;o1.O;|Q12853;|Q12854;O.1;o1.I;|Q12855;O.2;o1.O;o2.M;|Q12856;O.2;o1.O;o2.O;|Q12857;O.1;o1.I;|Q12858;O.1;o1.I;|Q12859;O.1;o1.O;|Q12860;O.1;o1.O;|Q12861;O.1;o1.O;|Q12862;|Q12863;|Q12864;O.1;o1.E;|Q12801;O.1;o1.O;|Q12865;O.1;o1.O;|Q12802;|Q12866;O.2;o1.M;o2.M;|Q12803;|Q12867;O.1;o1.I;|Q12804;O.1;o1.I;|Q12868;O.1;o1.M;|Q12805;O.1;o1.O;|Q12869;O.1;o1.M;|Q12806;|Q12870;O.1;o1.I;|Q12807;O.1;o1.E;|Q12871;|Q12808;|Q12872;O.1;o1.I;|Q12809;|Q12873;|Q12810;O.1;o1.O;|Q12874;O.1;o1.M;|Q12811;|Q12875;|Q12812;|Q12876;O.1;o1.O;|Q12813;O.1;o1.O;|Q12877;O.1;o1.I;|Q12814;O.1;o1.O;|Q12878;|Q12815;O.1;o1.M;|Q12879;|Q12816;O.2;o1.E;o2.I;|Q12880;|Q12817;O.2;o1.E;o2.I;|Q12881;|Q12818;O.1;o1.I;|Q12882;O.1;o1.I;|Q12819;O.1;o1.I;|Q12883;|Q12820;O.1;o1.O;|Q12884;|Q12821;O.2;o1.E;o2.I;|Q12885;|Q12822;O.2;o1.M;o2.M;|Q12886;O.1;o1.O;|Q12823;O.2;o1.O;o2.M;|Q12887;O.1;o1.O;|Q12824;|Q12888;O.2;o1.I;o2.I;|Q12825;|Q12889;|Q12826;|Q12890;|Q12827;O.1;o1.I;|Q12891;O.4;o1.I;o2.I;o3.I;o4.I;|Q12828;O.1;o1.I;|Q12892;O.1;o1.O;|Q12829;O.1;o1.O;|Q12893;O.3;o1.O;o2.O;o3.O;|Q12830;O.1;o1.I;|Q12894;|Q12831;O.1;o1.I;|Q12895;O.1;o1.I;|Q12832;O.1;o1.E;|Q12896;O.1;o1.M;|Q12833;O.1;o1.O;|Q12897;O.1;o1.M;|Q12834;|Q12898;|Q12835;O.2;o1.M;o2.M;|Q12899;|Q12836;O.1;o1.M;|Q12900;O.1;o1.I;|Q12837;|Q12838;O.1;o1.I;|Q12839;|Q12840;O.2;o1.O;o2.O;|Q12841;O.1;o1.I;|Q12842;O.1;o1.E;|Q12843;O.1;o1.O;|Q12844;O.1;o1.I;|Q12845;|Q12846;|Q12847;O.2;o1.M;o2.O;|Q12848;O.1;o1.O;|Q12849;|Q12850;|", -- [129]
	"|Q12915;O.2;o1.O;o2.O;|Q12979;O.1;o1.I;|Q12916;O.1;o1.O;|Q12980;O.1;o1.O;|Q12917;O.1;o1.I;|Q12981;Y.1;L.80;O.1;o1.I;|Q12918;O.3;o1.I;o2.I;o3.I;|Q12982;O.1;o1.O;|Q12919;O.4;o1.O;o2.M;o3.M;o4.M;|Q12983;O.1;o1.O;|Q12920;O.1;o1.O;|Q12984;O.1;o1.M;|Q12921;|Q12985;O.1;o1.I;|Q12922;O.1;o1.I;|Q12986;O.4;o1.O;o2.O;o3.O;o4.O;|Q12987;O.1;o1.O;|Q12924;O.1;o1.O;|Q12988;O.3;o1.O;o2.O;o3.O;|Q12925;O.1;o1.O;|Q12989;O.1;o1.M;|Q12926;O.1;o1.I;|Q12990;O.2;o1.I;o2.I;|Q12927;O.1;o1.O;|Q12991;O.1;o1.M;|Q12928;O.1;o1.I;|Q12992;O.1;o1.O;|Q12929;|Q12993;O.2;o1.I;o2.I;|Q12930;O.2;o1.I;o2.I;|Q12994;O.1;o1.O;|Q12931;O.1;o1.O;|Q12995;O.1;o1.O;|Q12932;O.1;o1.E;|Q12996;O.1;o1.O;|Q12933;O.1;o1.E;|Q12997;O.1;o1.M;|Q12934;O.1;o1.E;|Q12998;O.1;o1.O;|Q12935;O.1;o1.E;|Q12999;|Q12936;O.1;o1.E;|Q13000;O.1;o1.I;|Q12937;O.1;o1.O;|Q12938;|Q12939;O.1;o1.O;|Q12940;|Q12941;|Q12942;O.1;o1.M;|Q12943;O.1;o1.M;|Q12944;|Q12945;|Q12946;|Q12947;|Q12948;O.1;o1.E;|Q12949;O.1;o1.I;|Q12950;|Q12951;|Q12952;O.3;o1.I;o2.I;o3.I;|Q12953;O.1;o1.O;|Q12954;O.1;o1.E;|Q12955;O.4;o1.O;o2.O;o3.O;o4.O;|Q12956;|Q12957;O.2;o1.O;o2.M;|Q12958;Y.1;O.1;o1.I;|Q12959;Y.1;O.1;o1.I;L.80;|Q12960;Y.1;O.1;o1.I;|Q12961;O.1;o1.I;|Q12962;Y.1;O.1;o1.I;L.80;|Q12963;Y.1;O.1;o1.I;|Q12964;O.1;o1.I;|Q12901;O.1;o1.I;|Q12902;|Q12966;|Q12903;O.3;o1.O;o2.O;o3.O;|Q12967;O.1;o1.O;|Q12904;O.1;o1.O;|Q12968;O.1;o1.M;|Q12905;|Q12969;O.1;o1.M;|Q12906;O.1;o1.O;|Q12970;O.1;o1.O;|Q12907;O.1;o1.M;|Q12971;O.1;o1.M;|Q12965;O.3;o1.O;o2.O;o3.O;|Q12908;|Q12972;|Q12909;|Q12973;O.1;o1.E;|Q12910;O.2;o1.O;o2.M;|Q12974;T.G.4;L.77;|Q12911;O.1;o1.O;|Q12975;O.1;o1.I;|Q12912;|Q12976;|Q12913;|Q12977;O.2;o1.M;o2.M;Y.1;L.80;|Q12914;O.2;o1.I;o2.I;|Q12978;O.1;o1.O;|", -- [130]
	"|Q13043;O.1;o1.M;|Q13044;|Q13045;O.1;o1.O;|Q13046;O.1;o1.O;|Q13047;O.1;o1.E;|Q13048;O.1;o1.O;|Q13049;O.2;o1.I;o2.I;|Q13050;O.1;o1.I;|Q13051;O.1;o1.O;|Q13052;|Q13053;O.1;o1.E;|Q13054;|Q13055;O.2;o1.I;o2.I;|Q13056;O.1;o1.M;|Q13057;|Q13058;O.1;o1.O;|Q13059;O.2;o1.O;o2.M;|Q13060;|Q13061;|Q13062;|Q13063;|Q13064;O.1;o1.O;|Q13042;O.2;o1.O;o2.M;|Q13001;O.2;o1.I;o2.I;|Q13065;|Q13002;O.3;o1.I;o2.I;o3.I;|Q13066;|Q13003;O.1;o1.O;|Q13067;|Q13004;O.3;o1.I;o2.I;o3.I;|Q13068;|Q13005;O.2;o1.M;o2.M;L.80;|Q13069;O.1;o1.O;|Q13006;Y.1;O.1;o1.I;L.80;|Q13070;|Q13007;O.1;o1.M;|Q13071;O.1;o1.O;|Q13008;O.1;o1.O;|Q13072;|Q13009;|Q13073;|Q13010;O.2;o1.O;o2.R;|Q13074;O.1;o1.I;|Q13011;O.1;o1.M;|Q13075;|Q13012;|Q13076;|Q13013;|Q13077;|Q13014;|Q13078;O.1;o1.I;|Q13015;|Q13079;|Q13016;|Q13080;|Q13017;|Q13081;|Q13018;|Q13082;|Q13019;|Q13083;|Q13020;|Q13084;O.1;o1.O;|Q13021;|Q13085;|Q13086;O.2;o1.O;o2.M;|Q13022;|Q13023;|Q13087;O.1;o1.I;|Q13024;|Q13088;O.1;o1.I;|Q13025;|Q13089;O.1;o1.I;|Q13026;|Q13090;O.1;o1.I;|Q13027;|Q13091;O.1;o1.O;|Q13092;O.1;o1.I;|Q13028;|Q13029;|Q13093;O.1;o1.I;|Q13030;|Q13094;O.1;o1.I;|Q13031;|Q13095;O.1;o1.I;|Q13032;|Q13096;O.1;o1.O;|Q13033;|Q13097;|Q13034;O.3;o1.O;o2.O;o3.O;|Q13098;O.1;o1.I;|Q13035;O.3;o1.M;o2.M;o3.M;L.80;|Q13099;|Q13036;|Q13100;Y.1;O.1;o1.I;L.80;|Q13037;O.1;o1.O;|Q13038;O.1;o1.O;|Q13039;O.1;o1.O;|Q13040;O.1;o1.I;|Q13041;O.1;o1.I;|", -- [131]
	"|Q13107;Y.1;O.1;o1.I;|Q13171;O.1;o1.O;|Q13108;O.1;o1.I;|Q13172;O.1;o1.O;|Q13109;O.1;o1.O;|Q13173;|Q13110;O.1;o1.O;|Q13174;O.4;o1.I;o2.I;o3.I;o4.I;|Q13111;O.1;o1.I;|Q13175;|Q13112;O.1;o1.I;|Q13176;|Q13113;O.2;o1.I;o2.I;|Q13177;T.P;O.1;o1.O;|Q13114;O.1;o1.I;|Q13178;O.1;o1.O;|Q13115;O.1;o1.I;|Q13179;O.1;o1.O;|Q13116;O.1;o1.I;|Q13180;O.1;o1.O;|Q13117;O.1;o1.O;|Q13181;T.P;O.1;o1.E;|Q13118;O.3;o1.M;o2.M;o3.M;|Q13182;O.1;o1.O;|Q13119;O.4;o1.O;o2.O;o3.O;o4.O;|Q13183;O.1;o1.E;|Q13120;O.3;o1.O;o2.O;o3.O;|Q13184;|Q13121;O.1;o1.O;|Q13185;O.1;o1.O;|Q13122;O.1;o1.I;|Q13186;T.P;O.1;o1.O;|Q13123;|Q13187;O.2;o1.O;o2.O;|Q13124;|Q13188;|Q13125;O.3;o1.M;o2.M;o3.M;|Q13189;|Q13126;O.2;o1.O;o2.O;|Q13190;T.H;Y.1;L.80;O.1;o1.M;|Q13127;O.1;o1.O;|Q13191;O.1;o1.I;|Q13128;O.1;o1.O;|Q13192;O.1;o1.I;|Q13170;O.1;o1.O;|Q13129;O.1;o1.I;|Q13193;O.1;o1.I;|Q13130;O.2;o1.I;o2.I;|Q13194;O.1;o1.I;|Q13131;O.4;o1.I;o2.I;o3.I;o4.I;T.D;|Q13195;O.1;o1.I;|Q13132;T.D;O.1;o1.O;|Q13196;O.1;o1.I;|Q13133;O.1;o1.O;|Q13197;O.1;o1.I;|Q13134;O.2;o1.O;o2.O;|Q13198;O.1;o1.I;|Q13135;O.1;o1.I;|Q13199;O.1;o1.I;|Q13136;O.1;o1.I;|Q13200;O.1;o1.I;|Q13168;O.1;o1.E;|Q13137;O.1;o1.O;|Q13138;O.1;o1.I;|Q13139;|Q13140;O.1;o1.I;|Q13141;O.1;o1.E;|Q13142;O.1;o1.M;|Q13143;O.1;o1.O;|Q13144;O.1;o1.O;|Q13145;O.4;o1.O;o2.O;o3.O;o4.O;|Q13166;O.2;o1.M;o2.O;|Q13146;O.1;o1.O;|Q13147;O.1;o1.O;|Q13148;O.1;o1.I;|Q13149;T.D;O.1;o1.O;|Q13150;|Q13151;T.D;O.1;o1.O;|Q13152;O.2;o1.O;o2.O;|Q13153;O.1;o1.I;|Q13164;O.1;o1.M;|Q13154;O.1;o1.I;|Q13155;|Q13156;O.1;o1.I;|Q13157;|Q13158;|Q13159;O.1;o1.O;|Q13160;O.1;o1.O;|Q13161;O.1;o1.M;|Q13162;O.1;o1.M;|Q13163;O.1;o1.M;|Q13101;O.2;o1.I;o2.I;|Q13165;|Q13102;Y.1;O.1;o1.I;L.80;|Q13103;Y.1;O.1;o1.I;L.80;|Q13167;O.1;o1.I;|Q13104;|Q13105;|Q13169;O.1;o1.O;|Q13106;|", -- [132]
	"|Q13235;O.1;o1.M;|Q13299;O.1;o1.I;|Q13236;O.1;o1.O;|Q13300;O.1;o1.O;|Q13237;O.3;o1.O;o2.O;o3.O;|Q13238;|Q13239;O.1;o1.O;|Q13240;O.1;o1.M;|Q13241;O.1;o1.M;|Q13242;O.1;o1.I;|Q13243;Y.1;T.D;O.1;o1.M;|Q13244;T.D;Y.1;L.80;O.1;o1.M;|Q13245;O.1;o1.I;|Q13246;O.1;o1.I;|Q13247;O.1;o1.I;|Q13248;O.1;o1.I;|Q13249;O.1;o1.I;|Q13250;O.1;o1.I;|Q13251;O.1;o1.I;|Q13252;Y.1;T.H;O.1;o1.I;|Q13253;O.1;o1.I;|Q13254;O.1;o1.I;|Q13255;Y.1;T.H;O.1;o1.I;|Q13257;|Q13258;|Q13259;O.1;o1.M;|Q13260;|Q13261;O.1;o1.O;|Q13262;|Q13263;|Q13264;O.3;o1.O;o2.O;o3.O;|Q13201;O.1;o1.I;|Q13265;|Q13202;O.1;o1.I;|Q13266;|Q13203;|Q13267;|Q13204;O.1;o1.I;|Q13268;|Q13205;O.1;o1.I;|Q13269;|Q13206;O.1;o1.I;|Q13270;|Q13207;T.D;O.1;o1.O;|Q13271;|Q13272;|Q13273;O.2;o1.I;o2.I;|Q13210;|Q13274;O.1;o1.I;|Q13211;O.1;o1.O;|Q13275;O.1;o1.I;|Q13212;O.4;o1.I;o2.I;o3.I;o4.I;|Q13276;O.3;o1.O;o2.O;o3.O;|Q13213;|Q13277;O.1;o1.I;|Q13214;O.1;o1.E;|Q13278;O.1;o1.O;|Q13215;O.1;o1.E;|Q13279;O.1;o1.O;|Q13216;O.1;o1.E;|Q13280;O.1;o1.O;|Q13217;O.1;o1.E;|Q13281;O.1;o1.O;|Q13292;O.1;o1.O;|Q13282;|Q13218;O.1;o1.E;|Q13219;O.1;o1.E;|Q13283;O.1;o1.O;|Q13220;O.1;o1.O;|Q13284;O.1;o1.O;|Q13221;O.1;o1.E;|Q13285;O.1;o1.O;|Q13222;O.1;o1.O;|Q13286;|Q13223;O.1;o1.O;|Q13287;O.3;o1.O;o2.O;o3.O;|Q13224;|Q13288;O.3;o1.O;o2.O;o3.O;|Q13225;|Q13289;O.3;o1.O;o2.O;o3.O;|Q13290;|Q13226;|Q13227;|Q13291;O.1;o1.O;|Q13228;O.1;o1.O;|Q13229;O.1;o1.E;|Q13293;|Q13230;O.1;o1.O;|Q13294;O.1;o1.I;|Q13231;O.1;o1.O;|Q13295;O.1;o1.O;|Q13232;O.1;o1.O;|Q13296;|Q13233;O.1;o1.P;|Q13297;O.1;o1.O;|Q13234;O.1;o1.P;|Q13298;O.1;o1.O;|Q13256;O.1;o1.I;|", -- [133]
	"|Q13363;|Q13364;O.1;o1.O;|Q13301;O.1;o1.O;|Q13365;O.1;o1.O;|Q13302;O.1;o1.O;|Q13366;O.1;o1.I;|Q13303;O.1;o1.I;|Q13367;O.1;o1.O;|Q13304;O.1;o1.I;|Q13368;O.1;o1.O;|Q13305;O.3;o1.M;o2.M;o3.M;|Q13369;|Q13306;O.1;o1.O;|Q13370;|Q13307;O.3;o1.M;o2.M;o3.M;|Q13371;|Q13308;O.1;o1.I;|Q13372;|Q13309;O.1;o1.O;|Q13373;O.3;o1.M;o2.M;o3.M;|Q13310;O.1;o1.O;|Q13374;O.4;o1.M;o2.M;o3.M;o4.M;|Q13311;|Q13375;|Q13312;O.1;o1.M;|Q13376;O.4;o1.M;o2.M;o3.M;o4.M;|Q13313;O.1;o1.O;|Q13377;O.1;o1.E;|Q13314;O.1;o1.I;|Q13378;|Q13315;O.4;o1.O;o2.O;o3.O;o4.O;|Q13379;|Q13316;O.2;o1.M;o2.M;|Q13380;O.3;o1.M;o2.M;o3.M;|Q13317;|Q13381;O.4;o1.M;o2.M;o3.M;o4.M;|Q13318;O.1;o1.O;|Q13382;O.4;o1.M;o2.M;o3.M;o4.M;|Q13319;O.4;o1.O;o2.O;o3.O;o4.O;|Q13383;|Q13320;O.3;o1.O;o2.O;o3.O;|Q13384;O.1;o1.I;|Q13362;|Q13321;O.1;o1.O;|Q13385;O.1;o1.I;|Q13322;O.1;o1.O;|Q13386;|Q13323;O.1;o1.O;|Q13387;O.1;o1.M;|Q13324;|Q13388;|Q13325;|Q13389;|Q13326;|Q13390;|Q13327;|Q13391;O.1;o1.I;|Q13328;O.3;o1.I;o2.I;o3.I;|Q13392;|Q13329;O.1;o1.O;|Q13393;O.1;o1.I;|Q13330;O.1;o1.O;|Q13394;O.3;o1.M;o2.M;o3.M;|Q13331;O.1;o1.O;|Q13395;O.1;o1.O;|Q13332;O.1;o1.O;|Q13396;|Q13333;O.1;o1.I;|Q13397;O.3;o1.M;o2.M;o3.M;|Q13334;O.3;o1.M;o2.M;o3.M;|Q13398;O.1;o1.I;|Q13335;O.1;o1.O;|Q13399;|Q13336;O.1;o1.O;|Q13400;O.1;o1.O;|Q13337;O.1;o1.M;|Q13338;O.2;o1.M;o2.M;|Q13339;O.3;o1.I;o2.I;o3.I;|Q13340;|Q13341;|Q13342;O.1;o1.O;|Q13343;O.1;o1.E;|Q13344;O.1;o1.O;|Q13345;O.1;o1.I;|Q13346;O.1;o1.O;|Q13347;O.1;o1.I;|Q13348;|Q13349;O.3;o1.M;o2.M;o3.M;|Q13350;O.1;o1.O;|Q13351;O.4;o1.O;o2.O;o3.O;o4.O;|Q13352;O.1;o1.O;|Q13353;O.1;o1.O;|Q13354;O.4;o1.O;o2.O;o3.O;o4.O;|Q13355;O.3;o1.O;o2.O;o3.O;|Q13356;O.1;o1.O;|Q13357;O.1;o1.O;|Q13358;O.1;o1.O;|Q13359;O.1;o1.I;|Q13360;|Q13361;O.1;o1.O;|", -- [134]
	"|Q13427;O.1;o1.E;|Q13491;|Q13428;O.1;o1.E;|Q13492;|Q13429;O.1;o1.E;|Q13493;|Q13430;O.1;o1.M;|Q13494;|Q13431;O.2;o1.I;o2.I;|Q13495;|Q13432;O.2;o1.I;o2.I;|Q13496;|Q13433;|Q13497;|Q13434;|Q13498;|Q13435;|Q13499;|Q13436;|Q13500;|Q13437;|Q13438;|Q13439;|Q13440;|Q13441;|Q13442;|Q13443;|Q13444;|Q13445;|Q13446;|Q13447;|Q13448;|Q13449;|Q13450;|Q13451;|Q13452;|Q13453;|Q13454;|Q13455;|Q13456;|Q13457;|Q13458;|Q13459;|Q13460;|Q13461;|Q13462;|Q13463;|Q13464;|Q13401;|Q13465;|Q13422;Y.1;O.1;o1.O;|Q13402;|Q13466;|Q13403;O.1;o1.O;|Q13467;|Q13404;O.3;o1.M;o2.M;o3.M;|Q13468;|Q13405;Y.1;T.P;O.1;o1.E;|Q13469;|Q13406;O.3;o1.M;o2.M;o3.M;|Q13470;|Q13407;O.1;o1.E;|Q13471;|Q13408;O.3;o1.O;o2.O;o3.O;|Q13472;|Q13409;O.3;o1.O;o2.O;o3.O;|Q13473;|Q13420;|Q13410;O.3;o1.O;o2.O;o3.O;|Q13474;|Q13411;O.3;o1.O;o2.O;o3.O;|Q13475;O.6;o1.I;o2.I;o3.I;o4.I;o5.I;o6.I;|Q13412;|Q13476;O.6;o1.I;o2.I;o3.I;o4.I;o5.I;o6.I;|Q13413;O.1;o1.O;|Q13477;O.6;o1.I;o2.I;o3.I;o4.I;o5.I;o6.I;|Q13414;O.1;o1.O;|Q13478;O.6;o1.I;o2.I;o3.I;o4.I;o5.I;o6.I;|Q13415;|Q13479;O.1;o1.I;|Q13416;|Q13480;O.1;o1.I;|Q13417;O.1;o1.E;|Q13481;O.1;o1.E;|Q13482;O.1;o1.E;|Q13418;|Q13419;|Q13483;|Q13484;|Q13421;O.1;o1.I;|Q13485;|Q13486;|Q13423;Y.1;O.1;o1.M;|Q13487;|Q13424;Y.1;O.1;o1.M;|Q13488;|Q13425;O.1;o1.O;|Q13489;|Q13426;|Q13490;|", -- [135]
	"|Q13556;O.1;o1.I;|Q13559;O.1;o1.I;|Q13501;|Q13502;O.1;o1.I;|Q13503;O.1;o1.I;|Q13571;|Q13524;O.1;o1.O;|Q13592;O.1;o1.M;|Q13593;|Q13600;O.1;o1.I;|Q13538;O.1;o1.O;|Q13539;O.1;o1.O;|Q13548;|Q13549;O.2;o1.O;o2.O;|", -- [136]
	"|Q13684;|Q13685;|Q13622;|Q13686;|Q13687;|Q13688;|Q13625;O.3;o1.O;o2.O;o3.O;|Q13689;|Q13690;|Q13627;O.1;o1.I;|Q13691;|Q13692;O.1;o1.I;|Q13629;O.1;o1.I;|Q13693;|Q13694;|Q13631;|Q13695;|Q13696;|Q13633;O.1;o1.I;|Q13697;O.1;o1.I;|Q13634;O.1;o1.I;|Q13699;O.1;o1.O;|Q13700;|Q13682;O.2;o1.M;o2.M;|Q13641;O.1;o1.I;|Q13643;O.3;o1.O;o2.O;o3.O;|Q13649;O.1;o1.O;|Q13654;O.1;o1.I;|Q13662;O.1;o1.I;|Q13663;O.3;o1.O;o2.I;o3.I;|Q13664;O.1;o1.M;|Q13665;O.1;o1.I;|Q13666;O.1;o1.I;|Q13603;O.1;o1.I;|Q13667;|Q13604;|Q13668;|Q13669;O.1;o1.I;|Q13606;O.1;o1.I;|Q13670;O.1;o1.I;|Q13607;O.1;o1.E;|Q13671;O.1;o1.O;|Q13672;O.1;o1.I;|Q13609;O.1;o1.I;|Q13673;O.1;o1.I;|Q13610;O.1;o1.I;|Q13674;O.1;o1.I;|Q13611;O.1;o1.I;|Q13675;O.1;o1.I;|Q13676;O.1;o1.O;|Q13677;O.3;o1.O;o2.O;o3.O;|Q13614;|Q13678;O.1;o1.I;|Q13679;O.1;o1.O;|Q13616;O.1;o1.I;|Q13680;O.1;o1.O;|Q13681;O.1;o1.I;|", -- [137]
	"|Q13747;O.1;o1.I;|Q13748;O.1;o1.I;|Q13749;O.1;o1.M;|Q13750;O.1;o1.I;|Q13752;O.1;o1.I;|Q13753;Y.1;O.1;o1.I;|Q13754;Y.1;O.1;o1.I;|Q13755;Y.1;O.1;o1.M;|Q13756;Y.1;O.1;o1.I;|Q13757;O.1;o1.I;|Q13758;O.1;o1.I;|Q13759;O.1;o1.I;|Q13760;O.1;o1.M;|Q13761;O.1;o1.I;|Q13762;O.1;o1.I;|Q13763;O.1;o1.I;|Q13764;O.1;o1.I;|Q13701;|Q13765;O.1;o1.M;|Q13702;|Q13703;|Q13767;O.1;o1.I;|Q13704;|Q13768;O.1;o1.I;|Q13705;|Q13769;O.1;o1.I;|Q13706;|Q13770;O.1;o1.I;|Q13707;|Q13771;O.1;o1.M;|Q13708;|Q13772;O.1;o1.I;|Q13709;|Q13773;O.1;o1.I;|Q13710;|Q13774;O.1;o1.I;|Q13711;|Q13775;O.1;o1.I;|Q13776;O.1;o1.M;|Q13777;O.1;o1.I;|Q13713;O.1;o1.O;|Q13714;O.1;o1.I;|Q13778;O.1;o1.I;|Q13715;O.1;o1.I;|Q13779;O.1;o1.I;|Q13716;O.1;o1.I;|Q13780;O.1;o1.I;|Q13717;O.1;o1.I;|Q13781;O.1;o1.M;|Q13718;O.1;o1.I;|Q13782;O.1;o1.I;|Q13719;O.1;o1.I;|Q13783;O.1;o1.I;|Q13720;O.1;o1.I;|Q13784;O.1;o1.I;|Q13721;O.1;o1.I;|Q13785;O.1;o1.I;|Q13722;O.1;o1.I;|Q13786;O.1;o1.M;|Q13723;O.1;o1.O;|Q13787;O.1;o1.I;|Q13724;O.1;o1.O;|Q13788;O.2;o1.M;o2.M;|Q13789;O.1;o1.O;|Q13725;O.1;o1.O;|Q13726;O.1;o1.O;|Q13790;O.1;o1.I;|Q13727;O.1;o1.O;|Q13791;O.1;o1.O;|Q13728;O.1;o1.O;|Q13729;O.1;o1.O;|Q13793;O.1;o1.I;|Q13794;|Q13731;O.1;o1.O;|Q13795;|Q13732;|Q13733;|Q13734;|Q13735;|Q13736;|Q13737;|Q13738;|Q13739;|Q13740;|Q13741;O.1;o1.I;|Q13742;O.1;o1.I;|Q13743;O.1;o1.I;|Q13744;O.1;o1.M;|Q13745;O.1;o1.I;|Q13746;O.1;o1.I;|", -- [138]
	"|Q13811;O.1;o1.I;|Q13812;O.2;o1.M;o2.M;|Q13813;O.1;o1.O;|Q13814;O.1;o1.I;|Q13816;O.1;o1.E;|Q13817;|Q13818;|Q13819;|Q13820;|Q13821;O.1;o1.I;|Q13822;O.1;o1.I;|Q13823;O.1;o1.I;|Q13887;O.1;o1.I;|Q13824;O.1;o1.I;|Q13825;|Q13889;O.1;o1.O;|Q13826;|Q13827;O.1;o1.I;|Q13828;O.2;o1.O;o2.O;|Q13829;O.2;o1.O;o2.O;|Q13830;Y.1;O.1;o1.E;|Q13832;Y.1;O.1;o1.I;|Q13833;O.1;o1.I;|Q13834;O.1;o1.I;|Q13835;O.2;o1.O;o2.O;|Q13836;O.1;o1.I;|Q13837;O.2;o1.O;o2.O;|Q13838;O.2;o1.O;o2.O;|Q13839;O.2;o1.O;o2.O;|Q13840;O.1;o1.I;|Q13843;|Q13845;|Q13846;|Q13847;O.3;o1.M;o2.M;o3.M;|Q13850;O.1;o1.O;|Q13851;O.3;o1.M;o2.M;o3.M;|Q13852;O.3;o1.M;o2.M;o3.M;|Q13854;Y.1;O.3;o1.M;o2.M;o3.M;|Q13855;O.3;o1.M;o2.M;o3.M;|Q13856;O.3;o1.M;o2.M;o3.M;|Q13857;O.3;o1.M;o2.M;o3.M;|Q13858;O.3;o1.M;o2.M;o3.M;|Q13859;O.3;o1.M;o2.M;o3.M;|Q13860;O.3;o1.M;o2.M;o3.M;|Q13861;O.1;o1.M;|Q13862;O.1;o1.M;|Q13863;O.1;o1.M;|Q13864;O.1;o1.M;|Q13807;|Q13809;O.2;o1.M;o2.M;|Q13810;O.1;o1.O;|", -- [139]
	"|Q13950;O.1;o1.E;|Q13951;O.1;o1.E;|Q13952;O.1;o1.I;|Q13954;O.1;o1.E;|Q13955;O.1;o1.E;|Q13956;O.1;o1.E;|Q13957;O.1;o1.E;|Q13959;O.1;o1.I;|Q13960;O.1;o1.I;|Q13966;|Q13903;O.1;o1.O;|Q13904;O.1;o1.O;|Q13905;O.1;o1.I;|Q13906;O.3;o1.I;o2.I;o3.I;|Q13908;O.2;o1.I;o2.I;|Q13914;O.1;o1.I;|Q13915;O.1;o1.O;|Q13916;O.1;o1.O;|Q13917;O.1;o1.O;|Q13933;O.1;o1.E;|Q13986;|Q13990;|Q13927;O.1;o1.O;|Q13929;O.1;o1.E;|Q13930;O.1;o1.E;|Q13926;O.1;o1.O;|Q13931;|Q13932;|Q13997;O.1;o1.M;|Q13934;O.1;o1.E;|Q13937;O.1;o1.O;|Q13938;O.1;o1.O;|", -- [140]
	"|Q14074;O.1;o1.I;|Q14076;O.1;o1.I;|Q14077;O.1;o1.O;|Q14079;|Q14016;O.1;o1.E;|Q14080;O.1;o1.O;|Q14017;O.1;o1.I;|Q14081;|Q14082;|Q14083;|Q14084;|Q14085;|Q14022;|Q14086;|Q14023;O.2;o1.O;o2.I;|Q14087;|Q14024;O.2;o1.O;o2.I;|Q14088;|Q14089;|Q14090;O.1;o1.O;|Q14028;O.2;o1.O;o2.I;|Q14092;O.1;o1.I;|Q14030;O.2;o1.I;o2.I;|Q14095;|Q14032;O.2;o1.M;o2.M;|Q14096;O.2;o1.O;o2.O;|Q14033;O.2;o1.O;o2.I;|Q14035;O.2;o1.O;o2.I;|Q14036;|Q14100;O.4;o1.I;o2.I;o3.I;o4.I;|Q14037;O.2;o1.O;o2.I;|Q14040;O.2;o1.O;o2.I;|Q14041;O.2;o1.O;o2.I;|Q14043;O.2;o1.O;o2.I;|Q14047;O.2;o1.O;o2.I;|Q14048;O.1;o1.I;|Q14051;O.1;o1.I;|Q14054;O.1;o1.I;|Q14053;O.1;o1.I;|Q14055;O.1;o1.I;|Q14058;O.1;o1.I;|Q14059;O.1;o1.I;|Q14060;O.1;o1.I;|Q14061;O.1;o1.I;|Q14044;O.2;o1.I;o2.I;|Q14062;O.1;o1.I;|Q14064;O.1;o1.O;|Q14065;O.1;o1.O;|", -- [141]
	"|Q14199;T.H;Y.1;L.80;O.1;o1.I;|Q14136;O.2;o1.M;o2.M;|Q14200;|Q14140;O.1;o1.O;|Q14141;O.1;o1.O;|Q14142;O.2;o1.O;o2.O;|Q14143;O.1;o1.I;|Q14144;O.1;o1.O;|Q14145;O.1;o1.I;|Q14147;O.1;o1.I;|Q14148;O.1;o1.I;|Q14149;O.1;o1.I;|Q14150;O.1;o1.I;|Q14151;O.1;o1.O;|Q14152;O.2;o1.M;o2.M;|Q14160;|Q14163;O.1;o1.E;|Q14164;O.1;o1.E;|Q14101;O.1;o1.M;|Q14102;O.1;o1.M;|Q14166;O.1;o1.I;|Q14103;O.1;o1.I;|Q14167;O.1;o1.I;|Q14104;O.1;o1.M;|Q14168;O.1;o1.I;|Q14169;O.1;o1.I;|Q14105;O.1;o1.M;|Q14106;O.8;o1.M;o2.M;o3.I;o4.I;o5.I;o6.I;o7.I;o8.I;|Q14170;O.1;o1.I;|Q14107;O.1;o1.O;|Q14171;O.1;o1.I;|Q14108;O.2;o1.O;o2.M;|Q14172;O.1;o1.I;|Q14173;O.1;o1.I;|Q14174;O.1;o1.I;|Q14111;O.4;o1.I;o2.I;o3.I;o4.I;|Q14175;O.1;o1.I;|Q14112;O.1;o1.I;|Q14176;O.1;o1.I;|Q14177;O.1;o1.I;|Q14178;O.1;o1.E;|Q14179;O.1;o1.E;|Q14180;O.1;o1.E;|Q14181;O.1;o1.E;|Q14182;O.1;o1.E;|Q14119;O.2;o1.I;o2.I;|Q14183;O.1;o1.E;|Q14185;|Q14186;|Q14187;|", -- [142]
	"|Q14203;|", -- [143]
	[145] = "|Q14409;|Q14436;O.1;o1.I;|Q14437;|Q14438;O.3;o1.M;o2.M;o3.M;|",
	[243] = "|Q24216;O.1;o1.E;|Q24217;O.1;o1.E;|Q24218;O.1;o1.E;|Q24220;O.1;o1.E;|Q24222;O.1;o1.E;|Q24223;O.1;o1.E;|Q24225;O.1;o1.E;|Q24224;O.1;o1.E;|Q24226;O.1;o1.E;|Q24221;O.1;o1.E;|Q24219;O.1;o1.E;|Q24227;O.1;o1.E;|",
	[245] = "|Q24426;O.1;o1.E;|Q24427;O.1;o1.E;|Q24428;|Q24431;|Q24442;O.1;o1.I;|Q24429;|",
}


local _G = _G
local GetQuestLogTitle = _G.GetQuestLogTitle
local GetQuestLogSelection = _G.GetQuestLogSelection
local GetNumQuestLogEntries = _G.GetNumQuestLogEntries
local GetNumQuestLeaderBoards = _G.GetNumQuestLeaderBoards
local table = table
local string = string
local tonumber = tonumber
local tostring = tostring

local new, del, deepDel, doAll
do
	local list = setmetatable({}, {__mode='k'})
	function new(...)
		local t = next(list)
		if t then
			list[t] = nil
			for i = 1, select('#', ...) do
				t[i] = select(i, ...)
			end
			return t
		else
			return { ... }
		end
	end
	function del(t)
		setmetatable(t, nil)
		table.wipe(t)
		list[t] = true
		return nil
	end
	function deepDel(t)
		if type(t) ~= "table" then
			return nil
		end
		for k,v in pairs(t) do
			t[k] = deepDel(v)
		end
		return del(t)
	end
	function doAll(func, ...)
		for i=1, select('#', ...) do
			func(select(i, ...))
		end
	end
end

local frame
if lib.frame then
	frame = lib.frame
	frame:UnregisterAllEvents()
	frame:SetScript("OnEvent", nil)
	frame:SetScript("OnUpdate", nil)
else
	frame = CreateFrame("Frame", MAJOR_VERSION .. "_Frame")
	lib.frame = frame
end

frame:RegisterEvent("QUEST_LOG_UPDATE")
--frame:RegisterEvent("CHAT_MSG_ADDON") -- unfinished
--frame:RegisterEvent("CHAT_MSG_SYSTEM") -- unfinished
--frame:RegisterEvent("QUEST_COMPLETE") -- unfinished
frame:RegisterEvent("PARTY_MEMBERS_CHANGED") -- unfinished

frame:SetScript("OnEvent", function(this, event, ...)
	if not lib[event] then
		print("Function not found: LibQuestInfo:" .. event .. "()")
		print(...)
	else
		lib[event](...)
	end
end)

-- this is just a throttler for QUEST_LOG_UPDATE
local timeSoFar = 0;
local failload = {};

frame:SetScript("OnUpdate", function(this, timeSinceLast)
	timeSoFar = timeSoFar + timeSinceLast
	local first, found
	if not (timeSoFar > 0.25) then
		return
	end

	local currentid = LibQuestInfoTooltip.currentid

	for i = 1, LibQuestInfoTooltip:NumLines() do
		if not LibQuestInfoTooltip[i] then
			LibQuestInfoTooltip[i] = _G[LibQuestInfoTooltip:GetName() .. "TextLeft" .. i]
		end
	end

	if currentid then
		if not questcache[currentid] then
			questcache[currentid] = {}
		end
		questcache[currentid].title = LibQuestInfoTooltip[1] and LibQuestInfoTooltip[1]:GetText()
		if not questcache[currentid].title then
			failload[currentid] = failload[currentid] and failload[currentid] + 1 or 1;
		end
		if (questcache[currentid].title) ~= nil then
			loadqueue[currentid] = nil
			lib.callbacks:Fire("QuestLoaded", currentid)
		end
	end

	timeSoFar = 0
	local size = 0
	for k in pairs(loadqueue) do
		if failload[currentid] and failload[currentid] > 10 then
			questcache[currentid].title = "Quest doesn't exist (Quest ID: " .. currentid ..")";
			loadqueue[k] = nil;
		else
			size = size + 1
		end
	end

	LibQuestInfoTooltip:ClearLines();
	LibQuestInfoTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");

	if size > 0 then
		for qid in pairs(loadqueue) do
			if not LibQuestInfoTooltip.currentid then
				LibQuestInfoTooltip.currentid = qid
				LibQuestInfoTooltip:SetHyperlink("quest:" .. qid)
				return
			end
			if not first then
				first = qid
			end
			if found then
				LibQuestInfoTooltip.currentid = qid
				LibQuestInfoTooltip:SetHyperlink("quest:" .. qid)
				return
			end

			if qid == LibQuestInfoTooltip.currentid then
				found = true
			end
		end
		LibQuestInfoTooltip.currentid = first
		LibQuestInfoTooltip:SetHyperlink("quest:" .. first)
		return
	else
		this:Hide()
		LibQuestInfoTooltip.currentid = nil
	end
end)

if lib.callbacks then
	lib:UnregisterAll(lib) -- unregisters all callbacks that questinfo is doing on itself (comms stuff, mostly)
	lib:UnhookDialogs()
else
	lib.callbacks = LibStub("CallbackHandler-1.0"):New(lib, nil, nil, "UnregisterAll")
end

do
	local tooltip = lib.tooltip
	local function tooltip_line(link, line)
		if not tooltip then
			tooltip = CreateFrame("GameTooltip", MAJOR_VERSION.."_Tooltip", nil, "GameTooltipTemplate")
			tooltip:SetOwner(UIParent, "ANCHOR_NONE")
			lib.tooltip = tooltip
		end
		tooltip:ClearLines()
		tooltip:SetHyperlink(link)

		if tooltip:NumLines() < line then return false end
		return _G[MAJOR_VERSION.."_TooltipTextLeft"..line]:GetText()
	end
	lib.UID_to_name = setmetatable(lib.UID_to_name or {}, {__index = function(self, key)
		local link = (type(key) == 'string') and key or ('quest:'..key)
		local uid = string.match(link, '%d+')
		local name = tooltip_line(link, 1)
		if name then
			self[uid] = name
			return name
		end
		return false
	end,})
end



--
-- Events
--

local update_is_running

function lib:QUEST_LOG_UPDATE()
	update_is_running = true

	-- The quest log is scanned:
	local selectionId = GetQuestLogSelection()
	local numEntries, numQuests = GetNumQuestLogEntries()
	local zone

	if numEntries > 0 then
		for id = 1, numEntries do
			SelectQuestLogEntry(id)
			local name, level, tag, group, header, _, _, daily, unique_id = GetQuestLogTitle(id)
			if header then
				zone = lib:GetQuestCategoryKey(name);
			else

				-- update questzone
				lib:UpdateQuestZone(unique_id, zone)

				local q = {}
				q.L = level
				q.H = level
				q.M = level

				if group > 0 then
					q.group = group
				end

				if tag then
					q.tag = tag
					if  tag == GROUP then
						q.T = "G"
					elseif  tag == ELITE  then
							q.T = "E"
					elseif  tag == ITEM_HEROIC then
							q.T = "H"
					elseif  tag == RAID  then
							q.T = "R"
					elseif  tag == PVP  then
							q.T = "P"
					elseif  tag == LFG_TYPE_DUNGEON then
							q.T = "D"
					end
				end
				q.Y = daily and 1 or nil
				q.S = "S" .. MINOR_VERSION

				q.O = GetNumQuestLeaderBoards(id)
				if q.O and q.O > 0 then
					for o = 1, q.O do
						local otype = select(2,GetQuestLogLeaderBoard(o, id))
						q["o" .. o] = strupper(tostring(otype):sub(1,1))
					end
				end

				-- update QuestDB
				lib:UpdateQuestData(unique_id, lib:PackQuestData(q, unique_id))
			end
		end
		SelectQuestLogEntry(selectionId)
	end

	update_is_running = false
end

function lib:QUEST_COMPLETE()
	local questname = GetTitleText()

	lib.lastquest.qID = "unknown"
	lib.lastquest.name = questname
	lib.lastquest.chatname = string.format(ERR_QUEST_COMPLETE_S , questname)


	-- search for quest in questlog
	for uid, questdata in pairs(libQ.quests) do
		if questdata.title == questname then
			lib.lastquest.qID = uid
			break
		end
	end

	-- we don't have quest in questlog, search in found table
	if lib.lastquest.qID == "unknown" then
		for k, questdata in pairs(lib.db.global.questfound) do
			if questdata.name == questname then
				lib.lastquest.qID = questdata.id
				break
			end
		end
	end
end

-- Comms:
-- Someone joins party -> version info exchanged -> quest data shared

local last_incompatible_version = 0

--lib.party = lib.party or {}
--lib.party_quests = lib.party_quests or {}

local ChatThrottleLib = ChatThrottleLib
function lib:SendAddonMessage(contents, distribution, target, priority)
	if ChatThrottleLib and not lib.disable_comms then
		ChatThrottleLib:SendAddonMessage(priority or "NORMAL", "LibQuestInfo", contents, distribution or "PARTY", target);
	end
end

--/dump LibStub("LibQuixote-2.0").party_quests
function lib:PARTY_MEMBERS_CHANGED()
	local p = new()
	local sent, grouptype
--	print("Normal: P" .. GetNumPartyMembers() .. " R" .. GetNumRaidMembers() .. " Real: P" .. GetRealNumPartyMembers() .. " R" .. GetRealNumRaidMembers())
	if UnitInBattleground("player") then
		lib.inBG = true
	else
		lib.inBG = nil
	end

	if ( GetRealNumRaidMembers() > 0 ) then
		lib.grouptype = "raid"
		print("in Raid" .. GetRealNumRaidMembers() .. " player(s)")
		if GetRealNumRaidMembers() ~= lib.NumPartyPlayer then
			lib.NumPartyPlayer = 40;
		end
	elseif ( GetRealNumPartyMembers() > 0 ) then
		lib.grouptype = "party"
		print("in Party " .. GetRealNumPartyMembers() .. " player(s)")
		if GetRealNumPartyMembers() ~= lib.NumPartyPlayer then
			lib.NumPartyPlayer = GetRealNumPartyMembers();
		end
	else
		lib.grouptype = nil;
		print("not in Raid or Group")
		lib.NumPartyPlayer = 0
	end

	if not lib.inBG and lib.grouptype and lib.NumPartyPlayer > 0 then
		for i=1, lib.NumPartyPlayer do
			local name, realm = UnitName(lib.grouptype..i)
			if realm  and realm ~= "" then
				--print(realm)
				name = name .."-" .. realm
			end
--			print(tostring(lib.grouptype) .. i .. " - " .. tostring(name))
			if name then
				p[name] = true
				if lib.party[name] == nil and name ~= UNKNOWN then
					lib:SendAddonMessage("v"..MINOR_VERSION, "WHISPER", name)
					lib.party[name] = false -- to prevent spamming with pointless version data
		--			lib.party_quests[name] = new()
		--			lib.party_quests[name].waiting = true
				end
			end
		end
		for known in pairs(lib.party) do
			if not p[known] then
				lib.party[known] = deepDel(lib.party[known])
	--			lib.party_quests[known] = deepDel(lib.party_quests[known])
			end
		end
		del(p)
	end
end

local commhandlers = {
	--version
	--  implicit sync request; 'v' is never broadcast to the party, only to unknown party members
	--  upon receiving 'v', begin a sync to the sender
	v = function(msg, distribution, sender)
		local version = tonumber(msg)
		if version <= last_incompatible_version then
			lib.callbacks:Fire("Incompatible_Version", sender, version)
		end
		if version > MINOR_VERSION then
			lib.callbacks:Fire("New_Version", sender, version)
		end
		lib.worth_sending = true

		lib.party[sender] = version
		--And sync:
		for _, uid in pairs(lib.quest_ids) do
			lib:SendAddonMessage("q"..uid, "WHISPER", sender, "BULK")
			if lib.quest_objectives[uid] then
				for desc, o in pairs(lib.quest_objectives[uid]) do
					local isrep = o[3] == "reputation"
					local got = isrep and lib:GetReactionLevel(o[1]) or o[1] or '1'
					local need = isrep and lib:GetReactionLevel(o[2]) or o[2] or '1'
					lib:SendAddonMessage("o"..uid.."~"..desc.."~"..(isrep and "r" or "")..got.."/"..need, "WHISPER", sender, "BULK")
				end
			end
			lib:SendAddonMessage("r"..uid, "WHISPER", sender, "BULK")
		end
		lib:SendAddonMessage("d", "WHISPER", sender, "BULK")
	end,
	--quest gained
	q = function(msg, distribution, sender)
		local uid = tonumber(msg)
		if not lib.party_quests[sender] then return end
		local name = lib.UID_to_name[uid] -- Forces the tooltip request.
		lib.party_quests[sender][uid] = lib.party_quests[sender][uid] or new()
	end,
	--objective update
	o = function(msg, distribution, sender)
		local uid, objective, isrep, got, need = msg:match("^(.-)~(.-)~(r?)(%d-)/(%d-)$")
		uid = tonumber(uid)
		if not (lib.party_quests[sender] and lib.party_quests[sender][uid]) then return end
		local o = lib.party_quests[sender][uid][objective]
		isrep = isrep == 'r'
		got = isrep and lib:GetReactionName(tonumber(got)) or tonumber(got)
		need = isrep and lib:GetReactionName(tonumber(need)) or tonumber(need)
		if o then
			o[1] = got
			o[2] = need
			o[3] = isrep
			if not lib.party_quests[sender].waiting then
				lib.callbacks:Fire("Party_Objective_Update", sender, uid, lib.UID_to_name[uid], objective, got, need, isrep)
				lib.callbacks:Fire("Party_Update", sender, uid)
			end
		else
			lib.party_quests[sender][uid][objective] = new(got, need, isrep)
		end
	end,
	--sending a quest is complete
	r = function(msg, distribution, sender)
		local uid = tonumber(msg)
		if not (lib.party_quests[sender] and lib.party_quests[sender][uid]) then return end
		if not lib.party_quests[sender].waiting then
			lib.callbacks:Fire("Party_Quest_Gained", sender, uid, lib.UID_to_name[uid])
			lib.callbacks:Fire("Party_Update", sender, uid)
		end
	end,
	--quest finished
	c = function(msg, distribution, sender)
		local uid = tonumber(msg)
		lib.callbacks:Fire("Party_Quest_Complete", sender, uid, lib.UID_to_name[uid])
		lib.callbacks:Fire("Party_Update", sender, uid)
	end,
	--quest failed
	f = function(msg, distribution, sender)
		local uid = tonumber(msg)
		lib.callbacks:Fire("Party_Quest_Failed", sender, uid, lib.UID_to_name[uid])
		lib.callbacks:Fire("Party_Update", sender, uid)
	end,
	--quest lost
	l = function(msg, distribution, sender)
		local uid = tonumber(msg)
		if not (lib.party_quests[sender] and lib.party_quests[sender][uid]) then return end
		lib.callbacks:Fire("Party_Quest_Lost", sender, uid, lib.UID_to_name[uid])
		lib.callbacks:Fire("Party_Update", sender, uid)
		lib.party_quests[sender][uid] = deepDel(lib.party_quests[sender][uid])
	end,
	--sync done
	d = function(msg, distribution, sender)
		if not lib.party_quests[sender] then return end
		lib.party_quests[sender].waiting = nil
		lib.callbacks:Fire("Sync_Finished", sender)
		lib.callbacks:Fire("Party_Update", sender)
	end,
}

function lib:CHAT_MSG_ADDON(prefix, msg, distribution, sender)
	if prefix ~= "Quixote2" or sender == playerName or lib.disable_comms then return end
	--LibStub("AceConsole-2.0"):PrintLiteral(msg, distribution, sender)

	local commtype, remainder = msg:match("^(%a)(.*)")
	if (not commtype) or (not commhandlers[commtype]) then return end

	--/dump LibStub("LibQuixote-2.0").party_quests.Alibank
	commhandlers[commtype](remainder, distribution, sender)
end

function lib:CHAT_MSG_SYSTEM()

end

--
-- Debug Frame Events
-- @debug@

local Updatetable
function LibQuestInfoScrollFrame_OnLoad(frame)
	HybridScrollFrame_CreateButtons(frame, "LibQuestInfoButtonTemplate");
	local width = frame:GetWidth()
	frame.update = function() LibQuestInfoScrollFrame_Update() end

	for i=1, #frame.buttons do
		local button = frame.buttons[i]
		button:Reset()
		button:SetText(button:GetName())
		button:SetWidth(width)
		button:SetScript("OnClick", function(button)
			if button.HeaderKey then
				if Updatetable.Zonestate[button.HeaderKey] == 1 then
					Updatetable.Zonestate[button.HeaderKey] = 0
				else
					Updatetable.Zonestate[button.HeaderKey] = 1
				end
				LibQuestInfoScrollFrame_Update(1)
				return
			end

			if button.qID then
				LibQuestInfoLoadQuest(button.qID)
				LibQuestInfoFrameHighlight:Show()
				LibQuestInfoFrameHighlight:SetPoint("TOPLEFT", button, "TOPLEFT",0,0)
				LibQuestInfoFrameHighlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT",0,0)
				LibQuestInfoFrameHighlight.qid = button.qID;
			end

		end)
	end

	local buttonheigth = frame.buttons[1]:GetHeight()
	HybridScrollFrame_Update(frame, #frame.buttons * buttonheigth, #frame.buttons * buttonheigth);
end

function LibQuestInfoLoadQuest(qID)
	local QuestFrame = LibQuestInfoFrameEditFrame

	-- prepare default state
	-- hide  objectives
	for i = 1, 10, 1 do
		QuestFrame["OLabel" .. i]:Hide()
		QuestFrame["OBtn" .. i]:Hide()
	end

	QuestFrame.EditMinLevel:Hide()
	QuestFrame.MinLevelLabel:SetFontObject(GameFontDisableSmallLeft)


	--print("you clicked on qid: " .. qID)
	if not QuestFrame.Questdata or QuestFrame.Questdata.Q ~= qID then
		local questdata = string.match(QuestDB[ceil(qID/100)], "|(Q"..qID..";.-|)")
		QuestFrame.Questdata = lib:UnpackQuestData(questdata)
		QuestFrame.Questdata.Q = qID
	end

	ShowUIPanel(ItemRefTooltip);
	ItemRefTooltip:SetOwner(LibQuestInfoFrame, "ANCHOR_BOTTOM");
	ItemRefTooltip:SetHyperlink("quest:" .. qID)
	--ItemRefTooltip:Show()

	if QuestFrame.Questdata.O and QuestFrame.Questdata.O > 0 then
		for i = 1, QuestFrame.Questdata.O, 1 do
			QuestFrame["OLabel" .. i]:Show()
			QuestFrame["OBtn" .. i]:Show()
			QuestFrame["OBtn" .. i]:SetText(ObjectiveTypes[QuestFrame.Questdata["o" .. i]] or "Unknown")
		end
	end

	if QuestFrame.Questdata.T then
		QuestFrame.BtnTag:SetText(TagTypes[QuestFrame.Questdata.T] or "Unknown")
	else
		QuestFrame.BtnTag:SetText("None")
	end
	QuestFrame:Show()
end

function LibQuestInfoScrollFrame_Update(force)
	-- refresh datatable
	if not Updatetable or Updatetable.NextUpdate < time() or force then
		if not Updatetable then
			-- first time, create defaults
			Updatetable = {
				NextUpdate = time() + 300,
				Zones = {},
				Zonestate = {},
			}
		else
			for k in pairs(Updatetable.Zones) do
				Updatetable.Zones[k] = nil
			end
		end

		Updatetable.NextUpdate = time() + 300

		for zonekey in pairs(QuestZones) do
			-- zone
			local zonename = lib:GetQuestCategoryName(zonekey)

			if not Updatetable.Zonestate[zonename] then
				Updatetable.Zonestate[zonename] = 1
			else
				if Updatetable.Zonestate[zonename] ~= 1 then
					if not ZoneCache[zonename] then
						-- unserialize zonedata to display it
						ZoneCache[zonekey] = {strsplit('|', QuestZones[zonekey])};
						for k in ipairs(ZoneCache[zonekey]) do
							ZoneCache[zonekey][k] = tonumber(ZoneCache[zonekey][k])
						end
					end
					table.sort(ZoneCache[zonekey], function(a,b)
							if (lib:GetQuestName(a,1) or "ZZZ " .. a) < (lib:GetQuestName(b,1) or "ZZZ " .. b) then
								return true
							end
							return false
						end
					)
				elseif ZoneCache[zonename] and Updatetable.Zonestate[zonename] ~= 1 then
					-- clean up old zonecache to save memory
					for k in pairs(ZoneCache[zonename]) do
						ZoneCache[zonename][k] = nil;
					end
					ZoneCache[zonename] = nil;
				end
			end

			local found
			for k, zname in pairs(Updatetable.Zones) do
				if zname == zonename then
					found = true
					break
				end
			end

			if not found then
				table.insert(Updatetable.Zones, zonename)
			end
		end

		table.sort(Updatetable.Zones)
	end


	LibQuestInfoFrameHighlight:Hide();

	local SrollFrame = LibQuestInfoFrameScrollFrame
	local buttonheigth = SrollFrame.buttons[1]:GetHeight()

	local i = -HybridScrollFrame_GetOffset(SrollFrame) + 1

	for k, zonename in pairs(Updatetable.Zones) do
		local zonekey = lib:GetQuestCategoryKey(zonename)
		if i > 0 and i <= #SrollFrame.buttons then
			local button = SrollFrame.buttons[i]
			button:Reset()
			button.HeaderKey = zonename;
			button:SetButton(button)
			button:SetText(zonename)
			button.tag:SetText("#".. (QuestZones[zonekey] and lib:CountChars(QuestZones[zonekey], "|") or 0))
			if Updatetable.Zonestate[zonename] == 1 then
				button:SetNormalTexture([[Interface\Buttons\UI-PlusButton-UP]])
			end
--			QuestLogTitleButton_Resize(button)
		end
		i = i + 1
		if Updatetable.Zonestate[zonename] ~= 1 then
			for k, qid in pairs(ZoneCache[zonekey]) do
				if i > 0 and i <= #SrollFrame.buttons then
					local button = SrollFrame.buttons[i]
					button:Reset()
					button.qID = qid
					button:SetText(lib:GetQuestName(qid) or "Quest not in Cache! (QuestID: " .. qid .. ")")
					button.tag:SetText("qID: " .. qid)
					if button.qID == LibQuestInfoFrameHighlight.qid then
						LibQuestInfoFrameHighlight:Show()
						LibQuestInfoFrameHighlight:SetPoint("TOPLEFT", button, "TOPLEFT",0,0)
						LibQuestInfoFrameHighlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT",0,0)
					end
				end
				i = i + 1
				if i > #SrollFrame.buttons then
					break;
				end
			end
			if i > #SrollFrame.buttons then
				break;
			end
		end
	end

	for j = i, #SrollFrame.buttons do
		if j > 0 then
			SrollFrame.buttons[j]:Reset(SrollFrame.buttons[j])
			SrollFrame.buttons[j]:Disable()
		end
	end

	-- calc # lines
	local totallines = 0
	for k, zonename in pairs(Updatetable.Zones) do
		local zonekey = lib:GetQuestCategoryKey(zonename)
		totallines = totallines + 1
		if Updatetable.Zonestate[zonename] ~= 1 then
			totallines = totallines + (QuestZones[zonekey] and lib:CountChars(QuestZones[zonekey], "|") or 0)
		end
	end
	HybridScrollFrame_Update(SrollFrame, (totallines)  * buttonheigth, #SrollFrame.buttons * buttonheigth);


end

function LibQuestInfoFrame_OnLoad(self)
	self:SetScript("OnUpdate", function(self, elapsed)
		if not self.updatetime or self.updatetime < 0 then
			self.updatetime = 1
		end

		self.updatetime = self.updatetime - elapsed

		if self.updatetime > 0 then
			return
		end
		LibQuestInfoScrollFrame_Update()
	end)
end

function LibQuestInfoSelectFrame_OnShow(frame)

	if frame.mode == "Zone" then
		frame.Btn1:Hide()
		frame.Btn2:Hide()
		frame.Btn3:Hide()
		frame.Btn4:Hide()
		frame.Btn5:Hide()
		frame.Btn6:Hide()
		frame.Btn7:Hide()
		--frame.Zones:Show()
	else
		frame.Btn1:Show()
		frame.Btn1:SetText("None")
		frame.Btn2:Show()
		frame.Btn3:Show()
		frame.Btn4:Show()
		frame.Btn5:Show()
		frame.Btn6:Show()
		frame.Btn7:Show()
		frame:SetWidth(160)
		frame:SetHeight(135)
		--frame.Zones:Hide()
	end

	frame:ClearAllPoints()


	if frame.mode == "Tag" then
		frame:SetPoint("CENTER", frame:GetParent().BtnTag ,"CENTER",0,0)
		local i = 2
		for k, v in pairs(TagTypes) do
			frame["Btn" .. i]:SetText(v)
			frame["Btn" .. i].id = k
			i = i + 1
		end
	elseif frame.mode == "Zone" then
		frame:SetPoint("CENTER", frame:GetParent():GetParent() ,"CENTER",0,0)
		frame:SetHeight(frame:GetParent():GetParent():GetHeight())
		frame:SetWidth(frame:GetParent():GetParent():GetWidth())
	elseif frame.mode:sub(1,9) == "Objective" then
		local id = tonumber(frame.mode:sub(10))
		frame:SetID(id)
		frame:SetPoint("CENTER", frame:GetParent()["OBtn" .. id] ,"CENTER",0,0)
		local i = 2
		for k, v in pairs(ObjectiveTypes) do
			frame["Btn" .. i]:SetText(v)
			frame["Btn" .. i].id = k
			i = i + 1
		end
	end
--	print("show " .. frame:GetName())
end

function LibQuestInfoSelectFrameButton_OnClick(button)
	local SelectFrame = _G["LibQuestInfoFrameEditFrameSelectFrame"]
	local EditFrame = SelectFrame:GetParent()



	if SelectFrame.mode == "Tag" then
		EditFrame.Questdata.T = button.id
	elseif SelectFrame.mode == "Zone" then
		EditFrame.Questdata.Z = button.id
	elseif SelectFrame.mode:sub(1,9) == "Objective" then
		EditFrame.Questdata["o" .. button:GetID()] = button.id
	end

	--print("button clicked " .. EditFrame:GetName())
	SelectFrame:Hide()
	LibQuestInfoLoadQuest(EditFrame.Questdata.Q)
end

function LibQuestInfoTileButton_OnLoad(button)
	button:SetText(button:GetName())
	button.tag:Show()
	button.Reset = function(button)
		button:Enable()
		button:SetText("")
		button.tag:SetText("")
		button:SetNormalTexture("")
		button:SetHighlightTexture("")
		button:SetNormalFontObject(GameFontNormalLeft)
		button.HeaderKey = nil
		button.qID = nil
	end
	button.SetButton = function(button)
		button:SetNormalFontObject(GameFontHighlight)
		button:SetNormalTexture([[Interface\Buttons\UI-MinusButton-UP]])
		button:SetHighlightTexture([[Interface\Buttons\UI-PlusButton-Hilight]])
	end
end

function lib_cleanup()
	for k in pairs(beql.db.global.QuestData) do
		if beql.db.global.QuestData[k].O then
			for i = beql.db.global.QuestData[k].O + 1, 30 do
				beql.db.global.QuestData[k]["o" .. i] = nil
			end
		end

		if beql.db.global.QuestData[k].T then
			local flags = {strsplit(":", beql.db.global.QuestData[k].T)}
			if #flags > 1 then
				for m, v in pairs(flags) do
					if v == "Y" then
						beql.db.global.QuestData[k].Y = 1
					else
						beql.db.global.QuestData[k].T = v
					end
				end
			end
		end

		if beql.db.global.QuestData[k].zone then
			if not beql.db.global.QuestData[k].Z or beql.db.global.QuestData[k].Z == beql.db.global.QuestData[k].zone then
				beql.db.global.QuestData[k].Z = beql.db.global.QuestData[k].zone
				beql.db.global.QuestData[k].zone = nil
			end
		end

		if beql.db.global.QuestData[k].Z then
			lib.callbacks:Fire("ZoneUpdate", k, beql.db.global.QuestData[k].Z)
		else
			lib.callbacks:Fire("ZoneUpdate", k, "UNKNOWN")
		end

		if beql.db.global.QuestData[k].queststring then
			local q = lib:UnpackQuestData(beql.db.global.QuestData[k].queststring)
			for l in pairs(q) do
				if not beql.db.global.QuestData[k][l] or beql.db.global.QuestData[k][l] ~= q[l] then
					beql.db.global.QuestData[k][l] = q[l]
				end
			end
		end

		beql.db.global.QuestData[k].serial = lib:PackQuestData(beql.db.global.QuestData[k], k)
		lib.callbacks:Fire("QuestUpdate", k, beql.db.global.QuestData[k].serial)
	end
end

--@end-debug@
-- internal functions
--

--[[
	pack questdata into a string to save memory
	--]]

function lib:PackQuestData(datatable, qid)

	local queststring = "Q"..tostring(qid) .. ";"

	for _, k in ipairs(datakeys) do
		--print(k .. " " .. v)
		if datatable[k] then
			if k == "O" and datatable[k] > 0 then
				queststring = queststring .. "O." .. tostring(datatable[k]) .. ";"
				for i=1, datatable[k] do
					--print(i .. " " .. tostring(i) .. " " .. tostring(datatable["o"..i]))
					queststring = queststring .. "o" .. tostring(i) .. "."..  tostring(datatable["o"..i]) .. ";"
				end
			elseif k =="T" then
				if datatable[k] == "G" and datatable.group and datatable.group > 0 then
					queststring = queststring .. k .."." .. tostring(datatable[k]) .. "." .. tostring(datatable.group) .. ";"
				else
					queststring = queststring .. k .."." .. tostring(datatable[k]) .. ";"
				end
			elseif k == "L" or (k == "M" and datatable.L ~= datatable.M) or (k == "H" and datatable.L ~= datatable.H) or k == "Y" or k =="W" or k == "S" then
				queststring = queststring .. k .."." .. tostring(datatable[k]) .. ";"
			end
		end
	end
	return queststring .. "|"
end

function lib:UnpackQuestData(datastring)
	local tmptable = {strsplit(";", datastring)}

	local datatable = {}
	for _, v in ipairs(tmptable) do
		local key, data, group = strsplit(".", v)
		if group then
			datatable["group"] = tonumber(group)
		end

		if key == "L" or key == "M" or key == "O" or key == "H" then
			datatable[key] = tonumber(data)
		else
			datatable[key] = data
		end

	end

	if not datatable.S then
		datatable.S = "E" .. MINOR_VERSION
	end

	-- create level dumy's
	if not datatable.M and datatable.L then
		datatable.M = datatable.L
	end
	if not datatable.H and datatable.L then
		datatable.H = datatable.L
	end

	return datatable
end

function lib:LoadQuestData(qid)
	if not qid or type(qid) ~= 'number' then
		assert(false, 'usage: LibQuestInfo:LoadQuestData(QuestID)')
	end

	if not questcache[qid] or not questcache[qid] then
		-- get Datastring from DB
		local questdata = string.match(QuestDB[ceil(qid/100)], "|Q"..qid..";(.-)|")

		if not questdata then
			return false
		end

		questcache[qid] = lib:UnpackQuestData(questdata)
		questcache[qid].L = questcache[qid].L or -2

		if questcache[qid].L == -1 then
			if questcache[qid].M and UnitLevel("player") < questcache[qid].M then
				questcache[qid].L = questcache[qid].M
			else
				questcache[qid].L = UnitLevel("player")
			end
		end
	end

	return true

end

local priotable = { E = -1, P = 0, S = 1}

function lib:UpdateQuestData(qid, newquestdata)
	if not QuestDB[ceil(qid/100)] then
		QuestDB[ceil(qid/100)] = '';
	end

	if string.match(QuestDB[ceil(qid/100)], newquestdata) then
		-- quest not updated -> exit
		return false
	end

	local oldquestdata = string.match(QuestDB[ceil(qid/100)], "|(Q"..qid..";.-|)")

		-- quest doesn't exist in quest db, insert and exit
	if not oldquestdata then
		QuestDB[ceil(qid/100)] = QuestDB[ceil(qid/100)] .. newquestdata
		lib.callbacks:Fire("QuestDataUpdate", qid, newquestdata)
		return true
	end

	local newquest = lib:UnpackQuestData(newquestdata)
	local oldquest = lib:UnpackQuestData(oldquestdata)

	-- calc priority of questdata, bases on version of lib and source
	oldquest.prio = (priotable[string.match(oldquest.S, "(%a)")] or -1) + (string.match(oldquest.S, "(%d)") or 0)
	newquest.prio = (priotable[string.match(newquest.S, "(%a)")] or -1) + (string.match(newquest.S, "(%d)") or 0)

	local quest, priolevel
	if oldquest.prio > newquest.prio then
		priolevel = oldquest.prio - newquest.prio
		if priolevel == 1 then
			-- data is not to old, do a key based comparsion
			quest = CopyTable(oldquest)
		else
			return false
		end

	else
		priolevel = newquest.prio - oldquest.prio
		if priolevel == 1 or priolevel == 0 then
			-- data be is not to old, do a key based comparsion
			quest = CopyTable(newquest)
		else
			-- new data is newer, insert and exit
			QuestDB[ceil(qid/100)] = QuestDB[ceil(qid/100)] .. newquestdata
			lib.callbacks:Fire("QuestDataUpdate", qid, newquestdata)
			return true
		end
	end


	local changed

	print("Do extended Check for " .. qid)
	for _, key in ipairs({"O","L","M","H","Y","W","T","group"}) do
		if newquest[key] ~= oldquest[key] then
			if not changed then
				changed = {}
			end
			if key == "M" or key == "H" then
				changed.L = true
			else
				changed[key] = true
			end
		end
	end

	-- data not changed, but order is different
	if not changed then
		print("Data for " .. qid .. " not changed (" .. tostring(changed) .. ")")
		return false
	end




	--print(prolevel)
	for k in pairs(changed) do
		-- level
		if k == "L" then
			if not newquest.L or not oldquest.L then
				-- one quest has no leveldata, use leveldata from existning quest
				quest.L = newquest.L or oldquest.L
			else
				-- both quest have a level, set min/max questlevel
				-- this have nothing to do with gained XP Range
				quest.L = -1
				quest.M = min(oldquest.M, newquest.M)
				quest.H = max(oldquest.H, newquest.H)
			end
		elseif priolevel <= 1 then
			quest[k] = oldquest[k] or newquest[k] or nil
		end
	end



	lib.callbacks:Fire("QuestDataUpdate", qid, lib:PackQuestData(quest))

	return true
end

function lib:UpdateQuestZone(qid, newzonekey, oldzonekey)

	local changed

	if not oldzonekey or not (string.match(QuestZones[oldzonekey], "^" .. qid .."|") or string.match(QuestZones[oldzonekey], "|" .. qid .."|")) then
		for k in pairs(QuestZones) do
			if string.match(QuestZones[k], "^" .. qid .."|") or string.match(QuestZones[k], "|" .. qid .."|") then
				if newzonekey ~= k then
					--print("Update Questzone: " .. qid .. " OldZone: " .. k .. " Newzone: " .. newzonekey )
					changed = true;
				end
			end
		end
	end

	if not changed then
		return false
	end
end

function lib:CountChars (haystack, needle)
    haystack = strlower(tostring(haystack));
    needle = strlower(tostring(needle));
    local ret, pos = 0, 0
    for i = 1, strlen(haystack) do
        local s, e = strfind(haystack, needle, pos)
        if e then
            pos = e
            if s == e then
                pos = pos + 1
            end
        end
        if s and e then
            ret = ret + 1;
        else
            break -- If it hasn't found any more, there's no reason to continue
        end
    end
    return ret
end

-- Public API:

function lib:GetZonebyQid(qid)

	local zonekey--, zonename
	for k in pairs(QuestZones) do
		if string.match("|" .. QuestZones[k], "|"..qid.."|") then
			zonekey = k;
			--zonename = lib:GetQuestCategoryName(zonekey);
			return zonekey--, zonename;
		end
	end

	return nil, nil;
end

function lib:GetData(data)
	local qid, questdata = sting.match(data, "^Q(%d*);(.*)")


end

function lib:GetQuestLogTitleByID(qid, silent)
	if not qid or type(qid) ~= 'number' then
		if not silent then
			assert(false, "QuestID is not a number")
		end
		return
	end

	lib:LoadQuestData(qid)

	-- questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily
	return lib:GetQuestName(qid), questcache[qid].L, questcache[qid].T and TagTypes[questcache[qid].T] or nil, questcache[qid].group or 0, nil, nil, nil, questcache[qid].Y, qid
end

LQI = {
	k = {},
	z = {},
}

function lib:GetQuestCategoryKey(zonename)
	local keytype, keyname
	-- zone check
	if ZoneKeys[zonename] then
		return ZoneKeys[zonename]
	end

	-- check non zone key
	if NonZoneKeys[zonename] then
		return NonZoneKeys[zonename]
	end

	if ClassKeys[zonename] then
		return ClassKeys[zonename]
	end

	if not LQI.z[zonename] then
		LQI.z[zonename] = 1
	end


	--print("Unknown Zonename " .. zonename)
	return zonename
end

function lib:GetQuestCategoryName(zonekeyname)
	if ZoneNames[zonekeyname] then
		return ZoneNames[zonekeyname]
	end

	if NonZoneNames[zonekeyname] then
		return NonZoneNames[zonekeyname]
	end

	if ClassNames[zonekeyname] then
		return ClassNames[zonekeyname]
	end


	if string.gsub(zonekeyname, "UNKNOWN_", "") then
		return string.gsub(zonekeyname, "UNKNOWN_", "")
	end

	if not LQI.k[zonekeyname] then
		LQI.k[zonekeyname] = 1
	end
--	print("Unknown Zonekey " .. zonekeyname)
	return zonekeyname
end

function lib:GetQuestName(qid, tmp)
	if not questcache[qid] or not questcache[qid].title then
		if not tmp then
			if not loadqueue[qid] then
				loadqueue[qid] = true
			end
			lib.frame:Show()
			return nil
		end
		return nil
	else
		return questcache[qid].title
	end
end

-- Get a numeric equivalent to a reaction level, from Hated=1 to Exalted=8.
do
	local reactions = {
		[FACTION_STANDING_LABEL1] = 1,
		[FACTION_STANDING_LABEL1_FEMALE] = 1,
		[FACTION_STANDING_LABEL2] = 2,
		[FACTION_STANDING_LABEL2_FEMALE] = 2,
		[FACTION_STANDING_LABEL3] = 3,
		[FACTION_STANDING_LABEL3_FEMALE] = 3,
		[FACTION_STANDING_LABEL4] = 4,
		[FACTION_STANDING_LABEL4_FEMALE] = 4,
		[FACTION_STANDING_LABEL5] = 5,
		[FACTION_STANDING_LABEL5_FEMALE] = 5,
		[FACTION_STANDING_LABEL6] = 6,
		[FACTION_STANDING_LABEL6_FEMALE] = 6,
		[FACTION_STANDING_LABEL7] = 7,
		[FACTION_STANDING_LABEL7_FEMALE] = 7,
		[FACTION_STANDING_LABEL8] = 8,
		[FACTION_STANDING_LABEL8_FEMALE] = 8,
	}
	--------------------------------------------
	-- Arguments:
	-- string - faction standing text
	--
	-- Notes:
	-- Returns a numeric-equivalent for faction standings.
	--
	-- Returns:
	-- * number - number from 1 to 8 or -1 if invalid
	--------------------------------------------
	function lib:GetReactionLevel(leveltext)
		return leveltext and reactions[leveltext] or -1
	end

	function lib:GetReactionName(level)
		for react, index in pairs(reactions) do
			if index == level then
				return react
			end
		end
	end
end
